/*globals define, Image*/

define([
    "testsuite/qunit",
    "testsuite/qunit-fixture!de_epages/presentation/ui/unit-selector-test",
    "testsuite/sinon",
    "jquery",
    "ep",
    "de_epages/presentation/ui/unit-selector",
    "util/string",

    "$ready!"
], function(QUnit, fixture, sinon, $, ep, de_epages, str) {
    var getUnsortedList = function(searchList) {
            var ret;

            for(var i=0; i< searchList.length; i++) {
                if (searchList[i].nodeName && searchList[i].nodeName.toLowerCase() === "ul") {
                    // $ul = $(searchList[i]);
                    ret = $(searchList[i]);
                    break;
                }
            }

            return ret;
        },

        methods = [
                "_create",
                "_build",
                "_setOption",
                "handleBgClass",
                "getSelectedUnit",
                "removeCallback",
                "destroy"
            ];

    // set module wide fixture property
    QUnit.fixtureVar = fixture;

    QUnit.module("de_epages/presentation/ui/unit-selector", {
        setup: fixture.init,
        teardown: fixture.destroy
    });

    QUnit.asyncTest("widget test presence of methods", methods.length, function() {
        $.each(
            methods,
            function(i, fn) {
                QUnit.equal(
                    $.isFunction($.ui.presentationUnitSelector.prototype[fn]),
                    true,
                    "$.isFunction(" + fn + "):"
                );
            }
        );

        QUnit.start();
    });

    QUnit.asyncTest(
        "de_epages/presentation/ui/unit-selector - Test basic widget functionality and selected unit (using OrderUnit)",
        function() {
            /* !!! FIXTURE - comment out next lines if fixture is used !!! */
            // clear HTML fragments and initiate fixture (if fixture is used)
            $("#qunit-fixture")
                // move into view and make visible -> otherwise focus event will not work!!!
                .css({
                    'position': 'static',
                    'opacity': 0
                })
                .nextAll().remove();
            QUnit.fixtureVar.init();

            var $unitInput = ep("#" + "OrderUnitId"),
                unitName = "OrderUnit",
                unitId = "265",
                $ul,
                selectedItem,
                hiddenInput,
                initSpy,
                buildSpy,
                destroySpy;

            // register spies on widget methods => number of executions can be investigated
            initSpy = sinon.spy(
                $.ui.presentationUnitSelector.prototype,
                "_create"
            );
            buildSpy = sinon.spy(
                $.ui.presentationUnitSelector.prototype,
                "_build"
            );
            destroySpy = sinon.spy(
                $.ui.presentationUnitSelector.prototype,
                "destroy"
            );

            QUnit.equal(
                initSpy.callCount,
                0,
                "*_create* method to initialize widget was not called until yet."
            );
            QUnit.equal(
                buildSpy.callCount,
                0,
                "*_build* method to create the UI was not called until yet."
            );

            // wait until widget is instantiated
            $unitInput.presentationUnitSelector({
                unitHidden: {
                    name: unitName,
                    value: unitId
                },
                bgClass: "ep-ui-autocomplete-input-background",
                addClass: true
            });

            window.setTimeout(function() {
                hiddenInput = $("#" + unitName);

                $ul = getUnsortedList($unitInput.data("uiAutocomplete").bindings);

                QUnit.equal(
                    !!$unitInput.presentationUnitSelector("instance"),
                    true,
                    "presentationUnitSelector widget instance exists."
                );

                QUnit.equal(
                    !!$unitInput.autocomplete("instance"),
                    true,
                    "autocomplete widget instance exists."
                );

                QUnit.equal(
                    !!$unitInput.presentationUiInputaction("instance"),
                    true,
                    "presentationUiInputaction widget instance exists."
                );

                QUnit.equal(
                    initSpy.callCount,
                    1,
                    "*_create* method to initialize widget was called once."
                );
                QUnit.equal(
                    buildSpy.callCount,
                    1,
                    "*_build* method to create the UI was called once."
                );

                QUnit.equal(
                    !!(
                        $._data($unitInput[0], "events") &&
                        $._data($unitInput[0], "events").keydown
                    ),
                    true,
                    "keydown event was successfully registered on the input element."
                );
                QUnit.equal(
                    !!(
                        $._data($unitInput[0], "events") &&
                        $._data($unitInput[0], "events").focus
                    ),
                    true,
                    "focus event was successfully registered on the input element."
                );

                QUnit.ok(
                    hiddenInput.length === 1 &&
                        hiddenInput.attr("type") === "hidden",
                    'A hidden input element with name "' +
                        unitName +
                        '" was created and added to the DOM.'
                );

                QUnit.equal(
                    hiddenInput.val(),
                    unitId,
                    'Hidden input contains the unit ID "' +
                        unitId +
                        '" as value.'
                );

                // trigger events to fill autocomplete with values
                $unitInput.trigger("focus.unitSelector");

                window.setTimeout(function() {
                    // needs to be triggered twice ... don't know why?!
                    $unitInput.trigger("focus.unitSelector");
                    selectedItem = ep.config.units.units.filter(function(item) {
                        return item.value === hiddenInput.val();
                    })[0];

                    QUnit.equal(
                        $ul.find('li').length,
                        1,
                        "Autocompletion list contains 1 element."
                    );

                    QUnit.equal(
                        $ul.find('li')
                            .eq(0)
                            .text(),
                        selectedItem.label,
                        "Selected list item contains the correct text."
                    );
                    QUnit.equal(
                        $('samp[title="' + unitName + '"]')
                            .text(),
                        selectedItem.label,
                        "Label samp also contains the correct text."
                    );

                    $unitInput.presentationUnitSelector("destroy");

                    window.setTimeout(function() {
                        QUnit.equal(
                            destroySpy.callCount,
                            1,
                            "*destroy* method to initialize widget was called once."
                        );

                        QUnit.equal(
                            $unitInput.data("uiPresentationUnitSelector") === undefined,
                            true,
                            "presentationUnitSelector widget instance was destroyed."
                        );
                        QUnit.equal(
                            $unitInput.data("uiPresentationUiInputaction") === undefined,
                            true,
                            "presentationUiInputaction widget instance was destroyed."
                        );
                        QUnit.equal(
                            $unitInput.data("uiAutocomplete") === undefined,
                            true,
                            "Autocomplete widget instance was destroyed."
                        );
                        QUnit.equal(
                            $("#" + unitName).length,
                            0,
                            "Hidden input was removed from DOM."
                        );

                        QUnit.equal(
                            !!(
                                $._data($unitInput[0], "events") &&
                                $._data($unitInput[0], "events").keydown
                            ),
                            false,
                            "keydown event was successfully removed from the input element by *destroy*."
                        );
                        QUnit.equal(
                            !!(
                                $._data($unitInput[0], "events") &&
                                $._data($unitInput[0], "events").focus
                            ),
                            false,
                            "focus event was successfully removed from the input element by *destroy*."
                        );

                        QUnit.start();
                    }, 100);
                }, 100);
            }, 300);
        }
    );

    QUnit.asyncTest(
        "de_epages/presentation/ui/unit-selector - Test widget without selected unit (using RefUnit)",
        function() {
            /* !!! FIXTURE - comment out next lines if fixture is used !!! */
            // clear HTML fragments and initiate fixture (if fixture is used)
            $("#qunit-fixture")
                // move into view and make visible -> otherwise focus event will not work!!!
                .css({
                    'position': 'static',
                    'opacity': 0
                })
                .nextAll().remove();
            QUnit.fixtureVar.init();

            var $unitInput = ep("#" + "RefUnitId"),
                unitName = "RefUnit",
                unitId = "265",
                $ul,
                selectedItem,
                hiddenInput;

            // wait until widget is instantiated
            $unitInput.presentationUnitSelector({
                unitHidden: {
                    name: unitName,
                    // value: unitId
                },
                bgClass: "ep-ui-autocomplete-input-background",
                addClass: true
            });

            window.setTimeout(function() {
                hiddenInput = $("#" + unitName);

                $ul = getUnsortedList($unitInput.data("uiAutocomplete").bindings);

                QUnit.equal(
                    hiddenInput.val().length,
                    0,
                    'Hidden input contains no value.'
                );

                // trigger events to fill autocomplete with values
                $unitInput.trigger("focus.unitSelector");

                window.setTimeout(function() {
                    // needs to be triggered twice ... don't know why?!
                    $unitInput.trigger("focus.unitSelector");
                    QUnit.equal(
                        $ul.find('li').length,
                        ep.config.units.units.length,
                        "Autocompletion list without a search pattern contains all " + ep.config.units.units.length + " elements."
                    );

                    QUnit.equal(
                        $('samp[title="' + unitName + '"]')
                            .text().length,
                        0,
                        "Label samp is empty."
                    );

                    $unitInput.presentationUnitSelector("destroy");

                    QUnit.start();
                }, 100);
            }, 300);
        }
    );

    QUnit.asyncTest(
        "de_epages/presentation/ui/unit-selector - Test public methods",
        function() {
            /* !!! FIXTURE - comment out next lines if fixture is used !!! */
            // clear HTML fragments and initiate fixture (if fixture is used)
            $("#qunit-fixture").nextAll().remove();
            QUnit.fixtureVar.init();

            var $unitInput = ep("#" + "OrderUnitId"),
                unitName = "OrderUnit",
                unitId = "265",
                bgClass = "test-bg",
                selectedItem,
                hiddenInput;

            // wait until widget is instantiated
            $unitInput.presentationUnitSelector({
                unitHidden: {
                    name: unitName,
                    value: unitId
                },
                bgClass: bgClass,
                addClass: false,
                testCallbackFn: function() {
                    console.log("I am a test callback!");
                }
            });

            window.setTimeout(function() {
                hiddenInput = $("#" + unitName);
                // get selected item from test data
                selectedItem = ep.config.units.units.filter(function(item) {
                    return item.value === hiddenInput.val();
                })[0];

                // get selected item using public method
                QUnit.equal(
                    $unitInput.presentationUnitSelector("getSelectedUnit"),
                    selectedItem,
                    "Public method *getSelectedUnit* returns the correct selected unit."
                );

                // set background class
                $unitInput.presentationUnitSelector("handleBgClass", true);
                QUnit.ok(
                    $unitInput.hasClass(bgClass),
                    "Correct background class was added."
                );

                // unset background class
                $unitInput.presentationUnitSelector("handleBgClass");
                QUnit.ok(
                    !$unitInput.hasClass(bgClass),
                    "Background class was removed."
                );

                // check existance of callback function
                QUnit.ok(
                    typeof $unitInput.data("uiPresentationUnitSelector").options.testCallbackFn === "function",
                    "Callback function is present in options."
                );
                // remove callback function
                $unitInput.presentationUnitSelector("removeCallback", "testCallbackFn");
                QUnit.ok(
                    $unitInput.data("uiPresentationUnitSelector").options.testCallbackFn === undefined,
                    "Callback function was deleted from options."
                );

                $unitInput.presentationUnitSelector("destroy");

                QUnit.start();
            }, 300);
        }
    );
});
