/*globals require, window*/

require([
    'testsuite/qunit',
    'testsuite/sinon',
    'testsuite/qunit-fixture!de_epages/product/ui/pricepreview-test',
    'jquery',
    'ep',
    'de_epages/product/ui/pricepreview',
    "$dict!de_epages/product/dictionary",

    'ep/ui/validate',
    'jquery/i18n',
    'jquery/metaparse'
], function (QUnit, sinon, fixture, $, ep, de_epages, productDict) {
    'use strict';

    /*
     * @dictionary       de_epages.product.dictionary
     *
     * @translation      {IsGross}
     *                   {IsNet}
     *                   {WithoutShipping}
     */

    /*
     * @dictionary      ep.dict
     */

    var translations = {
        priceManufacturer: productDict.translate('PriceManufacturer'),
        pricepreviewHeadline: productDict.translate('PricePreviewHeadline'),
        isGross: productDict.translate('IsGross'),
        isNet: productDict.translate('IsNet'),
        withoutShipping: productDict.translate('WithoutShipping')
    };

    var methods = [
            '_create',
            '_calcPercent',
            '_calcPrice',
            '_formatCurrency',
            '_updateOptions',
            '_findUnitByLabel',
            '_findUnitById',
            '_registerEvents',
            '_render',
            'destroy'
        ],

        refUnits = {"units":[{"value":"281","label":"Byte","abbr":"Byte","alias":"byte","show":true},{"value":"282","label":"Kilobyte","abbr":"kByte","alias":"kilobyte","show":true},{"value":"283","label":"Megabyte","abbr":"MByte","alias":"megabyte","show":true},{"value":"284","label":"Gigabyte","abbr":"GByte","alias":"gigabyte","show":true},{"value":"285","label":"Terabyte","abbr":"TByte","alias":"terrabyte","show":true},{"value":"259","label":"Gramm","abbr":"g","alias":"gram","show":true},{"value":"261","label":"Kilogramm","abbr":"kg","alias":"kilogram","show":true},{"value":"260","label":"Milligramm","abbr":"mg","alias":"milligram","show":true},{"value":"264","label":"Unze(n)","abbr":"oz","alias":"ounce","show":true},{"value":"263","label":"Pfund","abbr":"lb","alias":"pound","show":true},{"value":"262","label":"Tonne(n)","abbr":"t","alias":"tone","show":true},{"value":"250","label":"Liter","abbr":"l","alias":"litre","show":true},{"value":"254","label":"Kubikfuß","abbr":"ft³","alias":"cubicfoot","show":true},{"value":"253","label":"Kubikzoll","abbr":"in³","alias":"cubicinch","show":true},{"value":"252","label":"Kubikmeter","abbr":"m³","alias":"cubicmetre","show":true},{"value":"255","label":"Kubikyard","abbr":"yd³","alias":"cubicyard","show":true},{"value":"256","label":"Flüssigunze(n)","abbr":"fl oz","alias":"fluidounce","show":true},{"value":"257","label":"Gallonen","abbr":"gal","alias":"gallon","show":true},{"value":"251","label":"Milliliter","abbr":"ml","alias":"millilitre","show":true},{"value":"258","label":"Quart","abbr":"qt","alias":"quart","show":true},{"value":"243","label":"Meter","abbr":"m","alias":"metre","show":true},{"value":"244","label":"Zentimeter","abbr":"cm","alias":"centimetre","show":true},{"value":"247","label":"Fuß","abbr":"ft","alias":"foot","show":true},{"value":"248","label":"Zoll","abbr":"in","alias":"inch","show":true},{"value":"246","label":"Kilometer","abbr":"km","alias":"kilometre","show":true},{"value":"245","label":"Millimeter","abbr":"mm","alias":"millimetre","show":true},{"value":"249","label":"Yard","abbr":"yd","alias":"yard","show":true},{"value":"265","label":"Stück","abbr":"","alias":"piece","show":false},{"value":"267","label":"Flasche(n)","abbr":"","alias":"bottle","show":true},{"value":"268","label":"Kiste(n)","abbr":"","alias":"box","show":true},{"value":"278","label":"Dose(n)","abbr":"","alias":"can","show":true},{"value":"279","label":"Kapsel(n)","abbr":"","alias":"capsule","show":true},{"value":"269","label":"Karton(s)","abbr":"","alias":"carton","show":true},{"value":"280","label":"Glas","abbr":"","alias":"glass","show":true},{"value":"275","label":"Kit(s)","abbr":"","alias":"kit","show":true},{"value":"277","label":"Packung(en)","abbr":"","alias":"pack","show":true},{"value":"276","label":"Paket(e)","abbr":"","alias":"packet","show":true},{"value":"266","label":"Paar","abbr":"","alias":"pair","show":true},{"value":"274","label":"Rolle(n)","abbr":"","alias":"roll","show":true},{"value":"272","label":"Set(s)","abbr":"","alias":"set","show":true},{"value":"270","label":"Blatt","abbr":"","alias":"sheet","show":true},{"value":"273","label":"Ticket(s)","abbr":"","alias":"ticket","show":true},{"value":"271","label":"VKE","abbr":"","alias":"unit","show":true},{"value":"292","label":"Sekunde(n)","abbr":"s","alias":"second","show":true},{"value":"295","label":"Tag(e)","abbr":"","alias":"day","show":true},{"value":"294","label":"Stunde(n)","abbr":"","alias":"hour","show":true},{"value":"293","label":"Minute(n)","abbr":"min","alias":"minute","show":true},{"value":"298","label":"Monat(e)","abbr":"","alias":"month","show":true},{"value":"296","label":"Nacht","abbr":"","alias":"night","show":true},{"value":"297","label":"Woche(n)","abbr":"","alias":"week","show":true},{"value":"299","label":"Jahr(e)","abbr":"","alias":"year","show":true},{"value":"286","label":"Quadratmeter","abbr":"m²","alias":"squaremetre","show":true},{"value":"287","label":"Quadratzentimeter","abbr":"cm²","alias":"squarecentimetre","show":true},{"value":"290","label":"Quadratfuß","abbr":"ft²","alias":"squarefoot","show":true},{"value":"289","label":"Quadratzoll","abbr":"in²","alias":"squareinch","show":true},{"value":"288","label":"Quadratmillimeter","abbr":"mm²","alias":"squaremillimetre","show":true},{"value":"291","label":"Quadratyard","abbr":"yd²","alias":"squareyard","show":true}],"unitsHashByLabel":{"Byte":"281","Kilobyte":"282","Megabyte":"283","Gigabyte":"284","Terabyte":"285","Gramm":"259","Kilogramm":"261","Milligramm":"260","Unze(n)":"264","Pfund":"263","Tonne(n)":"262","Liter":"250","Kubikfuß":"254","Kubikzoll":"253","Kubikmeter":"252","Kubikyard":"255","Flüssigunze(n)":"256","Gallonen":"257","Milliliter":"251","Quart":"258","Meter":"243","Zentimeter":"244","Fuß":"247","Zoll":"248","Kilometer":"246","Millimeter":"245","Yard":"249","Stück":"265","Flasche(n)":"267","Kiste(n)":"268","Dose(n)":"278","Kapsel(n)":"279","Karton(s)":"269","Glas":"280","Kit(s)":"275","Packung(en)":"277","Paket(e)":"276","Paar":"266","Rolle(n)":"274","Set(s)":"272","Blatt":"270","Ticket(s)":"273","VKE":"271","Sekunde(n)":"292","Tag(e)":"295","Stunde(n)":"294","Minute(n)":"293","Monat(e)":"298","Nacht":"296","Woche(n)":"297","Jahr(e)":"299","Quadratmeter":"286","Quadratzentimeter":"287","Quadratfuß":"290","Quadratzoll":"289","Quadratmillimeter":"288","Quadratyard":"291"},"selectedUnit":{"value":"259","label":"Gramm","abbr":"g"}};

    QUnit.module('de_epages/product/ui/pricepreview', {
        teardown: fixture.destroy
    });

    QUnit.fixtureVar = fixture;

    QUnit.test('de_epages/product/ui/pricepreview - check existence of pricepreview methods', methods.length, function() {
        $.each(methods, function() {
            QUnit.ok(
                $.isFunction($.ui.productUiPricePreview.prototype[this]),
                this + ' is a method of the pricepreview widget'
            );
        });
    });

    QUnit.asyncTest('de_epages/product/ui/pricepreview - widget basic and ui tests', function () {
        // clear HTML fragments and initiate fixture
        $('#qunit-fixture').nextAll().remove();
        QUnit.fixtureVar.init();

        window.setTimeout(function() {
            // metaparse form elements
            $('#qunit-fixture .ep-js').metaparse();

            var pricePreviewElem = $('.ep-pricepreview'),
                taxClassSelect = $('#' + 'TaxClass'),
                selectedIndex = taxClassSelect[0].selectedIndex,

                initialData = {
                    listPrice: 169.95,
                    manufacturerPrice: 225,
                    priceQuantity: 1,
                    orderUnitId: refUnits.selectedUnit.value,
                    texts: {
                        vatReleasedMessage: "umsatzsteuerbefreit nach §19 UStG",
                    },
                    vatReleased: 0,
                    isRRP: 1,
                    shipping: 1,
                    taxRate: 19,
                    taxModel: 0, // 0 = without MwSt (net), 1 = with MwSt (gross)
                    inputTaxModel: 0,
                    taxClass: true,  // false = without tax, true = with tax
                    ref: {
                        amount: 100,
                        contentAmount: 1000,
                        unit: 'g'
                    }
                },
                testData = {
                    listPrice: 200,
                    manufacturerPrice: 210,
                    vatReleased: 0,
                    isRRP: 0,
                    shipping: 1,
                    taxRate: 19,
                    taxModel: 0, // 0 = without MwSt (net), 1 = with MwSt (gross)
                    inputTaxModel: 0,
                    taxClass: true,  // false = without tax, true = with tax
                    ref: {
                        amount: 50,
                        contentAmount: 200,
                        unit: 'gal'
                    }
                },

                listPrice = $('input[name="Price"]').eq(0),
                manufacturerPrice = $('input[name="ManufacturerPrice"]').eq(0),
                taxRate = $('#TaxClass'),
                isRRP = $('input[name="IsManufacturerPriceRRP"]'),
                shipping = $('input[name="IsShippingLinkVisible"]'),
                refAmount = $('#' + 'RefAmount'),
                refUnit = $('#' + 'RefUnitId'),
                refContentAmount = $('input[name="RefContentAmount"]').eq(0),

                percentage,
                refPriceStr,
                createSpy;

            // prepare spies
            createSpy = sinon.spy($.ui.productUiPricePreview.prototype, '_create');

            pricePreviewElem.productUiPricePreview($.extend({
                units: refUnits.units,
                events: {
                    listPrice: listPrice,
                    manufacturerPrice: manufacturerPrice,
                    taxRate: taxRate,
                    isRRP: isRRP,
                    shipping: shipping,
                    refAmount: refAmount,
                    refUnit: refUnit,
                    refContentAmount: refContentAmount
                }
            }, initialData));

            // wait for fixture and widget
            window.setTimeout(function() {
                QUnit.ok(
                    pricePreviewElem.data('uiProductUiPricePreview') !== undefined,
                    'Widget exists'
                );
                // check if spy in create method was called
                QUnit.ok(
                    createSpy.callCount === 1,
                    '"_create" method was called ' + createSpy.callCount + ' times.'
                );

                // test manufacturer price
                QUnit.equal(
                    $.i18n.parseNumber($('.ep-pricepreview-price-row-reduced-manufacturer-price').text()),
                    initialData.manufacturerPrice,
                    'Rendered manufacturer price is equal to the initial value (' + initialData.manufacturerPrice + ').'
                );
                QUnit.ok(
                    (new RegExp("^" + translations.priceManufacturer)).test($('.ep-pricepreview-price-row-reduced-manufacturer').text()),
                    'Rendered manufacturer string contains "' + translations.priceManufacturer + ' - mark as RRP".'
                );

                // test calculation and content of percentage value
                percentage = parseInt(-1 * (100 - (100 * initialData.listPrice / initialData.manufacturerPrice)));
                QUnit.equal(
                    $.i18n.parseNumber($('.ep-pricepreview-price-row-reduced-percent').text()),
                    percentage,
                    'Rendered percentage value was correct calculated (' + percentage + ' %).'
                );
                // test list price
                QUnit.equal(
                    $.i18n.parseNumber($('.ep-pricepreview-price-row-listprice-value').text()),
                    initialData.listPrice,
                    'Rendered list price is equal to the initial value (' + initialData.listPrice + ').'
                );

                // test reference price and corresponding calculation
                refPriceStr = "(" + initialData.ref.amount + " " + initialData.ref.unit + " = "
                    + $.i18n.formatNumber(
                        (initialData.listPrice * initialData.ref.amount / initialData.ref.contentAmount),
                        'c2',
                        {
                            region: $.i18n.settings.region,
                            currency: $.i18n.settings.currency
                        }
                      )
                    + ")";
                QUnit.equal(
                    $('.ep-pricepreview-price-row-refprice').text(),
                    refPriceStr,
                    'Rendered reference price string is equal to "' + refPriceStr + '".'
                );

                // test shipping information
                QUnit.equal(
                    $('.ep-pricepreview-info-row .PseudoLink').length,
                    1,
                    'Display indication of delivery costs.'
                );

                QUnit.ok(
                    true,
                    '========= Set new values to input fields => re-renders the UI ========='
                );

                // enter test data => trigger events
                listPrice.val(testData.listPrice).trigger('input');
                manufacturerPrice.val(testData.manufacturerPrice).trigger('input');
                isRRP.eq(1).trigger('click').trigger('change');
                shipping.eq(1).trigger('click').trigger('change');
                refUnit.val("Gallonen").trigger('input').trigger('change');
                refAmount.val(testData.ref.amount).trigger('input');
                refContentAmount.val(testData.ref.contentAmount).trigger('input');

                window.setTimeout(function() {
                    // test manufacturer price
                    QUnit.equal(
                        $.i18n.parseNumber($('.ep-pricepreview-price-row-reduced-manufacturer-price').text()),
                        testData.manufacturerPrice,
                        'Rendered manufacturer price is equal to the initial value (' + testData.manufacturerPrice + ').'
                    );
                    QUnit.ok(
                        !(new RegExp("^" + translations.priceManufacturer)).test($('.ep-pricepreview-price-row-reduced-manufacturer').text()),
                        'Rendered manufacturer string does not contain "' + translations.priceManufacturer + ' - don\'t mark as RRP".'
                    );

                    // test calculation of percentage value
                    percentage = Math.round(-1 * (100 - (100 * testData.listPrice / testData.manufacturerPrice)), 0);
                    QUnit.equal(
                        $.i18n.parseNumber($('.ep-pricepreview-price-row-reduced-percent').text()),
                        percentage,
                        'Rendered percentage value was correct calculated (' + percentage + ' %).'
                    );
                    // test list price
                    QUnit.equal(
                        $.i18n.parseNumber($('.ep-pricepreview-price-row-listprice-value').text()),
                        testData.listPrice,
                        'Rendered list price is equal to the initial value (' + testData.listPrice + ').'
                    );

                    // test reference price and corresponding calculation
                    refPriceStr = "(" + testData.ref.amount + " " + testData.ref.unit + " = "
                        + $.i18n.formatNumber(
                            (testData.listPrice * testData.ref.amount / testData.ref.contentAmount),
                            'c2',
                            {
                                region: $.i18n.settings.region,
                                currency: $.i18n.settings.currency
                            }
                          )
                        + ")";
                    QUnit.equal(
                        $('.ep-pricepreview-price-row-refprice').text(),
                        refPriceStr,
                        'Rendered reference price string is now equal to "' + refPriceStr + '".'
                    );

                    // test shipping link/info
                    QUnit.equal(
                        $('.ep-pricepreview-info-row .PseudoLink').length,
                        0,
                        'Don\'t display indication of delivery costs.'
                    );

                    // destroy widget
                    pricePreviewElem.productUiPricePreview('destroy');

                    window.setTimeout(function() {
                        QUnit.ok(
                            pricePreviewElem.data('uiProductUiPricePreview') === undefined,
                            'Widget was destroyed'
                        );

                        QUnit.equal(
                            pricePreviewElem.find('*').length,
                            0,
                            "Inserted HTML was removed."
                        );

                        QUnit.start();
                    }, 50);
                }, 100);


            }, 50);
        }, 200);
    });

    QUnit.asyncTest('de_epages/product/ui/pricepreview - VAT info text if tax rate equals to 0', function () {
        // clear HTML fragments and initiate fixture
        $('#qunit-fixture').nextAll().remove();
        QUnit.fixtureVar.init();

        window.setTimeout(function() {
            // metaparse form elements
            $('#qunit-fixture .ep-js').metaparse();

            var pricePreviewElem = $('.ep-pricepreview'),
                taxClassSelect = $('#' + 'TaxClass'),
                selectedIndex = taxClassSelect[0].selectedIndex,

                initialData = {
                    listPrice: 169.95,
                    manufacturerPrice: 225,
                    texts: {
                        vatReleasedMessage: "umsatzsteuerbefreit nach §19 UStG",
                    },
                    vatReleased: 0,
                    isRRP: 1,
                    shipping: 1,
                    taxRate: 0,
                    taxModel: 0, // 0 = without MwSt (net), 1 = with MwSt (gross)
                    taxClass: true,  // false = without tax, true = with tax
                    isVATInfoVisible: 0,
                    ref: {
                        amount: 100,
                        contentAmount: 1000,
                        unit: 'g'
                    }
                },
                testData = {
                    listPrice: 200,
                    manufacturerPrice: 210,
                    vatReleased: 0,
                    isRRP: 0,
                    shipping: 1,
                    taxRate: 0,
                    taxModel: 0, // 0 = without MwSt (net), 1 = with MwSt (gross)
                    taxClass: true,  // false = without tax, true = with tax
                    ref: {
                        amount: 50,
                        contentAmount: 200,
                        unit: 'gal'
                    }
                },

                listPrice = $('input[name="Price"]').eq(0),
                manufacturerPrice = $('input[name="ManufacturerPrice"]').eq(0),
                taxRate = $('#TaxClass'),
                isRRP = $('input[name="IsManufacturerPriceRRP"]'),
                shipping = $('input[name="IsShippingLinkVisible"]'),
                isVATInfoVisible = $('input[name="IsVATInfoVisible"]'),
                refAmount = $('#' + 'RefAmount'),
                refUnit = $('#' + 'RefUnitId'),
                refContentAmount = $('input[name="RefContentAmount"]').eq(0),

                percentage,
                refPriceStr;

            // set tax rate to 0 percent
            taxRate[0].selectedIndex = 0;
            taxRate[0].options[0].selected = "selected";

            pricePreviewElem.productUiPricePreview($.extend({
                units: refUnits.units,
                events: {
                    listPrice: listPrice,
                    manufacturerPrice: manufacturerPrice,
                    taxRate: taxRate,
                    isRRP: isRRP,
                    shipping: shipping,
                    isVATInfoVisible: isVATInfoVisible,
                    refAmount: refAmount,
                    refUnit: refUnit,
                    refContentAmount: refContentAmount
                }
            }, initialData));

            // wait for fixture and widget
            window.setTimeout(function() {
                QUnit.ok(
                    pricePreviewElem.data('uiProductUiPricePreview') !== undefined,
                    'Widget exists'
                );

                // test VAT info text
                QUnit.equal(
                    taxRate[0].selectedIndex,
                    0,
                    'Tax rate is set to 0 percent (without VAT).'
                );
                QUnit.ok(
                    isVATInfoVisible[1].checked,
                    'Hide VAT information text is selected.'
                );
                QUnit.ok(
                    !(new RegExp("" + translations.isNet)).test($('.ep-pricepreview-info-row').text()),
                    'Rendered additional information string doesn\'t contain "' + translations.isNet + '" - information about VAT.'
                );

                // set VAT information text visible
                $('input[name="IsVATInfoVisible"]:first').trigger('click').trigger('change');


                window.setTimeout(function() {
                    QUnit.ok(
                        isVATInfoVisible[0].checked,
                        'Display VAT information text is selected.'
                    );
                    QUnit.ok(
                        (new RegExp("" + translations.isNet)).test($('.ep-pricepreview-info-row').text()),
                        'Rendered additional information string contains "' + translations.isNet + '" - information about VAT.'
                    );

                    // destroy widget
                    pricePreviewElem.productUiPricePreview('destroy');

                    window.setTimeout(function() {
                        QUnit.ok(
                            pricePreviewElem.data('uiProductUiPricePreview') === undefined,
                            'Widget was destroyed'
                        );

                        QUnit.equal(
                            pricePreviewElem.find('*').length,
                            0,
                            "Inserted HTML was removed."
                        );

                        QUnit.start();
                    }, 50);
                }, 100);


            }, 50);
        }, 200);
    });

    QUnit.asyncTest('de_epages/product/ui/pricepreview - test additional order unit information', function () {
        // clear HTML fragments and initiate fixture
        $('#qunit-fixture').nextAll().remove();
        QUnit.fixtureVar.init();

        window.setTimeout(function() {
            // metaparse form elements
            $('#qunit-fixture .ep-js').metaparse();

            var pricePreviewElem = $('.ep-pricepreview'),
                taxClassSelect = $('#' + 'TaxClass'),
                selectedIndex = taxClassSelect[0].selectedIndex,
                pieceId = "265",

                initialData = {
                    listPrice: 169.95,
                    manufacturerPrice: 225,
                    priceQuantity: 1,
                    orderUnitId: refUnits.selectedUnit.value,
                    texts: {
                        vatReleasedMessage: "umsatzsteuerbefreit nach §19 UStG",
                    },
                    vatReleased: 0,
                    isRRP: 1,
                    shipping: 1,
                    taxRate: 19,
                    taxModel: 0, // 0 = without MwSt (net), 1 = with MwSt (gross)
                    inputTaxModel: 0,
                    taxClass: true,  // false = without tax, true = with tax
                    ref: {
                        amount: 100,
                        contentAmount: 1000,
                        unit: 'g'
                    }
                },
                testData = {
                    listPrice: 200,
                    manufacturerPrice: 210,
                    priceQuantity: 2,
                    orderUnitId: "cubicfoot",  // cubicfoot
                    orderUnit: "254",  // cubicfoot
                    vatReleased: 0,
                    isRRP: 0,
                    shipping: 1,
                    taxRate: 19,
                    taxModel: 0, // 0 = without MwSt (net), 1 = with MwSt (gross)
                    inputTaxModel: 0,
                    taxClass: true,  // false = without tax, true = with tax
                    ref: {
                        amount: 50,
                        contentAmount: 200,
                        unit: 'gal'
                    }
                },

                listPrice = $('input[name="Price"]').eq(0),
                manufacturerPrice = $('input[name="ManufacturerPrice"]').eq(0),
                taxRate = $('#TaxClass'),
                isRRP = $('input[name="IsManufacturerPriceRRP"]'),
                shipping = $('input[name="IsShippingLinkVisible"]'),
                refAmount = $('#' + 'RefAmount'),
                refUnit = $('#' + 'RefUnitId'),
                refContentAmount = $('input[name="RefContentAmount"]').eq(0),
                priceQuantity = $('#' + 'PriceQuantity'),
                orderUnit = $('#' + 'OrderUnitId'),
                orderUnitId = $('#' + 'OrderUnit'),

                findUnitById = function(id) {
                    return refUnits.units.filter(function(unit) {
                        return unit.value === id;
                    });
                };

            pricePreviewElem.productUiPricePreview($.extend({
                units: refUnits.units,
                events: {
                    listPrice: listPrice,
                    manufacturerPrice: manufacturerPrice,
                    taxRate: taxRate,
                    isRRP: isRRP,
                    shipping: shipping,
                    refAmount: refAmount,
                    refUnit: refUnit,
                    refContentAmount: refContentAmount,
                    priceQuantity: priceQuantity,
                    // orderUnit: orderUnit,
                                        orderUnit: {
                                          watch: orderUnit,
                                          value: orderUnitId
                                        }
                }
            }, initialData));

            // wait for fixture and widget
            window.setTimeout(function() {
                QUnit.ok(
                    pricePreviewElem.data('uiProductUiPricePreview') !== undefined,
                    'Widget exists'
                );

                // test additional order unit info
                QUnit.equal(
                    $('.ep-pricepreview-price-row-listprice-orderunit').text().trim(),
                    '/ ' + (initialData.priceQuantity === 1 ? '' : initialData.priceQuantity) + ' ' + findUnitById(refUnits.selectedUnit.value)[0].abbr,
                    'Rendered order unit information is "' + '/ ' + (initialData.priceQuantity === 1 ? '' : initialData.priceQuantity) + ' ' + findUnitById(refUnits.selectedUnit.value)[0].abbr + '".'
                );

                QUnit.ok(
                    true,
                    '========= Set new order unit and price quantity to input fields => re-renders the UI ========='
                );

                // enter test data => trigger events
                priceQuantity.val(testData.priceQuantity).trigger('input');
                orderUnitId.val(testData.orderUnit).trigger('input').trigger('change');
                orderUnit.val(testData.orderUnitId).trigger('input').trigger('change');

                window.setTimeout(function() {

                    // test new additional order unit info
                    QUnit.equal(
                        $('.ep-pricepreview-price-row-listprice-orderunit').text().trim(),
                        '/ ' + testData.priceQuantity + ' ' + findUnitById(testData.orderUnit)[0].abbr,
                        'Rendered changed order unit information is "' + '/ ' + testData.priceQuantity + ' ' + findUnitById(testData.orderUnit)[0].abbr + '".'
                    );

                    QUnit.ok(
                        true,
                        '========= Change order unit to "piece" => re-renders the UI ========='
                    );

                    // enter test data => trigger events
                    orderUnitId.val("piece").trigger('input').trigger('change');
                    orderUnit.val(pieceId).trigger('input').trigger('change');

                    window.setTimeout(function() {

                        // test new additional order unit info
                        QUnit.equal(
                            $('.ep-pricepreview-price-row-listprice-orderunit').length,
                            0,
                            'No order unit was rendered for unit "piece".'
                        );

                        // destroy widget
                        pricePreviewElem.productUiPricePreview('destroy');

                        window.setTimeout(function() {
                            QUnit.ok(
                                pricePreviewElem.data('uiProductUiPricePreview') === undefined,
                                'Widget was destroyed'
                            );

                            QUnit.equal(
                                pricePreviewElem.find('*').length,
                                0,
                                "Inserted HTML was removed."
                            );

                            QUnit.start();
                        }, 50);
                    }, 100);
                }, 100);
            }, 50);
        }, 200);
    });
});