/*
 * @copyright       Copyright 2006-2010, epages GmbH, All Rights Reserved.
 *
 * @module           de_epages.shop.ui.userBlogWidget
 */

/*globals define*/
/*jslint nomen:true*/

define("de_epages/shop/ui/userblogwidget", [
    "jquery",
    "ep",
    "de_epages",
    "util/string",
    "$tmpl!./userblogwidget",

    "ep/ajax",
    "jquery/i18n",
    "ep/date",
    "ep/ui/tooltip",
    "ep/fn/busy",
    "de_epages/presentation/rss"
], function ($, ep, de_epages, str, tmplUserBlogWidget) {

    $.widget("ui.shopUiUserBlogWidget", {

        //define options of widget formally know as widget properties in dojos
        options: {
            url: null,
            visibleEntrys: 3,
            entryLength: 50
        },

        _create: function () {
            if (!this.options.url) {
                return;
            }

            $(this.element.parent()).busy();

            //getFeedData
            new de_epages.presentation.Rss(this.options.url, $.proxy(this, '_readFeedData'));
        },

        _readFeedData: function (feedData) {
            var self = this,
                o = this.options,
                items = feedData.items.slice(0, o.visibleEntrys),
                urls = [];

            $.each(items, function () {
                this.tooltipTitle = this.title;
                this.tooltipText = this.text;
                urls.push(this.link);

                this.tooltipDate = $.i18n.formatDate(this.pubDate, 'd', {region: ep.config.region});
                this.title = str.shrink(this.title, {length: o.entryLength, cut: false});
                this.text = str.shrink(this.text, {length: o.entryLength, cut: false});
            });

            // Get links for the dereferred ChangeAction
            ep.ajax({
                dataType: 'json',
                data: {
                    'ViewAction': 'JSONDerefer',
                    'DereferURLs': urls.join(',')
                }
            }).done(function (data) {
                if (data.length !== items.length) {
                    return;
                }

                $.each(items, function (index) {
                    this.link = data[index];
                });

                tmplUserBlogWidget({'items': items}).appendTo(self.element);

                self.element
                    .find('.ep-shopUiUserBlogWidget-item')
                    .each(function () {
                        var elem = $(this);
                        elem.find('.ep-shopUiUserBlogWidget-itemTooltip')
                            .uiTooltip({
                                context:      elem.find('.ep-shopUiUserBlogWidget-itemContent'),
                                interactive:  true,
                                addClass:     'ep-shopUiUserBlogWidget-itemTooltip',
                                showDelay:    1000,
                                hideDelay:    200
                            });
                    });

                $(self.element.parent()).busy('hide');
            });
        },

        destroy: function () {
            this.element.remove();
        }

    });

    return de_epages;

});
