/*
	Copyright (c) 2004-2011, The Dojo Foundation All Rights Reserved.
	Available via Academic Free License >= 2.1 OR the modified BSD license.
	see: http://dojotoolkit.org/license for details
*/

/*
	This is an optimized version of Dojo, built for deployment and not for
	development. To get sources and documentation, please visit:

		http://dojotoolkit.org
*/

if(!dojo._hasResource["dojo.dnd.common"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.dnd.common"] = true;
dojo.provide("dojo.dnd.common");

dojo.getObject("dnd", true, dojo);

dojo.dnd.getCopyKeyState = dojo.isCopyKey;

dojo.dnd._uniqueId = 0;
dojo.dnd.getUniqueId = function(){
	// summary:
	//		returns a unique string for use with any DOM element
	var id;
	do{
		id = dojo._scopeName + "Unique" + (++dojo.dnd._uniqueId);
	}while(dojo.byId(id));
	return id;
};

dojo.dnd._empty = {};

dojo.dnd.isFormElement = function(/*Event*/ e){
	// summary:
	//		returns true if user clicked on a form element
	var t = e.target;
	if(t.nodeType == 3 /*TEXT_NODE*/){
		t = t.parentNode;
	}
	return " button textarea input select option ".indexOf(" " + t.tagName.toLowerCase() + " ") >= 0;	// Boolean
};

}

if(!dojo._hasResource["dojo.window"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.window"] = true;
dojo.provide("dojo.window");

dojo.getObject("window", true, dojo);

dojo.window.getBox = function(){
	// summary:
	//		Returns the dimensions and scroll position of the viewable area of a browser window

	var scrollRoot = (dojo.doc.compatMode == 'BackCompat') ? dojo.body() : dojo.doc.documentElement;

	// get scroll position
	var scroll = dojo._docScroll(); // scrollRoot.scrollTop/Left should work
	return { w: scrollRoot.clientWidth, h: scrollRoot.clientHeight, l: scroll.x, t: scroll.y };
};

dojo.window.get = function(doc){
	// summary:
	// 		Get window object associated with document doc

	// In some IE versions (at least 6.0), document.parentWindow does not return a
	// reference to the real window object (maybe a copy), so we must fix it as well
	// We use IE specific execScript to attach the real window reference to
	// document._parentWindow for later use
	if(dojo.isIE && window !== document.parentWindow){
		/*
		In IE 6, only the variable "window" can be used to connect events (others
		may be only copies).
		*/
		doc.parentWindow.execScript("document._parentWindow = window;", "Javascript");
		//to prevent memory leak, unset it after use
		//another possibility is to add an onUnload handler which seems overkill to me (liucougar)
		var win = doc._parentWindow;
		doc._parentWindow = null;
		return win;	//	Window
	}

	return doc.parentWindow || doc.defaultView;	//	Window
};

dojo.window.scrollIntoView = function(/*DomNode*/ node, /*Object?*/ pos){
	// summary:
	//		Scroll the passed node into view, if it is not already.
	
	// don't rely on node.scrollIntoView working just because the function is there

	try{ // catch unexpected/unrecreatable errors (#7808) since we can recover using a semi-acceptable native method
		node = dojo.byId(node);
		var doc = node.ownerDocument || dojo.doc,
			body = doc.body || dojo.body(),
			html = doc.documentElement || body.parentNode,
			isIE = dojo.isIE, isWK = dojo.isWebKit;
		// if an untested browser, then use the native method
		if((!(dojo.isMoz || isIE || isWK || dojo.isOpera) || node == body || node == html) && (typeof node.scrollIntoView != "undefined")){
			node.scrollIntoView(false); // short-circuit to native if possible
			return;
		}
		var backCompat = doc.compatMode == 'BackCompat',
			clientAreaRoot = (isIE >= 9 && node.ownerDocument.parentWindow.frameElement)
				? ((html.clientHeight > 0 && html.clientWidth > 0 && (body.clientHeight == 0 || body.clientWidth == 0 || body.clientHeight > html.clientHeight || body.clientWidth > html.clientWidth)) ? html : body)
				: (backCompat ? body : html),
			scrollRoot = isWK ? body : clientAreaRoot,
			rootWidth = clientAreaRoot.clientWidth,
			rootHeight = clientAreaRoot.clientHeight,
			rtl = !dojo._isBodyLtr(),
			nodePos = pos || dojo.position(node),
			el = node.parentNode,
			isFixed = function(el){
				return ((isIE <= 6 || (isIE && backCompat))? false : (dojo.style(el, 'position').toLowerCase() == "fixed"));
			};
		if(isFixed(node)){ return; } // nothing to do

		while(el){
			if(el == body){ el = scrollRoot; }
			var elPos = dojo.position(el),
				fixedPos = isFixed(el);
	
			if(el == scrollRoot){
				elPos.w = rootWidth; elPos.h = rootHeight;
				if(scrollRoot == html && isIE && rtl){ elPos.x += scrollRoot.offsetWidth-elPos.w; } // IE workaround where scrollbar causes negative x
				if(elPos.x < 0 || !isIE){ elPos.x = 0; } // IE can have values > 0
				if(elPos.y < 0 || !isIE){ elPos.y = 0; }
			}else{
				var pb = dojo._getPadBorderExtents(el);
				elPos.w -= pb.w; elPos.h -= pb.h; elPos.x += pb.l; elPos.y += pb.t;
				var clientSize = el.clientWidth,
					scrollBarSize = elPos.w - clientSize;
				if(clientSize > 0 && scrollBarSize > 0){
					elPos.w = clientSize;
					elPos.x += (rtl && (isIE || el.clientLeft > pb.l/*Chrome*/)) ? scrollBarSize : 0;
				}
				clientSize = el.clientHeight;
				scrollBarSize = elPos.h - clientSize;
				if(clientSize > 0 && scrollBarSize > 0){
					elPos.h = clientSize;
				}
			}
			if(fixedPos){ // bounded by viewport, not parents
				if(elPos.y < 0){
					elPos.h += elPos.y; elPos.y = 0;
				}
				if(elPos.x < 0){
					elPos.w += elPos.x; elPos.x = 0;
				}
				if(elPos.y + elPos.h > rootHeight){
					elPos.h = rootHeight - elPos.y;
				}
				if(elPos.x + elPos.w > rootWidth){
					elPos.w = rootWidth - elPos.x;
				}
			}
			// calculate overflow in all 4 directions
			var l = nodePos.x - elPos.x, // beyond left: < 0
				t = nodePos.y - Math.max(elPos.y, 0), // beyond top: < 0
				r = l + nodePos.w - elPos.w, // beyond right: > 0
				bot = t + nodePos.h - elPos.h; // beyond bottom: > 0
			if(r * l > 0){
				var s = Math[l < 0? "max" : "min"](l, r);
				if(rtl && ((isIE == 8 && !backCompat) || isIE >= 9)){ s = -s; }
				nodePos.x += el.scrollLeft;
				el.scrollLeft += s;
				nodePos.x -= el.scrollLeft;
			}
			if(bot * t > 0){
				nodePos.y += el.scrollTop;
				el.scrollTop += Math[t < 0? "max" : "min"](t, bot);
				nodePos.y -= el.scrollTop;
			}
			el = (el != scrollRoot) && !fixedPos && el.parentNode;
		}
	}catch(error){
		console.error('scrollIntoView: ' + error);
		node.scrollIntoView(false);
	}
};

}

if(!dojo._hasResource["dojo.dnd.autoscroll"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.dnd.autoscroll"] = true;
dojo.provide("dojo.dnd.autoscroll");


dojo.getObject("dnd", true, dojo);

dojo.dnd.getViewport = dojo.window.getBox;

dojo.dnd.V_TRIGGER_AUTOSCROLL = 32;
dojo.dnd.H_TRIGGER_AUTOSCROLL = 32;

dojo.dnd.V_AUTOSCROLL_VALUE = 16;
dojo.dnd.H_AUTOSCROLL_VALUE = 16;

dojo.dnd.autoScroll = function(e){
	// summary:
	//		a handler for onmousemove event, which scrolls the window, if
	//		necesary
	// e: Event
	//		onmousemove event

	// FIXME: needs more docs!
	var v = dojo.window.getBox(), dx = 0, dy = 0;
	if(e.clientX < dojo.dnd.H_TRIGGER_AUTOSCROLL){
		dx = -dojo.dnd.H_AUTOSCROLL_VALUE;
	}else if(e.clientX > v.w - dojo.dnd.H_TRIGGER_AUTOSCROLL){
		dx = dojo.dnd.H_AUTOSCROLL_VALUE;
	}
	if(e.clientY < dojo.dnd.V_TRIGGER_AUTOSCROLL){
		dy = -dojo.dnd.V_AUTOSCROLL_VALUE;
	}else if(e.clientY > v.h - dojo.dnd.V_TRIGGER_AUTOSCROLL){
		dy = dojo.dnd.V_AUTOSCROLL_VALUE;
	}
	window.scrollBy(dx, dy);
};

dojo.dnd._validNodes = {"div": 1, "p": 1, "td": 1};
dojo.dnd._validOverflow = {"auto": 1, "scroll": 1};

dojo.dnd.autoScrollNodes = function(e){
	// summary:
	//		a handler for onmousemove event, which scrolls the first avaialble
	//		Dom element, it falls back to dojo.dnd.autoScroll()
	// e: Event
	//		onmousemove event

	// FIXME: needs more docs!
	for(var n = e.target; n;){
		if(n.nodeType == 1 && (n.tagName.toLowerCase() in dojo.dnd._validNodes)){
			var s = dojo.getComputedStyle(n);
			if(s.overflow.toLowerCase() in dojo.dnd._validOverflow){
				var b = dojo._getContentBox(n, s), t = dojo.position(n, true);
				//console.log(b.l, b.t, t.x, t.y, n.scrollLeft, n.scrollTop);
				var w = Math.min(dojo.dnd.H_TRIGGER_AUTOSCROLL, b.w / 2),
					h = Math.min(dojo.dnd.V_TRIGGER_AUTOSCROLL, b.h / 2),
					rx = e.pageX - t.x, ry = e.pageY - t.y, dx = 0, dy = 0;
				if(dojo.isWebKit || dojo.isOpera){
					// FIXME: this code should not be here, it should be taken into account
					// either by the event fixing code, or the dojo.position()
					// FIXME: this code doesn't work on Opera 9.5 Beta
					rx += dojo.body().scrollLeft;
					ry += dojo.body().scrollTop;
				}
				if(rx > 0 && rx < b.w){
					if(rx < w){
						dx = -w;
					}else if(rx > b.w - w){
						dx = w;
					}
				}
				//console.log("ry =", ry, "b.h =", b.h, "h =", h);
				if(ry > 0 && ry < b.h){
					if(ry < h){
						dy = -h;
					}else if(ry > b.h - h){
						dy = h;
					}
				}
				var oldLeft = n.scrollLeft, oldTop = n.scrollTop;
				n.scrollLeft = n.scrollLeft + dx;
				n.scrollTop  = n.scrollTop  + dy;
				if(oldLeft != n.scrollLeft || oldTop != n.scrollTop){ return; }
			}
		}
		try{
			n = n.parentNode;
		}catch(x){
			n = null;
		}
	}
	dojo.dnd.autoScroll(e);
};

}

if(!dojo._hasResource["dojo.dnd.Mover"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.dnd.Mover"] = true;
dojo.provide("dojo.dnd.Mover");




dojo.declare("dojo.dnd.Mover", null, {
	constructor: function(node, e, host){
		// summary:
		//		an object which makes a node follow the mouse, or touch-drag on touch devices.
		//		Used as a default mover, and as a base class for custom movers.
		// node: Node
		//		a node (or node's id) to be moved
		// e: Event
		//		a mouse event, which started the move;
		//		only pageX and pageY properties are used
		// host: Object?
		//		object which implements the functionality of the move,
		//	 	and defines proper events (onMoveStart and onMoveStop)
		this.node = dojo.byId(node);
		var pos = e.touches ? e.touches[0] : e;
		this.marginBox = {l: pos.pageX, t: pos.pageY};
		this.mouseButton = e.button;
		var h = (this.host = host), d = node.ownerDocument;
		this.events = [
			// At the start of a drag, onFirstMove is called, and then the following two
			// connects are disconnected
			dojo.connect(d, "onmousemove", this, "onFirstMove"),
			dojo.connect(d, "ontouchmove", this, "onFirstMove"),

			// These are called continually during the drag
			dojo.connect(d, "onmousemove", this, "onMouseMove"),
			dojo.connect(d, "ontouchmove", this, "onMouseMove"),

			// And these are called at the end of the drag
			dojo.connect(d, "onmouseup",   this, "onMouseUp"),
			dojo.connect(d, "ontouchend", this, "onMouseUp"),

			// cancel text selection and text dragging
			dojo.connect(d, "ondragstart",   dojo.stopEvent),
			dojo.connect(d.body, "onselectstart", dojo.stopEvent)
		];
		// notify that the move has started
		if(h && h.onMoveStart){
			h.onMoveStart(this);
		}
	},
	// mouse event processors
	onMouseMove: function(e){
		// summary:
		//		event processor for onmousemove/ontouchmove
		// e: Event
		//		mouse/touch event
		dojo.dnd.autoScroll(e);
		var m = this.marginBox,
			pos = e.touches ? e.touches[0] : e;
		this.host.onMove(this, {l: m.l + pos.pageX, t: m.t + pos.pageY}, e);
		dojo.stopEvent(e);
	},
	onMouseUp: function(e){
		if(dojo.isWebKit && dojo.isMac && this.mouseButton == 2 ?
				e.button == 0 : this.mouseButton == e.button){ // TODO Should condition be met for touch devices, too?
			this.destroy();
		}
		dojo.stopEvent(e);
	},
	// utilities
	onFirstMove: function(e){
		// summary:
		//		makes the node absolute; it is meant to be called only once.
		// 		relative and absolutely positioned nodes are assumed to use pixel units
		var s = this.node.style, l, t, h = this.host;
		switch(s.position){
			case "relative":
			case "absolute":
				// assume that left and top values are in pixels already
				l = Math.round(parseFloat(s.left)) || 0;
				t = Math.round(parseFloat(s.top)) || 0;
				break;
			default:
				s.position = "absolute";	// enforcing the absolute mode
				var m = dojo.marginBox(this.node);
				// event.pageX/pageY (which we used to generate the initial
				// margin box) includes padding and margin set on the body.
				// However, setting the node's position to absolute and then
				// doing dojo.marginBox on it *doesn't* take that additional
				// space into account - so we need to subtract the combined
				// padding and margin.  We use getComputedStyle and
				// _getMarginBox/_getContentBox to avoid the extra lookup of
				// the computed style.
				var b = dojo.doc.body;
				var bs = dojo.getComputedStyle(b);
				var bm = dojo._getMarginBox(b, bs);
				var bc = dojo._getContentBox(b, bs);
				l = m.l - (bc.l - bm.l);
				t = m.t - (bc.t - bm.t);
				break;
		}
		this.marginBox.l = l - this.marginBox.l;
		this.marginBox.t = t - this.marginBox.t;
		if(h && h.onFirstMove){
			h.onFirstMove(this, e);
		}
		
		// Disconnect onmousemove and ontouchmove events that call this function
		dojo.disconnect(this.events.shift());
		dojo.disconnect(this.events.shift());
	},
	destroy: function(){
		// summary:
		//		stops the move, deletes all references, so the object can be garbage-collected
		dojo.forEach(this.events, dojo.disconnect);
		// undo global settings
		var h = this.host;
		if(h && h.onMoveStop){
			h.onMoveStop(this);
		}
		// destroy objects
		this.events = this.node = this.host = null;
	}
});

}

if(!dojo._hasResource["dojo.dnd.Moveable"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.dnd.Moveable"] = true;
dojo.provide("dojo.dnd.Moveable");



/*=====
dojo.declare("dojo.dnd.__MoveableArgs", [], {
	// handle: Node||String
	//		A node (or node's id), which is used as a mouse handle.
	//		If omitted, the node itself is used as a handle.
	handle: null,

	// delay: Number
	//		delay move by this number of pixels
	delay: 0,

	// skip: Boolean
	//		skip move of form elements
	skip: false,

	// mover: Object
	//		a constructor of custom Mover
	mover: dojo.dnd.Mover
});
=====*/

dojo.declare("dojo.dnd.Moveable", null, {
	// object attributes (for markup)
	handle: "",
	delay: 0,
	skip: false,
	
	constructor: function(node, params){
		// summary:
		//		an object, which makes a node moveable
		// node: Node
		//		a node (or node's id) to be moved
		// params: dojo.dnd.__MoveableArgs?
		//		optional parameters
		this.node = dojo.byId(node);
		if(!params){ params = {}; }
		this.handle = params.handle ? dojo.byId(params.handle) : null;
		if(!this.handle){ this.handle = this.node; }
		this.delay = params.delay > 0 ? params.delay : 0;
		this.skip  = params.skip;
		this.mover = params.mover ? params.mover : dojo.dnd.Mover;
		this.events = [
			dojo.connect(this.handle, "onmousedown", this, "onMouseDown"),
			dojo.connect(this.handle, "ontouchstart", this, "onMouseDown"),
			// cancel text selection and text dragging
			dojo.connect(this.handle, "ondragstart",   this, "onSelectStart"),
			dojo.connect(this.handle, "onselectstart", this, "onSelectStart")
		];
	},

	// markup methods
	markupFactory: function(params, node){
		return new dojo.dnd.Moveable(node, params);
	},

	// methods
	destroy: function(){
		// summary:
		//		stops watching for possible move, deletes all references, so the object can be garbage-collected
		dojo.forEach(this.events, dojo.disconnect);
		this.events = this.node = this.handle = null;
	},
	
	// mouse event processors
	onMouseDown: function(e){
		// summary:
		//		event processor for onmousedown/ontouchstart, creates a Mover for the node
		// e: Event
		//		mouse/touch event
		if(this.skip && dojo.dnd.isFormElement(e)){ return; }
		if(this.delay){
			this.events.push(
				dojo.connect(this.handle, "onmousemove", this, "onMouseMove"),
				dojo.connect(this.handle, "ontouchmove", this, "onMouseMove"),
				dojo.connect(this.handle, "onmouseup", this, "onMouseUp"),
				dojo.connect(this.handle, "ontouchend", this, "onMouseUp")
			);
			var pos = e.touches ? e.touches[0] : e;
			this._lastX = pos.pageX;
			this._lastY = pos.pageY;
		}else{
			this.onDragDetected(e);
		}
		dojo.stopEvent(e);
	},
	onMouseMove: function(e){
		// summary:
		//		event processor for onmousemove/ontouchmove, used only for delayed drags
		// e: Event
		//		mouse/touch event
		var pos = e.touches ? e.touches[0] : e;
		if(Math.abs(pos.pageX - this._lastX) > this.delay || Math.abs(pos.pageY - this._lastY) > this.delay){
			this.onMouseUp(e);
			this.onDragDetected(e);
		}
		dojo.stopEvent(e);
	},
	onMouseUp: function(e){
		// summary:
		//		event processor for onmouseup, used only for delayed drags
		// e: Event
		//		mouse event
		for(var i = 0; i < 2; ++i){
			dojo.disconnect(this.events.pop());
		}
		dojo.stopEvent(e);
	},
	onSelectStart: function(e){
		// summary:
		//		event processor for onselectevent and ondragevent
		// e: Event
		//		mouse event
		if(!this.skip || !dojo.dnd.isFormElement(e)){
			dojo.stopEvent(e);
		}
	},
	
	// local events
	onDragDetected: function(/* Event */ e){
		// summary:
		//		called when the drag is detected;
		//		responsible for creation of the mover
		new this.mover(this.node, e, this);
	},
	onMoveStart: function(/* dojo.dnd.Mover */ mover){
		// summary:
		//		called before every move operation
		dojo.publish("/dnd/move/start", [mover]);
		dojo.addClass(dojo.body(), "dojoMove");
		dojo.addClass(this.node, "dojoMoveItem");
	},
	onMoveStop: function(/* dojo.dnd.Mover */ mover){
		// summary:
		//		called after every move operation
		dojo.publish("/dnd/move/stop", [mover]);
		dojo.removeClass(dojo.body(), "dojoMove");
		dojo.removeClass(this.node, "dojoMoveItem");
	},
	onFirstMove: function(/* dojo.dnd.Mover */ mover, /* Event */ e){
		// summary:
		//		called during the very first move notification;
		//		can be used to initialize coordinates, can be overwritten.
		
		// default implementation does nothing
	},
	onMove: function(/* dojo.dnd.Mover */ mover, /* Object */ leftTop, /* Event */ e){
		// summary:
		//		called during every move notification;
		//		should actually move the node; can be overwritten.
		this.onMoving(mover, leftTop);
		var s = mover.node.style;
		s.left = leftTop.l + "px";
		s.top  = leftTop.t + "px";
		this.onMoved(mover, leftTop);
	},
	onMoving: function(/* dojo.dnd.Mover */ mover, /* Object */ leftTop){
		// summary:
		//		called before every incremental move; can be overwritten.
		
		// default implementation does nothing
	},
	onMoved: function(/* dojo.dnd.Mover */ mover, /* Object */ leftTop){
		// summary:
		//		called after every incremental move; can be overwritten.
		
		// default implementation does nothing
	}
});

}

if(!dojo._hasResource["dojo.dnd.move"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.dnd.move"] = true;
dojo.provide("dojo.dnd.move");




/*=====
dojo.declare("dojo.dnd.move.__constrainedMoveableArgs", [dojo.dnd.__MoveableArgs], {
	// constraints: Function
	//		Calculates a constraint box.
	//		It is called in a context of the moveable object.
	constraints: function(){},

	// within: Boolean
	//		restrict move within boundaries.
	within: false
});
=====*/

dojo.declare("dojo.dnd.move.constrainedMoveable", dojo.dnd.Moveable, {
	// object attributes (for markup)
	constraints: function(){},
	within: false,
	
	// markup methods
	markupFactory: function(params, node){
		return new dojo.dnd.move.constrainedMoveable(node, params);
	},

	constructor: function(node, params){
		// summary:
		//		an object that makes a node moveable
		// node: Node
		//		a node (or node's id) to be moved
		// params: dojo.dnd.move.__constrainedMoveableArgs?
		//		an optional object with additional parameters;
		//		the rest is passed to the base class
		if(!params){ params = {}; }
		this.constraints = params.constraints;
		this.within = params.within;
	},
	onFirstMove: function(/* dojo.dnd.Mover */ mover){
		// summary:
		//		called during the very first move notification;
		//		can be used to initialize coordinates, can be overwritten.
		var c = this.constraintBox = this.constraints.call(this, mover);
		c.r = c.l + c.w;
		c.b = c.t + c.h;
		if(this.within){
			var mb = dojo._getMarginSize(mover.node);
			c.r -= mb.w;
			c.b -= mb.h;
		}
	},
	onMove: function(/* dojo.dnd.Mover */ mover, /* Object */ leftTop){
		// summary:
		//		called during every move notification;
		//		should actually move the node; can be overwritten.
		var c = this.constraintBox, s = mover.node.style;
		this.onMoving(mover, leftTop);
		leftTop.l = leftTop.l < c.l ? c.l : c.r < leftTop.l ? c.r : leftTop.l;
		leftTop.t = leftTop.t < c.t ? c.t : c.b < leftTop.t ? c.b : leftTop.t;
		s.left = leftTop.l + "px";
		s.top  = leftTop.t + "px";
		this.onMoved(mover, leftTop);
	}
});

/*=====
dojo.declare("dojo.dnd.move.__boxConstrainedMoveableArgs", [dojo.dnd.move.__constrainedMoveableArgs], {
	// box: Object
	//		a constraint box
	box: {}
});
=====*/

dojo.declare("dojo.dnd.move.boxConstrainedMoveable", dojo.dnd.move.constrainedMoveable, {
	// box:
	//		object attributes (for markup)
	box: {},
	
	// markup methods
	markupFactory: function(params, node){
		return new dojo.dnd.move.boxConstrainedMoveable(node, params);
	},

	constructor: function(node, params){
		// summary:
		//		an object, which makes a node moveable
		// node: Node
		//		a node (or node's id) to be moved
		// params: dojo.dnd.move.__boxConstrainedMoveableArgs?
		//		an optional object with parameters
		var box = params && params.box;
		this.constraints = function(){ return box; };
	}
});

/*=====
dojo.declare("dojo.dnd.move.__parentConstrainedMoveableArgs", [dojo.dnd.move.__constrainedMoveableArgs], {
	// area: String
	//		A parent's area to restrict the move.
	//		Can be "margin", "border", "padding", or "content".
	area: ""
});
=====*/

dojo.declare("dojo.dnd.move.parentConstrainedMoveable", dojo.dnd.move.constrainedMoveable, {
	// area:
	//		object attributes (for markup)
	area: "content",

	// markup methods
	markupFactory: function(params, node){
		return new dojo.dnd.move.parentConstrainedMoveable(node, params);
	},

	constructor: function(node, params){
		// summary:
		//		an object, which makes a node moveable
		// node: Node
		//		a node (or node's id) to be moved
		// params: dojo.dnd.move.__parentConstrainedMoveableArgs?
		//		an optional object with parameters
		var area = params && params.area;
		this.constraints = function(){
			var n = this.node.parentNode,
				s = dojo.getComputedStyle(n),
				mb = dojo._getMarginBox(n, s);
			if(area == "margin"){
				return mb;	// Object
			}
			var t = dojo._getMarginExtents(n, s);
			mb.l += t.l, mb.t += t.t, mb.w -= t.w, mb.h -= t.h;
			if(area == "border"){
				return mb;	// Object
			}
			t = dojo._getBorderExtents(n, s);
			mb.l += t.l, mb.t += t.t, mb.w -= t.w, mb.h -= t.h;
			if(area == "padding"){
				return mb;	// Object
			}
			t = dojo._getPadExtents(n, s);
			mb.l += t.l, mb.t += t.t, mb.w -= t.w, mb.h -= t.h;
			return mb;	// Object
		};
	}
});

// patching functions one level up for compatibility

dojo.dnd.constrainedMover = dojo.dnd.move.constrainedMover;
dojo.dnd.boxConstrainedMover = dojo.dnd.move.boxConstrainedMover;
dojo.dnd.parentConstrainedMover = dojo.dnd.move.parentConstrainedMover;

}

if(!dojo._hasResource["dojo.dnd.TimedMoveable"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.dnd.TimedMoveable"] = true;
dojo.provide("dojo.dnd.TimedMoveable");



/*=====
dojo.declare("dojo.dnd.__TimedMoveableArgs", [dojo.dnd.__MoveableArgs], {
	// timeout: Number
	//		delay move by this number of ms,
	//		accumulating position changes during the timeout
	timeout: 0
});
=====*/

(function(){
	// precalculate long expressions
	var oldOnMove = dojo.dnd.Moveable.prototype.onMove;
		
	dojo.declare("dojo.dnd.TimedMoveable", dojo.dnd.Moveable, {
		// summary:
		//		A specialized version of Moveable to support an FPS throttling.
		//		This class puts an upper restriction on FPS, which may reduce
		//		the CPU load. The additional parameter "timeout" regulates
		//		the delay before actually moving the moveable object.
		
		// object attributes (for markup)
		timeout: 40,	// in ms, 40ms corresponds to 25 fps
	
		constructor: function(node, params){
			// summary:
			//		an object that makes a node moveable with a timer
			// node: Node||String
			//		a node (or node's id) to be moved
			// params: dojo.dnd.__TimedMoveableArgs
			//		object with additional parameters.
			
			// sanitize parameters
			if(!params){ params = {}; }
			if(params.timeout && typeof params.timeout == "number" && params.timeout >= 0){
				this.timeout = params.timeout;
			}
		},
	
		// markup methods
		markupFactory: function(params, node){
			return new dojo.dnd.TimedMoveable(node, params);
		},
	
		onMoveStop: function(/* dojo.dnd.Mover */ mover){
			if(mover._timer){
				// stop timer
				clearTimeout(mover._timer)
				// reflect the last received position
				oldOnMove.call(this, mover, mover._leftTop)
			}
			dojo.dnd.Moveable.prototype.onMoveStop.apply(this, arguments);
		},
		onMove: function(/* dojo.dnd.Mover */ mover, /* Object */ leftTop){
			mover._leftTop = leftTop;
			if(!mover._timer){
				var _t = this;	// to avoid using dojo.hitch()
				mover._timer = setTimeout(function(){
					// we don't have any pending requests
					mover._timer = null;
					// reflect the last received position
					oldOnMove.call(_t, mover, mover._leftTop);
				}, this.timeout);
			}
		}
	});
})();

}

if(!dojo._hasResource["dojo.i18n"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.i18n"] = true;
dojo.provide("dojo.i18n");

dojo.getObject("i18n", true, dojo);

/*=====
dojo.i18n = {
	// summary: Utility classes to enable loading of resources for internationalization (i18n)
};
=====*/

// when using a real AMD loader, dojo.i18n.getLocalization is already defined by dojo/lib/backCompat
dojo.i18n.getLocalization = dojo.i18n.getLocalization || function(/*String*/packageName, /*String*/bundleName, /*String?*/locale){
	//	summary:
	//		Returns an Object containing the localization for a given resource
	//		bundle in a package, matching the specified locale.
	//	description:
	//		Returns a hash containing name/value pairs in its prototypesuch
	//		that values can be easily overridden.  Throws an exception if the
	//		bundle is not found.  Bundle must have already been loaded by
	//		`dojo.requireLocalization()` or by a build optimization step.  NOTE:
	//		try not to call this method as part of an object property
	//		definition (`var foo = { bar: dojo.i18n.getLocalization() }`).  In
	//		some loading situations, the bundle may not be available in time
	//		for the object definition.  Instead, call this method inside a
	//		function that is run after all modules load or the page loads (like
	//		in `dojo.addOnLoad()`), or in a widget lifecycle method.
	//	packageName:
	//		package which is associated with this resource
	//	bundleName:
	//		the base filename of the resource bundle (without the ".js" suffix)
	//	locale:
	//		the variant to load (optional).  By default, the locale defined by
	//		the host environment: dojo.locale

	locale = dojo.i18n.normalizeLocale(locale);

	// look for nearest locale match
	var elements = locale.split('-');
	var module = [packageName,"nls",bundleName].join('.');
		var bundle = dojo._loadedModules[module];
	if(bundle){
		var localization;
		for(var i = elements.length; i > 0; i--){
			var loc = elements.slice(0, i).join('_');
			if(bundle[loc]){
				localization = bundle[loc];
				break;
			}
		}
		if(!localization){
			localization = bundle.ROOT;
		}

		// make a singleton prototype so that the caller won't accidentally change the values globally
		if(localization){
			var clazz = function(){};
			clazz.prototype = localization;
			return new clazz(); // Object
		}
	}

	throw new Error("Bundle not found: " + bundleName + " in " + packageName+" , locale=" + locale);
};

dojo.i18n.normalizeLocale = function(/*String?*/locale){
	//	summary:
	//		Returns canonical form of locale, as used by Dojo.
	//
	//  description:
	//		All variants are case-insensitive and are separated by '-' as specified in [RFC 3066](http://www.ietf.org/rfc/rfc3066.txt).
	//		If no locale is specified, the dojo.locale is returned.  dojo.locale is defined by
	//		the user agent's locale unless overridden by djConfig.

	var result = locale ? locale.toLowerCase() : dojo.locale;
	if(result == "root"){
		result = "ROOT";
	}
	return result; // String
};

dojo.i18n._requireLocalization = function(/*String*/moduleName, /*String*/bundleName, /*String?*/locale, /*String?*/availableFlatLocales){
	//	summary:
	//		See dojo.requireLocalization()
	//	description:
	// 		Called by the bootstrap, but factored out so that it is only
	// 		included in the build when needed.

	var targetLocale = dojo.i18n.normalizeLocale(locale);
 	var bundlePackage = [moduleName, "nls", bundleName].join(".");
	// NOTE:
	//		When loading these resources, the packaging does not match what is
	//		on disk.  This is an implementation detail, as this is just a
	//		private data structure to hold the loaded resources.  e.g.
	//		`tests/hello/nls/en-us/salutations.js` is loaded as the object
	//		`tests.hello.nls.salutations.en_us={...}` The structure on disk is
	//		intended to be most convenient for developers and translators, but
	//		in memory it is more logical and efficient to store in a different
	//		order.  Locales cannot use dashes, since the resulting path will
	//		not evaluate as valid JS, so we translate them to underscores.

	//Find the best-match locale to load if we have available flat locales.
	var bestLocale = "";
	if(availableFlatLocales){
		var flatLocales = availableFlatLocales.split(",");
		for(var i = 0; i < flatLocales.length; i++){
			//Locale must match from start of string.
			//Using ["indexOf"] so customBase builds do not see
			//this as a dojo._base.array dependency.
			if(targetLocale["indexOf"](flatLocales[i]) == 0){
				if(flatLocales[i].length > bestLocale.length){
					bestLocale = flatLocales[i];
				}
			}
		}
		if(!bestLocale){
			bestLocale = "ROOT";
		}
	}

	//See if the desired locale is already loaded.
	var tempLocale = availableFlatLocales ? bestLocale : targetLocale;
	var bundle = dojo._loadedModules[bundlePackage];
	var localizedBundle = null;
	if(bundle){
		if(dojo.config.localizationComplete && bundle._built){return;}
		var jsLoc = tempLocale.replace(/-/g, '_');
		var translationPackage = bundlePackage+"."+jsLoc;
		localizedBundle = dojo._loadedModules[translationPackage];
	}

	if(!localizedBundle){
		bundle = dojo["provide"](bundlePackage);
		var syms = dojo._getModuleSymbols(moduleName);
		var modpath = syms.concat("nls").join("/");
		var parent;

		dojo.i18n._searchLocalePath(tempLocale, availableFlatLocales, function(loc){
			var jsLoc = loc.replace(/-/g, '_');
			var translationPackage = bundlePackage + "." + jsLoc;
			var loaded = false;
			if(!dojo._loadedModules[translationPackage]){
				// Mark loaded whether it's found or not, so that further load attempts will not be made
				dojo["provide"](translationPackage);
				var module = [modpath];
				if(loc != "ROOT"){module.push(loc);}
				module.push(bundleName);
				var filespec = module.join("/") + '.js';
				loaded = dojo._loadPath(filespec, null, function(hash){
					hash = hash.root || hash;
					// Use singleton with prototype to point to parent bundle, then mix-in result from loadPath
					var clazz = function(){};
					clazz.prototype = parent;
					bundle[jsLoc] = new clazz();
					for(var j in hash){ bundle[jsLoc][j] = hash[j]; }
				});
			}else{
				loaded = true;
			}
			if(loaded && bundle[jsLoc]){
				parent = bundle[jsLoc];
			}else{
				bundle[jsLoc] = parent;
			}

			if(availableFlatLocales){
				//Stop the locale path searching if we know the availableFlatLocales, since
				//the first call to this function will load the only bundle that is needed.
				return true;
			}
		});
	}

	//Save the best locale bundle as the target locale bundle when we know the
	//the available bundles.
	if(availableFlatLocales && targetLocale != bestLocale){
		bundle[targetLocale.replace(/-/g, '_')] = bundle[bestLocale.replace(/-/g, '_')];
	}
};

(function(){
	// If other locales are used, dojo.requireLocalization should load them as
	// well, by default.
	//
	// Override dojo.requireLocalization to do load the default bundle, then
	// iterate through the extraLocale list and load those translations as
	// well, unless a particular locale was requested.

	var extra = dojo.config.extraLocale;
	if(extra){
		if(!extra instanceof Array){
			extra = [extra];
		}

		var req = dojo.i18n._requireLocalization;
		dojo.i18n._requireLocalization = function(m, b, locale, availableFlatLocales){
			req(m,b,locale, availableFlatLocales);
			if(locale){return;}
			for(var i=0; i<extra.length; i++){
				req(m,b,extra[i], availableFlatLocales);
			}
		};
	}
})();

dojo.i18n._searchLocalePath = function(/*String*/locale, /*Boolean*/down, /*Function*/searchFunc){
	//	summary:
	//		A helper method to assist in searching for locale-based resources.
	//		Will iterate through the variants of a particular locale, either up
	//		or down, executing a callback function.  For example, "en-us" and
	//		true will try "en-us" followed by "en" and finally "ROOT".

	locale = dojo.i18n.normalizeLocale(locale);

	var elements = locale.split('-');
	var searchlist = [];
	for(var i = elements.length; i > 0; i--){
		searchlist.push(elements.slice(0, i).join('-'));
	}
	searchlist.push(false);
	if(down){searchlist.reverse();}

	for(var j = searchlist.length - 1; j >= 0; j--){
		var loc = searchlist[j] || "ROOT";
		var stop = searchFunc(loc);
		if(stop){ break; }
	}
};

dojo.i18n._preloadLocalizations = function(/*String*/bundlePrefix, /*Array*/localesGenerated){
	//	summary:
	//		Load built, flattened resource bundles, if available for all
	//		locales used in the page. Only called by built layer files.

	function preload(locale){
		locale = dojo.i18n.normalizeLocale(locale);
		dojo.i18n._searchLocalePath(locale, true, function(loc){
			for(var i=0; i<localesGenerated.length;i++){
				if(localesGenerated[i] == loc){
					dojo["require"](bundlePrefix+"_"+loc);
					return true; // Boolean
				}
			}
			return false; // Boolean
		});
	}
	preload();
	var extra = dojo.config.extraLocale||[];
	for(var i=0; i<extra.length; i++){
		preload(extra[i]);
	}
};

}

if(!dojo._hasResource["epages._workaround"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["epages._workaround"] = true;
dojo.provide("epages._workaround");

dojo['requireLocalization']("dojo.cldr", "gregorian");

epages._workaround.load = function(module) {
	if( module == 'dojo.parser' ){
		/* on run dojo parser run jQuery metaparse also */
		var parser = dojo.parser.parse;
		dojo.parser.parse = function( rootNode, args ){
			jQuery(document).on('metaparse',function () {
				jQuery(rootNode)
					.find('.ep-js')
					.metaparse();
				});
			return parser.call( dojo.parser, rootNode, args );
		};
		return;
	}

	if (module == 'epages.widget.Calendar' && dojo.version.revision == '0') {

		dijit._placeOnScreenAroundRect = function(
			/* DomNode */		node,
			/* Number */		x,
			/* Number */		y,
			/* Number */		width,
			/* Number */		height,
			/* Object */		aroundCorners,
			/* Function */		layoutNode){

			// summary:
			//		Like dijit.placeOnScreenAroundNode(), except it accepts coordinates
			//		of a rectangle to place node adjacent to.

			// TODO: combine with placeOnScreenAroundRectangle()

			// Generate list of possible positions for node
			var choices = [];
			for(var nodeCorner in aroundCorners){
				choices.push( {
					aroundCorner: nodeCorner,
					corner: aroundCorners[nodeCorner],
					pos: {
						x: x + (nodeCorner.charAt(1) == 'L' ? 0 : width),
						y: y + (nodeCorner.charAt(0) == 'T' ? 0 : height)
					}
				});
			}
		//Start of workaround
			// only correct popups (calendar) with {'BR':'TR','BL':'TL'}
			if (choices.length == 2 &&
				  choices[0].aroundCorner == 'BR' && choices[0].corner == 'TR' &&
					choices[1].aroundCorner == 'BL' && choices[1].corner == 'TL') {
						choices = choices.reverse();
			}
		//End of Workaround
			return dijit._place(node, choices, layoutNode);
		};

		return;
	}

	if (module == 'dijit.Tree' && dojo.version.revision == '20973') {

		//add publish to expand method of tree nodes
		dojo.connect(dijit._TreeNode.prototype, "expand", function() {
			dojo.publish(this.tree.id+'/nodeexpand',[this]);
		});

		return;
	}
	else if (module == 'dojo.cldr.supplemental') {
		dojo.cldr.supplemental.getFirstDayOfWeek = function(/*String?*/locale){
			if (dojo.i18n.normalizeLocale(locale) == djConfig.locale){
				return epages.vars.Locale.firstDayOfWeek;
			}
			// summary: Returns a zero-based index for first day of the week
			// description:
			//		Returns a zero-based index for first day of the week, as used by the local (Gregorian) calendar.
			//		e.g. Sunday (returns 0), or Monday (returns 1)

			// from http://www.unicode.org/cldr/data/common/supplemental/supplementalData.xml:supplementalData/weekData/firstDay
			var firstDay = {/*default is 1=Monday*/
				mv:5,
				ae:6,af:6,bh:6,dj:6,dz:6,eg:6,er:6,et:6,iq:6,ir:6,jo:6,ke:6,kw:6,lb:6,ly:6,ma:6,om:6,qa:6,sa:6,
				sd:6,so:6,tn:6,ye:6,
				as:0,au:0,az:0,bw:0,ca:0,cn:0,fo:0,ge:0,gl:0,gu:0,hk:0,ie:0,il:0,is:0,jm:0,jp:0,kg:0,kr:0,la:0,
				mh:0,mo:0,mp:0,mt:0,nz:0,ph:0,pk:0,sg:0,th:0,tt:0,tw:0,um:0,us:0,uz:0,vi:0,za:0,zw:0,
				et:0,mw:0,ng:0,tj:0,
				// variant. do not use?		gb:0,
				sy:4
			};

			var country = dojo.cldr.supplemental._region(locale);
			var dow = firstDay[country];
			return (dow === undefined) ? 1 : dow; /*Number*/
		};

		return;
	}

	if(module == 'dijit._Widget' && dojo.version.revision == '20973'){
		dijit._Widget.prototype._deferredConnects = {};

		return;
	}
};

dojo.connect(dojo, "require", epages._workaround, "load");




}

if(!dojo._hasResource["epages.epages"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["epages.epages"] = true;
/*  EPages JavaScript module
 *  (c) 2006 ePages GmbH
 *--------------------------------------------------------------------------*/
dojo.provide("epages.epages");

(function() {
	// is this called in an iframe?
	// these vars are set, when epages is loaded in an iframe

	window.parentDojo = undefined;
	window.restrictedParent = undefined;

	try {
		window.parentDojo = parent.dojo ? parent.dojo : undefined;
		if(parent != self && !parent.dojo) {
			window.restrictedParent = true; // IE never throws an error inside of this try-catch statement
			dojo._global_omit_module_check = true; // dojo's module check would fail for IE in that case
		}
	}catch(ex) {
		window.restrictedParent = true;	// page embedded into cross site iframe -> resticted access
	}

	if(parent != self && window.parentDojo !== undefined) {
		parent.epages._windows.push(window);
		for( var i=0,iLength=parent.epages.onIframeStart.length ; i<iLength ; i++ ) {
			parent.epages.onIframeStart[i](parent, window);
		}
		// epages NavBars are globals
		if(epages.NavBars == undefined){
			epages.NavBars = {};
		}
		epages.NavBars = parent.epages.NavBars;
	}
})();

dojo.subscribe('epages/addGlobalSymbol', function(gvn, win) {
	if (win === undefined) {
		console.warn('no target window for addGlobalSymbol');
		return;
	}
	if (win[gvn] != null){
		return;
	}

	if(window.parentDojo){
		win[gvn] = parent[gvn];
	}
});

dojo.mixin( epages, {
	onIframeStart: [],
	onUnloadFunctions: [],
	topDojo: (parent != self && window.parentDojo !== undefined) ? window.parentDojo : dojo,
	topWindow: parent,
	constants: {
		preventCache: false,
		doFlashForward: true
	},
	theme: {
		name: 'epages',
		url:  dojo.moduleUrl('dijit.themes', 'epages')
	},
	// all known windows using epages
	_windows: [ window ],
	// all global vars
	_gvs: {},
	zIndex: 450,

	buildNode: function(attributeArray /* Array */, nodeOrTagRef /* String or DOM node */, content /* String - Optional */, containerNodeName /* String */) {
		// summary:
		//		Injects or expands a node in the current page source.
		// description:
		//		Since we're using attributes that are not valid XHTML syntax we have to cloak
		//		the instantiation of nodes with such attributes attached.
		//
		//		In the future this method could be used to increase the rendering performance.
		//		If the global configuration attribute "djConfig.parseOnLoad" is set to "false"
		//		we could take care of the parsing process in here (and therefore would save
		//		time which is now spent for the global dojo.query("[dojoType]") method).
		// attributeArray:
		//		An array with subarrays of the attributes to be attached to the node.
		// nodeOrTagRef:
		//		Can be a plain tag name > A new node with the specified tag will be created.
		//		Can be a node id as string > The specified node will be extended.
		//		Can be a node reference itself > The specified node will be extended.
		// content:
		//		The new content of the node.
		// example:
		//		epages.buildNode([
		//			["type", "text"],
		//			["dojoType", "epages.widget.NumberSpinner"],
		//			["name", "Quantity"],
		//			["class", "Quantity"],
		//			["value", "#MinOrder[float]"],
		//			["delta", "#IntervalOrder[float]"]
		//		], "input");

		if(attributeArray && attributeArray.length && nodeOrTagRef ) {

			var node;						// main node reference
			var containerNode;	// temporary parent node (never inserted to the DOM)

			if(typeof(nodeOrTagRef) === "string") {
				var nodeRef = $(nodeOrTagRef);
				if(nodeRef) {
					// an id string is passed
					node = nodeRef;
				}else {
					// a tag name is passed
					node = document.createElement(nodeOrTagRef);
					containerNode = document.createElement("div");
				}
			}else {
				// a node reference is passed
				node = nodeOrTagRef;
			}

			if(content) { // rewrites an existing node
				node.innerHTML = content;
			}

			for( var i=0,iLength=attributeArray.length ; i<iLength ; i++ ) {
				if(attributeArray[i][0] === "style") {
					node.style.cssText = attributeArray[i][1];
				}else if(attributeArray[i][0] === "name" && epages.Browser.engine === "MSIE") {
					// somehow the native attribute "name" is written uppercase in the MSIE. Hence we're attaching it separately for this browser.
					node.Name = attributeArray[i][1];
				}else {
					node.setAttribute(attributeArray[i][0], attributeArray[i][1]);
				}
			}

			if(containerNode) {
				if(!containerNodeName) {
					console.warn("error in buildNode method: no containerNodeName string given as container for creating new DOM nodes");
					return;
				}
				containerNode.appendChild(node);
				//document.write(containerNode.innerHTML);
				dojo.create('div', { 'innerHTML' : containerNode.innerHTML, 'style' : 'display:inline' }, containerNodeName, 'before');
			}
		}
	},

	flashForward: function () {
		return epages.constants.doFlashForward;
	},

	setTheme: function (/*String*/theme) {
		var body = dojo.body();
		var old = epages.theme.name;
		if (body){
			dojo.removeClass(body, epages.theme.name);
		} else {
			dojo.addOnLoad(function() {
				var body = dojo.body();
				if (body) {
					dojo.removeClass(body, epages.theme.name);
					dojo.addClass(body, theme);
				}
			});
		}
		epages.theme.name= theme;
		epages.theme.url = dojo.moduleUrl('dijit.themes', theme);
		if (body){
			dojo.addClass(body, theme);
		}
	},

	themeUrl: function (/*String*/path) {
		return epages.vars.IconsRoot;
	},

	reload: function() {
		window.location.hash += "0";
		var url = window.location.href;
		window.location.href = url.substring(0, url.indexOf('#'));
	},

	locationHref: function(uri) {
		window.location.href = uri.substr(0,1) == '?' ? epages.vars.BaseUrl + uri : uri;
	},

	setFormAction: function(formRef, newAction) {
		formRef.action = newAction.substr(0,1) == '?' ? epages.vars.BaseUrl + newAction : newAction;
	},

	locationWebRoot: function(uri) {
		return window.location.protocol + '//' + window.location.host + uri;
	},

	Class: function() {
		return function() {
			this.initialize.apply(this, arguments);
		};
	},

	changeSampleText: function(newdata,group) {
	// this function replaces the data in samp tags (newdata) which are specified
	// by their title attributes (group)
		if(!newdata){
			newdata="";
		}
		var sampletags = document.getElementsByTagName("samp");
		for( var i=0,iLength=sampletags.length ; i<iLength ; i++ ) {
			if(sampletags[i].getAttribute("title") == group) {
				if(sampletags[i].firstChild) {
					sampletags[i].firstChild.data = newdata;
				} else {
					var mydata = document.createTextNode(newdata);
					sampletags[i].appendChild(mydata);
				}
			}
		}
	},

	toggleNodes: function(className /* class of toggled elements */, clearSpace /* true = "display" else "visibility" */, classLimiter /* element that is been scanned for className */) {
		if(typeof className == "string") {
			if(typeof classLimiter == "undefined") {classLimiter = document;}
			if(typeof classLimiter == "string") {classLimiter = document.getElementById(classLimiter);}
			var domElements = classLimiter.getElementsByTagName("*");
			for( var i=0,iLength=domElements.length ; i<iLength ; i++ ) {
				if(domElements[i].attributes && domElements[i].attributes["class"] &&
						domElements[i].attributes["class"].nodeValue.indexOf(className) > -1) {
					if(clearSpace) {
						if(domElements[i].className.match(/HideNode/)) {
								domElements[i].className = domElements[i].className.replace(/HideNode/,'ShowNode');
						}else{
							if(domElements[i].className.match(/ShowNode/)) {
								domElements[i].className = domElements[i].className.replace(/ShowNode/,'HideNode');
							}else {
								if(domElements[i].style.display == "none"){
									domElements[i].style.display = "";
								}else if(domElements[i].style.display != "none"){
									domElements[i].style.display = "none";
								}
							}
						}
						if(domElements[i].className.match(/HideElement/)) {
								domElements[i].className = domElements[i].className.replace(/HideElement/,'ShowElement');
						}
					}else{
						if(domElements[i].style.visibility == "hidden") {
							domElements[i].style.visibility = "visible";
							if(domElements[i].style.display == "none"){
								domElements[i].style.display = "";
							}
							domElements[i].className = domElements[i].className.replace(/HideNode/,'ShowNode');
							domElements[i].className = domElements[i].className.replace(/HideElement/,'ShowElement');
						}else{
							domElements[i].style.visibility = "hidden";
						}
					}
				}
			}
		}
	},

	changeDelimiter: function(DelimiterSymbol, SelectId){
			switch(DelimiterSymbol) {
					case ',' :
					document.getElementById(SelectId).options[0].selected = 'selected';
					break;
					case ';' :
					document.getElementById(SelectId).options[1].selected = 'selected';
					break;
					default :
					document.getElementById(SelectId).options[2].selected = 'selected';
					break;
			}
	},

	toggleLink: function(LinkName){
		if(document.getElementsByName(LinkName)[0].className == 'ShowLink') {
			document.getElementsByName(LinkName)[0].className = 'HideLink';
		}
		else {
			document.getElementsByName(LinkName)[0].className = 'ShowLink';
		}
	},

	/* - MBO Search (Toolbar) -------------------------------------------------------------*/
	toggleExtender: function(object){
		// toggles between the image file names of the extend button
		// object => image node
		var splittedFilename = object.src.split("/");
		var lastItemNo = splittedFilename.length-1;
		var lastItem = splittedFilename[lastItemNo];

		if(lastItem=="toolbar_ico_s_extend.png") {
			splittedFilename[lastItemNo]="toolbar_ico_s_extend_open.png";
		} else {
			splittedFilename[lastItemNo]="toolbar_ico_s_extend.png";
		}
		object.src = splittedFilename.join("/");
	},

	getDocumentWindow: function(doc) {
		return (doc.parentWindow) ? doc.parentWindow : doc.defaultView;
	},

	// returns the url for the iframe wrapper
	// this is useful for external iframes to override/set the charset if it is missing
	getIframeWrapperUrl: function(/*string*/url, /*string?*/charset) {
		var src = '?iframeUrl=' + encodeURIComponent(url);
		if (charset){
			src += '&charset=' + encodeURIComponent(charset);
		}
		return dojo.moduleUrl('epages', 'iframe_wrapper.html') + src;
	},

	require: function(/*string*/packageName,/*modul*/modulName){
		//summary:
		//improve dojo.require to load shrinksafe packages
		//description:
		//check if modul is loaded
		//load package if it is not loaded
		//if package was not loaded correct do normal dojo.require

		// EPG-17181 : just do dojo.require
		if(modulName) {
			return dojo["require"](modulName);
		}
	},

	checkUnit: function(node,unitNode){
		//summary:
		//checks if unitNode has a value if node has a value
		//in error case DialogError class is added
		if(node && unitNode){
			if(unitNode.value === '' && node.value !== ''){
				dojo.addClass(unitNode, 'DialogError');
			}
			else{
				dojo.removeClass(unitNode, 'DialogError');
			}
		}
	}

});

// set default vars if not exists
if( !epages.vars ){
	epages.vars = { Locale: {} };
}

dojo.addOnLoad(function () { epages.setTheme(epages.theme.name); });

// load workarounds
dojo["require"]("epages._workaround");

$ = dojo.byId;

dojo.addOnLoad(function() {
	if(top == self){
		epages.vars.dojoUnloadStuff = window.onbeforeunload;
		window.onbeforeunload = function () {
			var l = false;
			var d = dojo;
			// for each registered epages.onUnloadFunctions element, exec the function and determine whether at least one function returned true
			d.forEach(epages.onUnloadFunctions, function (el) { l |= el(); });

			if (l) {
				d.require('epages.io.translation');
				return epages.io.dictionary.get("LoseChanges");
			}else {
				dojo["require"]("epages.browser");
				if(epages.Browser.engine == "MSIE" && epages.Browser.version > 6) {
					document.getElementsByTagName("body")[0].className += " SuspendUserInteraction";
					setTimeout(function() {
						dojo.removeClass(document.getElementsByTagName("body")[0], "SuspendUserInteraction");
					}, 2000);
					window.onunload = function() {
						if(epages.vars.dojoUnloadStuff) {epages.vars.dojoUnloadStuff();}
					};
				}
			}
			return;
		};
	}
});

epages.parentByTagName = function( node, tagName ){
	while( node.nodeName.toLocaleLowerCase()!=tagName ){
		node = node.parentNode;
		if( node.parentNode==document.body ){
			break;
		}
	}
	return node;
};

// set console (sometimes not set in time and hence throws an error)
try{
window.console = window.console || {};
window.console.debug = window.console.debug || function(){return;};
window.console.warn = window.console.warn || function(){return;};
}
catch(e){}

}

if(!dojo._hasResource['epages.preload']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['epages.preload'] = true;
/*
	Copyright (c) 2006-2007, ePages GmbH
	All Rights Reserved.
*/
dojo.provide('epages.preload');


epages.Preload = {
	// summary:
	//  preloads the given sources
	// description:
	// |preloads given sources listed in the "files" array
	// |never call the _preloadFiles method directly - simply push the necessary file into the "files" array
	// |there are two required properties: "file" and "priority"
	// |  the "file" property is the path / src to your file
	// |  the "priority" property states the importance of that particular file (e. g. images usually have got a lower priority than js packages)
	// example:
	// | dojo.addOnLoad(function() {
	// |   epages.Preload.add({file:"ep_megapackage.js", priority: 9});
	// | });
	start: function() {
		// summary:
		//  starts / restarts the preloading
		this.stopped = false;
		this._preloadFiles();
	},
	stop: function() {
		// summary:
		//  stops the preload operation on the next queued element
		this.stopped = true;
	},
	add: function(fileObject) {
		// summary:
		//  adds a file to the "files" array
		if(fileObject.file && fileObject.priority) {
			var isAlreadyAdded = false;
			for( var i=0,iLength=this.files.length ; i<iLength ; i++ ) {
				if(this.files[i].file == fileObject.file) {
					isAlreadyAdded = true
					break;
				}
			}
			if(!isAlreadyAdded) {
				this.files.push(fileObject);
			}
		}
	},
	stopped: false,
	_preloadFiles: function() {
		if(!this.stopped && this.files.length > 0) {
			this.nextPreloadFile = this.files[0];
			for( var i=0,iLength=this.files.length ; i<iLength ; i++ ) {
				if(this.files[i].priority >= this.nextPreloadFile.priority) {
					this.nextPreloadFile = this.files[i];
				}
			}
			if(this.nextPreloadFile.priority != -1) {
				this.nextPreloadFile.priority = -1;
				dojo.xhrGet({
					url:  this.nextPreloadFile.file,
					sync: false,
					load: function(){
						epages.Preload._preloadFiles();
						if(djConfig.isDebug){
							console.log('epages.Preload loaded : ',epages.Preload.nextPreloadFile.file);
						}
					},
					error: function(){
						epages.Preload._preloadFiles();
						if(djConfig.isDebug){
							console.warn('epages.Preload could not load : ',epages.Preload.nextPreloadFile.file);
						}
					}
				});
			}
			// Since this is only used once on page load, we do not need to watch for changes in the *files* array,
			// and even if we had to, we should use some sort of event handling instead of timeout monitoring.
			/*else {
				// since we are through we can be a bit lazy - but we surely have to monitor the "files" array for realtime changes
				setTimeout(function() {epages.Preload._preloadFiles()}, 500);
			}*/
		}
	},
	files: [],
	nextPreloadFile: undefined
};

dojo.addOnLoad(function() {
	if(epages.vars.BusyLayerActive){
		dojo.subscribe("uimessage/stopBusy",function(owner, timeout, duration){
			setTimeout(function(){
				epages.Preload.start();
			},timeout);
		});
	}
	else{
		setTimeout(function() {
			epages.Preload.start();
		}, 100);
	}
})

}

if(!dojo._hasResource["dijit._base.manager"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.manager"] = true;
dojo.provide("dijit._base.manager");


dojo.declare("dijit.WidgetSet", null, {
	// summary:
	//		A set of widgets indexed by id. A default instance of this class is
	//		available as `dijit.registry`
	//
	// example:
	//		Create a small list of widgets:
	//		|	var ws = new dijit.WidgetSet();
	//		|	ws.add(dijit.byId("one"));
	//		| 	ws.add(dijit.byId("two"));
	//		|	// destroy both:
	//		|	ws.forEach(function(w){ w.destroy(); });
	//
	// example:
	//		Using dijit.registry:
	//		|	dijit.registry.forEach(function(w){ /* do something */ });

	constructor: function(){
		this._hash = {};
		this.length = 0;
	},

	add: function(/*dijit._Widget*/ widget){
		// summary:
		//		Add a widget to this list. If a duplicate ID is detected, a error is thrown.
		//
		// widget: dijit._Widget
		//		Any dijit._Widget subclass.
		if(this._hash[widget.id]){
			throw new Error("Tried to register widget with id==" + widget.id + " but that id is already registered");
		}
		this._hash[widget.id] = widget;
		this.length++;
	},

	remove: function(/*String*/ id){
		// summary:
		//		Remove a widget from this WidgetSet. Does not destroy the widget; simply
		//		removes the reference.
		if(this._hash[id]){
			delete this._hash[id];
			this.length--;
		}
	},

	forEach: function(/*Function*/ func, /* Object? */thisObj){
		// summary:
		//		Call specified function for each widget in this set.
		//
		// func:
		//		A callback function to run for each item. Is passed the widget, the index
		//		in the iteration, and the full hash, similar to `dojo.forEach`.
		//
		// thisObj:
		//		An optional scope parameter
		//
		// example:
		//		Using the default `dijit.registry` instance:
		//		|	dijit.registry.forEach(function(widget){
		//		|		console.log(widget.declaredClass);
		//		|	});
		//
		// returns:
		//		Returns self, in order to allow for further chaining.

		thisObj = thisObj || dojo.global;
		var i = 0, id;
		for(id in this._hash){
			func.call(thisObj, this._hash[id], i++, this._hash);
		}
		return this;	// dijit.WidgetSet
	},

	filter: function(/*Function*/ filter, /* Object? */thisObj){
		// summary:
		//		Filter down this WidgetSet to a smaller new WidgetSet
		//		Works the same as `dojo.filter` and `dojo.NodeList.filter`
		//
		// filter:
		//		Callback function to test truthiness. Is passed the widget
		//		reference and the pseudo-index in the object.
		//
		// thisObj: Object?
		//		Option scope to use for the filter function.
		//
		// example:
		//		Arbitrary: select the odd widgets in this list
		//		|	dijit.registry.filter(function(w, i){
		//		|		return i % 2 == 0;
		//		|	}).forEach(function(w){ /* odd ones */ });

		thisObj = thisObj || dojo.global;
		var res = new dijit.WidgetSet(), i = 0, id;
		for(id in this._hash){
			var w = this._hash[id];
			if(filter.call(thisObj, w, i++, this._hash)){
				res.add(w);
			}
		}
		return res; // dijit.WidgetSet
	},

	byId: function(/*String*/ id){
		// summary:
		//		Find a widget in this list by it's id.
		// example:
		//		Test if an id is in a particular WidgetSet
		//		| var ws = new dijit.WidgetSet();
		//		| ws.add(dijit.byId("bar"));
		//		| var t = ws.byId("bar") // returns a widget
		//		| var x = ws.byId("foo"); // returns undefined

		return this._hash[id];	// dijit._Widget
	},

	byClass: function(/*String*/ cls){
		// summary:
		//		Reduce this widgetset to a new WidgetSet of a particular `declaredClass`
		//
		// cls: String
		//		The Class to scan for. Full dot-notated string.
		//
		// example:
		//		Find all `dijit.TitlePane`s in a page:
		//		|	dijit.registry.byClass("dijit.TitlePane").forEach(function(tp){ tp.close(); });

		var res = new dijit.WidgetSet(), id, widget;
		for(id in this._hash){
			widget = this._hash[id];
			if(widget.declaredClass == cls){
				res.add(widget);
			}
		 }
		 return res; // dijit.WidgetSet
},

	toArray: function(){
		// summary:
		//		Convert this WidgetSet into a true Array
		//
		// example:
		//		Work with the widget .domNodes in a real Array
		//		|	dojo.map(dijit.registry.toArray(), function(w){ return w.domNode; });

		var ar = [];
		for(var id in this._hash){
			ar.push(this._hash[id]);
		}
		return ar;	// dijit._Widget[]
},

	map: function(/* Function */func, /* Object? */thisObj){
		// summary:
		//		Create a new Array from this WidgetSet, following the same rules as `dojo.map`
		// example:
		//		|	var nodes = dijit.registry.map(function(w){ return w.domNode; });
		//
		// returns:
		//		A new array of the returned values.
		return dojo.map(this.toArray(), func, thisObj); // Array
	},

	every: function(func, thisObj){
		// summary:
		// 		A synthetic clone of `dojo.every` acting explicitly on this WidgetSet
		//
		// func: Function
		//		A callback function run for every widget in this list. Exits loop
		//		when the first false return is encountered.
		//
		// thisObj: Object?
		//		Optional scope parameter to use for the callback

		thisObj = thisObj || dojo.global;
		var x = 0, i;
		for(i in this._hash){
			if(!func.call(thisObj, this._hash[i], x++, this._hash)){
				return false; // Boolean
			}
		}
		return true; // Boolean
	},

	some: function(func, thisObj){
		// summary:
		// 		A synthetic clone of `dojo.some` acting explictly on this WidgetSet
		//
		// func: Function
		//		A callback function run for every widget in this list. Exits loop
		//		when the first true return is encountered.
		//
		// thisObj: Object?
		//		Optional scope parameter to use for the callback

		thisObj = thisObj || dojo.global;
		var x = 0, i;
		for(i in this._hash){
			if(func.call(thisObj, this._hash[i], x++, this._hash)){
				return true; // Boolean
			}
		}
		return false; // Boolean
	}

});

(function(){

	/*=====
	dijit.registry = {
		// summary:
		//		A list of widgets on a page.
		// description:
		//		Is an instance of `dijit.WidgetSet`
	};
	=====*/
	dijit.registry = new dijit.WidgetSet();

	var hash = dijit.registry._hash,
		attr = dojo.attr,
		hasAttr = dojo.hasAttr,
		style = dojo.style;

	dijit.byId = function(/*String|dijit._Widget*/ id){
		// summary:
		//		Returns a widget by it's id, or if passed a widget, no-op (like dojo.byId())
		return typeof id == "string" ? hash[id] : id; // dijit._Widget
	};

	var _widgetTypeCtr = {};
	dijit.getUniqueId = function(/*String*/widgetType){
		// summary:
		//		Generates a unique id for a given widgetType
	
		var id;
		do{
			id = widgetType + "_" +
				(widgetType in _widgetTypeCtr ?
					++_widgetTypeCtr[widgetType] : _widgetTypeCtr[widgetType] = 0);
		}while(hash[id]);
		return dijit._scopeName == "dijit" ? id : dijit._scopeName + "_" + id; // String
	};
	
	dijit.findWidgets = function(/*DomNode*/ root){
		// summary:
		//		Search subtree under root returning widgets found.
		//		Doesn't search for nested widgets (ie, widgets inside other widgets).
	
		var outAry = [];
	
		function getChildrenHelper(root){
			for(var node = root.firstChild; node; node = node.nextSibling){
				if(node.nodeType == 1){
					var widgetId = node.getAttribute("widgetId");
					if(widgetId){
						var widget = hash[widgetId];
						if(widget){	// may be null on page w/multiple dojo's loaded
							outAry.push(widget);
						}
					}else{
						getChildrenHelper(node);
					}
				}
			}
		}
	
		getChildrenHelper(root);
		return outAry;
	};
	
	dijit._destroyAll = function(){
		// summary:
		//		Code to destroy all widgets and do other cleanup on page unload
	
		// Clean up focus manager lingering references to widgets and nodes
		dijit._curFocus = null;
		dijit._prevFocus = null;
		dijit._activeStack = [];
	
		// Destroy all the widgets, top down
		dojo.forEach(dijit.findWidgets(dojo.body()), function(widget){
			// Avoid double destroy of widgets like Menu that are attached to <body>
			// even though they are logically children of other widgets.
			if(!widget._destroyed){
				if(widget.destroyRecursive){
					widget.destroyRecursive();
				}else if(widget.destroy){
					widget.destroy();
				}
			}
		});
	};
	
	if(dojo.isIE){
		// Only run _destroyAll() for IE because we think it's only necessary in that case,
		// and because it causes problems on FF.  See bug #3531 for details.
		dojo.addOnWindowUnload(function(){
			dijit._destroyAll();
		});
	}
	
	dijit.byNode = function(/*DOMNode*/ node){
		// summary:
		//		Returns the widget corresponding to the given DOMNode
		return hash[node.getAttribute("widgetId")]; // dijit._Widget
	};
	
	dijit.getEnclosingWidget = function(/*DOMNode*/ node){
		// summary:
		//		Returns the widget whose DOM tree contains the specified DOMNode, or null if
		//		the node is not contained within the DOM tree of any widget
		while(node){
			var id = node.getAttribute && node.getAttribute("widgetId");
			if(id){
				return hash[id];
			}
			node = node.parentNode;
		}
		return null;
	};

	var shown = (dijit._isElementShown = function(/*Element*/ elem){
		var s = style(elem);
		return (s.visibility != "hidden")
			&& (s.visibility != "collapsed")
			&& (s.display != "none")
			&& (attr(elem, "type") != "hidden");
	});
	
	dijit.hasDefaultTabStop = function(/*Element*/ elem){
		// summary:
		//		Tests if element is tab-navigable even without an explicit tabIndex setting
	
		// No explicit tabIndex setting, need to investigate node type
		switch(elem.nodeName.toLowerCase()){
			case "a":
				// An <a> w/out a tabindex is only navigable if it has an href
				return hasAttr(elem, "href");
			case "area":
			case "button":
			case "input":
			case "object":
			case "select":
			case "textarea":
				// These are navigable by default
				return true;
			case "iframe":
				// If it's an editor <iframe> then it's tab navigable.
				var body;
				try{
					// non-IE
					var contentDocument = elem.contentDocument;
					if("designMode" in contentDocument && contentDocument.designMode == "on"){
						return true;
					}
					body = contentDocument.body;
				}catch(e1){
					// contentWindow.document isn't accessible within IE7/8
					// if the iframe.src points to a foreign url and this
					// page contains an element, that could get focus
					try{
						body = elem.contentWindow.document.body;
					}catch(e2){
						return false;
					}
				}
				return body.contentEditable == 'true' || (body.firstChild && body.firstChild.contentEditable == 'true');
			default:
				return elem.contentEditable == 'true';
		}
	};
	
	var isTabNavigable = (dijit.isTabNavigable = function(/*Element*/ elem){
		// summary:
		//		Tests if an element is tab-navigable
	
		// TODO: convert (and rename method) to return effective tabIndex; will save time in _getTabNavigable()
		if(attr(elem, "disabled")){
			return false;
		}else if(hasAttr(elem, "tabIndex")){
			// Explicit tab index setting
			return attr(elem, "tabIndex") >= 0; // boolean
		}else{
			// No explicit tabIndex setting, so depends on node type
			return dijit.hasDefaultTabStop(elem);
		}
	});

	dijit._getTabNavigable = function(/*DOMNode*/ root){
		// summary:
		//		Finds descendants of the specified root node.
		//
		// description:
		//		Finds the following descendants of the specified root node:
		//		* the first tab-navigable element in document order
		//		  without a tabIndex or with tabIndex="0"
		//		* the last tab-navigable element in document order
		//		  without a tabIndex or with tabIndex="0"
		//		* the first element in document order with the lowest
		//		  positive tabIndex value
		//		* the last element in document order with the highest
		//		  positive tabIndex value
		var first, last, lowest, lowestTabindex, highest, highestTabindex, radioSelected = {};
		function radioName(node) {
			// If this element is part of a radio button group, return the name for that group.
			return node && node.tagName.toLowerCase() == "input" &&
				node.type && node.type.toLowerCase() == "radio" &&
				node.name && node.name.toLowerCase();
		}
		var walkTree = function(/*DOMNode*/parent){
			dojo.query("> *", parent).forEach(function(child){
				// Skip hidden elements, and also non-HTML elements (those in custom namespaces) in IE,
				// since show() invokes getAttribute("type"), which crash on VML nodes in IE.
				if((dojo.isIE && child.scopeName!=="HTML") || !shown(child)){
					return;
				}

				if(isTabNavigable(child)){
					var tabindex = attr(child, "tabIndex");
					if(!hasAttr(child, "tabIndex") || tabindex == 0){
						if(!first){ first = child; }
						last = child;
					}else if(tabindex > 0){
						if(!lowest || tabindex < lowestTabindex){
							lowestTabindex = tabindex;
							lowest = child;
						}
						if(!highest || tabindex >= highestTabindex){
							highestTabindex = tabindex;
							highest = child;
						}
					}
					var rn = radioName(child);
					if(dojo.attr(child, "checked") && rn) {
						radioSelected[rn] = child;
					}
				}
				if(child.nodeName.toUpperCase() != 'SELECT'){
					walkTree(child);
				}
			});
		};
		if(shown(root)){ walkTree(root) }
		function rs(node) {
			// substitute checked radio button for unchecked one, if there is a checked one with the same name.
			return radioSelected[radioName(node)] || node;
		}
		return { first: rs(first), last: rs(last), lowest: rs(lowest), highest: rs(highest) };
	}
	dijit.getFirstInTabbingOrder = function(/*String|DOMNode*/ root){
		// summary:
		//		Finds the descendant of the specified root node
		//		that is first in the tabbing order
		var elems = dijit._getTabNavigable(dojo.byId(root));
		return elems.lowest ? elems.lowest : elems.first; // DomNode
	};
	
	dijit.getLastInTabbingOrder = function(/*String|DOMNode*/ root){
		// summary:
		//		Finds the descendant of the specified root node
		//		that is last in the tabbing order
		var elems = dijit._getTabNavigable(dojo.byId(root));
		return elems.last ? elems.last : elems.highest; // DomNode
	};
	
	/*=====
	dojo.mixin(dijit, {
		// defaultDuration: Integer
		//		The default animation speed (in ms) to use for all Dijit
		//		transitional animations, unless otherwise specified
		//		on a per-instance basis. Defaults to 200, overrided by
		//		`djConfig.defaultDuration`
		defaultDuration: 200
	});
	=====*/
	
	dijit.defaultDuration = dojo.config["defaultDuration"] || 200;

})();

}

if(!dojo._hasResource["dijit._base.focus"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.focus"] = true;
dojo.provide("dijit._base.focus");




// summary:
//		These functions are used to query or set the focus and selection.
//
//		Also, they trace when widgets become activated/deactivated,
//		so that the widget can fire _onFocus/_onBlur events.
//		"Active" here means something similar to "focused", but
//		"focus" isn't quite the right word because we keep track of
//		a whole stack of "active" widgets.  Example: ComboButton --> Menu -->
//		MenuItem.  The onBlur event for ComboButton doesn't fire due to focusing
//		on the Menu or a MenuItem, since they are considered part of the
//		ComboButton widget.  It only happens when focus is shifted
//		somewhere completely different.

dojo.mixin(dijit, {
	// _curFocus: DomNode
	//		Currently focused item on screen
	_curFocus: null,

	// _prevFocus: DomNode
	//		Previously focused item on screen
	_prevFocus: null,

	isCollapsed: function(){
		// summary:
		//		Returns true if there is no text selected
		return dijit.getBookmark().isCollapsed;
	},

	getBookmark: function(){
		// summary:
		//		Retrieves a bookmark that can be used with moveToBookmark to return to the same range
		var bm, rg, tg, sel = dojo.doc.selection, cf = dijit._curFocus;

		if(dojo.global.getSelection){
			//W3C Range API for selections.
			sel = dojo.global.getSelection();
			if(sel){
				if(sel.isCollapsed){
					tg = cf? cf.tagName : "";
					if(tg){
						//Create a fake rangelike item to restore selections.
						tg = tg.toLowerCase();
						if(tg == "textarea" ||
								(tg == "input" && (!cf.type || cf.type.toLowerCase() == "text"))){
							sel = {
								start: cf.selectionStart,
								end: cf.selectionEnd,
								node: cf,
								pRange: true
							};
							return {isCollapsed: (sel.end <= sel.start), mark: sel}; //Object.
						}
					}
					bm = {isCollapsed:true};
					if(sel.rangeCount){
						bm.mark = sel.getRangeAt(0).cloneRange();
					}
				}else{
					rg = sel.getRangeAt(0);
					bm = {isCollapsed: false, mark: rg.cloneRange()};
				}
			}
		}else if(sel){
			// If the current focus was a input of some sort and no selection, don't bother saving
			// a native bookmark.  This is because it causes issues with dialog/page selection restore.
			// So, we need to create psuedo bookmarks to work with.
			tg = cf ? cf.tagName : "";
			tg = tg.toLowerCase();
			if(cf && tg && (tg == "button" || tg == "textarea" || tg == "input")){
				if(sel.type && sel.type.toLowerCase() == "none"){
					return {
						isCollapsed: true,
						mark: null
					}
				}else{
					rg = sel.createRange();
					return {
						isCollapsed: rg.text && rg.text.length?false:true,
						mark: {
							range: rg,
							pRange: true
						}
					};
				}
			}
			bm = {};

			//'IE' way for selections.
			try{
				// createRange() throws exception when dojo in iframe
				//and nothing selected, see #9632
				rg = sel.createRange();
				bm.isCollapsed = !(sel.type == 'Text' ? rg.htmlText.length : rg.length);
			}catch(e){
				bm.isCollapsed = true;
				return bm;
			}
			if(sel.type.toUpperCase() == 'CONTROL'){
				if(rg.length){
					bm.mark=[];
					var i=0,len=rg.length;
					while(i<len){
						bm.mark.push(rg.item(i++));
					}
				}else{
					bm.isCollapsed = true;
					bm.mark = null;
				}
			}else{
				bm.mark = rg.getBookmark();
			}
		}else{
			console.warn("No idea how to store the current selection for this browser!");
		}
		return bm; // Object
	},

	moveToBookmark: function(/*Object*/bookmark){
		// summary:
		//		Moves current selection to a bookmark
		// bookmark:
		//		This should be a returned object from dijit.getBookmark()

		var _doc = dojo.doc,
			mark = bookmark.mark;
		if(mark){
			if(dojo.global.getSelection){
				//W3C Rangi API (FF, WebKit, Opera, etc)
				var sel = dojo.global.getSelection();
				if(sel && sel.removeAllRanges){
					if(mark.pRange){
						var r = mark;
						var n = r.node;
						n.selectionStart = r.start;
						n.selectionEnd = r.end;
					}else{
						sel.removeAllRanges();
						sel.addRange(mark);
					}
				}else{
					console.warn("No idea how to restore selection for this browser!");
				}
			}else if(_doc.selection && mark){
				//'IE' way.
				var rg;
				if(mark.pRange){
					rg = mark.range;
				}else if(dojo.isArray(mark)){
					rg = _doc.body.createControlRange();
					//rg.addElement does not have call/apply method, so can not call it directly
					//rg is not available in "range.addElement(item)", so can't use that either
					dojo.forEach(mark, function(n){
						rg.addElement(n);
					});
				}else{
					rg = _doc.body.createTextRange();
					rg.moveToBookmark(mark);
				}
				rg.select();
			}
		}
	},

	getFocus: function(/*Widget?*/ menu, /*Window?*/ openedForWindow){
		// summary:
		//		Called as getFocus(), this returns an Object showing the current focus
		//		and selected text.
		//
		//		Called as getFocus(widget), where widget is a (widget representing) a button
		//		that was just pressed, it returns where focus was before that button
		//		was pressed.   (Pressing the button may have either shifted focus to the button,
		//		or removed focus altogether.)   In this case the selected text is not returned,
		//		since it can't be accurately determined.
		//
		// menu: dijit._Widget or {domNode: DomNode} structure
		//		The button that was just pressed.  If focus has disappeared or moved
		//		to this button, returns the previous focus.  In this case the bookmark
		//		information is already lost, and null is returned.
		//
		// openedForWindow:
		//		iframe in which menu was opened
		//
		// returns:
		//		A handle to restore focus/selection, to be passed to `dijit.focus`
		var node = !dijit._curFocus || (menu && dojo.isDescendant(dijit._curFocus, menu.domNode)) ? dijit._prevFocus : dijit._curFocus;
		return {
			node: node,
			bookmark: (node == dijit._curFocus) && dojo.withGlobal(openedForWindow || dojo.global, dijit.getBookmark),
			openedForWindow: openedForWindow
		}; // Object
	},

	focus: function(/*Object || DomNode */ handle){
		// summary:
		//		Sets the focused node and the selection according to argument.
		//		To set focus to an iframe's content, pass in the iframe itself.
		// handle:
		//		object returned by get(), or a DomNode

		if(!handle){ return; }

		var node = "node" in handle ? handle.node : handle,		// because handle is either DomNode or a composite object
			bookmark = handle.bookmark,
			openedForWindow = handle.openedForWindow,
			collapsed = bookmark ? bookmark.isCollapsed : false;

		// Set the focus
		// Note that for iframe's we need to use the <iframe> to follow the parentNode chain,
		// but we need to set focus to iframe.contentWindow
		if(node){
			var focusNode = (node.tagName.toLowerCase() == "iframe") ? node.contentWindow : node;
			if(focusNode && focusNode.focus){
				try{
					// Gecko throws sometimes if setting focus is impossible,
					// node not displayed or something like that
					focusNode.focus();
				}catch(e){/*quiet*/}
			}
			dijit._onFocusNode(node);
		}

		// set the selection
		// do not need to restore if current selection is not empty
		// (use keyboard to select a menu item) or if previous selection was collapsed
		// as it may cause focus shift (Esp in IE).
		if(bookmark && dojo.withGlobal(openedForWindow || dojo.global, dijit.isCollapsed) && !collapsed){
			if(openedForWindow){
				openedForWindow.focus();
			}
			try{
				dojo.withGlobal(openedForWindow || dojo.global, dijit.moveToBookmark, null, [bookmark]);
			}catch(e2){
				/*squelch IE internal error, see http://trac.dojotoolkit.org/ticket/1984 */
			}
		}
	},

	// _activeStack: dijit._Widget[]
	//		List of currently active widgets (focused widget and it's ancestors)
	_activeStack: [],

	registerIframe: function(/*DomNode*/ iframe){
		// summary:
		//		Registers listeners on the specified iframe so that any click
		//		or focus event on that iframe (or anything in it) is reported
		//		as a focus/click event on the <iframe> itself.
		// description:
		//		Currently only used by editor.
		// returns:
		//		Handle to pass to unregisterIframe()
		return dijit.registerWin(iframe.contentWindow, iframe);
	},

	unregisterIframe: function(/*Object*/ handle){
		// summary:
		//		Unregisters listeners on the specified iframe created by registerIframe.
		//		After calling be sure to delete or null out the handle itself.
		// handle:
		//		Handle returned by registerIframe()

		dijit.unregisterWin(handle);
	},

	registerWin: function(/*Window?*/targetWindow, /*DomNode?*/ effectiveNode){
		// summary:
		//		Registers listeners on the specified window (either the main
		//		window or an iframe's window) to detect when the user has clicked somewhere
		//		or focused somewhere.
		// description:
		//		Users should call registerIframe() instead of this method.
		// targetWindow:
		//		If specified this is the window associated with the iframe,
		//		i.e. iframe.contentWindow.
		// effectiveNode:
		//		If specified, report any focus events inside targetWindow as
		//		an event on effectiveNode, rather than on evt.target.
		// returns:
		//		Handle to pass to unregisterWin()

		// TODO: make this function private in 2.0; Editor/users should call registerIframe(),

		var mousedownListener = function(evt){
			dijit._justMouseDowned = true;
			setTimeout(function(){ dijit._justMouseDowned = false; }, 0);
			
			// workaround weird IE bug where the click is on an orphaned node
			// (first time clicking a Select/DropDownButton inside a TooltipDialog)
			if(dojo.isIE && evt && evt.srcElement && evt.srcElement.parentNode == null){
				return;
			}

			dijit._onTouchNode(effectiveNode || evt.target || evt.srcElement, "mouse");
		};
		//dojo.connect(targetWindow, "onscroll", ???);

		// Listen for blur and focus events on targetWindow's document.
		// IIRC, I'm using attachEvent() rather than dojo.connect() because focus/blur events don't bubble
		// through dojo.connect(), and also maybe to catch the focus events early, before onfocus handlers
		// fire.
		// Connect to <html> (rather than document) on IE to avoid memory leaks, but document on other browsers because
		// (at least for FF) the focus event doesn't fire on <html> or <body>.
		var doc = dojo.isIE ? targetWindow.document.documentElement : targetWindow.document;
		if(doc){
			if(dojo.isIE){
				targetWindow.document.body.attachEvent('onmousedown', mousedownListener);
				var activateListener = function(evt){
					// IE reports that nodes like <body> have gotten focus, even though they have tabIndex=-1,
					// Should consider those more like a mouse-click than a focus....
					if(evt.srcElement.tagName.toLowerCase() != "#document" &&
						dijit.isTabNavigable(evt.srcElement)){
						dijit._onFocusNode(effectiveNode || evt.srcElement);
					}else{
						dijit._onTouchNode(effectiveNode || evt.srcElement);
					}
				};
				doc.attachEvent('onactivate', activateListener);
				var deactivateListener =  function(evt){
					dijit._onBlurNode(effectiveNode || evt.srcElement);
				};
				doc.attachEvent('ondeactivate', deactivateListener);

				return function(){
					targetWindow.document.detachEvent('onmousedown', mousedownListener);
					doc.detachEvent('onactivate', activateListener);
					doc.detachEvent('ondeactivate', deactivateListener);
					doc = null;	// prevent memory leak (apparent circular reference via closure)
				};
			}else{
				doc.body.addEventListener('mousedown', mousedownListener, true);
				var focusListener = function(evt){
					dijit._onFocusNode(effectiveNode || evt.target);
				};
				doc.addEventListener('focus', focusListener, true);
				var blurListener = function(evt){
					dijit._onBlurNode(effectiveNode || evt.target);
				};
				doc.addEventListener('blur', blurListener, true);

				return function(){
					doc.body.removeEventListener('mousedown', mousedownListener, true);
					doc.removeEventListener('focus', focusListener, true);
					doc.removeEventListener('blur', blurListener, true);
					doc = null;	// prevent memory leak (apparent circular reference via closure)
				};
			}
		}
	},

	unregisterWin: function(/*Handle*/ handle){
		// summary:
		//		Unregisters listeners on the specified window (either the main
		//		window or an iframe's window) according to handle returned from registerWin().
		//		After calling be sure to delete or null out the handle itself.

		// Currently our handle is actually a function
		handle && handle();
	},

	_onBlurNode: function(/*DomNode*/ node){
		// summary:
		// 		Called when focus leaves a node.
		//		Usually ignored, _unless_ it *isn't* follwed by touching another node,
		//		which indicates that we tabbed off the last field on the page,
		//		in which case every widget is marked inactive
		dijit._prevFocus = dijit._curFocus;
		dijit._curFocus = null;

		if(dijit._justMouseDowned){
			// the mouse down caused a new widget to be marked as active; this blur event
			// is coming late, so ignore it.
			return;
		}

		// if the blur event isn't followed by a focus event then mark all widgets as inactive.
		if(dijit._clearActiveWidgetsTimer){
			clearTimeout(dijit._clearActiveWidgetsTimer);
		}
		dijit._clearActiveWidgetsTimer = setTimeout(function(){
			delete dijit._clearActiveWidgetsTimer;
			dijit._setStack([]);
			dijit._prevFocus = null;
		}, 100);
	},

	_onTouchNode: function(/*DomNode*/ node, /*String*/ by){
		// summary:
		//		Callback when node is focused or mouse-downed
		// node:
		//		The node that was touched.
		// by:
		//		"mouse" if the focus/touch was caused by a mouse down event

		// ignore the recent blurNode event
		if(dijit._clearActiveWidgetsTimer){
			clearTimeout(dijit._clearActiveWidgetsTimer);
			delete dijit._clearActiveWidgetsTimer;
		}

		// compute stack of active widgets (ex: ComboButton --> Menu --> MenuItem)
		var newStack=[];
		try{
			while(node){
				var popupParent = dojo.attr(node, "dijitPopupParent");
				if(popupParent){
					node=dijit.byId(popupParent).domNode;
				}else if(node.tagName && node.tagName.toLowerCase() == "body"){
					// is this the root of the document or just the root of an iframe?
					if(node === dojo.body()){
						// node is the root of the main document
						break;
					}
					// otherwise, find the iframe this node refers to (can't access it via parentNode,
					// need to do this trick instead). window.frameElement is supported in IE/FF/Webkit
					node=dojo.window.get(node.ownerDocument).frameElement;
				}else{
					// if this node is the root node of a widget, then add widget id to stack,
					// except ignore clicks on disabled widgets (actually focusing a disabled widget still works,
					// to support MenuItem)
					var id = node.getAttribute && node.getAttribute("widgetId"),
						widget = id && dijit.byId(id);
					if(widget && !(by == "mouse" && widget.get("disabled"))){
						newStack.unshift(id);
					}
					node=node.parentNode;
				}
			}
		}catch(e){ /* squelch */ }

		dijit._setStack(newStack, by);
	},

	_onFocusNode: function(/*DomNode*/ node){
		// summary:
		//		Callback when node is focused

		if(!node){
			return;
		}

		if(node.nodeType == 9){
			// Ignore focus events on the document itself.  This is here so that
			// (for example) clicking the up/down arrows of a spinner
			// (which don't get focus) won't cause that widget to blur. (FF issue)
			return;
		}

		dijit._onTouchNode(node);

		if(node == dijit._curFocus){ return; }
		if(dijit._curFocus){
			dijit._prevFocus = dijit._curFocus;
		}
		dijit._curFocus = node;
		dojo.publish("focusNode", [node]);
	},

	_setStack: function(/*String[]*/ newStack, /*String*/ by){
		// summary:
		//		The stack of active widgets has changed.  Send out appropriate events and records new stack.
		// newStack:
		//		array of widget id's, starting from the top (outermost) widget
		// by:
		//		"mouse" if the focus/touch was caused by a mouse down event

		var oldStack = dijit._activeStack;
		dijit._activeStack = newStack;

		// compare old stack to new stack to see how many elements they have in common
		for(var nCommon=0; nCommon<Math.min(oldStack.length, newStack.length); nCommon++){
			if(oldStack[nCommon] != newStack[nCommon]){
				break;
			}
		}

		var widget;
		// for all elements that have gone out of focus, send blur event
		for(var i=oldStack.length-1; i>=nCommon; i--){
			widget = dijit.byId(oldStack[i]);
			if(widget){
				widget._focused = false;
				widget.set("focused", false);
				widget._hasBeenBlurred = true;
				if(widget._onBlur){
					widget._onBlur(by);
				}
				dojo.publish("widgetBlur", [widget, by]);
			}
		}

		// for all element that have come into focus, send focus event
		for(i=nCommon; i<newStack.length; i++){
			widget = dijit.byId(newStack[i]);
			if(widget){
				widget._focused = true;
				widget.set("focused", true);
				if(widget._onFocus){
					widget._onFocus(by);
				}
				dojo.publish("widgetFocus", [widget, by]);
			}
		}
	}
});

// register top window and all the iframes it contains
dojo.addOnLoad(function(){
	var handle = dijit.registerWin(window);
	if(dojo.isIE){
		dojo.addOnWindowUnload(function(){
			dijit.unregisterWin(handle);
			handle = null;
		})
	}
});

}

if(!dojo._hasResource["dojo.AdapterRegistry"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.AdapterRegistry"] = true;
dojo.provide("dojo.AdapterRegistry");


dojo.AdapterRegistry = function(/*Boolean?*/ returnWrappers){
	//	summary:
	//		A registry to make contextual calling/searching easier.
	//	description:
	//		Objects of this class keep list of arrays in the form [name, check,
	//		wrap, directReturn] that are used to determine what the contextual
	//		result of a set of checked arguments is. All check/wrap functions
	//		in this registry should be of the same arity.
	//	example:
	//	|	// create a new registry
	//	|	var reg = new dojo.AdapterRegistry();
	//	|	reg.register("handleString",
	//	|		dojo.isString,
	//	|		function(str){
	//	|			// do something with the string here
	//	|		}
	//	|	);
	//	|	reg.register("handleArr",
	//	|		dojo.isArray,
	//	|		function(arr){
	//	|			// do something with the array here
	//	|		}
	//	|	);
	//	|
	//	|	// now we can pass reg.match() *either* an array or a string and
	//	|	// the value we pass will get handled by the right function
	//	|	reg.match("someValue"); // will call the first function
	//	|	reg.match(["someValue"]); // will call the second

	this.pairs = [];
	this.returnWrappers = returnWrappers || false; // Boolean
};

dojo.extend(dojo.AdapterRegistry, {
	register: function(/*String*/ name, /*Function*/ check, /*Function*/ wrap, /*Boolean?*/ directReturn, /*Boolean?*/ override){
		//	summary:
		//		register a check function to determine if the wrap function or
		//		object gets selected
		//	name:
		//		a way to identify this matcher.
		//	check:
		//		a function that arguments are passed to from the adapter's
		//		match() function.  The check function should return true if the
		//		given arguments are appropriate for the wrap function.
		//	directReturn:
		//		If directReturn is true, the value passed in for wrap will be
		//		returned instead of being called. Alternately, the
		//		AdapterRegistry can be set globally to "return not call" using
		//		the returnWrappers property. Either way, this behavior allows
		//		the registry to act as a "search" function instead of a
		//		function interception library.
		//	override:
		//		If override is given and true, the check function will be given
		//		highest priority. Otherwise, it will be the lowest priority
		//		adapter.
		this.pairs[((override) ? "unshift" : "push")]([name, check, wrap, directReturn]);
	},

	match: function(/* ... */){
		// summary:
		//		Find an adapter for the given arguments. If no suitable adapter
		//		is found, throws an exception. match() accepts any number of
		//		arguments, all of which are passed to all matching functions
		//		from the registered pairs.
		for(var i = 0; i < this.pairs.length; i++){
			var pair = this.pairs[i];
			if(pair[1].apply(this, arguments)){
				if((pair[3])||(this.returnWrappers)){
					return pair[2];
				}else{
					return pair[2].apply(this, arguments);
				}
			}
		}
		throw new Error("No match found");
	},

	unregister: function(name){
		// summary: Remove a named adapter from the registry

		// FIXME: this is kind of a dumb way to handle this. On a large
		// registry this will be slow-ish and we can use the name as a lookup
		// should we choose to trade memory for speed.
		for(var i = 0; i < this.pairs.length; i++){
			var pair = this.pairs[i];
			if(pair[0] == name){
				this.pairs.splice(i, 1);
				return true;
			}
		}
		return false;
	}
});

}

if(!dojo._hasResource["dijit._base.place"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.place"] = true;
dojo.provide("dijit._base.place");




dijit.getViewport = function(){
	// summary:
	//		Returns the dimensions and scroll position of the viewable area of a browser window

	return dojo.window.getBox();
};

/*=====
dijit.__Position = function(){
	// x: Integer
	//		horizontal coordinate in pixels, relative to document body
	// y: Integer
	//		vertical coordinate in pixels, relative to document body

	thix.x = x;
	this.y = y;
}
=====*/


dijit.placeOnScreen = function(
	/* DomNode */			node,
	/* dijit.__Position */	pos,
	/* String[] */			corners,
	/* dijit.__Position? */	padding){
	// summary:
	//		Positions one of the node's corners at specified position
	//		such that node is fully visible in viewport.
	// description:
	//		NOTE: node is assumed to be absolutely or relatively positioned.
	//	pos:
	//		Object like {x: 10, y: 20}
	//	corners:
	//		Array of Strings representing order to try corners in, like ["TR", "BL"].
	//		Possible values are:
	//			* "BL" - bottom left
	//			* "BR" - bottom right
	//			* "TL" - top left
	//			* "TR" - top right
	//	padding:
	//		set padding to put some buffer around the element you want to position.
	// example:
	//		Try to place node's top right corner at (10,20).
	//		If that makes node go (partially) off screen, then try placing
	//		bottom left corner at (10,20).
	//	|	placeOnScreen(node, {x: 10, y: 20}, ["TR", "BL"])

	var choices = dojo.map(corners, function(corner){
		var c = { corner: corner, pos: {x:pos.x,y:pos.y} };
		if(padding){
			c.pos.x += corner.charAt(1) == 'L' ? padding.x : -padding.x;
			c.pos.y += corner.charAt(0) == 'T' ? padding.y : -padding.y;
		}
		return c;
	});

	return dijit._place(node, choices);
}

dijit._place = function(/*DomNode*/ node, choices, layoutNode, /*Object*/ aroundNodeCoords){
	// summary:
	//		Given a list of spots to put node, put it at the first spot where it fits,
	//		of if it doesn't fit anywhere then the place with the least overflow
	// choices: Array
	//		Array of elements like: {corner: 'TL', pos: {x: 10, y: 20} }
	//		Above example says to put the top-left corner of the node at (10,20)
	// layoutNode: Function(node, aroundNodeCorner, nodeCorner, size)
	//		for things like tooltip, they are displayed differently (and have different dimensions)
	//		based on their orientation relative to the parent.   This adjusts the popup based on orientation.
	//		It also passes in the available size for the popup, which is useful for tooltips to
	//		tell them that their width is limited to a certain amount.   layoutNode() may return a value expressing
	//		how much the popup had to be modified to fit into the available space.   This is used to determine
	//		what the best placement is.
	// aroundNodeCoords: Object
	//		Size of aroundNode, ex: {w: 200, h: 50}

	// get {x: 10, y: 10, w: 100, h:100} type obj representing position of
	// viewport over document
	var view = dojo.window.getBox();

	// This won't work if the node is inside a <div style="position: relative">,
	// so reattach it to dojo.doc.body.   (Otherwise, the positioning will be wrong
	// and also it might get cutoff)
	if(!node.parentNode || String(node.parentNode.tagName).toLowerCase() != "body"){
		dojo.body().appendChild(node);
	}

	var best = null;
	dojo.some(choices, function(choice){
		var corner = choice.corner;
		var pos = choice.pos;
		var overflow = 0;

		// calculate amount of space available given specified position of node
		var spaceAvailable = {
			w: corner.charAt(1) == 'L' ? (view.l + view.w) - pos.x : pos.x - view.l,
			h: corner.charAt(1) == 'T' ? (view.t + view.h) - pos.y : pos.y - view.t
		};

		// configure node to be displayed in given position relative to button
		// (need to do this in order to get an accurate size for the node, because
		// a tooltip's size changes based on position, due to triangle)
		if(layoutNode){
			var res = layoutNode(node, choice.aroundCorner, corner, spaceAvailable, aroundNodeCoords);
			overflow = typeof res == "undefined" ? 0 : res;
		}

		// get node's size
		var style = node.style;
		var oldDisplay = style.display;
		var oldVis = style.visibility;
		style.visibility = "hidden";
		style.display = "";
		var mb = dojo.marginBox(node);
		style.display = oldDisplay;
		style.visibility = oldVis;

		// coordinates and size of node with specified corner placed at pos,
		// and clipped by viewport
		var startX = Math.max(view.l, corner.charAt(1) == 'L' ? pos.x : (pos.x - mb.w)),
			startY = Math.max(view.t, corner.charAt(0) == 'T' ? pos.y : (pos.y - mb.h)),
			endX = Math.min(view.l + view.w, corner.charAt(1) == 'L' ? (startX + mb.w) : pos.x),
			endY = Math.min(view.t + view.h, corner.charAt(0) == 'T' ? (startY + mb.h) : pos.y),
			width = endX - startX,
			height = endY - startY;

		overflow += (mb.w - width) + (mb.h - height);

		if(best == null || overflow < best.overflow){
			best = {
				corner: corner,
				aroundCorner: choice.aroundCorner,
				x: startX,
				y: startY,
				w: width,
				h: height,
				overflow: overflow,
				spaceAvailable: spaceAvailable
			};
		}
		
		return !overflow;
	});

	// In case the best position is not the last one we checked, need to call
	// layoutNode() again.
	if(best.overflow && layoutNode){
		layoutNode(node, best.aroundCorner, best.corner, best.spaceAvailable, aroundNodeCoords);
	}

	// And then position the node.   Do this last, after the layoutNode() above
	// has sized the node, due to browser quirks when the viewport is scrolled
	// (specifically that a Tooltip will shrink to fit as though the window was
	// scrolled to the left).
	//
	// In RTL mode, set style.right rather than style.left so in the common case,
	// window resizes move the popup along with the aroundNode.
	var l = dojo._isBodyLtr(),
		s = node.style;
	s.top = best.y + "px";
	s[l ? "left" : "right"] = (l ? best.x : view.w - best.x - best.w) + "px";
	
	return best;
}

dijit.placeOnScreenAroundNode = function(
	/* DomNode */		node,
	/* DomNode */		aroundNode,
	/* Object */		aroundCorners,
	/* Function? */		layoutNode){

	// summary:
	//		Position node adjacent or kitty-corner to aroundNode
	//		such that it's fully visible in viewport.
	//
	// description:
	//		Place node such that corner of node touches a corner of
	//		aroundNode, and that node is fully visible.
	//
	// aroundCorners:
	//		Ordered list of pairs of corners to try matching up.
	//		Each pair of corners is represented as a key/value in the hash,
	//		where the key corresponds to the aroundNode's corner, and
	//		the value corresponds to the node's corner:
	//
	//	|	{ aroundNodeCorner1: nodeCorner1, aroundNodeCorner2: nodeCorner2, ...}
	//
	//		The following strings are used to represent the four corners:
	//			* "BL" - bottom left
	//			* "BR" - bottom right
	//			* "TL" - top left
	//			* "TR" - top right
	//
	// layoutNode: Function(node, aroundNodeCorner, nodeCorner)
	//		For things like tooltip, they are displayed differently (and have different dimensions)
	//		based on their orientation relative to the parent.   This adjusts the popup based on orientation.
	//
	// example:
	//	|	dijit.placeOnScreenAroundNode(node, aroundNode, {'BL':'TL', 'TR':'BR'});
	//		This will try to position node such that node's top-left corner is at the same position
	//		as the bottom left corner of the aroundNode (ie, put node below
	//		aroundNode, with left edges aligned).  If that fails it will try to put
	// 		the bottom-right corner of node where the top right corner of aroundNode is
	//		(ie, put node above aroundNode, with right edges aligned)
	//

	// get coordinates of aroundNode
	aroundNode = dojo.byId(aroundNode);
	var aroundNodePos = dojo.position(aroundNode, true);

	// place the node around the calculated rectangle
	return dijit._placeOnScreenAroundRect(node,
		aroundNodePos.x, aroundNodePos.y, aroundNodePos.w, aroundNodePos.h,	// rectangle
		aroundCorners, layoutNode);
};

/*=====
dijit.__Rectangle = function(){
	// x: Integer
	//		horizontal offset in pixels, relative to document body
	// y: Integer
	//		vertical offset in pixels, relative to document body
	// width: Integer
	//		width in pixels
	// height: Integer
	//		height in pixels

	this.x = x;
	this.y = y;
	this.width = width;
	this.height = height;
}
=====*/


dijit.placeOnScreenAroundRectangle = function(
	/* DomNode */			node,
	/* dijit.__Rectangle */	aroundRect,
	/* Object */			aroundCorners,
	/* Function */			layoutNode){

	// summary:
	//		Like dijit.placeOnScreenAroundNode(), except that the "around"
	//		parameter is an arbitrary rectangle on the screen (x, y, width, height)
	//		instead of a dom node.

	return dijit._placeOnScreenAroundRect(node,
		aroundRect.x, aroundRect.y, aroundRect.width, aroundRect.height,	// rectangle
		aroundCorners, layoutNode);
};

dijit._placeOnScreenAroundRect = function(
	/* DomNode */		node,
	/* Number */		x,
	/* Number */		y,
	/* Number */		width,
	/* Number */		height,
	/* Object */		aroundCorners,
	/* Function */		layoutNode){

	// summary:
	//		Like dijit.placeOnScreenAroundNode(), except it accepts coordinates
	//		of a rectangle to place node adjacent to.

	// TODO: combine with placeOnScreenAroundRectangle()

	// Generate list of possible positions for node
	var choices = [];
	for(var nodeCorner in aroundCorners){
		choices.push( {
			aroundCorner: nodeCorner,
			corner: aroundCorners[nodeCorner],
			pos: {
				x: x + (nodeCorner.charAt(1) == 'L' ? 0 : width),
				y: y + (nodeCorner.charAt(0) == 'T' ? 0 : height)
			}
		});
	}

	return dijit._place(node, choices, layoutNode, {w: width, h: height});
};

dijit.placementRegistry= new dojo.AdapterRegistry();
dijit.placementRegistry.register("node",
	function(n, x){
		return typeof x == "object" &&
			typeof x.offsetWidth != "undefined" && typeof x.offsetHeight != "undefined";
	},
	dijit.placeOnScreenAroundNode);
dijit.placementRegistry.register("rect",
	function(n, x){
		return typeof x == "object" &&
			"x" in x && "y" in x && "width" in x && "height" in x;
	},
	dijit.placeOnScreenAroundRectangle);

dijit.placeOnScreenAroundElement = function(
	/* DomNode */		node,
	/* Object */		aroundElement,
	/* Object */		aroundCorners,
	/* Function */		layoutNode){

	// summary:
	//		Like dijit.placeOnScreenAroundNode(), except it accepts an arbitrary object
	//		for the "around" argument and finds a proper processor to place a node.

	return dijit.placementRegistry.match.apply(dijit.placementRegistry, arguments);
};

dijit.getPopupAroundAlignment = function(/*Array*/ position, /*Boolean*/ leftToRight){
	// summary:
	//		Transforms the passed array of preferred positions into a format suitable for passing as the aroundCorners argument to dijit.placeOnScreenAroundElement.
	//
	// position: String[]
	//		This variable controls the position of the drop down.
	//		It's an array of strings with the following values:
	//
	//			* before: places drop down to the left of the target node/widget, or to the right in
	//			  the case of RTL scripts like Hebrew and Arabic
	//			* after: places drop down to the right of the target node/widget, or to the left in
	//			  the case of RTL scripts like Hebrew and Arabic
	//			* above: drop down goes above target node
	//			* below: drop down goes below target node
	//
	//		The list is positions is tried, in order, until a position is found where the drop down fits
	//		within the viewport.
	//
	// leftToRight: Boolean
	//		Whether the popup will be displaying in leftToRight mode.
	//
	var align = {};
	dojo.forEach(position, function(pos){
		switch(pos){
			case "after":
				align[leftToRight ? "BR" : "BL"] = leftToRight ? "BL" : "BR";
				break;
			case "before":
				align[leftToRight ? "BL" : "BR"] = leftToRight ? "BR" : "BL";
				break;
			case "below-alt":
				leftToRight = !leftToRight;
				// fall through
			case "below":
				// first try to align left borders, next try to align right borders (or reverse for RTL mode)
				align[leftToRight ? "BL" : "BR"] = leftToRight ? "TL" : "TR";
				align[leftToRight ? "BR" : "BL"] = leftToRight ? "TR" : "TL";
				break;
			case "above-alt":
				leftToRight = !leftToRight;
				// fall through
			case "above":
			default:
				// first try to align left borders, next try to align right borders (or reverse for RTL mode)
				align[leftToRight ? "TL" : "TR"] = leftToRight ? "BL" : "BR";
				align[leftToRight ? "TR" : "TL"] = leftToRight ? "BR" : "BL";
				break;
		}
	});
	return align;
};

}

if(!dojo._hasResource["dijit._base.window"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.window"] = true;
dojo.provide("dijit._base.window");



dijit.getDocumentWindow = function(doc){
	return dojo.window.get(doc);
};

}

if(!dojo._hasResource["dijit._base.popup"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.popup"] = true;
dojo.provide("dijit._base.popup");





/*=====
dijit.popup.__OpenArgs = function(){
	// popup: Widget
	//		widget to display
	// parent: Widget
	//		the button etc. that is displaying this popup
	// around: DomNode
	//		DOM node (typically a button); place popup relative to this node.  (Specify this *or* "x" and "y" parameters.)
	// x: Integer
	//		Absolute horizontal position (in pixels) to place node at.  (Specify this *or* "around" parameter.)
	// y: Integer
	//		Absolute vertical position (in pixels) to place node at.  (Specify this *or* "around" parameter.)
	// orient: Object|String
	//		When the around parameter is specified, orient should be an
	//		ordered list of tuples of the form (around-node-corner, popup-node-corner).
	//		dijit.popup.open() tries to position the popup according to each tuple in the list, in order,
	//		until the popup appears fully within the viewport.
	//
	//		The default value is {BL:'TL', TL:'BL'}, which represents a list of two tuples:
	//			1. (BL, TL)
	//			2. (TL, BL)
	//		where BL means "bottom left" and "TL" means "top left".
	//		So by default, it first tries putting the popup below the around node, left-aligning them,
	//		and then tries to put it above the around node, still left-aligning them.   Note that the
	//		default is horizontally reversed when in RTL mode.
	//
	//		When an (x,y) position is specified rather than an around node, orient is either
	//		"R" or "L".  R (for right) means that it tries to put the popup to the right of the mouse,
	//		specifically positioning the popup's top-right corner at the mouse position, and if that doesn't
	//		fit in the viewport, then it tries, in order, the bottom-right corner, the top left corner,
	//		and the top-right corner.
	// onCancel: Function
	//		callback when user has canceled the popup by
	//			1. hitting ESC or
	//			2. by using the popup widget's proprietary cancel mechanism (like a cancel button in a dialog);
	//			   i.e. whenever popupWidget.onCancel() is called, args.onCancel is called
	// onClose: Function
	//		callback whenever this popup is closed
	// onExecute: Function
	//		callback when user "executed" on the popup/sub-popup by selecting a menu choice, etc. (top menu only)
	// padding: dijit.__Position
	//		adding a buffer around the opening position. This is only useful when around is not set.
	this.popup = popup;
	this.parent = parent;
	this.around = around;
	this.x = x;
	this.y = y;
	this.orient = orient;
	this.onCancel = onCancel;
	this.onClose = onClose;
	this.onExecute = onExecute;
	this.padding = padding;
}
=====*/

dijit.popup = {
	// summary:
	//		This singleton is used to show/hide widgets as popups.

	// _stack: dijit._Widget[]
	//		Stack of currently popped up widgets.
	//		(someone opened _stack[0], and then it opened _stack[1], etc.)
	_stack: [],
	
	// _beginZIndex: Number
	//		Z-index of the first popup.   (If first popup opens other
	//		popups they get a higher z-index.)
	_beginZIndex: 100000,

	_idGen: 1,

	_createWrapper: function(/*Widget || DomNode*/ widget){
		// summary:
		//		Initialization for widgets that will be used as popups.
		//		Puts widget inside a wrapper DIV (if not already in one),
		//		and returns pointer to that wrapper DIV.

		var wrapper = widget.declaredClass ? widget._popupWrapper : (widget.parentNode && dojo.hasClass(widget.parentNode, "dijitPopup")),
			node = widget.domNode || widget;

		if(!wrapper){
			// Create wrapper <div> for when this widget [in the future] will be used as a popup.
			// This is done early because of IE bugs where creating/moving DOM nodes causes focus
			// to go wonky, see tests/robot/Toolbar.html to reproduce
			wrapper = dojo.create("div",{
				"class":"dijitPopup",
				style:{ display: "none"},
				role: "presentation"
			}, dojo.body());
			wrapper.appendChild(node);

			var s = node.style;
			s.display = "";
			s.visibility = "";
			s.position = "";
			s.top = "0px";

			if(widget.declaredClass){		// TODO: in 2.0 change signature to always take widget, then remove if()
				widget._popupWrapper = wrapper;
				dojo.connect(widget, "destroy", function(){
					dojo.destroy(wrapper);
					delete widget._popupWrapper;
				});
			}
		}
		
		return wrapper;
	},

	moveOffScreen: function(/*Widget || DomNode*/ widget){
		// summary:
		//		Moves the popup widget off-screen.
		//		Do not use this method to hide popups when not in use, because
		//		that will create an accessibility issue: the offscreen popup is
		//		still in the tabbing order.

		// Create wrapper if not already there
		var wrapper = this._createWrapper(widget);

		dojo.style(wrapper, {
			visibility: "hidden",
			top: "-9999px",		// prevent transient scrollbar causing misalign (#5776), and initial flash in upper left (#10111)
			display: ""
		});
	},

	hide: function(/*dijit._Widget*/ widget){
		// summary:
		//		Hide this popup widget (until it is ready to be shown).
		//		Initialization for widgets that will be used as popups
		//
		// 		Also puts widget inside a wrapper DIV (if not already in one)
		//
		//		If popup widget needs to layout it should
		//		do so when it is made visible, and popup._onShow() is called.

		// Create wrapper if not already there
		var wrapper = this._createWrapper(widget);

		dojo.style(wrapper, "display", "none");
	},
		
	getTopPopup: function(){
		// summary:
		//		Compute the closest ancestor popup that's *not* a child of another popup.
		//		Ex: For a TooltipDialog with a button that spawns a tree of menus, find the popup of the button.
		var stack = this._stack;
		for(var pi=stack.length-1; pi > 0 && stack[pi].parent === stack[pi-1].widget; pi--){
			/* do nothing, just trying to get right value for pi */
		}
		return stack[pi];
	},

	open: function(/*dijit.popup.__OpenArgs*/ args){
		// summary:
		//		Popup the widget at the specified position
		//
		// example:
		//		opening at the mouse position
		//		|		dijit.popup.open({popup: menuWidget, x: evt.pageX, y: evt.pageY});
		//
		// example:
		//		opening the widget as a dropdown
		//		|		dijit.popup.open({parent: this, popup: menuWidget, around: this.domNode, onClose: function(){...}});
		//
		//		Note that whatever widget called dijit.popup.open() should also listen to its own _onBlur callback
		//		(fired from _base/focus.js) to know that focus has moved somewhere else and thus the popup should be closed.

		var stack = this._stack,
			widget = args.popup,
			orient = args.orient || (
				(args.parent ? args.parent.isLeftToRight() : dojo._isBodyLtr()) ?
				{'BL':'TL', 'BR':'TR', 'TL':'BL', 'TR':'BR'} :
				{'BR':'TR', 'BL':'TL', 'TR':'BR', 'TL':'BL'}
			),
			around = args.around,
			id = (args.around && args.around.id) ? (args.around.id+"_dropdown") : ("popup_"+this._idGen++);

		// If we are opening a new popup that isn't a child of a currently opened popup, then
		// close currently opened popup(s).   This should happen automatically when the old popups
		// gets the _onBlur() event, except that the _onBlur() event isn't reliable on IE, see [22198].
		while(stack.length && (!args.parent || !dojo.isDescendant(args.parent.domNode, stack[stack.length-1].widget.domNode))){
			dijit.popup.close(stack[stack.length-1].widget);
		}

		// Get pointer to popup wrapper, and create wrapper if it doesn't exist
		var wrapper = this._createWrapper(widget);


		dojo.attr(wrapper, {
			id: id,
			style: {
				zIndex: this._beginZIndex + stack.length
			},
			"class": "dijitPopup " + (widget.baseClass || widget["class"] || "").split(" ")[0] +"Popup",
			dijitPopupParent: args.parent ? args.parent.id : ""
		});

		if(dojo.isIE || dojo.isMoz){
			if(!widget.bgIframe){
				// setting widget.bgIframe triggers cleanup in _Widget.destroy()
				widget.bgIframe = new dijit.BackgroundIframe(wrapper);
			}
		}

		// position the wrapper node and make it visible
		var best = around ?
			dijit.placeOnScreenAroundElement(wrapper, around, orient, widget.orient ? dojo.hitch(widget, "orient") : null) :
			dijit.placeOnScreen(wrapper, args, orient == 'R' ? ['TR','BR','TL','BL'] : ['TL','BL','TR','BR'], args.padding);

		wrapper.style.display = "";
		wrapper.style.visibility = "visible";
		widget.domNode.style.visibility = "visible";	// counteract effects from _HasDropDown

		var handlers = [];

		// provide default escape and tab key handling
		// (this will work for any widget, not just menu)
		handlers.push(dojo.connect(wrapper, "onkeypress", this, function(evt){
			if(evt.charOrCode == dojo.keys.ESCAPE && args.onCancel){
				dojo.stopEvent(evt);
				args.onCancel();
			}else if(evt.charOrCode === dojo.keys.TAB){
				dojo.stopEvent(evt);
				var topPopup = this.getTopPopup();
				if(topPopup && topPopup.onCancel){
					topPopup.onCancel();
				}
			}
		}));

		// watch for cancel/execute events on the popup and notify the caller
		// (for a menu, "execute" means clicking an item)
		if(widget.onCancel){
			handlers.push(dojo.connect(widget, "onCancel", args.onCancel));
		}

		handlers.push(dojo.connect(widget, widget.onExecute ? "onExecute" : "onChange", this, function(){
			var topPopup = this.getTopPopup();
			if(topPopup && topPopup.onExecute){
				topPopup.onExecute();
			}
		}));

		stack.push({
			widget: widget,
			parent: args.parent,
			onExecute: args.onExecute,
			onCancel: args.onCancel,
 			onClose: args.onClose,
			handlers: handlers
		});

		if(widget.onOpen){
			// TODO: in 2.0 standardize onShow() (used by StackContainer) and onOpen() (used here)
			widget.onOpen(best);
		}

		return best;
	},

	close: function(/*dijit._Widget?*/ popup){
		// summary:
		//		Close specified popup and any popups that it parented.
		//		If no popup is specified, closes all popups.

		var stack = this._stack;

		// Basically work backwards from the top of the stack closing popups
		// until we hit the specified popup, but IIRC there was some issue where closing
		// a popup would cause others to close too.  Thus if we are trying to close B in [A,B,C]
		// closing C might close B indirectly and then the while() condition will run where stack==[A]...
		// so the while condition is constructed defensively.
		while((popup && dojo.some(stack, function(elem){return elem.widget == popup;})) ||
			(!popup && stack.length)){
			var top = stack.pop(),
				widget = top.widget,
				onClose = top.onClose;

			if(widget.onClose){
				// TODO: in 2.0 standardize onHide() (used by StackContainer) and onClose() (used here)
				widget.onClose();
			}
			dojo.forEach(top.handlers, dojo.disconnect);

			// Hide the widget and it's wrapper unless it has already been destroyed in above onClose() etc.
			if(widget && widget.domNode){
				this.hide(widget);
			}
                        
			if(onClose){
				onClose();
			}
		}
	}
};

// TODO: remove dijit._frames, it isn't being used much, since popups never release their
// iframes (see [22236])
dijit._frames = new function(){
	// summary:
	//		cache of iframes

	var queue = [];

	this.pop = function(){
		var iframe;
		if(queue.length){
			iframe = queue.pop();
			iframe.style.display="";
		}else{
			if(dojo.isIE < 9){
				var burl = dojo.config["dojoBlankHtmlUrl"] || (dojo.moduleUrl("dojo", "resources/blank.html")+"") || "javascript:\"\"";
				var html="<iframe src='" + burl + "'"
					+ " style='position: absolute; left: 0px; top: 0px;"
					+ "z-index: -1; filter:Alpha(Opacity=\"0\");'>";
				iframe = dojo.doc.createElement(html);
			}else{
			 	iframe = dojo.create("iframe");
				iframe.src = 'javascript:""';
				iframe.className = "dijitBackgroundIframe";
				dojo.style(iframe, "opacity", 0.1);
			}
			iframe.tabIndex = -1; // Magic to prevent iframe from getting focus on tab keypress - as style didn't work.
			dijit.setWaiRole(iframe,"presentation");
		}
		return iframe;
	};

	this.push = function(iframe){
		iframe.style.display="none";
		queue.push(iframe);
	}
}();


dijit.BackgroundIframe = function(/*DomNode*/ node){
	// summary:
	//		For IE/FF z-index schenanigans. id attribute is required.
	//
	// description:
	//		new dijit.BackgroundIframe(node)
	//			Makes a background iframe as a child of node, that fills
	//			area (and position) of node

	if(!node.id){ throw new Error("no id"); }
	if(dojo.isIE || dojo.isMoz){
		var iframe = (this.iframe = dijit._frames.pop());
		node.appendChild(iframe);
		if(dojo.isIE<7 || dojo.isQuirks){
			this.resize(node);
			this._conn = dojo.connect(node, 'onresize', this, function(){
				this.resize(node);
			});
		}else{
			dojo.style(iframe, {
				width: '100%',
				height: '100%'
			});
		}
	}
};

dojo.extend(dijit.BackgroundIframe, {
	resize: function(node){
		// summary:
		// 		Resize the iframe so it's the same size as node.
		//		Needed on IE6 and IE/quirks because height:100% doesn't work right.
		if(this.iframe){
			dojo.style(this.iframe, {
				width: node.offsetWidth + 'px',
				height: node.offsetHeight + 'px'
			});
		}
	},
	destroy: function(){
		// summary:
		//		destroy the iframe
		if(this._conn){
			dojo.disconnect(this._conn);
			this._conn = null;
		}
		if(this.iframe){
			dijit._frames.push(this.iframe);
			delete this.iframe;
		}
	}
});

}

if(!dojo._hasResource["dijit._base.scroll"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.scroll"] = true;
dojo.provide("dijit._base.scroll");



dijit.scrollIntoView = function(/*DomNode*/ node, /*Object?*/ pos){
	// summary:
	//		Scroll the passed node into view, if it is not already.
	//		Deprecated, use `dojo.window.scrollIntoView` instead.
	
	dojo.window.scrollIntoView(node, pos);
};

}

if(!dojo._hasResource["dojo.uacss"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.uacss"] = true;
dojo.provide("dojo.uacss");


(function(){
	// summary:
	//		Applies pre-set CSS classes to the top-level HTML node, based on:
	// 			- browser (ex: dj_ie)
	//			- browser version (ex: dj_ie6)
	//			- box model (ex: dj_contentBox)
	//			- text direction (ex: dijitRtl)
	//
	//		In addition, browser, browser version, and box model are
	//		combined with an RTL flag when browser text is RTL.  ex: dj_ie-rtl.

	var d = dojo,
		html = d.doc.documentElement,
		ie = d.isIE,
		opera = d.isOpera,
		maj = Math.floor,
		ff = d.isFF,
		boxModel = d.boxModel.replace(/-/,''),

		classes = {
			dj_ie: ie,
			dj_ie6: maj(ie) == 6,
			dj_ie7: maj(ie) == 7,
			dj_ie8: maj(ie) == 8,
			dj_ie9: maj(ie) == 9,
			dj_quirks: d.isQuirks,
			dj_iequirks: ie && d.isQuirks,

			// NOTE: Opera not supported by dijit
			dj_opera: opera,

			dj_khtml: d.isKhtml,

			dj_webkit: d.isWebKit,
			dj_safari: d.isSafari,
			dj_chrome: d.isChrome,

			dj_gecko: d.isMozilla,
			dj_ff3: maj(ff) == 3
		}; // no dojo unsupported browsers

	classes["dj_" + boxModel] = true;

	// apply browser, browser version, and box model class names
	var classStr = "";
	for(var clz in classes){
		if(classes[clz]){
			classStr += clz + " ";
		}
	}
	html.className = d.trim(html.className + " " + classStr);

	// If RTL mode, then add dj_rtl flag plus repeat existing classes with -rtl extension.
	// We can't run the code below until the <body> tag has loaded (so we can check for dir=rtl).
	// Unshift() is to run sniff code before the parser.
	dojo._loaders.unshift(function(){
		if(!dojo._isBodyLtr()){
			var rtlClassStr = "dj_rtl dijitRtl " + classStr.replace(/ /g, "-rtl ")
			html.className = d.trim(html.className + " " + rtlClassStr);
		}
	});
})();

}

if(!dojo._hasResource["dijit._base.sniff"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.sniff"] = true;
dojo.provide("dijit._base.sniff");



// summary:
//		Applies pre-set CSS classes to the top-level HTML node, see
//		`dojo.uacss` for details.
//
//		Simply doing a require on this module will
//		establish this CSS.  Modified version of Morris' CSS hack.

}

if(!dojo._hasResource["dijit._base.typematic"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.typematic"] = true;
dojo.provide("dijit._base.typematic");


dijit.typematic = {
	// summary:
	//		These functions are used to repetitively call a user specified callback
	//		method when a specific key or mouse click over a specific DOM node is
	//		held down for a specific amount of time.
	//		Only 1 such event is allowed to occur on the browser page at 1 time.

	_fireEventAndReload: function(){
		this._timer = null;
		this._callback(++this._count, this._node, this._evt);
		
		// Schedule next event, timer is at most minDelay (default 10ms) to avoid
		// browser overload (particularly avoiding starving DOH robot so it never gets to send a mouseup)
		this._currentTimeout = Math.max(
			this._currentTimeout < 0 ? this._initialDelay :
				(this._subsequentDelay > 1 ? this._subsequentDelay : Math.round(this._currentTimeout * this._subsequentDelay)),
			this._minDelay);
		this._timer = setTimeout(dojo.hitch(this, "_fireEventAndReload"), this._currentTimeout);
	},

	trigger: function(/*Event*/ evt, /*Object*/ _this, /*DOMNode*/ node, /*Function*/ callback, /*Object*/ obj, /*Number*/ subsequentDelay, /*Number*/ initialDelay, /*Number?*/ minDelay){
		// summary:
		//		Start a timed, repeating callback sequence.
		//		If already started, the function call is ignored.
		//		This method is not normally called by the user but can be
		//		when the normal listener code is insufficient.
		// evt:
		//		key or mouse event object to pass to the user callback
		// _this:
		//		pointer to the user's widget space.
		// node:
		//		the DOM node object to pass the the callback function
		// callback:
		//		function to call until the sequence is stopped called with 3 parameters:
		// count:
		//		integer representing number of repeated calls (0..n) with -1 indicating the iteration has stopped
		// node:
		//		the DOM node object passed in
		// evt:
		//		key or mouse event object
		// obj:
		//		user space object used to uniquely identify each typematic sequence
		// subsequentDelay (optional):
		//		if > 1, the number of milliseconds until the 3->n events occur
		//		or else the fractional time multiplier for the next event's delay, default=0.9
		// initialDelay (optional):
		//		the number of milliseconds until the 2nd event occurs, default=500ms
		// minDelay (optional):
		//		the maximum delay in milliseconds for event to fire, default=10ms
		if(obj != this._obj){
			this.stop();
			this._initialDelay = initialDelay || 500;
			this._subsequentDelay = subsequentDelay || 0.90;
			this._minDelay = minDelay || 10;
			this._obj = obj;
			this._evt = evt;
			this._node = node;
			this._currentTimeout = -1;
			this._count = -1;
			this._callback = dojo.hitch(_this, callback);
			this._fireEventAndReload();
			this._evt = dojo.mixin({faux: true}, evt);
		}
	},

	stop: function(){
		// summary:
		//		Stop an ongoing timed, repeating callback sequence.
		if(this._timer){
			clearTimeout(this._timer);
			this._timer = null;
		}
		if(this._obj){
			this._callback(-1, this._node, this._evt);
			this._obj = null;
		}
	},

	addKeyListener: function(/*DOMNode*/ node, /*Object*/ keyObject, /*Object*/ _this, /*Function*/ callback, /*Number*/ subsequentDelay, /*Number*/ initialDelay, /*Number?*/ minDelay){
		// summary:
		//		Start listening for a specific typematic key.
		//		See also the trigger method for other parameters.
		// keyObject:
		//		an object defining the key to listen for:
		// 		charOrCode:
		//			the printable character (string) or keyCode (number) to listen for.
		// 		keyCode:
		//			(deprecated - use charOrCode) the keyCode (number) to listen for (implies charCode = 0).
		// 		charCode:
		//			(deprecated - use charOrCode) the charCode (number) to listen for.
		// 		ctrlKey:
		//			desired ctrl key state to initiate the callback sequence:
		//			- pressed (true)
		//			- released (false)
		//			- either (unspecified)
		// 		altKey:
		//			same as ctrlKey but for the alt key
		// 		shiftKey:
		//			same as ctrlKey but for the shift key
		// returns:
		//		an array of dojo.connect handles
		if(keyObject.keyCode){
			keyObject.charOrCode = keyObject.keyCode;
			dojo.deprecated("keyCode attribute parameter for dijit.typematic.addKeyListener is deprecated. Use charOrCode instead.", "", "2.0");
		}else if(keyObject.charCode){
			keyObject.charOrCode = String.fromCharCode(keyObject.charCode);
			dojo.deprecated("charCode attribute parameter for dijit.typematic.addKeyListener is deprecated. Use charOrCode instead.", "", "2.0");
		}
		return [
			dojo.connect(node, "onkeypress", this, function(evt){
				if(evt.charOrCode == keyObject.charOrCode &&
				(keyObject.ctrlKey === undefined || keyObject.ctrlKey == evt.ctrlKey) &&
				(keyObject.altKey === undefined || keyObject.altKey == evt.altKey) &&
				(keyObject.metaKey === undefined || keyObject.metaKey == (evt.metaKey || false)) && // IE doesn't even set metaKey
				(keyObject.shiftKey === undefined || keyObject.shiftKey == evt.shiftKey)){
					dojo.stopEvent(evt);
					dijit.typematic.trigger(evt, _this, node, callback, keyObject, subsequentDelay, initialDelay, minDelay);
				}else if(dijit.typematic._obj == keyObject){
					dijit.typematic.stop();
				}
			}),
			dojo.connect(node, "onkeyup", this, function(evt){
				if(dijit.typematic._obj == keyObject){
					dijit.typematic.stop();
				}
			})
		];
	},

	addMouseListener: function(/*DOMNode*/ node, /*Object*/ _this, /*Function*/ callback, /*Number*/ subsequentDelay, /*Number*/ initialDelay, /*Number?*/ minDelay){
		// summary:
		//		Start listening for a typematic mouse click.
		//		See the trigger method for other parameters.
		// returns:
		//		an array of dojo.connect handles
		var dc = dojo.connect;
		return [
			dc(node, "mousedown", this, function(evt){
				dojo.stopEvent(evt);
				dijit.typematic.trigger(evt, _this, node, callback, node, subsequentDelay, initialDelay, minDelay);
			}),
			dc(node, "mouseup", this, function(evt){
				dojo.stopEvent(evt);
				dijit.typematic.stop();
			}),
			dc(node, "mouseout", this, function(evt){
				dojo.stopEvent(evt);
				dijit.typematic.stop();
			}),
			dc(node, "mousemove", this, function(evt){
				evt.preventDefault();
			}),
			dc(node, "dblclick", this, function(evt){
				dojo.stopEvent(evt);
				if(dojo.isIE){
					dijit.typematic.trigger(evt, _this, node, callback, node, subsequentDelay, initialDelay, minDelay);
					setTimeout(dojo.hitch(this, dijit.typematic.stop), 50);
				}
			})
		];
	},

	addListener: function(/*Node*/ mouseNode, /*Node*/ keyNode, /*Object*/ keyObject, /*Object*/ _this, /*Function*/ callback, /*Number*/ subsequentDelay, /*Number*/ initialDelay, /*Number?*/ minDelay){
		// summary:
		//		Start listening for a specific typematic key and mouseclick.
		//		This is a thin wrapper to addKeyListener and addMouseListener.
		//		See the addMouseListener and addKeyListener methods for other parameters.
		// mouseNode:
		//		the DOM node object to listen on for mouse events.
		// keyNode:
		//		the DOM node object to listen on for key events.
		// returns:
		//		an array of dojo.connect handles
		return this.addKeyListener(keyNode, keyObject, _this, callback, subsequentDelay, initialDelay, minDelay).concat(
			this.addMouseListener(mouseNode, _this, callback, subsequentDelay, initialDelay, minDelay));
	}
};

}

if(!dojo._hasResource["dijit._base.wai"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.wai"] = true;
dojo.provide("dijit._base.wai");


dijit.wai = {
	onload: function(){
		// summary:
		//		Detects if we are in high-contrast mode or not

		// This must be a named function and not an anonymous
		// function, so that the widget parsing code can make sure it
		// registers its onload function after this function.
		// DO NOT USE "this" within this function.

		// create div for testing if high contrast mode is on or images are turned off
		var div = dojo.create("div",{
			id: "a11yTestNode",
			style:{
				cssText:'border: 1px solid;'
					+ 'border-color:red green;'
					+ 'position: absolute;'
					+ 'height: 5px;'
					+ 'top: -999px;'
					+ 'background-image: url("' + (dojo.config.blankGif || dojo.moduleUrl("dojo", "resources/blank.gif")) + '");'
			}
		}, dojo.body());

		// test it
		var cs = dojo.getComputedStyle(div);
		if(cs){
			var bkImg = cs.backgroundImage;
			var needsA11y = (cs.borderTopColor == cs.borderRightColor) || (bkImg != null && (bkImg == "none" || bkImg == "url(invalid-url:)" ));
			dojo[needsA11y ? "addClass" : "removeClass"](dojo.body(), "dijit_a11y");
			if(dojo.isIE){
				div.outerHTML = "";		// prevent mixed-content warning, see http://support.microsoft.com/kb/925014
			}else{
				dojo.body().removeChild(div);
			}
		}
	}
};

// Test if computer is in high contrast mode.
// Make sure the a11y test runs first, before widgets are instantiated.
if(dojo.isIE || dojo.isMoz){	// NOTE: checking in Safari messes things up
	dojo._loaders.unshift(dijit.wai.onload);
}

dojo.mixin(dijit, {
	hasWaiRole: function(/*Element*/ elem, /*String?*/ role){
		// summary:
		//		Determines if an element has a particular role.
		// returns:
		//		True if elem has the specific role attribute and false if not.
		// 		For backwards compatibility if role parameter not provided,
		// 		returns true if has a role
		var waiRole = this.getWaiRole(elem);
		return role ? (waiRole.indexOf(role) > -1) : (waiRole.length > 0);
	},

	getWaiRole: function(/*Element*/ elem){
		// summary:
		//		Gets the role for an element (which should be a wai role).
		// returns:
		//		The role of elem or an empty string if elem
		//		does not have a role.
		 return dojo.trim((dojo.attr(elem, "role") || "").replace("wairole:",""));
	},

	setWaiRole: function(/*Element*/ elem, /*String*/ role){
		// summary:
		//		Sets the role on an element.
		// description:
		//		Replace existing role attribute with new role.

			dojo.attr(elem, "role", role);
	},

	removeWaiRole: function(/*Element*/ elem, /*String*/ role){
		// summary:
		//		Removes the specified role from an element.
		// 		Removes role attribute if no specific role provided (for backwards compat.)

		var roleValue = dojo.attr(elem, "role");
		if(!roleValue){ return; }
		if(role){
			var t = dojo.trim((" " + roleValue + " ").replace(" " + role + " ", " "));
			dojo.attr(elem, "role", t);
		}else{
			elem.removeAttribute("role");
		}
	},

	hasWaiState: function(/*Element*/ elem, /*String*/ state){
		// summary:
		//		Determines if an element has a given state.
		// description:
		//		Checks for an attribute called "aria-"+state.
		// returns:
		//		true if elem has a value for the given state and
		//		false if it does not.

		return elem.hasAttribute ? elem.hasAttribute("aria-"+state) : !!elem.getAttribute("aria-"+state);
	},

	getWaiState: function(/*Element*/ elem, /*String*/ state){
		// summary:
		//		Gets the value of a state on an element.
		// description:
		//		Checks for an attribute called "aria-"+state.
		// returns:
		//		The value of the requested state on elem
		//		or an empty string if elem has no value for state.

		return elem.getAttribute("aria-"+state) || "";
	},

	setWaiState: function(/*Element*/ elem, /*String*/ state, /*String*/ value){
		// summary:
		//		Sets a state on an element.
		// description:
		//		Sets an attribute called "aria-"+state.

		elem.setAttribute("aria-"+state, value);
	},

	removeWaiState: function(/*Element*/ elem, /*String*/ state){
		// summary:
		//		Removes a state from an element.
		// description:
		//		Sets an attribute called "aria-"+state.

		elem.removeAttribute("aria-"+state);
	}
});

}

if(!dojo._hasResource["dijit._base"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base"] = true;
dojo.provide("dijit._base");












}

if(!dojo._hasResource["dojo.date.stamp"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.date.stamp"] = true;
dojo.provide("dojo.date.stamp");

dojo.getObject("date.stamp", true, dojo);

// Methods to convert dates to or from a wire (string) format using well-known conventions

dojo.date.stamp.fromISOString = function(/*String*/formattedString, /*Number?*/defaultTime){
	//	summary:
	//		Returns a Date object given a string formatted according to a subset of the ISO-8601 standard.
	//
	//	description:
	//		Accepts a string formatted according to a profile of ISO8601 as defined by
	//		[RFC3339](http://www.ietf.org/rfc/rfc3339.txt), except that partial input is allowed.
	//		Can also process dates as specified [by the W3C](http://www.w3.org/TR/NOTE-datetime)
	//		The following combinations are valid:
	//
	//			* dates only
	//			|	* yyyy
	//			|	* yyyy-MM
	//			|	* yyyy-MM-dd
	// 			* times only, with an optional time zone appended
	//			|	* THH:mm
	//			|	* THH:mm:ss
	//			|	* THH:mm:ss.SSS
	// 			* and "datetimes" which could be any combination of the above
	//
	//		timezones may be specified as Z (for UTC) or +/- followed by a time expression HH:mm
	//		Assumes the local time zone if not specified.  Does not validate.  Improperly formatted
	//		input may return null.  Arguments which are out of bounds will be handled
	// 		by the Date constructor (e.g. January 32nd typically gets resolved to February 1st)
	//		Only years between 100 and 9999 are supported.
	//
  	//	formattedString:
	//		A string such as 2005-06-30T08:05:00-07:00 or 2005-06-30 or T08:05:00
	//
	//	defaultTime:
	//		Used for defaults for fields omitted in the formattedString.
	//		Uses 1970-01-01T00:00:00.0Z by default.

	if(!dojo.date.stamp._isoRegExp){
		dojo.date.stamp._isoRegExp =
//TODO: could be more restrictive and check for 00-59, etc.
			/^(?:(\d{4})(?:-(\d{2})(?:-(\d{2}))?)?)?(?:T(\d{2}):(\d{2})(?::(\d{2})(.\d+)?)?((?:[+-](\d{2}):(\d{2}))|Z)?)?$/;
	}

	var match = dojo.date.stamp._isoRegExp.exec(formattedString),
		result = null;

	if(match){
		match.shift();
		if(match[1]){match[1]--;} // Javascript Date months are 0-based
		if(match[6]){match[6] *= 1000;} // Javascript Date expects fractional seconds as milliseconds

		if(defaultTime){
			// mix in defaultTime.  Relatively expensive, so use || operators for the fast path of defaultTime === 0
			defaultTime = new Date(defaultTime);
			dojo.forEach(dojo.map(["FullYear", "Month", "Date", "Hours", "Minutes", "Seconds", "Milliseconds"], function(prop){
				return defaultTime["get" + prop]();
			}), function(value, index){
				match[index] = match[index] || value;
			});
		}
		result = new Date(match[0]||1970, match[1]||0, match[2]||1, match[3]||0, match[4]||0, match[5]||0, match[6]||0); //TODO: UTC defaults
		if(match[0] < 100){
			result.setFullYear(match[0] || 1970);
		}

		var offset = 0,
			zoneSign = match[7] && match[7].charAt(0);
		if(zoneSign != 'Z'){
			offset = ((match[8] || 0) * 60) + (Number(match[9]) || 0);
			if(zoneSign != '-'){ offset *= -1; }
		}
		if(zoneSign){
			offset -= result.getTimezoneOffset();
		}
		if(offset){
			result.setTime(result.getTime() + offset * 60000);
		}
	}

	return result; // Date or null
};

/*=====
	dojo.date.stamp.__Options = function(){
		//	selector: String
		//		"date" or "time" for partial formatting of the Date object.
		//		Both date and time will be formatted by default.
		//	zulu: Boolean
		//		if true, UTC/GMT is used for a timezone
		//	milliseconds: Boolean
		//		if true, output milliseconds
		this.selector = selector;
		this.zulu = zulu;
		this.milliseconds = milliseconds;
	}
=====*/

dojo.date.stamp.toISOString = function(/*Date*/dateObject, /*dojo.date.stamp.__Options?*/options){
	//	summary:
	//		Format a Date object as a string according a subset of the ISO-8601 standard
	//
	//	description:
	//		When options.selector is omitted, output follows [RFC3339](http://www.ietf.org/rfc/rfc3339.txt)
	//		The local time zone is included as an offset from GMT, except when selector=='time' (time without a date)
	//		Does not check bounds.  Only years between 100 and 9999 are supported.
	//
	//	dateObject:
	//		A Date object

	var _ = function(n){ return (n < 10) ? "0" + n : n; };
	options = options || {};
	var formattedDate = [],
		getter = options.zulu ? "getUTC" : "get",
		date = "";
	if(options.selector != "time"){
		var year = dateObject[getter+"FullYear"]();
		date = ["0000".substr((year+"").length)+year, _(dateObject[getter+"Month"]()+1), _(dateObject[getter+"Date"]())].join('-');
	}
	formattedDate.push(date);
	if(options.selector != "date"){
		var time = [_(dateObject[getter+"Hours"]()), _(dateObject[getter+"Minutes"]()), _(dateObject[getter+"Seconds"]())].join(':');
		var millis = dateObject[getter+"Milliseconds"]();
		if(options.milliseconds){
			time += "."+ (millis < 100 ? "0" : "") + _(millis);
		}
		if(options.zulu){
			time += "Z";
		}else if(options.selector != "time"){
			var timezoneOffset = dateObject.getTimezoneOffset();
			var absOffset = Math.abs(timezoneOffset);
			time += (timezoneOffset > 0 ? "-" : "+") +
				_(Math.floor(absOffset/60)) + ":" + _(absOffset%60);
		}
		formattedDate.push(time);
	}
	return formattedDate.join('T'); // String
};

}

if(!dojo._hasResource["dojo.parser"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.parser"] = true;
dojo.provide("dojo.parser");



new Date("X"); // workaround for #11279, new Date("") == NaN

dojo.parser = new function(){
	// summary:
	//		The Dom/Widget parsing package

	var d = dojo;

	function val2type(/*Object*/ value){
		// summary:
		//		Returns name of type of given value.

		if(d.isString(value)){ return "string"; }
		if(typeof value == "number"){ return "number"; }
		if(typeof value == "boolean"){ return "boolean"; }
		if(d.isFunction(value)){ return "function"; }
		if(d.isArray(value)){ return "array"; } // typeof [] == "object"
		if(value instanceof Date) { return "date"; } // assume timestamp
		if(value instanceof d._Url){ return "url"; }
		return "object";
	}

	function str2obj(/*String*/ value, /*String*/ type){
		// summary:
		//		Convert given string value to given type
		switch(type){
			case "string":
				return value;
			case "number":
				return value.length ? Number(value) : NaN;
			case "boolean":
				// for checked/disabled value might be "" or "checked".	 interpret as true.
				return typeof value == "boolean" ? value : !(value.toLowerCase()=="false");
			case "function":
				if(d.isFunction(value)){
					// IE gives us a function, even when we say something like onClick="foo"
					// (in which case it gives us an invalid function "function(){ foo }").
					//	Therefore, convert to string
					value=value.toString();
					value=d.trim(value.substring(value.indexOf('{')+1, value.length-1));
				}
				try{
					if(value === "" || value.search(/[^\w\.]+/i) != -1){
						// The user has specified some text for a function like "return x+5"
						return new Function(value);
					}else{
						// The user has specified the name of a function like "myOnClick"
						// or a single word function "return"
						return d.getObject(value, false) || new Function(value);
					}
				}catch(e){ return new Function(); }
			case "array":
				return value ? value.split(/\s*,\s*/) : [];
			case "date":
				switch(value){
					case "": return new Date("");	// the NaN of dates
					case "now": return new Date();	// current date
					default: return d.date.stamp.fromISOString(value);
				}
			case "url":
				return d.baseUrl + value;
			default:
				return d.fromJson(value);
		}
	}

	var dummyClass = {}, instanceClasses = {
		// map from fully qualified name (like "dijit.Button") to structure like
		// { cls: dijit.Button, params: {label: "string", disabled: "boolean"} }
	};

	// Widgets like BorderContainer add properties to _Widget via dojo.extend().
	// If BorderContainer is loaded after _Widget's parameter list has been cached,
	// we need to refresh that parameter list (for _Widget and all widgets that extend _Widget).
	// TODO: remove this in 2.0, when we stop caching parameters.
	d.connect(d, "extend", function(){
		instanceClasses = {};
	});

	function getProtoInfo(cls, params){
		// cls: A prototype
		//		The prototype of the class to check props on
		// params: Object
		//		The parameters object to mix found parameters onto.
		for(var name in cls){
			if(name.charAt(0)=="_"){ continue; }	// skip internal properties
			if(name in dummyClass){ continue; }		// skip "constructor" and "toString"
			params[name] = val2type(cls[name]);
		}
		return params;
	}

	function getClassInfo(/*String*/ className, /*Boolean*/ skipParamsLookup){
		// summary:
		//		Maps a widget name string like "dijit.form.Button" to the widget constructor itself,
		//		and a list of that widget's parameters and their types
		// className:
		//		fully qualified name (like "dijit.form.Button")
		// returns:
		//		structure like
		//			{
		//				cls: dijit.Button,
		//				params: { label: "string", disabled: "boolean"}
		//			}

		var c = instanceClasses[className];
		if(!c){
			// get pointer to widget class
			var cls = d.getObject(className), params = null;
			if(!cls){ return null; }		// class not defined [yet]
			if(!skipParamsLookup){ // from fastpath, we don't need to lookup the attrs on the proto because they are explicit
				params = getProtoInfo(cls.prototype, {})
			}
			c = { cls: cls, params: params };
			
		}else if(!skipParamsLookup && !c.params){
			// if we're calling getClassInfo and have a cls proto, but no params info, scan that cls for params now
			// and update the pointer in instanceClasses[className]. This happens when a widget appears in another
			// widget's template which still uses dojoType, but an instance of the widget appears prior with a data-dojo-type,
			// skipping this lookup the first time.
			c.params = getProtoInfo(c.cls.prototype, {});
		}
		
		return c;
	}

	this._functionFromScript = function(script, attrData){
		// summary:
		//		Convert a <script type="dojo/method" args="a, b, c"> ... </script>
		//		into a function
		// script: DOMNode
		//		The <script> DOMNode
		// attrData: String
		//		For HTML5 compliance, searches for attrData + "args" (typically
		//		"data-dojo-args") instead of "args"
		var preamble = "";
		var suffix = "";
		var argsStr = (script.getAttribute(attrData + "args") || script.getAttribute("args"));
		if(argsStr){
			d.forEach(argsStr.split(/\s*,\s*/), function(part, idx){
				preamble += "var "+part+" = arguments["+idx+"]; ";
			});
		}
		var withStr = script.getAttribute("with");
		if(withStr && withStr.length){
			d.forEach(withStr.split(/\s*,\s*/), function(part){
				preamble += "with("+part+"){";
				suffix += "}";
			});
		}
		return new Function(preamble+script.innerHTML+suffix);
	};

	this.instantiate = function(/* Array */nodes, /* Object? */mixin, /* Object? */args){
		// summary:
		//		Takes array of nodes, and turns them into class instances and
		//		potentially calls a startup method to allow them to connect with
		//		any children.
		// nodes: Array
		//		Array of nodes or objects like
		//	|		{
		//	|			type: "dijit.form.Button",
		//	|			node: DOMNode,
		//	|			scripts: [ ... ],	// array of <script type="dojo/..."> children of node
		//	|			inherited: { ... }	// settings inherited from ancestors like dir, theme, etc.
		//	|		}
		// mixin: Object?
		//		An object that will be mixed in with each node in the array.
		//		Values in the mixin will override values in the node, if they
		//		exist.
		// args: Object?
		//		An object used to hold kwArgs for instantiation.
		//		See parse.args argument for details.

		var thelist = [],
		mixin = mixin||{};
		args = args||{};

		// TODO: for 2.0 default to data-dojo- regardless of scopeName (or maybe scopeName won't exist in 2.0)
		var attrName = (args.scope || d._scopeName) + "Type",	// typically "dojoType"
			attrData = "data-" + (args.scope || d._scopeName) + "-";	// typically "data-dojo-"

		d.forEach(nodes, function(obj){
			if(!obj){ return; }

			// Get pointers to DOMNode, dojoType string, and clsInfo (metadata about the dojoType), etc.
			var node, type, clsInfo, clazz, scripts, fastpath;
			if(obj.node){
				// new format of nodes[] array, object w/lots of properties pre-computed for me
				node = obj.node;
				type = obj.type;
				fastpath = obj.fastpath;
				clsInfo = obj.clsInfo || (type && getClassInfo(type, fastpath));
				clazz = clsInfo && clsInfo.cls;
				scripts = obj.scripts;
			}else{
				// old (backwards compatible) format of nodes[] array, simple array of DOMNodes. no fastpath/data-dojo-type support here.
				node = obj;
				type = attrName in mixin ? mixin[attrName] : node.getAttribute(attrName);
				clsInfo = type && getClassInfo(type);
				clazz = clsInfo && clsInfo.cls;
				scripts = (clazz && (clazz._noScript || clazz.prototype._noScript) ? [] :
							d.query("> script[type^='dojo/']", node));
			}
			if(!clsInfo){
				throw new Error("Could not load class '" + type);
			}

			// Setup hash to hold parameter settings for this widget.	Start with the parameter
			// settings inherited from ancestors ("dir" and "lang").
			// Inherited setting may later be overridden by explicit settings on node itself.
			var params = {};
				
			if(args.defaults){
				// settings for the document itself (or whatever subtree is being parsed)
				d._mixin(params, args.defaults);
			}
			if(obj.inherited){
				// settings from dir=rtl or lang=... on a node above this node
				d._mixin(params, obj.inherited);
			}
			
			// mix things found in data-dojo-props into the params
			if(fastpath){
				var extra = node.getAttribute(attrData + "props");
				if(extra && extra.length){
					try{
						extra = d.fromJson.call(args.propsThis, "{" + extra + "}");
						d._mixin(params, extra);
					}catch(e){
						// give the user a pointer to their invalid parameters. FIXME: can we kill this in production?
						throw new Error(e.toString() + " in data-dojo-props='" + extra + "'");
					}
				}

				// For the benefit of _Templated, check if node has data-dojo-attach-point/data-dojo-attach-event
				// and mix those in as though they were parameters
				var attachPoint = node.getAttribute(attrData + "attach-point");
				if(attachPoint){
					params.dojoAttachPoint = attachPoint;
				}
				var attachEvent = node.getAttribute(attrData + "attach-event");
				if(attachEvent){
					params.dojoAttachEvent = attachEvent;
				}
				dojo.mixin(params, mixin);
			}else{
				// FIXME: we need something like "deprecateOnce()" to throw dojo.deprecation for something.
				// remove this logic in 2.0
				// read parameters (ie, attributes) specified on DOMNode

				var attributes = node.attributes;

				// clsInfo.params lists expected params like {"checked": "boolean", "n": "number"}
				for(var name in clsInfo.params){
					var item = name in mixin ? { value:mixin[name], specified:true } : attributes.getNamedItem(name);
					if(!item || (!item.specified && (!dojo.isIE || name.toLowerCase()!="value"))){ continue; }
					var value = item.value;
					// Deal with IE quirks for 'class' and 'style'
					switch(name){
					case "class":
						value = "className" in mixin ? mixin.className : node.className;
						break;
					case "style":
						value = "style" in mixin ? mixin.style : (node.style && node.style.cssText); // FIXME: Opera?
					}
					var _type = clsInfo.params[name];
					if(typeof value == "string"){
						params[name] = str2obj(value, _type);
					}else{
						params[name] = value;
					}
				}
			}

			// Process <script type="dojo/*"> script tags
			// <script type="dojo/method" event="foo"> tags are added to params, and passed to
			// the widget on instantiation.
			// <script type="dojo/method"> tags (with no event) are executed after instantiation
			// <script type="dojo/connect" event="foo"> tags are dojo.connected after instantiation
			// note: dojo/* script tags cannot exist in self closing widgets, like <input />
			var connects = [],	// functions to connect after instantiation
				calls = [];		// functions to call after instantiation

			d.forEach(scripts, function(script){
				node.removeChild(script);
				// FIXME: drop event="" support in 2.0. use data-dojo-event="" instead
				var event = (script.getAttribute(attrData + "event") || script.getAttribute("event")),
					type = script.getAttribute("type"),
					nf = d.parser._functionFromScript(script, attrData);
				if(event){
					if(type == "dojo/connect"){
						connects.push({event: event, func: nf});
					}else{
						params[event] = nf;
					}
				}else{
					calls.push(nf);
				}
			});

			var markupFactory = clazz.markupFactory || clazz.prototype && clazz.prototype.markupFactory;
			// create the instance
			var instance = markupFactory ? markupFactory(params, node, clazz) : new clazz(params, node);
			thelist.push(instance);

			// map it to the JS namespace if that makes sense
			// FIXME: in 2.0, drop jsId support. use data-dojo-id instead
			var jsname = (node.getAttribute(attrData + "id") || node.getAttribute("jsId"));
			if(jsname){
				d.setObject(jsname, instance);
			}

			// process connections and startup functions
			d.forEach(connects, function(connect){
				d.connect(instance, connect.event, null, connect.func);
			});
			d.forEach(calls, function(func){
				func.call(instance);
			});
		});

		// Call startup on each top level instance if it makes sense (as for
		// widgets).  Parent widgets will recursively call startup on their
		// (non-top level) children
		if(!mixin._started){
			// TODO: for 2.0, when old instantiate() API is desupported, store parent-child
			// relationships in the nodes[] array so that no getParent() call is needed.
			// Note that will  require a parse() call from ContentPane setting a param that the
			// ContentPane is the parent widget (so that the parse doesn't call startup() on the
			// ContentPane's children)
			d.forEach(thelist, function(instance){
				if( !args.noStart && instance  &&
					dojo.isFunction(instance.startup) &&
					!instance._started &&
					(!instance.getParent || !instance.getParent())
				){
					instance.startup();
				}
			});
		}
		return thelist;
	};

	this.parse = function(rootNode, args){
		// summary:
		//		Scan the DOM for class instances, and instantiate them.
		//
		// description:
		//		Search specified node (or root node) recursively for class instances,
		//		and instantiate them. Searches for either data-dojo-type="Class" or
		//		dojoType="Class" where "Class" is a a fully qualified class name,
		//		like `dijit.form.Button`
		//
		//		Using `data-dojo-type`:
		//		Attributes using can be mixed into the parameters used to instantitate the
		//		Class by using a `data-dojo-props` attribute on the node being converted.
		//		`data-dojo-props` should be a string attribute to be converted from JSON.
		//
		//		Using `dojoType`:
		//		Attributes are read from the original domNode and converted to appropriate
		//		types by looking up the Class prototype values. This is the default behavior
		//		from Dojo 1.0 to Dojo 1.5. `dojoType` support is deprecated, and will
		//		go away in Dojo 2.0.
		//
		// rootNode: DomNode?
		//		A default starting root node from which to start the parsing. Can be
		//		omitted, defaulting to the entire document. If omitted, the `args`
		//		object can be passed in this place. If the `args` object has a
		//		`rootNode` member, that is used.
		//
		// args: Object
		//		a kwArgs object passed along to instantiate()
		//
		//			* noStart: Boolean?
		//				when set will prevent the parser from calling .startup()
		//				when locating the nodes.
		//			* rootNode: DomNode?
		//				identical to the function's `rootNode` argument, though
		//				allowed to be passed in via this `args object.
		//			* template: Boolean
		//				If true, ignores ContentPane's stopParser flag and parses contents inside of
		//				a ContentPane inside of a template.   This allows dojoAttachPoint on widgets/nodes
		//				nested inside the ContentPane to work.
		//			* inherited: Object
		//				Hash possibly containing dir and lang settings to be applied to
		//				parsed widgets, unless there's another setting on a sub-node that overrides
		//			* scope: String
		//				Root for attribute names to search for.   If scopeName is dojo,
		//				will search for data-dojo-type (or dojoType).   For backwards compatibility
		//				reasons defaults to dojo._scopeName (which is "dojo" except when
		//				multi-version support is used, when it will be something like dojo16, dojo20, etc.)
		//			* propsThis: Object
		//				If specified, "this" referenced from data-dojo-props will refer to propsThis.
		//				Intended for use from the widgets-in-template feature of `dijit._Templated`
		//
		// example:
		//		Parse all widgets on a page:
		//	|		dojo.parser.parse();
		//
		// example:
		//		Parse all classes within the node with id="foo"
		//	|		dojo.parser.parse(dojo.byId('foo'));
		//
		// example:
		//		Parse all classes in a page, but do not call .startup() on any
		//		child
		//	|		dojo.parser.parse({ noStart: true })
		//
		// example:
		//		Parse all classes in a node, but do not call .startup()
		//	|		dojo.parser.parse(someNode, { noStart:true });
		//	|		// or
		//	|		dojo.parser.parse({ noStart:true, rootNode: someNode });

		// determine the root node based on the passed arguments.
		var root;
		if(!args && rootNode && rootNode.rootNode){
			args = rootNode;
			root = args.rootNode;
		}else{
			root = rootNode;
		}
		root = root ? dojo.byId(root) : dojo.body();
		args = args || {};

		var attrName = (args.scope || d._scopeName) + "Type",		// typically "dojoType"
			attrData = "data-" + (args.scope || d._scopeName) + "-";	// typically "data-dojo-"

		function scan(parent, list){
			// summary:
			//		Parent is an Object representing a DOMNode, with or without a dojoType specified.
			//		Scan parent's children looking for nodes with dojoType specified, storing in list[].
			//		If parent has a dojoType, also collects <script type=dojo/*> children and stores in parent.scripts[].
			// parent: Object
			//		Object representing the parent node, like
			//	|	{
			//	|		node: DomNode,			// scan children of this node
			//	|		inherited: {dir: "rtl"},	// dir/lang setting inherited from above node
			//	|
			//	|		// attributes only set if node has dojoType specified
			//	|		scripts: [],			// empty array, put <script type=dojo/*> in here
			//	|		clsInfo: { cls: dijit.form.Button, ...}
			//	|	}
			// list: DomNode[]
			//		Output array of objects (same format as parent) representing nodes to be turned into widgets

			// Effective dir and lang settings on parent node, either set directly or inherited from grandparent
			var inherited = dojo.clone(parent.inherited);
			dojo.forEach(["dir", "lang"], function(name){
				// TODO: what if this is a widget and dir/lang are declared in data-dojo-props?
				var val = parent.node.getAttribute(name);
				if(val){
					inherited[name] = val;
				}
			});

			// if parent is a widget, then search for <script type=dojo/*> tags and put them in scripts[].
			var scripts = parent.clsInfo && !parent.clsInfo.cls.prototype._noScript ? parent.scripts : null;

			// unless parent is a widget with the stopParser flag set, continue search for dojoType, recursively
			var recurse = (!parent.clsInfo || !parent.clsInfo.cls.prototype.stopParser) || (args && args.template);

			// scan parent's children looking for dojoType and <script type=dojo/*>
			for(var child = parent.node.firstChild; child; child = child.nextSibling){
				if(child.nodeType == 1){
					// FIXME: desupport dojoType in 2.0. use data-dojo-type instead
					var type, html5 = recurse && child.getAttribute(attrData + "type");
					if(html5){
						type = html5;
					}else{
						// fallback to backward compatible mode, using dojoType. remove in 2.0
						type = recurse && child.getAttribute(attrName);
					}
					
					var fastpath = html5 == type;

					if(type){
						// if dojoType/data-dojo-type specified, add to output array of nodes to instantiate
						var params = {
							"type": type,
							fastpath: fastpath,
							clsInfo: getClassInfo(type, fastpath), // note: won't find classes declared via dojo.Declaration
							node: child,
							scripts: [], // <script> nodes that are parent's children
							inherited: inherited // dir & lang attributes inherited from parent
						};
						list.push(params);

						// Recurse, collecting <script type="dojo/..."> children, and also looking for
						// descendant nodes with dojoType specified (unless the widget has the stopParser flag),
						scan(params, list);
					}else if(scripts && child.nodeName.toLowerCase() == "script"){
						// if <script type="dojo/...">, save in scripts[]
						type = child.getAttribute("type");
						if (type && /^dojo\/\w/i.test(type)) {
							scripts.push(child);
						}
					}else if(recurse){
						// Recurse, looking for grandchild nodes with dojoType specified
						scan({
							node: child,
							inherited: inherited
						}, list);
					}
				}
			}
		}

		// Ignore bogus entries in inherited hash like {dir: ""}
		var inherited = {};
		if(args && args.inherited){
			for(var key in args.inherited){
				if(args.inherited[key]){ inherited[key] = args.inherited[key]; }
			}
		}

		// Make list of all nodes on page w/dojoType specified
		var list = [];
		scan({
			node: root,
			inherited: inherited
		}, list);

		// go build the object instances
		var mixin = args && args.template ? {template: true} : null;
		return this.instantiate(list, mixin, args); // Array
	};
}();

//Register the parser callback. It should be the first callback
//after the a11y test.

(function(){
	var parseRunner = function(){
		if(dojo.config.parseOnLoad){
			dojo.parser.parse();
		}
	};

	// FIXME: need to clobber cross-dependency!!
	if(dojo.getObject("dijit.wai.onload") === dojo._loaders[0]){
		dojo._loaders.splice(1, 0, parseRunner);
	}else{
		dojo._loaders.unshift(parseRunner);
	}
})();

}

if(!dojo._hasResource["dojo.string"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.string"] = true;
dojo.provide("dojo.string");

dojo.getObject("string", true, dojo);

/*=====
dojo.string = {
	// summary: String utilities for Dojo
};
=====*/

dojo.string.rep = function(/*String*/str, /*Integer*/num){
	//	summary:
	//		Efficiently replicate a string `n` times.
	//	str:
	//		the string to replicate
	//	num:
	//		number of times to replicate the string
	
	if(num <= 0 || !str){ return ""; }
	
	var buf = [];
	for(;;){
		if(num & 1){
			buf.push(str);
		}
		if(!(num >>= 1)){ break; }
		str += str;
	}
	return buf.join("");	// String
};

dojo.string.pad = function(/*String*/text, /*Integer*/size, /*String?*/ch, /*Boolean?*/end){
	//	summary:
	//		Pad a string to guarantee that it is at least `size` length by
	//		filling with the character `ch` at either the start or end of the
	//		string. Pads at the start, by default.
	//	text:
	//		the string to pad
	//	size:
	//		length to provide padding
	//	ch:
	//		character to pad, defaults to '0'
	//	end:
	//		adds padding at the end if true, otherwise pads at start
	//	example:
	//	|	// Fill the string to length 10 with "+" characters on the right.  Yields "Dojo++++++".
	//	|	dojo.string.pad("Dojo", 10, "+", true);

	if(!ch){
		ch = '0';
	}
	var out = String(text),
		pad = dojo.string.rep(ch, Math.ceil((size - out.length) / ch.length));
	return end ? out + pad : pad + out;	// String
};

dojo.string.substitute = function(	/*String*/		template,
									/*Object|Array*/map,
									/*Function?*/	transform,
									/*Object?*/		thisObject){
	//	summary:
	//		Performs parameterized substitutions on a string. Throws an
	//		exception if any parameter is unmatched.
	//	template:
	//		a string with expressions in the form `${key}` to be replaced or
	//		`${key:format}` which specifies a format function. keys are case-sensitive.
	//	map:
	//		hash to search for substitutions
	//	transform:
	//		a function to process all parameters before substitution takes
	//		place, e.g. mylib.encodeXML
	//	thisObject:
	//		where to look for optional format function; default to the global
	//		namespace
	//	example:
	//		Substitutes two expressions in a string from an Array or Object
	//	|	// returns "File 'foo.html' is not found in directory '/temp'."
	//	|	// by providing substitution data in an Array
	//	|	dojo.string.substitute(
	//	|		"File '${0}' is not found in directory '${1}'.",
	//	|		["foo.html","/temp"]
	//	|	);
	//	|
	//	|	// also returns "File 'foo.html' is not found in directory '/temp'."
	//	|	// but provides substitution data in an Object structure.  Dotted
	//	|	// notation may be used to traverse the structure.
	//	|	dojo.string.substitute(
	//	|		"File '${name}' is not found in directory '${info.dir}'.",
	//	|		{ name: "foo.html", info: { dir: "/temp" } }
	//	|	);
	//	example:
	//		Use a transform function to modify the values:
	//	|	// returns "file 'foo.html' is not found in directory '/temp'."
	//	|	dojo.string.substitute(
	//	|		"${0} is not found in ${1}.",
	//	|		["foo.html","/temp"],
	//	|		function(str){
	//	|			// try to figure out the type
	//	|			var prefix = (str.charAt(0) == "/") ? "directory": "file";
	//	|			return prefix + " '" + str + "'";
	//	|		}
	//	|	);
	//	example:
	//		Use a formatter
	//	|	// returns "thinger -- howdy"
	//	|	dojo.string.substitute(
	//	|		"${0:postfix}", ["thinger"], null, {
	//	|			postfix: function(value, key){
	//	|				return value + " -- howdy";
	//	|			}
	//	|		}
	//	|	);

	thisObject = thisObject || dojo.global;
	transform = transform ?
		dojo.hitch(thisObject, transform) : function(v){ return v; };

	return template.replace(/\$\{([^\s\:\}]+)(?:\:([^\s\:\}]+))?\}/g,
		function(match, key, format){
			var value = dojo.getObject(key, false, map);
			if(format){
				value = dojo.getObject(format, false, thisObject).call(thisObject, value, key);
			}
			return transform(value, key).toString();
		}); // String
};

/*=====
dojo.string.trim = function(str){
	//	summary:
	//		Trims whitespace from both sides of the string
	//	str: String
	//		String to be trimmed
	//	returns: String
	//		Returns the trimmed string
	//	description:
	//		This version of trim() was taken from [Steven Levithan's blog](http://blog.stevenlevithan.com/archives/faster-trim-javascript).
	//		The short yet performant version of this function is dojo.trim(),
	//		which is part of Dojo base.  Uses String.prototype.trim instead, if available.
	return "";	// String
}
=====*/

dojo.string.trim = String.prototype.trim ?
	dojo.trim : // aliasing to the native function
	function(str){
		str = str.replace(/^\s+/, '');
		for(var i = str.length - 1; i >= 0; i--){
			if(/\S/.test(str.charAt(i))){
				str = str.substring(0, i + 1);
				break;
			}
		}
		return str;
	};

}

if(!dojo._hasResource["dojo.Stateful"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.Stateful"] = true;
dojo.provide("dojo.Stateful");


dojo.declare("dojo.Stateful", null, {
	// summary:
	//		Base class for objects that provide named properties with optional getter/setter
	//		control and the ability to watch for property changes
	// example:
	//	|	var obj = new dojo.Stateful();
	//	|	obj.watch("foo", function(){
	//	|		console.log("foo changed to " + this.get("foo"));
	//	|	});
	//	|	obj.set("foo","bar");
	postscript: function(mixin){
		if(mixin){
			dojo.mixin(this, mixin);
		}
	},
	
	get: function(/*String*/name){
		// summary:
		//		Get a property on a Stateful instance.
		//	name:
		//		The property to get.
		// description:
		//		Get a named property on a Stateful object. The property may
		//		potentially be retrieved via a getter method in subclasses. In the base class
		// 		this just retrieves the object's property.
		// 		For example:
		//	|	stateful = new dojo.Stateful({foo: 3});
		//	|	stateful.get("foo") // returns 3
		//	|	stateful.foo // returns 3
		
		return this[name];
	},
	set: function(/*String*/name, /*Object*/value){
		// summary:
		//		Set a property on a Stateful instance
		//	name:
		//		The property to set.
		//	value:
		//		The value to set in the property.
		// description:
		//		Sets named properties on a stateful object and notifies any watchers of
		// 		the property. A programmatic setter may be defined in subclasses.
		// 		For example:
		//	|	stateful = new dojo.Stateful();
		//	|	stateful.watch(function(name, oldValue, value){
		//	|		// this will be called on the set below
		//	|	}
		//	|	stateful.set(foo, 5);
		//
		//	set() may also be called with a hash of name/value pairs, ex:
		//	|	myObj.set({
		//	|		foo: "Howdy",
		//	|		bar: 3
		//	|	})
		//	This is equivalent to calling set(foo, "Howdy") and set(bar, 3)
		if(typeof name === "object"){
			for(var x in name){
				this.set(x, name[x]);
			}
			return this;
		}
		var oldValue = this[name];
		this[name] = value;
		if(this._watchCallbacks){
			this._watchCallbacks(name, oldValue, value);
		}
		return this;
	},
	watch: function(/*String?*/name, /*Function*/callback){
		// summary:
		//		Watches a property for changes
		//	name:
		//		Indicates the property to watch. This is optional (the callback may be the
		// 		only parameter), and if omitted, all the properties will be watched
		// returns:
		//		An object handle for the watch. The unwatch method of this object
		// 		can be used to discontinue watching this property:
		//		|	var watchHandle = obj.watch("foo", callback);
		//		|	watchHandle.unwatch(); // callback won't be called now
		//	callback:
		//		The function to execute when the property changes. This will be called after
		//		the property has been changed. The callback will be called with the |this|
		//		set to the instance, the first argument as the name of the property, the
		// 		second argument as the old value and the third argument as the new value.
		
		var callbacks = this._watchCallbacks;
		if(!callbacks){
			var self = this;
			callbacks = this._watchCallbacks = function(name, oldValue, value, ignoreCatchall){
				var notify = function(propertyCallbacks){
					if(propertyCallbacks){
                        propertyCallbacks = propertyCallbacks.slice();
						for(var i = 0, l = propertyCallbacks.length; i < l; i++){
							try{
								propertyCallbacks[i].call(self, name, oldValue, value);
							}catch(e){
								console.error(e);
							}
						}
					}
				};
				notify(callbacks['_' + name]);
				if(!ignoreCatchall){
					notify(callbacks["*"]); // the catch-all
				}
			}; // we use a function instead of an object so it will be ignored by JSON conversion
		}
		if(!callback && typeof name === "function"){
			callback = name;
			name = "*";
		}else{
			// prepend with dash to prevent name conflicts with function (like "name" property)
			name = '_' + name;
		}
		var propertyCallbacks = callbacks[name];
		if(typeof propertyCallbacks !== "object"){
			propertyCallbacks = callbacks[name] = [];
		}
		propertyCallbacks.push(callback);
		return {
			unwatch: function(){
				propertyCallbacks.splice(dojo.indexOf(propertyCallbacks, callback), 1);
			}
		};
	}
	
});

}

if(!dojo._hasResource["dijit._WidgetBase"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._WidgetBase"] = true;
dojo.provide("dijit._WidgetBase");




(function(){

dojo.declare("dijit._WidgetBase", dojo.Stateful, {
	// summary:
	//		Future base class for all Dijit widgets.
	//		_Widget extends this class adding support for various features needed by desktop.

	// id: [const] String
	//		A unique, opaque ID string that can be assigned by users or by the
	//		system. If the developer passes an ID which is known not to be
	//		unique, the specified ID is ignored and the system-generated ID is
	//		used instead.
	id: "",

	// lang: [const] String
	//		Rarely used.  Overrides the default Dojo locale used to render this widget,
	//		as defined by the [HTML LANG](http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang) attribute.
	//		Value must be among the list of locales specified during by the Dojo bootstrap,
	//		formatted according to [RFC 3066](http://www.ietf.org/rfc/rfc3066.txt) (like en-us).
	lang: "",

	// dir: [const] String
	//		Bi-directional support, as defined by the [HTML DIR](http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir)
	//		attribute. Either left-to-right "ltr" or right-to-left "rtl".  If undefined, widgets renders in page's
	//		default direction.
	dir: "",

	// class: String
	//		HTML class attribute
	"class": "",

	// style: String||Object
	//		HTML style attributes as cssText string or name/value hash
	style: "",

	// title: String
	//		HTML title attribute.
	//
	//		For form widgets this specifies a tooltip to display when hovering over
	//		the widget (just like the native HTML title attribute).
	//
	//		For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,
	//		etc., it's used to specify the tab label, accordion pane title, etc.
	title: "",

	// tooltip: String
	//		When this widget's title attribute is used to for a tab label, accordion pane title, etc.,
	//		this specifies the tooltip to appear when the mouse is hovered over that text.
	tooltip: "",

	// baseClass: [protected] String
	//		Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate
	//		widget state.
	baseClass: "",

	// srcNodeRef: [readonly] DomNode
	//		pointer to original DOM node
	srcNodeRef: null,

	// domNode: [readonly] DomNode
	//		This is our visible representation of the widget! Other DOM
	//		Nodes may by assigned to other properties, usually through the
	//		template system's dojoAttachPoint syntax, but the domNode
	//		property is the canonical "top level" node in widget UI.
	domNode: null,

	// containerNode: [readonly] DomNode
	//		Designates where children of the source DOM node will be placed.
	//		"Children" in this case refers to both DOM nodes and widgets.
	//		For example, for myWidget:
	//
	//		|	<div dojoType=myWidget>
	//		|		<b> here's a plain DOM node
	//		|		<span dojoType=subWidget>and a widget</span>
	//		|		<i> and another plain DOM node </i>
	//		|	</div>
	//
	//		containerNode would point to:
	//
	//		|		<b> here's a plain DOM node
	//		|		<span dojoType=subWidget>and a widget</span>
	//		|		<i> and another plain DOM node </i>
	//
	//		In templated widgets, "containerNode" is set via a
	//		dojoAttachPoint assignment.
	//
	//		containerNode must be defined for any widget that accepts innerHTML
	//		(like ContentPane or BorderContainer or even Button), and conversely
	//		is null for widgets that don't, like TextBox.
	containerNode: null,

/*=====
	// _started: Boolean
	//		startup() has completed.
	_started: false,
=====*/

	// attributeMap: [protected] Object
	//		attributeMap sets up a "binding" between attributes (aka properties)
	//		of the widget and the widget's DOM.
	//		Changes to widget attributes listed in attributeMap will be
	//		reflected into the DOM.
	//
	//		For example, calling set('title', 'hello')
	//		on a TitlePane will automatically cause the TitlePane's DOM to update
	//		with the new title.
	//
	//		attributeMap is a hash where the key is an attribute of the widget,
	//		and the value reflects a binding to a:
	//
	//		- DOM node attribute
	// |		focus: {node: "focusNode", type: "attribute"}
	// 		Maps this.focus to this.focusNode.focus
	//
	//		- DOM node innerHTML
	//	|		title: { node: "titleNode", type: "innerHTML" }
	//		Maps this.title to this.titleNode.innerHTML
	//
	//		- DOM node innerText
	//	|		title: { node: "titleNode", type: "innerText" }
	//		Maps this.title to this.titleNode.innerText
	//
	//		- DOM node CSS class
	// |		myClass: { node: "domNode", type: "class" }
	//		Maps this.myClass to this.domNode.className
	//
	//		If the value is an array, then each element in the array matches one of the
	//		formats of the above list.
	//
	//		There are also some shorthands for backwards compatibility:
	//		- string --> { node: string, type: "attribute" }, for example:
	//	|	"focusNode" ---> { node: "focusNode", type: "attribute" }
	//		- "" --> { node: "domNode", type: "attribute" }
	attributeMap: {id:"", dir:"", lang:"", "class":"", style:"", title:""},

	// _blankGif: [protected] String
	//		Path to a blank 1x1 image.
	//		Used by <img> nodes in templates that really get their image via CSS background-image.
	_blankGif: (dojo.config.blankGif || dojo.moduleUrl("dojo", "resources/blank.gif")).toString(),

	//////////// INITIALIZATION METHODS ///////////////////////////////////////

	postscript: function(/*Object?*/params, /*DomNode|String*/srcNodeRef){
		// summary:
		//		Kicks off widget instantiation.  See create() for details.
		// tags:
		//		private
		this.create(params, srcNodeRef);
	},

	create: function(/*Object?*/params, /*DomNode|String?*/srcNodeRef){
		// summary:
		//		Kick off the life-cycle of a widget
		// params:
		//		Hash of initialization parameters for widget, including
		//		scalar values (like title, duration etc.) and functions,
		//		typically callbacks like onClick.
		// srcNodeRef:
		//		If a srcNodeRef (DOM node) is specified:
		//			- use srcNodeRef.innerHTML as my contents
		//			- if this is a behavioral widget then apply behavior
		//			  to that srcNodeRef
		//			- otherwise, replace srcNodeRef with my generated DOM
		//			  tree
		// description:
		//		Create calls a number of widget methods (postMixInProperties, buildRendering, postCreate,
		//		etc.), some of which of you'll want to override. See http://docs.dojocampus.org/dijit/_Widget
		//		for a discussion of the widget creation lifecycle.
		//
		//		Of course, adventurous developers could override create entirely, but this should
		//		only be done as a last resort.
		// tags:
		//		private

		// store pointer to original DOM tree
		this.srcNodeRef = dojo.byId(srcNodeRef);

		// For garbage collection.  An array of handles returned by Widget.connect()
		// Each handle returned from Widget.connect() is an array of handles from dojo.connect()
		this._connects = [];

		// For garbage collection.  An array of handles returned by Widget.subscribe()
		// The handle returned from Widget.subscribe() is the handle returned from dojo.subscribe()
		this._subscribes = [];

		// mix in our passed parameters
		if(this.srcNodeRef && (typeof this.srcNodeRef.id == "string")){ this.id = this.srcNodeRef.id; }
		if(params){
			this.params = params;
			dojo._mixin(this, params);
		}
		this.postMixInProperties();

		// generate an id for the widget if one wasn't specified
		// (be sure to do this before buildRendering() because that function might
		// expect the id to be there.)
		if(!this.id){
			this.id = dijit.getUniqueId(this.declaredClass.replace(/\./g,"_"));
		}
		dijit.registry.add(this);

		this.buildRendering();

		if(this.domNode){
			// Copy attributes listed in attributeMap into the [newly created] DOM for the widget.
			// Also calls custom setters for all attributes with custom setters.
			this._applyAttributes();

			// If srcNodeRef was specified, then swap out original srcNode for this widget's DOM tree.
			// For 2.0, move this after postCreate().  postCreate() shouldn't depend on the
			// widget being attached to the DOM since it isn't when a widget is created programmatically like
			// new MyWidget({}).   See #11635.
			var source = this.srcNodeRef;
			if(source && source.parentNode && this.domNode !== source){
				source.parentNode.replaceChild(this.domNode, source);
			}
		}

		if(this.domNode){
			// Note: for 2.0 may want to rename widgetId to dojo._scopeName + "_widgetId",
			// assuming that dojo._scopeName even exists in 2.0
			this.domNode.setAttribute("widgetId", this.id);
		}
		this.postCreate();

		// If srcNodeRef has been processed and removed from the DOM (e.g. TemplatedWidget) then delete it to allow GC.
		if(this.srcNodeRef && !this.srcNodeRef.parentNode){
			delete this.srcNodeRef;
		}

		this._created = true;
	},

	_applyAttributes: function(){
		// summary:
		//		Step during widget creation to copy all widget attributes to the
		//		DOM as per attributeMap and _setXXXAttr functions.
		// description:
		//		Skips over blank/false attribute values, unless they were explicitly specified
		//		as parameters to the widget, since those are the default anyway,
		//		and setting tabIndex="" is different than not setting tabIndex at all.
		//
		//		It processes the attributes in the attribute map first, and then
		//		it goes through and processes the attributes for the _setXXXAttr
		//		functions that have been specified
		// tags:
		//		private
		var condAttrApply = function(attr, scope){
			if((scope.params && attr in scope.params) || scope[attr]){
				scope.set(attr, scope[attr]);
			}
		};

		// Do the attributes in attributeMap
		for(var attr in this.attributeMap){
			condAttrApply(attr, this);
		}

		// And also any attributes with custom setters
		dojo.forEach(this._getSetterAttributes(), function(a){
			if(!(a in this.attributeMap)){
				condAttrApply(a, this);
			}
		}, this);
	},

	_getSetterAttributes: function(){
		// summary:
		//		Returns list of attributes with custom setters for this widget
		var ctor = this.constructor;
		if(!ctor._setterAttrs){
			var r = (ctor._setterAttrs = []),
				attrs,
				proto = ctor.prototype;
			for(var fxName in proto){
				if(dojo.isFunction(proto[fxName]) && (attrs = fxName.match(/^_set([a-zA-Z]*)Attr$/)) && attrs[1]){
					r.push(attrs[1].charAt(0).toLowerCase() + attrs[1].substr(1));
				}
			}
		}
		return ctor._setterAttrs;	// String[]
	},

	postMixInProperties: function(){
		// summary:
		//		Called after the parameters to the widget have been read-in,
		//		but before the widget template is instantiated. Especially
		//		useful to set properties that are referenced in the widget
		//		template.
		// tags:
		//		protected
	},

	buildRendering: function(){
		// summary:
		//		Construct the UI for this widget, setting this.domNode
		// description:
		//		Most widgets will mixin `dijit._Templated`, which implements this
		//		method.
		// tags:
		//		protected

		if(!this.domNode){
			// Create root node if it wasn't created by _Templated
			this.domNode = this.srcNodeRef || dojo.create('div');
		}

		// baseClass is a single class name or occasionally a space-separated list of names.
		// Add those classes to the DOMNode.  If RTL mode then also add with Rtl suffix.
		// TODO: make baseClass custom setter
		if(this.baseClass){
			var classes = this.baseClass.split(" ");
			if(!this.isLeftToRight()){
				classes = classes.concat( dojo.map(classes, function(name){ return name+"Rtl"; }));
			}
			dojo.addClass(this.domNode, classes);
		}
	},

	postCreate: function(){
		// summary:
		//		Processing after the DOM fragment is created
		// description:
		//		Called after the DOM fragment has been created, but not necessarily
		//		added to the document.  Do not include any operations which rely on
		//		node dimensions or placement.
		// tags:
		//		protected
	},

	startup: function(){
		// summary:
		//		Processing after the DOM fragment is added to the document
		// description:
		//		Called after a widget and its children have been created and added to the page,
		//		and all related widgets have finished their create() cycle, up through postCreate().
		//		This is useful for composite widgets that need to control or layout sub-widgets.
		//		Many layout widgets can use this as a wiring phase.
		this._started = true;
	},

	//////////// DESTROY FUNCTIONS ////////////////////////////////

	destroyRecursive: function(/*Boolean?*/ preserveDom){
		// summary:
		// 		Destroy this widget and its descendants
		// description:
		//		This is the generic "destructor" function that all widget users
		// 		should call to cleanly discard with a widget. Once a widget is
		// 		destroyed, it is removed from the manager object.
		// preserveDom:
		//		If true, this method will leave the original DOM structure
		//		alone of descendant Widgets. Note: This will NOT work with
		//		dijit._Templated widgets.

		this._beingDestroyed = true;
		this.destroyDescendants(preserveDom);
		this.destroy(preserveDom);
	},

	destroy: function(/*Boolean*/ preserveDom){
		// summary:
		// 		Destroy this widget, but not its descendants.
		//		This method will, however, destroy internal widgets such as those used within a template.
		// preserveDom: Boolean
		//		If true, this method will leave the original DOM structure alone.
		//		Note: This will not yet work with _Templated widgets

		this._beingDestroyed = true;
		this.uninitialize();
		var d = dojo,
			dfe = d.forEach,
			dun = d.unsubscribe;
		dfe(this._connects, function(array){
			dfe(array, d.disconnect);
		});
		dfe(this._subscribes, function(handle){
			dun(handle);
		});

		// destroy widgets created as part of template, etc.
		dfe(this._supportingWidgets || [], function(w){
			if(w.destroyRecursive){
				w.destroyRecursive();
			}else if(w.destroy){
				w.destroy();
			}
		});

		this.destroyRendering(preserveDom);
		dijit.registry.remove(this.id);
		this._destroyed = true;
	},

	destroyRendering: function(/*Boolean?*/ preserveDom){
		// summary:
		//		Destroys the DOM nodes associated with this widget
		// preserveDom:
		//		If true, this method will leave the original DOM structure alone
		//		during tear-down. Note: this will not work with _Templated
		//		widgets yet.
		// tags:
		//		protected

		if(this.bgIframe){
			this.bgIframe.destroy(preserveDom);
			delete this.bgIframe;
		}

		if(this.domNode){
			if(preserveDom){
				dojo.removeAttr(this.domNode, "widgetId");
			}else{
				dojo.destroy(this.domNode);
			}
			delete this.domNode;
		}

		if(this.srcNodeRef){
			if(!preserveDom){
				dojo.destroy(this.srcNodeRef);
			}
			delete this.srcNodeRef;
		}
	},

	destroyDescendants: function(/*Boolean?*/ preserveDom){
		// summary:
		//		Recursively destroy the children of this widget and their
		//		descendants.
		// preserveDom:
		//		If true, the preserveDom attribute is passed to all descendant
		//		widget's .destroy() method. Not for use with _Templated
		//		widgets.

		// get all direct descendants and destroy them recursively
		dojo.forEach(this.getChildren(), function(widget){
			if(widget.destroyRecursive){
				widget.destroyRecursive(preserveDom);
			}
		});
	},

	uninitialize: function(){
		// summary:
		//		Stub function. Override to implement custom widget tear-down
		//		behavior.
		// tags:
		//		protected
		return false;
	},

	////////////////// GET/SET, CUSTOM SETTERS, ETC. ///////////////////

	_setClassAttr: function(/*String*/ value){
		// summary:
		//		Custom setter for the CSS "class" attribute
		// tags:
		//		protected
		var mapNode = this[this.attributeMap["class"] || 'domNode'];
		dojo.replaceClass(mapNode, value, this["class"]);
		this._set("class", value);
	},

	_setStyleAttr: function(/*String||Object*/ value){
		// summary:
		//		Sets the style attribute of the widget according to value,
		//		which is either a hash like {height: "5px", width: "3px"}
		//		or a plain string
		// description:
		//		Determines which node to set the style on based on style setting
		//		in attributeMap.
		// tags:
		//		protected

		var mapNode = this[this.attributeMap.style || 'domNode'];

		// Note: technically we should revert any style setting made in a previous call
		// to his method, but that's difficult to keep track of.

		if(dojo.isObject(value)){
			dojo.style(mapNode, value);
		}else{
			if(mapNode.style.cssText){
				mapNode.style.cssText += "; " + value;
			}else{
				mapNode.style.cssText = value;
			}
		}

		this._set("style", value);
	},

	_attrToDom: function(/*String*/ attr, /*String*/ value){
		// summary:
		//		Reflect a widget attribute (title, tabIndex, duration etc.) to
		//		the widget DOM, as specified in attributeMap.
		//		Note some attributes like "type"
		//		cannot be processed this way as they are not mutable.
		//
		// tags:
		//		private

		var commands = this.attributeMap[attr];
		dojo.forEach(dojo.isArray(commands) ? commands : [commands], function(command){

			// Get target node and what we are doing to that node
			var mapNode = this[command.node || command || "domNode"];	// DOM node
			var type = command.type || "attribute";	// class, innerHTML, innerText, or attribute

			switch(type){
				case "attribute":
					if(dojo.isFunction(value)){ // functions execute in the context of the widget
						value = dojo.hitch(this, value);
					}

					// Get the name of the DOM node attribute; usually it's the same
					// as the name of the attribute in the widget (attr), but can be overridden.
					// Also maps handler names to lowercase, like onSubmit --> onsubmit
					var attrName = command.attribute ? command.attribute :
						(/^on[A-Z][a-zA-Z]*$/.test(attr) ? attr.toLowerCase() : attr);

					dojo.attr(mapNode, attrName, value);
					break;
				case "innerText":
					mapNode.innerHTML = "";
					mapNode.appendChild(dojo.doc.createTextNode(value));
					break;
				case "innerHTML":
					mapNode.innerHTML = value;
					break;
				case "class":
					dojo.replaceClass(mapNode, value, this[attr]);
					break;
			}
		}, this);
	},

	get: function(name){
		// summary:
		//		Get a property from a widget.
		//	name:
		//		The property to get.
		// description:
		//		Get a named property from a widget. The property may
		//		potentially be retrieved via a getter method. If no getter is defined, this
		// 		just retrieves the object's property.
		// 		For example, if the widget has a properties "foo"
		//		and "bar" and a method named "_getFooAttr", calling:
		//	|	myWidget.get("foo");
		//		would be equivalent to writing:
		//	|	widget._getFooAttr();
		//		and:
		//	|	myWidget.get("bar");
		//		would be equivalent to writing:
		//	|	widget.bar;
		var names = this._getAttrNames(name);
		return this[names.g] ? this[names.g]() : this[name];
	},
	
	set: function(name, value){
		// summary:
		//		Set a property on a widget
		//	name:
		//		The property to set.
		//	value:
		//		The value to set in the property.
		// description:
		//		Sets named properties on a widget which may potentially be handled by a
		// 		setter in the widget.
		// 		For example, if the widget has a properties "foo"
		//		and "bar" and a method named "_setFooAttr", calling:
		//	|	myWidget.set("foo", "Howdy!");
		//		would be equivalent to writing:
		//	|	widget._setFooAttr("Howdy!");
		//		and:
		//	|	myWidget.set("bar", 3);
		//		would be equivalent to writing:
		//	|	widget.bar = 3;
		//
		//	set() may also be called with a hash of name/value pairs, ex:
		//	|	myWidget.set({
		//	|		foo: "Howdy",
		//	|		bar: 3
		//	|	})
		//	This is equivalent to calling set(foo, "Howdy") and set(bar, 3)

		if(typeof name === "object"){
			for(var x in name){
				this.set(x, name[x]);
			}
			return this;
		}
		var names = this._getAttrNames(name);
		if(this[names.s]){
			// use the explicit setter
			var result = this[names.s].apply(this, Array.prototype.slice.call(arguments, 1));
		}else{
			// if param is specified as DOM node attribute, copy it
			if(name in this.attributeMap){
				this._attrToDom(name, value);
			}
			this._set(name, value);
		}
		return result || this;
	},
	
	_attrPairNames: {},		// shared between all widgets
	_getAttrNames: function(name){
		// summary:
		//		Helper function for get() and set().
		//		Caches attribute name values so we don't do the string ops every time.
		// tags:
		//		private

		var apn = this._attrPairNames;
		if(apn[name]){ return apn[name]; }
		var uc = name.charAt(0).toUpperCase() + name.substr(1);
		return (apn[name] = {
			n: name+"Node",
			s: "_set"+uc+"Attr",
			g: "_get"+uc+"Attr"
		});
	},

	_set: function(/*String*/ name, /*anything*/ value){
		// summary:
		//		Helper function to set new value for specified attribute, and call handlers
		//		registered with watch() if the value has changed.
		var oldValue = this[name];
		this[name] = value;
		if(this._watchCallbacks && this._created && value !== oldValue){
			this._watchCallbacks(name, oldValue, value);
		}
	},

	toString: function(){
		// summary:
		//		Returns a string that represents the widget
		// description:
		//		When a widget is cast to a string, this method will be used to generate the
		//		output. Currently, it does not implement any sort of reversible
		//		serialization.
		return '[Widget ' + this.declaredClass + ', ' + (this.id || 'NO ID') + ']'; // String
	},

	getDescendants: function(){
		// summary:
		//		Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.
		//		This method should generally be avoided as it returns widgets declared in templates, which are
		//		supposed to be internal/hidden, but it's left here for back-compat reasons.

		return this.containerNode ? dojo.query('[widgetId]', this.containerNode).map(dijit.byNode) : []; // dijit._Widget[]
	},

	getChildren: function(){
		// summary:
		//		Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.
		//		Does not return nested widgets, nor widgets that are part of this widget's template.
		return this.containerNode ? dijit.findWidgets(this.containerNode) : []; // dijit._Widget[]
	},

	connect: function(
			/*Object|null*/ obj,
			/*String|Function*/ event,
			/*String|Function*/ method){
		// summary:
		//		Connects specified obj/event to specified method of this object
		//		and registers for disconnect() on widget destroy.
		// description:
		//		Provide widget-specific analog to dojo.connect, except with the
		//		implicit use of this widget as the target object.
		//		Events connected with `this.connect` are disconnected upon
		//		destruction.
		// returns:
		//		A handle that can be passed to `disconnect` in order to disconnect before
		//		the widget is destroyed.
		// example:
		//	|	var btn = new dijit.form.Button();
		//	|	// when foo.bar() is called, call the listener we're going to
		//	|	// provide in the scope of btn
		//	|	btn.connect(foo, "bar", function(){
		//	|		console.debug(this.toString());
		//	|	});
		// tags:
		//		protected

		var handles = [dojo._connect(obj, event, this, method)];
		this._connects.push(handles);
		return handles;		// _Widget.Handle
	},

	disconnect: function(/* _Widget.Handle */ handles){
		// summary:
		//		Disconnects handle created by `connect`.
		//		Also removes handle from this widget's list of connects.
		// tags:
		//		protected
		for(var i=0; i<this._connects.length; i++){
			if(this._connects[i] == handles){
				dojo.forEach(handles, dojo.disconnect);
				this._connects.splice(i, 1);
				return;
			}
		}
	},

	subscribe: function(
			/*String*/ topic,
			/*String|Function*/ method){
		// summary:
		//		Subscribes to the specified topic and calls the specified method
		//		of this object and registers for unsubscribe() on widget destroy.
		// description:
		//		Provide widget-specific analog to dojo.subscribe, except with the
		//		implicit use of this widget as the target object.
		// example:
		//	|	var btn = new dijit.form.Button();
		//	|	// when /my/topic is published, this button changes its label to
		//	|   // be the parameter of the topic.
		//	|	btn.subscribe("/my/topic", function(v){
		//	|		this.set("label", v);
		//	|	});
		var handle = dojo.subscribe(topic, this, method);

		// return handles for Any widget that may need them
		this._subscribes.push(handle);
		return handle;
	},

	unsubscribe: function(/*Object*/ handle){
		// summary:
		//		Unsubscribes handle created by this.subscribe.
		//		Also removes handle from this widget's list of subscriptions
		for(var i=0; i<this._subscribes.length; i++){
			if(this._subscribes[i] == handle){
				dojo.unsubscribe(handle);
				this._subscribes.splice(i, 1);
				return;
			}
		}
	},

	isLeftToRight: function(){
		// summary:
		//		Return this widget's explicit or implicit orientation (true for LTR, false for RTL)
		// tags:
		//		protected
		return this.dir ? (this.dir == "ltr") : dojo._isBodyLtr(); //Boolean
	},

	placeAt: function(/* String|DomNode|_Widget */reference, /* String?|Int? */position){
		// summary:
		//		Place this widget's domNode reference somewhere in the DOM based
		//		on standard dojo.place conventions, or passing a Widget reference that
		//		contains and addChild member.
		//
		// description:
		//		A convenience function provided in all _Widgets, providing a simple
		//		shorthand mechanism to put an existing (or newly created) Widget
		//		somewhere in the dom, and allow chaining.
		//
		// reference:
		//		The String id of a domNode, a domNode reference, or a reference to a Widget posessing
		//		an addChild method.
		//
		// position:
		//		If passed a string or domNode reference, the position argument
		//		accepts a string just as dojo.place does, one of: "first", "last",
		//		"before", or "after".
		//
		//		If passed a _Widget reference, and that widget reference has an ".addChild" method,
		//		it will be called passing this widget instance into that method, supplying the optional
		//		position index passed.
		//
		// returns:
		//		dijit._Widget
		//		Provides a useful return of the newly created dijit._Widget instance so you
		//		can "chain" this function by instantiating, placing, then saving the return value
		//		to a variable.
		//
		// example:
		// | 	// create a Button with no srcNodeRef, and place it in the body:
		// | 	var button = new dijit.form.Button({ label:"click" }).placeAt(dojo.body());
		// | 	// now, 'button' is still the widget reference to the newly created button
		// | 	dojo.connect(button, "onClick", function(e){ console.log('click'); });
		//
		// example:
		// |	// create a button out of a node with id="src" and append it to id="wrapper":
		// | 	var button = new dijit.form.Button({},"src").placeAt("wrapper");
		//
		// example:
		// |	// place a new button as the first element of some div
		// |	var button = new dijit.form.Button({ label:"click" }).placeAt("wrapper","first");
		//
		// example:
		// |	// create a contentpane and add it to a TabContainer
		// |	var tc = dijit.byId("myTabs");
		// |	new dijit.layout.ContentPane({ href:"foo.html", title:"Wow!" }).placeAt(tc)

		if(reference.declaredClass && reference.addChild){
			reference.addChild(this, position);
		}else{
			dojo.place(this.domNode, reference, position);
		}
		return this;
	}
});

})();

}

if(!dojo._hasResource["dijit._Widget"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._Widget"] = true;
dojo.provide("dijit._Widget");





////////////////// DEFERRED CONNECTS ///////////////////

// This code is to assist deferring dojo.connect() calls in widgets (connecting to events on the widgets'
// DOM nodes) until someone actually needs to monitor that event.
dojo.connect(dojo, "_connect",
	function(/*dijit._Widget*/ widget, /*String*/ event){
		if(widget && dojo.isFunction(widget._onConnect)){
			widget._onConnect(event);
		}
	});

dijit._connectOnUseEventHandler = function(/*Event*/ event){};

////////////////// ONDIJITCLICK SUPPORT ///////////////////

// Keep track of where the last keydown event was, to help avoid generating
// spurious ondijitclick events when:
// 1. focus is on a <button> or <a>
// 2. user presses then releases the ENTER key
// 3. onclick handler fires and shifts focus to another node, with an ondijitclick handler
// 4. onkeyup event fires, causing the ondijitclick handler to fire
dijit._lastKeyDownNode = null;
if(dojo.isIE){
	(function(){
		var keydownCallback = function(evt){
			dijit._lastKeyDownNode = evt.srcElement;
		};
		dojo.doc.attachEvent('onkeydown', keydownCallback);
		dojo.addOnWindowUnload(function(){
			dojo.doc.detachEvent('onkeydown', keydownCallback);
		});
	})();
}else{
	dojo.doc.addEventListener('keydown', function(evt){
		dijit._lastKeyDownNode = evt.target;
	}, true);
}

(function(){

dojo.declare("dijit._Widget", dijit._WidgetBase, {
	// summary:
	//		Base class for all Dijit widgets.
	//
	//		Extends _WidgetBase, adding support for:
	//			- deferred connections
	//				A call like dojo.connect(myWidget, "onMouseMove", func)
	//				will essentially do a dojo.connect(myWidget.domNode, "onMouseMove", func)
	//			- ondijitclick
	//				Support new dojoAttachEvent="ondijitclick: ..." that is triggered by a mouse click or a SPACE/ENTER keypress
	//			- focus related functions
	//				In particular, the onFocus()/onBlur() callbacks.   Driven internally by
	//				dijit/_base/focus.js.
	//			- deprecated methods
	//			- onShow(), onHide(), onClose()
	//
	//		Also, by loading code in dijit/_base, turns on:
	//			- browser sniffing (putting browser id like .dj_ie on <html> node)
	//			- high contrast mode sniffing (add .dijit_a11y class to <body> if machine is in high contrast mode)
	

	////////////////// DEFERRED CONNECTS ///////////////////

	// _deferredConnects: [protected] Object
	//		attributeMap addendum for event handlers that should be connected only on first use
	_deferredConnects: {
		onClick: "",
		onDblClick: "",
		onKeyDown: "",
		onKeyPress: "",
		onKeyUp: "",
		onMouseMove: "",
		onMouseDown: "",
		onMouseOut: "",
		onMouseOver: "",
		onMouseLeave: "",
		onMouseEnter: "",
		onMouseUp: ""
	},

	onClick: dijit._connectOnUseEventHandler,
	/*=====
	onClick: function(event){
		// summary:
		//		Connect to this function to receive notifications of mouse click events.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onDblClick: dijit._connectOnUseEventHandler,
	/*=====
	onDblClick: function(event){
		// summary:
		//		Connect to this function to receive notifications of mouse double click events.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onKeyDown: dijit._connectOnUseEventHandler,
	/*=====
	onKeyDown: function(event){
		// summary:
		//		Connect to this function to receive notifications of keys being pressed down.
		// event:
		//		key Event
		// tags:
		//		callback
	},
	=====*/
	onKeyPress: dijit._connectOnUseEventHandler,
	/*=====
	onKeyPress: function(event){
		// summary:
		//		Connect to this function to receive notifications of printable keys being typed.
		// event:
		//		key Event
		// tags:
		//		callback
	},
	=====*/
	onKeyUp: dijit._connectOnUseEventHandler,
	/*=====
	onKeyUp: function(event){
		// summary:
		//		Connect to this function to receive notifications of keys being released.
		// event:
		//		key Event
		// tags:
		//		callback
	},
	=====*/
	onMouseDown: dijit._connectOnUseEventHandler,
	/*=====
	onMouseDown: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse button is pressed down.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseMove: dijit._connectOnUseEventHandler,
	/*=====
	onMouseMove: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves over nodes contained within this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseOut: dijit._connectOnUseEventHandler,
	/*=====
	onMouseOut: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves off of nodes contained within this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseOver: dijit._connectOnUseEventHandler,
	/*=====
	onMouseOver: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves onto nodes contained within this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseLeave: dijit._connectOnUseEventHandler,
	/*=====
	onMouseLeave: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves off of this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseEnter: dijit._connectOnUseEventHandler,
	/*=====
	onMouseEnter: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves onto this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseUp: dijit._connectOnUseEventHandler,
	/*=====
	onMouseUp: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse button is released.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/

	create: function(/*Object?*/params, /*DomNode|String?*/srcNodeRef){
		// To avoid double-connects, remove entries from _deferredConnects
		// that have been setup manually by a subclass (ex, by dojoAttachEvent).
		// If a subclass has redefined a callback (ex: onClick) then assume it's being
		// connected to manually.
		this._deferredConnects = dojo.clone(this._deferredConnects);
		for(var attr in this.attributeMap){
			delete this._deferredConnects[attr]; // can't be in both attributeMap and _deferredConnects
		}
		for(attr in this._deferredConnects){
			if(this[attr] !== dijit._connectOnUseEventHandler){
				delete this._deferredConnects[attr];	// redefined, probably dojoAttachEvent exists
			}
		}

		this.inherited(arguments);

		if(this.domNode){
			// If the developer has specified a handler as a widget parameter
			// (ex: new Button({onClick: ...})
			// then naturally need to connect from DOM node to that handler immediately,
			for(attr in this.params){
				this._onConnect(attr);
			}
		}
	},

	_onConnect: function(/*String*/ event){
		// summary:
		//		Called when someone connects to one of my handlers.
		//		"Turn on" that handler if it isn't active yet.
		//
		//		This is also called for every single initialization parameter
		//		so need to do nothing for parameters like "id".
		// tags:
		//		private
		if(event in this._deferredConnects){
			var mapNode = this[this._deferredConnects[event] || 'domNode'];
			this.connect(mapNode, event.toLowerCase(), event);
			delete this._deferredConnects[event];
		}
	},

	////////////////// FOCUS RELATED ///////////////////
	// _onFocus() and _onBlur() are called by the focus manager

	// focused: [readonly] Boolean
	//		This widget or a widget it contains has focus, or is "active" because
	//		it was recently clicked.
	focused: false,

	isFocusable: function(){
		// summary:
		//		Return true if this widget can currently be focused
		//		and false if not
		return this.focus && (dojo.style(this.domNode, "display") != "none");
	},

	onFocus: function(){
		// summary:
		//		Called when the widget becomes "active" because
		//		it or a widget inside of it either has focus, or has recently
		//		been clicked.
		// tags:
		//		callback
	},

	onBlur: function(){
		// summary:
		//		Called when the widget stops being "active" because
		//		focus moved to something outside of it, or the user
		//		clicked somewhere outside of it, or the widget was
		//		hidden.
		// tags:
		//		callback
	},

	_onFocus: function(e){
		// summary:
		//		This is where widgets do processing for when they are active,
		//		such as changing CSS classes.  See onFocus() for more details.
		// tags:
		//		protected
		this.onFocus();
	},

	_onBlur: function(){
		// summary:
		//		This is where widgets do processing for when they stop being active,
		//		such as changing CSS classes.  See onBlur() for more details.
		// tags:
		//		protected
		this.onBlur();
	},

	////////////////// DEPRECATED METHODS ///////////////////

	setAttribute: function(/*String*/ attr, /*anything*/ value){
		// summary:
		//		Deprecated.  Use set() instead.
		// tags:
		//		deprecated
		dojo.deprecated(this.declaredClass+"::setAttribute(attr, value) is deprecated. Use set() instead.", "", "2.0");
		this.set(attr, value);
	},

	attr: function(/*String|Object*/name, /*Object?*/value){
		// summary:
		//		Set or get properties on a widget instance.
		//	name:
		//		The property to get or set. If an object is passed here and not
		//		a string, its keys are used as names of attributes to be set
		//		and the value of the object as values to set in the widget.
		//	value:
		//		Optional. If provided, attr() operates as a setter. If omitted,
		//		the current value of the named property is returned.
		// description:
		//		This method is deprecated, use get() or set() directly.

		// Print deprecation warning but only once per calling function
		if(dojo.config.isDebug){
			var alreadyCalledHash = arguments.callee._ach || (arguments.callee._ach = {}),
				caller = (arguments.callee.caller || "unknown caller").toString();
			if(!alreadyCalledHash[caller]){
				dojo.deprecated(this.declaredClass + "::attr() is deprecated. Use get() or set() instead, called from " +
				caller, "", "2.0");
				alreadyCalledHash[caller] = true;
			}
		}

		var args = arguments.length;
		if(args >= 2 || typeof name === "object"){ // setter
			return this.set.apply(this, arguments);
		}else{ // getter
			return this.get(name);
		}
	},
	
	////////////////// ONDIJITCLICK SUPPORT ///////////////////

	// nodesWithKeyClick: [private] String[]
	//		List of nodes that correctly handle click events via native browser support,
	//		and don't need dijit's help
	nodesWithKeyClick: ["input", "button"],

	connect: function(
			/*Object|null*/ obj,
			/*String|Function*/ event,
			/*String|Function*/ method){
		// summary:
		//		Connects specified obj/event to specified method of this object
		//		and registers for disconnect() on widget destroy.
		// description:
		//		Provide widget-specific analog to dojo.connect, except with the
		//		implicit use of this widget as the target object.
		//		This version of connect also provides a special "ondijitclick"
		//		event which triggers on a click or space or enter keyup.
		//		Events connected with `this.connect` are disconnected upon
		//		destruction.
		// returns:
		//		A handle that can be passed to `disconnect` in order to disconnect before
		//		the widget is destroyed.
		// example:
		//	|	var btn = new dijit.form.Button();
		//	|	// when foo.bar() is called, call the listener we're going to
		//	|	// provide in the scope of btn
		//	|	btn.connect(foo, "bar", function(){
		//	|		console.debug(this.toString());
		//	|	});
		// tags:
		//		protected

		var d = dojo,
			dc = d._connect,
			handles = this.inherited(arguments, [obj, event == "ondijitclick" ? "onclick" : event, method]);

		if(event == "ondijitclick"){
			// add key based click activation for unsupported nodes.
			// do all processing onkey up to prevent spurious clicks
			// for details see comments at top of this file where _lastKeyDownNode is defined
			if(d.indexOf(this.nodesWithKeyClick, obj.nodeName.toLowerCase()) == -1){ // is NOT input or button
				var m = d.hitch(this, method);
				handles.push(
					dc(obj, "onkeydown", this, function(e){
						//console.log(this.id + ": onkeydown, e.target = ", e.target, ", lastKeyDownNode was ", dijit._lastKeyDownNode, ", equality is ", (e.target === dijit._lastKeyDownNode));
						if((e.keyCode == d.keys.ENTER || e.keyCode == d.keys.SPACE) &&
							!e.ctrlKey && !e.shiftKey && !e.altKey && !e.metaKey){
							// needed on IE for when focus changes between keydown and keyup - otherwise dropdown menus do not work
							dijit._lastKeyDownNode = e.target;
							
							// Stop event to prevent scrolling on space key in IE.
							// But don't do this for _HasDropDown because it surpresses the onkeypress
							// event needed to open the drop down when the user presses the SPACE key.
							if(!("openDropDown" in this && obj == this._buttonNode)){
								e.preventDefault();
							}
						}
			 		}),
					dc(obj, "onkeyup", this, function(e){
						//console.log(this.id + ": onkeyup, e.target = ", e.target, ", lastKeyDownNode was ", dijit._lastKeyDownNode, ", equality is ", (e.target === dijit._lastKeyDownNode));
						if( (e.keyCode == d.keys.ENTER || e.keyCode == d.keys.SPACE) &&
							e.target == dijit._lastKeyDownNode &&	// === breaks greasemonkey
							!e.ctrlKey && !e.shiftKey && !e.altKey && !e.metaKey){
								//need reset here or have problems in FF when focus returns to trigger element after closing popup/alert
								dijit._lastKeyDownNode = null;
								return m(e);
						}
					})
				);
			}
		}

		return handles;		// _Widget.Handle
	},

	////////////////// MISCELLANEOUS METHODS ///////////////////

	_onShow: function(){
		// summary:
		//		Internal method called when this widget is made visible.
		//		See `onShow` for details.
		this.onShow();
	},

	onShow: function(){
		// summary:
		//		Called when this widget becomes the selected pane in a
		//		`dijit.layout.TabContainer`, `dijit.layout.StackContainer`,
		//		`dijit.layout.AccordionContainer`, etc.
		//
		//		Also called to indicate display of a `dijit.Dialog`, `dijit.TooltipDialog`, or `dijit.TitlePane`.
		// tags:
		//		callback
	},

	onHide: function(){
		// summary:
			//		Called when another widget becomes the selected pane in a
			//		`dijit.layout.TabContainer`, `dijit.layout.StackContainer`,
			//		`dijit.layout.AccordionContainer`, etc.
			//
			//		Also called to indicate hide of a `dijit.Dialog`, `dijit.TooltipDialog`, or `dijit.TitlePane`.
			// tags:
			//		callback
	},

	onClose: function(){
		// summary:
		//		Called when this widget is being displayed as a popup (ex: a Calendar popped
		//		up from a DateTextBox), and it is hidden.
		//		This is called from the dijit.popup code, and should not be called directly.
		//
		//		Also used as a parameter for children of `dijit.layout.StackContainer` or subclasses.
		//		Callback if a user tries to close the child.   Child will be closed if this function returns true.
		// tags:
		//		extension

		return true;		// Boolean
	}
});

})();

}

if(!dojo._hasResource["dojo.cache"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.cache"] = true;
dojo.provide("dojo.cache");


/*=====
dojo.cache = {
	// summary:
	// 		A way to cache string content that is fetchable via `dojo.moduleUrl`.
};
=====*/

	var cache = {};
	dojo.cache = function(/*String||Object*/module, /*String*/url, /*String||Object?*/value){
		// summary:
		// 		A getter and setter for storing the string content associated with the
		// 		module and url arguments.
		// description:
		// 		module and url are used to call `dojo.moduleUrl()` to generate a module URL.
		// 		If value is specified, the cache value for the moduleUrl will be set to
		// 		that value. Otherwise, dojo.cache will fetch the moduleUrl and store it
		// 		in its internal cache and return that cached value for the URL. To clear
		// 		a cache value pass null for value. Since XMLHttpRequest (XHR) is used to fetch the
		// 		the URL contents, only modules on the same domain of the page can use this capability.
		// 		The build system can inline the cache values though, to allow for xdomain hosting.
		// module: String||Object
		// 		If a String, the module name to use for the base part of the URL, similar to module argument
		// 		to `dojo.moduleUrl`. If an Object, something that has a .toString() method that
		// 		generates a valid path for the cache item. For example, a dojo._Url object.
		// url: String
		// 		The rest of the path to append to the path derived from the module argument. If
		// 		module is an object, then this second argument should be the "value" argument instead.
		// value: String||Object?
		// 		If a String, the value to use in the cache for the module/url combination.
		// 		If an Object, it can have two properties: value and sanitize. The value property
		// 		should be the value to use in the cache, and sanitize can be set to true or false,
		// 		to indicate if XML declarations should be removed from the value and if the HTML
		// 		inside a body tag in the value should be extracted as the real value. The value argument
		// 		or the value property on the value argument are usually only used by the build system
		// 		as it inlines cache content.
		//	example:
		//		To ask dojo.cache to fetch content and store it in the cache (the dojo["cache"] style
		// 		of call is used to avoid an issue with the build system erroneously trying to intern
		// 		this example. To get the build system to intern your dojo.cache calls, use the
		// 		"dojo.cache" style of call):
		// 		|	//If template.html contains "<h1>Hello</h1>" that will be
		// 		|	//the value for the text variable.
		//		|	var text = dojo["cache"]("my.module", "template.html");
		//	example:
		//		To ask dojo.cache to fetch content and store it in the cache, and sanitize the input
		// 		 (the dojo["cache"] style of call is used to avoid an issue with the build system
		// 		erroneously trying to intern this example. To get the build system to intern your
		// 		dojo.cache calls, use the "dojo.cache" style of call):
		// 		|	//If template.html contains "<html><body><h1>Hello</h1></body></html>", the
		// 		|	//text variable will contain just "<h1>Hello</h1>".
		//		|	var text = dojo["cache"]("my.module", "template.html", {sanitize: true});
		//	example:
		//		Same example as previous, but demostrates how an object can be passed in as
		//		the first argument, then the value argument can then be the second argument.
		// 		|	//If template.html contains "<html><body><h1>Hello</h1></body></html>", the
		// 		|	//text variable will contain just "<h1>Hello</h1>".
		//		|	var text = dojo["cache"](new dojo._Url("my/module/template.html"), {sanitize: true});

		//Module could be a string, or an object that has a toString() method
		//that will return a useful path. If it is an object, then the "url" argument
		//will actually be the value argument.
		if(typeof module == "string"){
			var pathObj = dojo.moduleUrl(module, url);
		}else{
			pathObj = module;
			value = url;
		}
		var key = pathObj.toString();

		var val = value;
		if(value != undefined && !dojo.isString(value)){
			val = ("value" in value ? value.value : undefined);
		}

		var sanitize = value && value.sanitize ? true : false;

		if(typeof val == "string"){
			//We have a string, set cache value
			val = cache[key] = sanitize ? dojo.cache._sanitize(val) : val;
		}else if(val === null){
			//Remove cached value
			delete cache[key];
		}else{
			//Allow cache values to be empty strings. If key property does
			//not exist, fetch it.
			if(!(key in cache)){
				val = dojo._getText(key);
				cache[key] = sanitize ? dojo.cache._sanitize(val) : val;
			}
			val = cache[key];
		}
		return val; //String
	};

	dojo.cache._sanitize = function(/*String*/val){
		// summary:
		//		Strips <?xml ...?> declarations so that external SVG and XML
		// 		documents can be added to a document without worry. Also, if the string
		//		is an HTML document, only the part inside the body tag is returned.
		// description:
		// 		Copied from dijit._Templated._sanitizeTemplateString.
		if(val){
			val = val.replace(/^\s*<\?xml(\s)+version=[\'\"](\d)*.(\d)*[\'\"](\s)*\?>/im, "");
			var matches = val.match(/<body[^>]*>\s*([\s\S]+)\s*<\/body>/im);
			if(matches){
				val = matches[1];
			}
		}else{
			val = "";
		}
		return val; //String
	};

}

if(!dojo._hasResource["dijit._Templated"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._Templated"] = true;
dojo.provide("dijit._Templated");






dojo.declare("dijit._Templated",
	null,
	{
		// summary:
		//		Mixin for widgets that are instantiated from a template

		// templateString: [protected] String
		//		A string that represents the widget template. Pre-empts the
		//		templatePath. In builds that have their strings "interned", the
		//		templatePath is converted to an inline templateString, thereby
		//		preventing a synchronous network call.
		//
		//		Use in conjunction with dojo.cache() to load from a file.
		templateString: null,

		// templatePath: [protected deprecated] String
		//		Path to template (HTML file) for this widget relative to dojo.baseUrl.
		//		Deprecated: use templateString with dojo.cache() instead.
		templatePath: null,

		// widgetsInTemplate: [protected] Boolean
		//		Should we parse the template to find widgets that might be
		//		declared in markup inside it?  False by default.
		widgetsInTemplate: false,

		// skipNodeCache: [protected] Boolean
		//		If using a cached widget template node poses issues for a
		//		particular widget class, it can set this property to ensure
		//		that its template is always re-built from a string
		_skipNodeCache: false,

		// _earlyTemplatedStartup: Boolean
		//		A fallback to preserve the 1.0 - 1.3 behavior of children in
		//		templates having their startup called before the parent widget
		//		fires postCreate. Defaults to 'false', causing child widgets to
		//		have their .startup() called immediately before a parent widget
		//		.startup(), but always after the parent .postCreate(). Set to
		//		'true' to re-enable to previous, arguably broken, behavior.
		_earlyTemplatedStartup: false,

/*=====
		// _attachPoints: [private] String[]
		//		List of widget attribute names associated with dojoAttachPoint=... in the
		//		template, ex: ["containerNode", "labelNode"]
 		_attachPoints: [],
 =====*/

/*=====
		// _attachEvents: [private] Handle[]
		//		List of connections associated with dojoAttachEvent=... in the
		//		template
 		_attachEvents: [],
 =====*/

		constructor: function(){
			this._attachPoints = [];
			this._attachEvents = [];
		},

		_stringRepl: function(tmpl){
			// summary:
			//		Does substitution of ${foo} type properties in template string
			// tags:
			//		private
			var className = this.declaredClass, _this = this;
			// Cache contains a string because we need to do property replacement
			// do the property replacement
			return dojo.string.substitute(tmpl, this, function(value, key){
				if(key.charAt(0) == '!'){ value = dojo.getObject(key.substr(1), false, _this); }
				if(typeof value == "undefined"){ throw new Error(className+" template:"+key); } // a debugging aide
				if(value == null){ return ""; }

				// Substitution keys beginning with ! will skip the transform step,
				// in case a user wishes to insert unescaped markup, e.g. ${!foo}
				return key.charAt(0) == "!" ? value :
					// Safer substitution, see heading "Attribute values" in
					// http://www.w3.org/TR/REC-html40/appendix/notes.html#h-B.3.2
					value.toString().replace(/"/g,"&quot;"); //TODO: add &amp? use encodeXML method?
			}, this);
		},

		buildRendering: function(){
			// summary:
			//		Construct the UI for this widget from a template, setting this.domNode.
			// tags:
			//		protected

			// Lookup cached version of template, and download to cache if it
			// isn't there already.  Returns either a DomNode or a string, depending on
			// whether or not the template contains ${foo} replacement parameters.
			var cached = dijit._Templated.getCachedTemplate(this.templatePath, this.templateString, this._skipNodeCache);

			var node;
			if(dojo.isString(cached)){
				node = dojo._toDom(this._stringRepl(cached));
				if(node.nodeType != 1){
					// Flag common problems such as templates with multiple top level nodes (nodeType == 11)
					throw new Error("Invalid template: " + cached);
				}
			}else{
				// if it's a node, all we have to do is clone it
				node = cached.cloneNode(true);
			}

			this.domNode = node;

			// Call down to _Widget.buildRendering() to get base classes assigned
			// TODO: change the baseClass assignment to attributeMap
			this.inherited(arguments);

			// recurse through the node, looking for, and attaching to, our
			// attachment points and events, which should be defined on the template node.
			this._attachTemplateNodes(node);

			if(this.widgetsInTemplate){
				// Store widgets that we need to start at a later point in time
				var cw = (this._startupWidgets = dojo.parser.parse(node, {
					noStart: !this._earlyTemplatedStartup,
					template: true,
					inherited: {dir: this.dir, lang: this.lang},
					propsThis: this,	// so data-dojo-props of widgets in the template can reference "this" to refer to me
					scope: "dojo"	// even in multi-version mode templates use dojoType/data-dojo-type
				}));

				this._supportingWidgets = dijit.findWidgets(node);

				this._attachTemplateNodes(cw, function(n,p){
					return n[p];
				});
			}

			this._fillContent(this.srcNodeRef);
		},

		_fillContent: function(/*DomNode*/ source){
			// summary:
			//		Relocate source contents to templated container node.
			//		this.containerNode must be able to receive children, or exceptions will be thrown.
			// tags:
			//		protected
			var dest = this.containerNode;
			if(source && dest){
				while(source.hasChildNodes()){
					dest.appendChild(source.firstChild);
				}
			}
		},

		_attachTemplateNodes: function(rootNode, getAttrFunc){
			// summary:
			//		Iterate through the template and attach functions and nodes accordingly.
			//		Alternately, if rootNode is an array of widgets, then will process dojoAttachPoint
			//		etc. for those widgets.
			// description:
			//		Map widget properties and functions to the handlers specified in
			//		the dom node and it's descendants. This function iterates over all
			//		nodes and looks for these properties:
			//			* dojoAttachPoint
			//			* dojoAttachEvent
			//			* waiRole
			//			* waiState
			// rootNode: DomNode|Array[Widgets]
			//		the node to search for properties. All children will be searched.
			// getAttrFunc: Function?
			//		a function which will be used to obtain property for a given
			//		DomNode/Widget
			// tags:
			//		private

			getAttrFunc = getAttrFunc || function(n,p){ return n.getAttribute(p); };

			var nodes = dojo.isArray(rootNode) ? rootNode : (rootNode.all || rootNode.getElementsByTagName("*"));
			var x = dojo.isArray(rootNode) ? 0 : -1;
			for(; x<nodes.length; x++){
				var baseNode = (x == -1) ? rootNode : nodes[x];
				if(this.widgetsInTemplate && (getAttrFunc(baseNode, "dojoType") || getAttrFunc(baseNode, "data-dojo-type"))){
					continue;
				}
				// Process dojoAttachPoint
				var attachPoint = getAttrFunc(baseNode, "dojoAttachPoint") || getAttrFunc(baseNode, "data-dojo-attach-point");
				if(attachPoint){
					var point, points = attachPoint.split(/\s*,\s*/);
					while((point = points.shift())){
						if(dojo.isArray(this[point])){
							this[point].push(baseNode);
						}else{
							this[point]=baseNode;
						}
						this._attachPoints.push(point);
					}
				}

				// Process dojoAttachEvent
				var attachEvent = getAttrFunc(baseNode, "dojoAttachEvent") || getAttrFunc(baseNode, "data-dojo-attach-event");;
				if(attachEvent){
					// NOTE: we want to support attributes that have the form
					// "domEvent: nativeEvent; ..."
					var event, events = attachEvent.split(/\s*,\s*/);
					var trim = dojo.trim;
					while((event = events.shift())){
						if(event){
							var thisFunc = null;
							if(event.indexOf(":") != -1){
								// oh, if only JS had tuple assignment
								var funcNameArr = event.split(":");
								event = trim(funcNameArr[0]);
								thisFunc = trim(funcNameArr[1]);
							}else{
								event = trim(event);
							}
							if(!thisFunc){
								thisFunc = event;
							}
							this._attachEvents.push(this.connect(baseNode, event, thisFunc));
						}
					}
				}

				// waiRole, waiState
				// TODO: remove this in 2.0, templates are now using role=... and aria-XXX=... attributes directicly
				var role = getAttrFunc(baseNode, "waiRole");
				if(role){
					dijit.setWaiRole(baseNode, role);
				}
				var values = getAttrFunc(baseNode, "waiState");
				if(values){
					dojo.forEach(values.split(/\s*,\s*/), function(stateValue){
						if(stateValue.indexOf('-') != -1){
							var pair = stateValue.split('-');
							dijit.setWaiState(baseNode, pair[0], pair[1]);
						}
					});
				}
			}
		},

		startup: function(){
			dojo.forEach(this._startupWidgets, function(w){
				if(w && !w._started && w.startup){
					w.startup();
				}
			});
			this.inherited(arguments);
		},

		destroyRendering: function(){
			// Delete all attach points to prevent IE6 memory leaks.
			dojo.forEach(this._attachPoints, function(point){
				delete this[point];
			}, this);
			this._attachPoints = [];

			// And same for event handlers
			dojo.forEach(this._attachEvents, this.disconnect, this);
			this._attachEvents = [];
			
			this.inherited(arguments);
		}
	}
);

// key is either templatePath or templateString; object is either string or DOM tree
dijit._Templated._templateCache = {};

dijit._Templated.getCachedTemplate = function(templatePath, templateString, alwaysUseString){
	// summary:
	//		Static method to get a template based on the templatePath or
	//		templateString key
	// templatePath: String||dojo.uri.Uri
	//		The URL to get the template from.
	// templateString: String?
	//		a string to use in lieu of fetching the template from a URL. Takes precedence
	//		over templatePath
	// returns: Mixed
	//		Either string (if there are ${} variables that need to be replaced) or just
	//		a DOM tree (if the node can be cloned directly)

	// is it already cached?
	var tmplts = dijit._Templated._templateCache;
	var key = templateString || templatePath;
	var cached = tmplts[key];
	if(cached){
		try{
			// if the cached value is an innerHTML string (no ownerDocument) or a DOM tree created within the current document, then use the current cached value
			if(!cached.ownerDocument || cached.ownerDocument == dojo.doc){
				// string or node of the same document
				return cached;
			}
		}catch(e){ /* squelch */ } // IE can throw an exception if cached.ownerDocument was reloaded
		dojo.destroy(cached);
	}

	// If necessary, load template string from template path
	if(!templateString){
		templateString = dojo.cache(templatePath, {sanitize: true});
	}
	templateString = dojo.string.trim(templateString);

	if(alwaysUseString || templateString.match(/\$\{([^\}]+)\}/g)){
		// there are variables in the template so all we can do is cache the string
		return (tmplts[key] = templateString); //String
	}else{
		// there are no variables in the template so we can cache the DOM tree
		var node = dojo._toDom(templateString);
		if(node.nodeType != 1){
			throw new Error("Invalid template: " + templateString);
		}
		return (tmplts[key] = node); //Node
	}
};

if(dojo.isIE){
	dojo.addOnWindowUnload(function(){
		var cache = dijit._Templated._templateCache;
		for(var key in cache){
			var value = cache[key];
			if(typeof value == "object"){ // value is either a string or a DOM node template
				dojo.destroy(value);
			}
			delete cache[key];
		}
	});
}

// These arguments can be specified for widgets which are used in templates.
// Since any widget can be specified as sub widgets in template, mix it
// into the base widget class.  (This is a hack, but it's effective.)
dojo.extend(dijit._Widget,{
	dojoAttachEvent: "",
	dojoAttachPoint: "",
	waiRole: "",
	waiState:""
});

}

if(!dojo._hasResource["dijit._Container"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._Container"] = true;
dojo.provide("dijit._Container");


dojo.declare("dijit._Container",
	null,
	{
		// summary:
		//		Mixin for widgets that contain a set of widget children.
		// description:
		//		Use this mixin for widgets that needs to know about and
		//		keep track of their widget children. Suitable for widgets like BorderContainer
		//		and TabContainer which contain (only) a set of child widgets.
		//
		//		It's not suitable for widgets like ContentPane
		//		which contains mixed HTML (plain DOM nodes in addition to widgets),
		//		and where contained widgets are not necessarily directly below
		//		this.containerNode.   In that case calls like addChild(node, position)
		//		wouldn't make sense.

		// isContainer: [protected] Boolean
		//		Indicates that this widget acts as a "parent" to the descendant widgets.
		//		When the parent is started it will call startup() on the child widgets.
		//		See also `isLayoutContainer`.
		isContainer: true,

		buildRendering: function(){
			this.inherited(arguments);
			if(!this.containerNode){
				// all widgets with descendants must set containerNode
	 				this.containerNode = this.domNode;
			}
		},

		addChild: function(/*dijit._Widget*/ widget, /*int?*/ insertIndex){
			// summary:
			//		Makes the given widget a child of this widget.
			// description:
			//		Inserts specified child widget's dom node as a child of this widget's
			//		container node, and possibly does other processing (such as layout).

			var refNode = this.containerNode;
			if(insertIndex && typeof insertIndex == "number"){
				var children = this.getChildren();
				if(children && children.length >= insertIndex){
					refNode = children[insertIndex-1].domNode;
					insertIndex = "after";
				}
			}
			dojo.place(widget.domNode, refNode, insertIndex);

			// If I've been started but the child widget hasn't been started,
			// start it now.  Make sure to do this after widget has been
			// inserted into the DOM tree, so it can see that it's being controlled by me,
			// so it doesn't try to size itself.
			if(this._started && !widget._started){
				widget.startup();
			}
		},

		removeChild: function(/*Widget or int*/ widget){
			// summary:
			//		Removes the passed widget instance from this widget but does
			//		not destroy it.  You can also pass in an integer indicating
			//		the index within the container to remove

			if(typeof widget == "number"){
				widget = this.getChildren()[widget];
			}

			if(widget){
				var node = widget.domNode;
				if(node && node.parentNode){
					node.parentNode.removeChild(node); // detach but don't destroy
				}
			}
		},

		hasChildren: function(){
			// summary:
			//		Returns true if widget has children, i.e. if this.containerNode contains something.
			return this.getChildren().length > 0;	// Boolean
		},

		destroyDescendants: function(/*Boolean*/ preserveDom){
			// summary:
			//      Destroys all the widgets inside this.containerNode,
			//      but not this widget itself
			dojo.forEach(this.getChildren(), function(child){ child.destroyRecursive(preserveDom); });
		},

		_getSiblingOfChild: function(/*dijit._Widget*/ child, /*int*/ dir){
			// summary:
			//		Get the next or previous widget sibling of child
			// dir:
			//		if 1, get the next sibling
			//		if -1, get the previous sibling
			// tags:
			//      private
			var node = child.domNode,
				which = (dir>0 ? "nextSibling" : "previousSibling");
			do{
				node = node[which];
			}while(node && (node.nodeType != 1 || !dijit.byNode(node)));
			return node && dijit.byNode(node);	// dijit._Widget
		},

		getIndexOfChild: function(/*dijit._Widget*/ child){
			// summary:
			//		Gets the index of the child in this container or -1 if not found
			return dojo.indexOf(this.getChildren(), child);	// int
		},

		startup: function(){
			// summary:
			//		Called after all the widgets have been instantiated and their
			//		dom nodes have been inserted somewhere under dojo.doc.body.
			//
			//		Widgets should override this method to do any initialization
			//		dependent on other widgets existing, and then call
			//		this superclass method to finish things off.
			//
			//		startup() in subclasses shouldn't do anything
			//		size related because the size of the widget hasn't been set yet.

			if(this._started){ return; }

			// Startup all children of this widget
			dojo.forEach(this.getChildren(), function(child){ child.startup(); });

			this.inherited(arguments);
		}
	}
);

}

if(!dojo._hasResource["dijit._Contained"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._Contained"] = true;
dojo.provide("dijit._Contained");


dojo.declare("dijit._Contained",
		null,
		{
			// summary:
			//		Mixin for widgets that are children of a container widget
			//
			// example:
			// | 	// make a basic custom widget that knows about it's parents
			// |	dojo.declare("my.customClass",[dijit._Widget,dijit._Contained],{});

			getParent: function(){
				// summary:
				//		Returns the parent widget of this widget, assuming the parent
				//		specifies isContainer
				var parent = dijit.getEnclosingWidget(this.domNode.parentNode);
				return parent && parent.isContainer ? parent : null;
			},

			_getSibling: function(/*String*/ which){
				// summary:
				//      Returns next or previous sibling
				// which:
				//      Either "next" or "previous"
				// tags:
				//      private
				var node = this.domNode;
				do{
					node = node[which+"Sibling"];
				}while(node && node.nodeType != 1);
				return node && dijit.byNode(node);	// dijit._Widget
			},

			getPreviousSibling: function(){
				// summary:
				//		Returns null if this is the first child of the parent,
				//		otherwise returns the next element sibling to the "left".

				return this._getSibling("previous"); // dijit._Widget
			},

			getNextSibling: function(){
				// summary:
				//		Returns null if this is the last child of the parent,
				//		otherwise returns the next element sibling to the "right".

				return this._getSibling("next"); // dijit._Widget
			},

			getIndexInParent: function(){
				// summary:
				//		Returns the index of this widget within its container parent.
				//		It returns -1 if the parent does not exist, or if the parent
				//		is not a dijit._Container

				var p = this.getParent();
				if(!p || !p.getIndexOfChild){
					return -1; // int
				}
				return p.getIndexOfChild(this); // int
			}
		}
	);

}

if(!dojo._hasResource["dijit.layout._LayoutWidget"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.layout._LayoutWidget"] = true;
dojo.provide("dijit.layout._LayoutWidget");





dojo.declare("dijit.layout._LayoutWidget",
	[dijit._Widget, dijit._Container, dijit._Contained],
	{
		// summary:
		//		Base class for a _Container widget which is responsible for laying out its children.
		//		Widgets which mixin this code must define layout() to manage placement and sizing of the children.

		// baseClass: [protected extension] String
		//		This class name is applied to the widget's domNode
		//		and also may be used to generate names for sub nodes,
		//		for example dijitTabContainer-content.
		baseClass: "dijitLayoutContainer",

		// isLayoutContainer: [protected] Boolean
		//		Indicates that this widget is going to call resize() on its
		//		children widgets, setting their size, when they become visible.
		isLayoutContainer: true,

		buildRendering: function(){
			this.inherited(arguments);
			dojo.addClass(this.domNode, "dijitContainer");
		},

		startup: function(){
			// summary:
			//		Called after all the widgets have been instantiated and their
			//		dom nodes have been inserted somewhere under dojo.doc.body.
			//
			//		Widgets should override this method to do any initialization
			//		dependent on other widgets existing, and then call
			//		this superclass method to finish things off.
			//
			//		startup() in subclasses shouldn't do anything
			//		size related because the size of the widget hasn't been set yet.

			if(this._started){ return; }

			// Need to call inherited first - so that child widgets get started
			// up correctly
			this.inherited(arguments);

			// If I am a not being controlled by a parent layout widget...
			var parent = this.getParent && this.getParent()
			if(!(parent && parent.isLayoutContainer)){
				// Do recursive sizing and layout of all my descendants
				// (passing in no argument to resize means that it has to glean the size itself)
				this.resize();

				// Since my parent isn't a layout container, and my style *may be* width=height=100%
				// or something similar (either set directly or via a CSS class),
				// monitor when my size changes so that I can re-layout.
				// For browsers where I can't directly monitor when my size changes,
				// monitor when the viewport changes size, which *may* indicate a size change for me.
				this.connect(dojo.isIE ? this.domNode : dojo.global, 'onresize', function(){
					// Using function(){} closure to ensure no arguments to resize.
					this.resize();
				});
			}
		},

		resize: function(changeSize, resultSize){
			// summary:
			//		Call this to resize a widget, or after its size has changed.
			// description:
			//		Change size mode:
			//			When changeSize is specified, changes the marginBox of this widget
			//			and forces it to relayout its contents accordingly.
			//			changeSize may specify height, width, or both.
			//
			//			If resultSize is specified it indicates the size the widget will
			//			become after changeSize has been applied.
			//
			//		Notification mode:
			//			When changeSize is null, indicates that the caller has already changed
			//			the size of the widget, or perhaps it changed because the browser
			//			window was resized.  Tells widget to relayout its contents accordingly.
			//
			//			If resultSize is also specified it indicates the size the widget has
			//			become.
			//
			//		In either mode, this method also:
			//			1. Sets this._borderBox and this._contentBox to the new size of
			//				the widget.  Queries the current domNode size if necessary.
			//			2. Calls layout() to resize contents (and maybe adjust child widgets).
			//
			// changeSize: Object?
			//		Sets the widget to this margin-box size and position.
			//		May include any/all of the following properties:
			//	|	{w: int, h: int, l: int, t: int}
			//
			// resultSize: Object?
			//		The margin-box size of this widget after applying changeSize (if
			//		changeSize is specified).  If caller knows this size and
			//		passes it in, we don't need to query the browser to get the size.
			//	|	{w: int, h: int}

			var node = this.domNode;

			// set margin box size, unless it wasn't specified, in which case use current size
			if(changeSize){
				dojo.marginBox(node, changeSize);

				// set offset of the node
				if(changeSize.t){ node.style.top = changeSize.t + "px"; }
				if(changeSize.l){ node.style.left = changeSize.l + "px"; }
			}

			// If either height or width wasn't specified by the user, then query node for it.
			// But note that setting the margin box and then immediately querying dimensions may return
			// inaccurate results, so try not to depend on it.
			var mb = resultSize || {};
			dojo.mixin(mb, changeSize || {});	// changeSize overrides resultSize
			if( !("h" in mb) || !("w" in mb) ){
				mb = dojo.mixin(dojo.marginBox(node), mb);	// just use dojo.marginBox() to fill in missing values
			}

			// Compute and save the size of my border box and content box
			// (w/out calling dojo.contentBox() since that may fail if size was recently set)
			var cs = dojo.getComputedStyle(node);
			var me = dojo._getMarginExtents(node, cs);
			var be = dojo._getBorderExtents(node, cs);
			var bb = (this._borderBox = {
				w: mb.w - (me.w + be.w),
				h: mb.h - (me.h + be.h)
			});
			var pe = dojo._getPadExtents(node, cs);
			this._contentBox = {
				l: dojo._toPixelValue(node, cs.paddingLeft),
				t: dojo._toPixelValue(node, cs.paddingTop),
				w: bb.w - pe.w,
				h: bb.h - pe.h
			};

			// Callback for widget to adjust size of its children
			this.layout();
		},

		layout: function(){
			// summary:
			//		Widgets override this method to size and position their contents/children.
			//		When this is called this._contentBox is guaranteed to be set (see resize()).
			//
			//		This is called after startup(), and also when the widget's size has been
			//		changed.
			// tags:
			//		protected extension
		},

		_setupChild: function(/*dijit._Widget*/child){
			// summary:
			//		Common setup for initial children and children which are added after startup
			// tags:
			//		protected extension

			var cls = this.baseClass + "-child "
				+ (child.baseClass ? this.baseClass + "-" + child.baseClass : "");
			dojo.addClass(child.domNode, cls);
		},

		addChild: function(/*dijit._Widget*/ child, /*Integer?*/ insertIndex){
			// Overrides _Container.addChild() to call _setupChild()
			this.inherited(arguments);
			if(this._started){
				this._setupChild(child);
			}
		},

		removeChild: function(/*dijit._Widget*/ child){
			// Overrides _Container.removeChild() to remove class added by _setupChild()
			var cls = this.baseClass + "-child"
					+ (child.baseClass ?
						" " + this.baseClass + "-" + child.baseClass : "");
			dojo.removeClass(child.domNode, cls);
			
			this.inherited(arguments);
		}
	}
);

dijit.layout.marginBox2contentBox = function(/*DomNode*/ node, /*Object*/ mb){
	// summary:
	//		Given the margin-box size of a node, return its content box size.
	//		Functions like dojo.contentBox() but is more reliable since it doesn't have
	//		to wait for the browser to compute sizes.
	var cs = dojo.getComputedStyle(node);
	var me = dojo._getMarginExtents(node, cs);
	var pb = dojo._getPadBorderExtents(node, cs);
	return {
		l: dojo._toPixelValue(node, cs.paddingLeft),
		t: dojo._toPixelValue(node, cs.paddingTop),
		w: mb.w - (me.w + pb.w),
		h: mb.h - (me.h + pb.h)
	};
};

(function(){
	var capitalize = function(word){
		return word.substring(0,1).toUpperCase() + word.substring(1);
	};

	var size = function(widget, dim){
		// size the child
		var newSize = widget.resize ? widget.resize(dim) : dojo.marginBox(widget.domNode, dim);

		// record child's size
		if(newSize){
			// if the child returned it's new size then use that
			dojo.mixin(widget, newSize);
		}else{
			// otherwise, call marginBox(), but favor our own numbers when we have them.
			// the browser lies sometimes
			dojo.mixin(widget, dojo.marginBox(widget.domNode));
			dojo.mixin(widget, dim);
		}
	};

	dijit.layout.layoutChildren = function(/*DomNode*/ container, /*Object*/ dim, /*Widget[]*/ children,
			/*String?*/ changedRegionId, /*Number?*/ changedRegionSize){
		// summary
		//		Layout a bunch of child dom nodes within a parent dom node
		// container:
		//		parent node
		// dim:
		//		{l, t, w, h} object specifying dimensions of container into which to place children
		// children:
		//		an array of Widgets or at least objects containing:
		//			* domNode: pointer to DOM node to position
		//			* region or layoutAlign: position to place DOM node
		//			* resize(): (optional) method to set size of node
		//			* id: (optional) Id of widgets, referenced from resize object, below.
		// changedRegionId:
		//		If specified, the slider for the region with the specified id has been dragged, and thus
		//		the region's height or width should be adjusted according to changedRegionSize
		// changedRegionSize:
		//		See changedRegionId.

		// copy dim because we are going to modify it
		dim = dojo.mixin({}, dim);

		dojo.addClass(container, "dijitLayoutContainer");

		// Move "client" elements to the end of the array for layout.  a11y dictates that the author
		// needs to be able to put them in the document in tab-order, but this algorithm requires that
		// client be last.    TODO: move these lines to LayoutContainer?   Unneeded other places I think.
		children = dojo.filter(children, function(item){ return item.region != "center" && item.layoutAlign != "client"; })
			.concat(dojo.filter(children, function(item){ return item.region == "center" || item.layoutAlign == "client"; }));

		// set positions/sizes
		dojo.forEach(children, function(child){
			var elm = child.domNode,
				pos = (child.region || child.layoutAlign);

			// set elem to upper left corner of unused space; may move it later
			var elmStyle = elm.style;
			elmStyle.left = dim.l+"px";
			elmStyle.top = dim.t+"px";
			elmStyle.position = "absolute";

			dojo.addClass(elm, "dijitAlign" + capitalize(pos));

			// Size adjustments to make to this child widget
			var sizeSetting = {};

			// Check for optional size adjustment due to splitter drag (height adjustment for top/bottom align
			// panes and width adjustment for left/right align panes.
			if(changedRegionId && changedRegionId == child.id){
				sizeSetting[child.region == "top" || child.region == "bottom" ? "h" : "w"] = changedRegionSize;
			}

			// set size && adjust record of remaining space.
			// note that setting the width of a <div> may affect its height.
			if(pos == "top" || pos == "bottom"){
				sizeSetting.w = dim.w;
				size(child, sizeSetting);
				dim.h -= child.h;
				if(pos == "top"){
					dim.t += child.h;
				}else{
					elmStyle.top = dim.t + dim.h + "px";
				}
			}else if(pos == "left" || pos == "right"){
				sizeSetting.h = dim.h;
				size(child, sizeSetting);
				dim.w -= child.w;
				if(pos == "left"){
					dim.l += child.w;
				}else{
					elmStyle.left = dim.l + dim.w + "px";
				}
			}else if(pos == "client" || pos == "center"){
				size(child, dim);
			}
		});
	};

})();

}

if(!dojo._hasResource["dijit._CssStateMixin"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._CssStateMixin"] = true;
dojo.provide("dijit._CssStateMixin");


dojo.declare("dijit._CssStateMixin", [], {
	// summary:
	//		Mixin for widgets to set CSS classes on the widget DOM nodes depending on hover/mouse press/focus
	//		state changes, and also higher-level state changes such becoming disabled or selected.
	//
	// description:
	//		By mixing this class into your widget, and setting the this.baseClass attribute, it will automatically
	//		maintain CSS classes on the widget root node (this.domNode) depending on hover,
	//		active, focus, etc. state.   Ex: with a baseClass of dijitButton, it will apply the classes
	//		dijitButtonHovered and dijitButtonActive, as the user moves the mouse over the widget and clicks it.
	//
	//		It also sets CSS like dijitButtonDisabled based on widget semantic state.
	//
	//		By setting the cssStateNodes attribute, a widget can also track events on subnodes (like buttons
	//		within the widget).

	// cssStateNodes: [protected] Object
	//		List of sub-nodes within the widget that need CSS classes applied on mouse hover/press and focus
	//.
	//		Each entry in the hash is a an attachpoint names (like "upArrowButton") mapped to a CSS class names
	//		(like "dijitUpArrowButton"). Example:
	//	|		{
	//	|			"upArrowButton": "dijitUpArrowButton",
	//	|			"downArrowButton": "dijitDownArrowButton"
	//	|		}
	//		The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it
	//		is hovered, etc.
	cssStateNodes: {},

	// hovering: [readonly] Boolean
	//		True if cursor is over this widget
	hovering: false,
	
	// active: [readonly] Boolean
	//		True if mouse was pressed while over this widget, and hasn't been released yet
	active: false,

	_applyAttributes: function(){
		// This code would typically be in postCreate(), but putting in _applyAttributes() for
		// performance: so the class changes happen before DOM is inserted into the document.
		// Change back to postCreate() in 2.0.  See #11635.

		this.inherited(arguments);

		// Automatically monitor mouse events (essentially :hover and :active) on this.domNode
		dojo.forEach(["onmouseenter", "onmouseleave", "onmousedown"], function(e){
			this.connect(this.domNode, e, "_cssMouseEvent");
		}, this);
		
		// Monitoring changes to disabled, readonly, etc. state, and update CSS class of root node
		dojo.forEach(["disabled", "readOnly", "checked", "selected", "focused", "state", "hovering", "active"], function(attr){
			this.watch(attr, dojo.hitch(this, "_setStateClass"));
		}, this);

		// Events on sub nodes within the widget
		for(var ap in this.cssStateNodes){
			this._trackMouseState(this[ap], this.cssStateNodes[ap]);
		}
		// Set state initially; there's probably no hover/active/focus state but widget might be
		// disabled/readonly/checked/selected so we want to set CSS classes for those conditions.
		this._setStateClass();
	},

	_cssMouseEvent: function(/*Event*/ event){
		// summary:
		//	Sets hovering and active properties depending on mouse state,
		//	which triggers _setStateClass() to set appropriate CSS classes for this.domNode.

		if(!this.disabled){
			switch(event.type){
				case "mouseenter":
				case "mouseover":	// generated on non-IE browsers even though we connected to mouseenter
					this._set("hovering", true);
					this._set("active", this._mouseDown);
					break;

				case "mouseleave":
				case "mouseout":	// generated on non-IE browsers even though we connected to mouseleave
					this._set("hovering", false);
					this._set("active", false);
					break;

				case "mousedown" :
					this._set("active", true);
					this._mouseDown = true;
					// Set a global event to handle mouseup, so it fires properly
					// even if the cursor leaves this.domNode before the mouse up event.
					// Alternately could set active=false on mouseout.
					var mouseUpConnector = this.connect(dojo.body(), "onmouseup", function(){
						this._mouseDown = false;
						this._set("active", false);
						this.disconnect(mouseUpConnector);
					});
					break;
			}
		}
	},

	_setStateClass: function(){
		// summary:
		//		Update the visual state of the widget by setting the css classes on this.domNode
		//		(or this.stateNode if defined) by combining this.baseClass with
		//		various suffixes that represent the current widget state(s).
		//
		// description:
		//		In the case where a widget has multiple
		//		states, it sets the class based on all possible
		//	 	combinations.  For example, an invalid form widget that is being hovered
		//		will be "dijitInput dijitInputInvalid dijitInputHover dijitInputInvalidHover".
		//
		//		The widget may have one or more of the following states, determined
		//		by this.state, this.checked, this.valid, and this.selected:
		//			- Error - ValidationTextBox sets this.state to "Error" if the current input value is invalid
		//			- Incomplete - ValidationTextBox sets this.state to "Incomplete" if the current input value is not finished yet
		//			- Checked - ex: a checkmark or a ToggleButton in a checked state, will have this.checked==true
		//			- Selected - ex: currently selected tab will have this.selected==true
		//
		//		In addition, it may have one or more of the following states,
		//		based on this.disabled and flags set in _onMouse (this.active, this.hovering) and from focus manager (this.focused):
		//			- Disabled	- if the widget is disabled
		//			- Active		- if the mouse (or space/enter key?) is being pressed down
		//			- Focused		- if the widget has focus
		//			- Hover		- if the mouse is over the widget

		// Compute new set of classes
		var newStateClasses = this.baseClass.split(" ");

		function multiply(modifier){
			newStateClasses = newStateClasses.concat(dojo.map(newStateClasses, function(c){ return c+modifier; }), "dijit"+modifier);
		}

		if(!this.isLeftToRight()){
			// For RTL mode we need to set an addition class like dijitTextBoxRtl.
			multiply("Rtl");
		}

		if(this.checked){
			multiply("Checked");
		}
		if(this.state){
			multiply(this.state);
		}
		if(this.selected){
			multiply("Selected");
		}

		if(this.disabled){
			multiply("Disabled");
		}else if(this.readOnly){
			multiply("ReadOnly");
		}else{
			if(this.active){
				multiply("Active");
			}else if(this.hovering){
				multiply("Hover");
			}
		}

		if(this._focused){
			multiply("Focused");
		}

		// Remove old state classes and add new ones.
		// For performance concerns we only write into domNode.className once.
		var tn = this.stateNode || this.domNode,
			classHash = {};	// set of all classes (state and otherwise) for node

		dojo.forEach(tn.className.split(" "), function(c){ classHash[c] = true; });

		if("_stateClasses" in this){
			dojo.forEach(this._stateClasses, function(c){ delete classHash[c]; });
		}

		dojo.forEach(newStateClasses, function(c){ classHash[c] = true; });

		var newClasses = [];
		for(var c in classHash){
			newClasses.push(c);
		}
		tn.className = newClasses.join(" ");

		this._stateClasses = newStateClasses;
	},

	_trackMouseState: function(/*DomNode*/ node, /*String*/ clazz){
		// summary:
		//		Track mouse/focus events on specified node and set CSS class on that node to indicate
		//		current state.   Usually not called directly, but via cssStateNodes attribute.
		// description:
		//		Given class=foo, will set the following CSS class on the node
		//			- fooActive: if the user is currently pressing down the mouse button while over the node
		//			- fooHover: if the user is hovering the mouse over the node, but not pressing down a button
		//			- fooFocus: if the node is focused
		//
		//		Note that it won't set any classes if the widget is disabled.
		// node: DomNode
		//		Should be a sub-node of the widget, not the top node (this.domNode), since the top node
		//		is handled specially and automatically just by mixing in this class.
		// clazz: String
		//		CSS class name (ex: dijitSliderUpArrow).

		// Current state of node (initially false)
		// NB: setting specifically to false because dojo.toggleClass() needs true boolean as third arg
		var hovering=false, active=false, focused=false;

		var self = this,
			cn = dojo.hitch(this, "connect", node);

		function setClass(){
			var disabled = ("disabled" in self && self.disabled) || ("readonly" in self && self.readonly);
			dojo.toggleClass(node, clazz+"Hover", hovering && !active && !disabled);
			dojo.toggleClass(node, clazz+"Active", active && !disabled);
			dojo.toggleClass(node, clazz+"Focused", focused && !disabled);
		}

		// Mouse
		cn("onmouseenter", function(){
			hovering = true;
			setClass();
		});
		cn("onmouseleave", function(){
			hovering = false;
			active = false;
			setClass();
		});
		cn("onmousedown", function(){
			active = true;
			setClass();
		});
		cn("onmouseup", function(){
			active = false;
			setClass();
		});

		// Focus
		cn("onfocus", function(){
			focused = true;
			setClass();
		});
		cn("onblur", function(){
			focused = false;
			setClass();
		});

		// Just in case widget is enabled/disabled while it has focus/hover/active state.
		// Maybe this is overkill.
		this.watch("disabled", setClass);
		this.watch("readOnly", setClass);
	}
});

}

if(!dojo._hasResource["dijit.form._FormWidget"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form._FormWidget"] = true;
dojo.provide("dijit.form._FormWidget");






dojo.declare("dijit.form._FormWidget", [dijit._Widget, dijit._Templated, dijit._CssStateMixin],
	{
	// summary:
	//		Base class for widgets corresponding to native HTML elements such as <checkbox> or <button>,
	//		which can be children of a <form> node or a `dijit.form.Form` widget.
	//
	// description:
	//		Represents a single HTML element.
	//		All these widgets should have these attributes just like native HTML input elements.
	//		You can set them during widget construction or afterwards, via `dijit._Widget.attr`.
	//
	//		They also share some common methods.

	// name: [const] String
	//		Name used when submitting form; same as "name" attribute or plain HTML elements
	name: "",

	// alt: String
	//		Corresponds to the native HTML <input> element's attribute.
	alt: "",

	// value: String
	//		Corresponds to the native HTML <input> element's attribute.
	value: "",

	// type: String
	//		Corresponds to the native HTML <input> element's attribute.
	type: "text",

	// tabIndex: Integer
	//		Order fields are traversed when user hits the tab key
	tabIndex: "0",

	// disabled: Boolean
	//		Should this widget respond to user input?
	//		In markup, this is specified as "disabled='disabled'", or just "disabled".
	disabled: false,

	// intermediateChanges: Boolean
	//		Fires onChange for each value change or only on demand
	intermediateChanges: false,

	// scrollOnFocus: Boolean
	//		On focus, should this widget scroll into view?
	scrollOnFocus: true,

	// These mixins assume that the focus node is an INPUT, as many but not all _FormWidgets are.
	attributeMap: dojo.delegate(dijit._Widget.prototype.attributeMap, {
		value: "focusNode",
		id: "focusNode",
		tabIndex: "focusNode",
		alt: "focusNode",
		title: "focusNode"
	}),

	postMixInProperties: function(){
		// Setup name=foo string to be referenced from the template (but only if a name has been specified)
		// Unfortunately we can't use attributeMap to set the name due to IE limitations, see #8660
		// Regarding escaping, see heading "Attribute values" in
		// http://www.w3.org/TR/REC-html40/appendix/notes.html#h-B.3.2
		this.nameAttrSetting = this.name ? ('name="' + this.name.replace(/'/g, "&quot;") + '"') : '';
		this.inherited(arguments);
	},

	postCreate: function(){
		this.inherited(arguments);
		this.connect(this.domNode, "onmousedown", "_onMouseDown");
	},

	_setDisabledAttr: function(/*Boolean*/ value){
		this._set("disabled", value);
		dojo.attr(this.focusNode, 'disabled', value);
		if(this.valueNode){
			dojo.attr(this.valueNode, 'disabled', value);
		}
		dijit.setWaiState(this.focusNode, "disabled", value);

		if(value){
			// reset these, because after the domNode is disabled, we can no longer receive
			// mouse related events, see #4200
			this._set("hovering", false);
			this._set("active", false);

			// clear tab stop(s) on this widget's focusable node(s)  (ComboBox has two focusable nodes)
			var attachPointNames = "tabIndex" in this.attributeMap ? this.attributeMap.tabIndex : "focusNode";
			dojo.forEach(dojo.isArray(attachPointNames) ? attachPointNames : [attachPointNames], function(attachPointName){
				var node = this[attachPointName];
				// complex code because tabIndex=-1 on a <div> doesn't work on FF
				if(dojo.isWebKit || dijit.hasDefaultTabStop(node)){	// see #11064 about webkit bug
					node.setAttribute('tabIndex', "-1");
				}else{
					node.removeAttribute('tabIndex');
				}
			}, this);
		}else{
			if(this.tabIndex != ""){
				this.focusNode.setAttribute('tabIndex', this.tabIndex);
			}
		}
	},

	setDisabled: function(/*Boolean*/ disabled){
		// summary:
		//		Deprecated.  Use set('disabled', ...) instead.
		dojo.deprecated("setDisabled("+disabled+") is deprecated. Use set('disabled',"+disabled+") instead.", "", "2.0");
		this.set('disabled', disabled);
	},

	_onFocus: function(e){
		if(this.scrollOnFocus){
			dojo.window.scrollIntoView(this.domNode);
		}
		this.inherited(arguments);
	},

	isFocusable: function(){
		// summary:
		//		Tells if this widget is focusable or not.  Used internally by dijit.
		// tags:
		//		protected
		return !this.disabled && this.focusNode && (dojo.style(this.domNode, "display") != "none");
	},

	focus: function(){
		// summary:
		//		Put focus on this widget
		if(!this.disabled){
			dijit.focus(this.focusNode);
		}
	},

	compare: function(/*anything*/ val1, /*anything*/ val2){
		// summary:
		//		Compare 2 values (as returned by get('value') for this widget).
		// tags:
		//		protected
		if(typeof val1 == "number" && typeof val2 == "number"){
			return (isNaN(val1) && isNaN(val2)) ? 0 : val1 - val2;
		}else if(val1 > val2){
			return 1;
		}else if(val1 < val2){
			return -1;
		}else{
			return 0;
		}
	},

	onChange: function(newValue){
		// summary:
		//		Callback when this widget's value is changed.
		// tags:
		//		callback
	},

	// _onChangeActive: [private] Boolean
	//		Indicates that changes to the value should call onChange() callback.
	//		This is false during widget initialization, to avoid calling onChange()
	//		when the initial value is set.
	_onChangeActive: false,

	_handleOnChange: function(/*anything*/ newValue, /*Boolean?*/ priorityChange){
		// summary:
		//		Called when the value of the widget is set.  Calls onChange() if appropriate
		// newValue:
		//		the new value
		// priorityChange:
		//		For a slider, for example, dragging the slider is priorityChange==false,
		//		but on mouse up, it's priorityChange==true.  If intermediateChanges==false,
		//		onChange is only called form priorityChange=true events.
		// tags:
		//		private
		if(this._lastValueReported == undefined && (priorityChange === null || !this._onChangeActive)){
			// this block executes not for a change, but during initialization,
			// and is used to store away the original value (or for ToggleButton, the original checked state)
			this._resetValue = this._lastValueReported = newValue;
		}
		this._pendingOnChange = this._pendingOnChange
			|| (typeof newValue != typeof this._lastValueReported)
			|| (this.compare(newValue, this._lastValueReported) != 0);
		if((this.intermediateChanges || priorityChange || priorityChange === undefined) && this._pendingOnChange){
			this._lastValueReported = newValue;
			this._pendingOnChange = false;
			if(this._onChangeActive){
				if(this._onChangeHandle){
					clearTimeout(this._onChangeHandle);
				}
				// setTimout allows hidden value processing to run and
				// also the onChange handler can safely adjust focus, etc
				this._onChangeHandle = setTimeout(dojo.hitch(this,
					function(){
						this._onChangeHandle = null;
						this.onChange(newValue);
					}), 0); // try to collapse multiple onChange's fired faster than can be processed
			}
		}
	},

	create: function(){
		// Overrides _Widget.create()
		this.inherited(arguments);
		this._onChangeActive = true;
	},

	destroy: function(){
		if(this._onChangeHandle){ // destroy called before last onChange has fired
			clearTimeout(this._onChangeHandle);
			this.onChange(this._lastValueReported);
		}
		this.inherited(arguments);
	},

	setValue: function(/*String*/ value){
		// summary:
		//		Deprecated.  Use set('value', ...) instead.
		dojo.deprecated("dijit.form._FormWidget:setValue("+value+") is deprecated.  Use set('value',"+value+") instead.", "", "2.0");
		this.set('value', value);
	},

	getValue: function(){
		// summary:
		//		Deprecated.  Use get('value') instead.
		dojo.deprecated(this.declaredClass+"::getValue() is deprecated. Use get('value') instead.", "", "2.0");
		return this.get('value');
	},
	
	_onMouseDown: function(e){
		// If user clicks on the button, even if the mouse is released outside of it,
		// this button should get focus (to mimics native browser buttons).
		// This is also needed on chrome because otherwise buttons won't get focus at all,
		// which leads to bizarre focus restore on Dialog close etc.
		if(!e.ctrlKey && dojo.mouseButtons.isLeft(e) && this.isFocusable()){ // !e.ctrlKey to ignore right-click on mac
			// Set a global event to handle mouseup, so it fires properly
			// even if the cursor leaves this.domNode before the mouse up event.
			var mouseUpConnector = this.connect(dojo.body(), "onmouseup", function(){
				if (this.isFocusable()) {
					this.focus();
				}
				this.disconnect(mouseUpConnector);
			});
		}
	}
});

dojo.declare("dijit.form._FormValueWidget", dijit.form._FormWidget,
{
	// summary:
	//		Base class for widgets corresponding to native HTML elements such as <input> or <select> that have user changeable values.
	// description:
	//		Each _FormValueWidget represents a single input value, and has a (possibly hidden) <input> element,
	//		to which it serializes it's input value, so that form submission (either normal submission or via FormBind?)
	//		works as expected.

	// Don't attempt to mixin the 'type', 'name' attributes here programatically -- they must be declared
	// directly in the template as read by the parser in order to function. IE is known to specifically
	// require the 'name' attribute at element creation time.  See #8484, #8660.
	// TODO: unclear what that {value: ""} is for; FormWidget.attributeMap copies value to focusNode,
	// so maybe {value: ""} is so the value *doesn't* get copied to focusNode?
	// Seems like we really want value removed from attributeMap altogether
	// (although there's no easy way to do that now)

	// readOnly: Boolean
	//		Should this widget respond to user input?
	//		In markup, this is specified as "readOnly".
	//		Similar to disabled except readOnly form values are submitted.
	readOnly: false,

	attributeMap: dojo.delegate(dijit.form._FormWidget.prototype.attributeMap, {
		value: "",
		readOnly: "focusNode"
	}),

	_setReadOnlyAttr: function(/*Boolean*/ value){
		dojo.attr(this.focusNode, 'readOnly', value);
		dijit.setWaiState(this.focusNode, "readonly", value);
		this._set("readOnly", value);
	},

	postCreate: function(){
		this.inherited(arguments);

		if(dojo.isIE < 9 || (dojo.isIE && dojo.isQuirks)){ // IE won't stop the event with keypress
			this.connect(this.focusNode || this.domNode, "onkeydown", this._onKeyDown);
		}
		// Update our reset value if it hasn't yet been set (because this.set()
		// is only called when there *is* a value)
		if(this._resetValue === undefined){
			this._lastValueReported = this._resetValue = this.value;
		}
	},

	_setValueAttr: function(/*anything*/ newValue, /*Boolean?*/ priorityChange){
		// summary:
		//		Hook so set('value', value) works.
		// description:
		//		Sets the value of the widget.
		//		If the value has changed, then fire onChange event, unless priorityChange
		//		is specified as null (or false?)
		this._handleOnChange(newValue, priorityChange);
	},

	_handleOnChange: function(/*anything*/ newValue, /*Boolean?*/ priorityChange){
		// summary:
		//		Called when the value of the widget has changed.  Saves the new value in this.value,
		//		and calls onChange() if appropriate.   See _FormWidget._handleOnChange() for details.
		this._set("value", newValue);
		this.inherited(arguments);
	},

	undo: function(){
		// summary:
		//		Restore the value to the last value passed to onChange
		this._setValueAttr(this._lastValueReported, false);
	},

	reset: function(){
		// summary:
		//		Reset the widget's value to what it was at initialization time
		this._hasBeenBlurred = false;
		this._setValueAttr(this._resetValue, true);
	},

	_onKeyDown: function(e){
		if(e.keyCode == dojo.keys.ESCAPE && !(e.ctrlKey || e.altKey || e.metaKey)){
			var te;
			if(dojo.isIE){
				e.preventDefault(); // default behavior needs to be stopped here since keypress is too late
				te = document.createEventObject();
				te.keyCode = dojo.keys.ESCAPE;
				te.shiftKey = e.shiftKey;
				e.srcElement.fireEvent('onkeypress', te);
			}
		}
	},

	_layoutHackIE7: function(){
		// summary:
		//		Work around table sizing bugs on IE7 by forcing redraw

		if(dojo.isIE == 7){ // fix IE7 layout bug when the widget is scrolled out of sight
			var domNode = this.domNode;
			var parent = domNode.parentNode;
			var pingNode = domNode.firstChild || domNode; // target node most unlikely to have a custom filter
			var origFilter = pingNode.style.filter; // save custom filter, most likely nothing
			var _this = this;
			while(parent && parent.clientHeight == 0){ // search for parents that haven't rendered yet
				(function ping(){
					var disconnectHandle = _this.connect(parent, "onscroll",
						function(e){
							_this.disconnect(disconnectHandle); // only call once
							pingNode.style.filter = (new Date()).getMilliseconds(); // set to anything that's unique
							setTimeout(function(){ pingNode.style.filter = origFilter }, 0); // restore custom filter, if any
						}
					);
				})();
				parent = parent.parentNode;
			}
		}
	}
});

}

if(!dojo._hasResource["dijit.dijit"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.dijit"] = true;
dojo.provide("dijit.dijit");









/*=====
dijit.dijit = {
	// summary:
	//		A roll-up for common dijit methods
	// description:
	//	A rollup file for the build system including the core and common
	//	dijit files.
	//
	// example:
	// | <script type="text/javascript" src="js/dojo/dijit/dijit.js"></script>
	//
};
=====*/

// All the stuff in _base (these are the function that are guaranteed available without an explicit dojo.require)

// And some other stuff that we tend to pull in all the time anyway

}

if(!dojo._hasResource["dojo.fx.Toggler"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.fx.Toggler"] = true;
dojo.provide("dojo.fx.Toggler");


dojo.declare("dojo.fx.Toggler", null, {
	// summary:
	//		A simple `dojo.Animation` toggler API.
	//
	// description:
	//		class constructor for an animation toggler. It accepts a packed
	//		set of arguments about what type of animation to use in each
	//		direction, duration, etc. All available members are mixed into
	//		these animations from the constructor (for example, `node`,
	//		`showDuration`, `hideDuration`).
	//
	// example:
	//	|	var t = new dojo.fx.Toggler({
	//	|		node: "nodeId",
	//	|		showDuration: 500,
	//	|		// hideDuration will default to "200"
	//	|		showFunc: dojo.fx.wipeIn,
	//	|		// hideFunc will default to "fadeOut"
	//	|	});
	//	|	t.show(100); // delay showing for 100ms
	//	|	// ...time passes...
	//	|	t.hide();

	// node: DomNode
	//		the node to target for the showing and hiding animations
	node: null,

	// showFunc: Function
	//		The function that returns the `dojo.Animation` to show the node
	showFunc: dojo.fadeIn,

	// hideFunc: Function
	//		The function that returns the `dojo.Animation` to hide the node
	hideFunc: dojo.fadeOut,

	// showDuration:
	//		Time in milliseconds to run the show Animation
	showDuration: 200,

	// hideDuration:
	//		Time in milliseconds to run the hide Animation
	hideDuration: 200,

	// FIXME: need a policy for where the toggler should "be" the next
	// time show/hide are called if we're stopped somewhere in the
	// middle.
	// FIXME: also would be nice to specify individual showArgs/hideArgs mixed into
	// each animation individually.
	// FIXME: also would be nice to have events from the animations exposed/bridged

	/*=====
	_showArgs: null,
	_showAnim: null,

	_hideArgs: null,
	_hideAnim: null,

	_isShowing: false,
	_isHiding: false,
	=====*/

	constructor: function(args){
		var _t = this;

		dojo.mixin(_t, args);
		_t.node = args.node;
		_t._showArgs = dojo.mixin({}, args);
		_t._showArgs.node = _t.node;
		_t._showArgs.duration = _t.showDuration;
		_t.showAnim = _t.showFunc(_t._showArgs);

		_t._hideArgs = dojo.mixin({}, args);
		_t._hideArgs.node = _t.node;
		_t._hideArgs.duration = _t.hideDuration;
		_t.hideAnim = _t.hideFunc(_t._hideArgs);

		dojo.connect(_t.showAnim, "beforeBegin", dojo.hitch(_t.hideAnim, "stop", true));
		dojo.connect(_t.hideAnim, "beforeBegin", dojo.hitch(_t.showAnim, "stop", true));
	},

	show: function(delay){
		// summary: Toggle the node to showing
		// delay: Integer?
		//		Ammount of time to stall playing the show animation
		return this.showAnim.play(delay || 0);
	},

	hide: function(delay){
		// summary: Toggle the node to hidden
		// delay: Integer?
		//		Ammount of time to stall playing the hide animation
		return this.hideAnim.play(delay || 0);
	}
});

}

if(!dojo._hasResource["dojo.fx"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.fx"] = true;
dojo.provide("dojo.fx");



/*=====
dojo.fx = {
	// summary: Effects library on top of Base animations
};
=====*/
(function(){
	
	var d = dojo,
		_baseObj = {
			_fire: function(evt, args){
				if(this[evt]){
					this[evt].apply(this, args||[]);
				}
				return this;
			}
		};

	var _chain = function(animations){
		this._index = -1;
		this._animations = animations||[];
		this._current = this._onAnimateCtx = this._onEndCtx = null;

		this.duration = 0;
		d.forEach(this._animations, function(a){
			this.duration += a.duration;
			if(a.delay){ this.duration += a.delay; }
		}, this);
	};
	d.extend(_chain, {
		_onAnimate: function(){
			this._fire("onAnimate", arguments);
		},
		_onEnd: function(){
			d.disconnect(this._onAnimateCtx);
			d.disconnect(this._onEndCtx);
			this._onAnimateCtx = this._onEndCtx = null;
			if(this._index + 1 == this._animations.length){
				this._fire("onEnd");
			}else{
				// switch animations
				this._current = this._animations[++this._index];
				this._onAnimateCtx = d.connect(this._current, "onAnimate", this, "_onAnimate");
				this._onEndCtx = d.connect(this._current, "onEnd", this, "_onEnd");
				this._current.play(0, true);
			}
		},
		play: function(/*int?*/ delay, /*Boolean?*/ gotoStart){
			if(!this._current){ this._current = this._animations[this._index = 0]; }
			if(!gotoStart && this._current.status() == "playing"){ return this; }
			var beforeBegin = d.connect(this._current, "beforeBegin", this, function(){
					this._fire("beforeBegin");
				}),
				onBegin = d.connect(this._current, "onBegin", this, function(arg){
					this._fire("onBegin", arguments);
				}),
				onPlay = d.connect(this._current, "onPlay", this, function(arg){
					this._fire("onPlay", arguments);
					d.disconnect(beforeBegin);
					d.disconnect(onBegin);
					d.disconnect(onPlay);
				});
			if(this._onAnimateCtx){
				d.disconnect(this._onAnimateCtx);
			}
			this._onAnimateCtx = d.connect(this._current, "onAnimate", this, "_onAnimate");
			if(this._onEndCtx){
				d.disconnect(this._onEndCtx);
			}
			this._onEndCtx = d.connect(this._current, "onEnd", this, "_onEnd");
			this._current.play.apply(this._current, arguments);
			return this;
		},
		pause: function(){
			if(this._current){
				var e = d.connect(this._current, "onPause", this, function(arg){
						this._fire("onPause", arguments);
						d.disconnect(e);
					});
				this._current.pause();
			}
			return this;
		},
		gotoPercent: function(/*Decimal*/percent, /*Boolean?*/ andPlay){
			this.pause();
			var offset = this.duration * percent;
			this._current = null;
			d.some(this._animations, function(a){
				if(a.duration <= offset){
					this._current = a;
					return true;
				}
				offset -= a.duration;
				return false;
			});
			if(this._current){
				this._current.gotoPercent(offset / this._current.duration, andPlay);
			}
			return this;
		},
		stop: function(/*boolean?*/ gotoEnd){
			if(this._current){
				if(gotoEnd){
					for(; this._index + 1 < this._animations.length; ++this._index){
						this._animations[this._index].stop(true);
					}
					this._current = this._animations[this._index];
				}
				var e = d.connect(this._current, "onStop", this, function(arg){
						this._fire("onStop", arguments);
						d.disconnect(e);
					});
				this._current.stop();
			}
			return this;
		},
		status: function(){
			return this._current ? this._current.status() : "stopped";
		},
		destroy: function(){
			if(this._onAnimateCtx){ d.disconnect(this._onAnimateCtx); }
			if(this._onEndCtx){ d.disconnect(this._onEndCtx); }
		}
	});
	d.extend(_chain, _baseObj);

	dojo.fx.chain = function(/*dojo.Animation[]*/ animations){
		// summary:
		//		Chain a list of `dojo.Animation`s to run in sequence
		//
		// description:
		//		Return a `dojo.Animation` which will play all passed
		//		`dojo.Animation` instances in sequence, firing its own
		//		synthesized events simulating a single animation. (eg:
		//		onEnd of this animation means the end of the chain,
		//		not the individual animations within)
		//
		// example:
		//	Once `node` is faded out, fade in `otherNode`
		//	|	dojo.fx.chain([
		//	|		dojo.fadeIn({ node:node }),
		//	|		dojo.fadeOut({ node:otherNode })
		//	|	]).play();
		//
		return new _chain(animations) // dojo.Animation
	};

	var _combine = function(animations){
		this._animations = animations||[];
		this._connects = [];
		this._finished = 0;

		this.duration = 0;
		d.forEach(animations, function(a){
			var duration = a.duration;
			if(a.delay){ duration += a.delay; }
			if(this.duration < duration){ this.duration = duration; }
			this._connects.push(d.connect(a, "onEnd", this, "_onEnd"));
		}, this);
		
		this._pseudoAnimation = new d.Animation({curve: [0, 1], duration: this.duration});
		var self = this;
		d.forEach(["beforeBegin", "onBegin", "onPlay", "onAnimate", "onPause", "onStop", "onEnd"],
			function(evt){
				self._connects.push(d.connect(self._pseudoAnimation, evt,
					function(){ self._fire(evt, arguments); }
				));
			}
		);
	};
	d.extend(_combine, {
		_doAction: function(action, args){
			d.forEach(this._animations, function(a){
				a[action].apply(a, args);
			});
			return this;
		},
		_onEnd: function(){
			if(++this._finished > this._animations.length){
				this._fire("onEnd");
			}
		},
		_call: function(action, args){
			var t = this._pseudoAnimation;
			t[action].apply(t, args);
		},
		play: function(/*int?*/ delay, /*Boolean?*/ gotoStart){
			this._finished = 0;
			this._doAction("play", arguments);
			this._call("play", arguments);
			return this;
		},
		pause: function(){
			this._doAction("pause", arguments);
			this._call("pause", arguments);
			return this;
		},
		gotoPercent: function(/*Decimal*/percent, /*Boolean?*/ andPlay){
			var ms = this.duration * percent;
			d.forEach(this._animations, function(a){
				a.gotoPercent(a.duration < ms ? 1 : (ms / a.duration), andPlay);
			});
			this._call("gotoPercent", arguments);
			return this;
		},
		stop: function(/*boolean?*/ gotoEnd){
			this._doAction("stop", arguments);
			this._call("stop", arguments);
			return this;
		},
		status: function(){
			return this._pseudoAnimation.status();
		},
		destroy: function(){
			d.forEach(this._connects, dojo.disconnect);
		}
	});
	d.extend(_combine, _baseObj);

	dojo.fx.combine = function(/*dojo.Animation[]*/ animations){
		// summary:
		//		Combine a list of `dojo.Animation`s to run in parallel
		//
		// description:
		//		Combine an array of `dojo.Animation`s to run in parallel,
		//		providing a new `dojo.Animation` instance encompasing each
		//		animation, firing standard animation events.
		//
		// example:
		//	Fade out `node` while fading in `otherNode` simultaneously
		//	|	dojo.fx.combine([
		//	|		dojo.fadeIn({ node:node }),
		//	|		dojo.fadeOut({ node:otherNode })
		//	|	]).play();
		//
		// example:
		//	When the longest animation ends, execute a function:
		//	|	var anim = dojo.fx.combine([
		//	|		dojo.fadeIn({ node: n, duration:700 }),
		//	|		dojo.fadeOut({ node: otherNode, duration: 300 })
		//	|	]);
		//	|	dojo.connect(anim, "onEnd", function(){
		//	|		// overall animation is done.
		//	|	});
		//	|	anim.play(); // play the animation
		//
		return new _combine(animations); // dojo.Animation
	};

	dojo.fx.wipeIn = function(/*Object*/ args){
		// summary:
		//		Expand a node to it's natural height.
		//
		// description:
		//		Returns an animation that will expand the
		//		node defined in 'args' object from it's current height to
		//		it's natural height (with no scrollbar).
		//		Node must have no margin/border/padding.
		//
		// args: Object
		//		A hash-map of standard `dojo.Animation` constructor properties
		//		(such as easing: node: duration: and so on)
		//
		// example:
		//	|	dojo.fx.wipeIn({
		//	|		node:"someId"
		//	|	}).play()
		var node = args.node = d.byId(args.node), s = node.style, o;

		var anim = d.animateProperty(d.mixin({
			properties: {
				height: {
					// wrapped in functions so we wait till the last second to query (in case value has changed)
					start: function(){
						// start at current [computed] height, but use 1px rather than 0
						// because 0 causes IE to display the whole panel
						o = s.overflow;
						s.overflow = "hidden";
						if(s.visibility == "hidden" || s.display == "none"){
							s.height = "1px";
							s.display = "";
							s.visibility = "";
							return 1;
						}else{
							var height = d.style(node, "height");
							return Math.max(height, 1);
						}
					},
					end: function(){
						return node.scrollHeight;
					}
				}
			}
		}, args));

		d.connect(anim, "onEnd", function(){
			s.height = "auto";
			s.overflow = o;
		});

		return anim; // dojo.Animation
	};

	dojo.fx.wipeOut = function(/*Object*/ args){
		// summary:
		//		Shrink a node to nothing and hide it.
		//
		// description:
		//		Returns an animation that will shrink node defined in "args"
		//		from it's current height to 1px, and then hide it.
		//
		// args: Object
		//		A hash-map of standard `dojo.Animation` constructor properties
		//		(such as easing: node: duration: and so on)
		//
		// example:
		//	|	dojo.fx.wipeOut({ node:"someId" }).play()
		
		var node = args.node = d.byId(args.node), s = node.style, o;
		
		var anim = d.animateProperty(d.mixin({
			properties: {
				height: {
					end: 1 // 0 causes IE to display the whole panel
				}
			}
		}, args));

		d.connect(anim, "beforeBegin", function(){
			o = s.overflow;
			s.overflow = "hidden";
			s.display = "";
		});
		d.connect(anim, "onEnd", function(){
			s.overflow = o;
			s.height = "auto";
			s.display = "none";
		});

		return anim; // dojo.Animation
	};

	dojo.fx.slideTo = function(/*Object*/ args){
		// summary:
		//		Slide a node to a new top/left position
		//
		// description:
		//		Returns an animation that will slide "node"
		//		defined in args Object from its current position to
		//		the position defined by (args.left, args.top).
		//
		// args: Object
		//		A hash-map of standard `dojo.Animation` constructor properties
		//		(such as easing: node: duration: and so on). Special args members
		//		are `top` and `left`, which indicate the new position to slide to.
		//
		// example:
		//	|	dojo.fx.slideTo({ node: node, left:"40", top:"50", units:"px" }).play()

		var node = args.node = d.byId(args.node),
			top = null, left = null;

		var init = (function(n){
			return function(){
				var cs = d.getComputedStyle(n);
				var pos = cs.position;
				top = (pos == 'absolute' ? n.offsetTop : parseInt(cs.top) || 0);
				left = (pos == 'absolute' ? n.offsetLeft : parseInt(cs.left) || 0);
				if(pos != 'absolute' && pos != 'relative'){
					var ret = d.position(n, true);
					top = ret.y;
					left = ret.x;
					n.style.position="absolute";
					n.style.top=top+"px";
					n.style.left=left+"px";
				}
			};
		})(node);
		init();

		var anim = d.animateProperty(d.mixin({
			properties: {
				top: args.top || 0,
				left: args.left || 0
			}
		}, args));
		d.connect(anim, "beforeBegin", anim, init);

		return anim; // dojo.Animation
	};

})();

}

if(!dojo._hasResource["dojo.fx.easing"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.fx.easing"] = true;
dojo.provide("dojo.fx.easing");

dojo.getObject("fx.easing", true, dojo);

dojo.fx.easing = {
	// summary:
	//		Collection of easing functions to use beyond the default
	//		`dojo._defaultEasing` function.
	//
	// description:
	//
	//		Easing functions are used to manipulate the iteration through
	//		an `dojo.Animation`s _Line. _Line being the properties of an Animation,
	//		and the easing function progresses through that Line determing
	//		how quickly (or slowly) it should go. Or more accurately: modify
	//		the value of the _Line based on the percentage of animation completed.
	//
	//		All functions follow a simple naming convention of "ease type" + "when".
	//		If the name of the function ends in Out, the easing described appears
	//		towards the end of the animation. "In" means during the beginning,
	//		and InOut means both ranges of the Animation will applied, both
	//		beginning and end.
	//
	//		One does not call the easing function directly, it must be passed to
	//		the `easing` property of an animation.
	//
	//	example:
	//	|	
	//	|	var anim = dojo.fadeOut({
	//	|		node: 'node',
	//	|		duration: 2000,
	//	|		//	note there is no ()
	//	|		easing: dojo.fx.easing.quadIn
	//	|	}).play();
	//
	
	linear: function(/* Decimal? */n){
		// summary: A linear easing function
		return n;
	},

	quadIn: function(/* Decimal? */n){
		return Math.pow(n, 2);
	},

	quadOut: function(/* Decimal? */n){
		return n * (n - 2) * -1;
	},

	quadInOut: function(/* Decimal? */n){
		n = n * 2;
		if(n < 1){ return Math.pow(n, 2) / 2; }
		return -1 * ((--n) * (n - 2) - 1) / 2;
	},

	cubicIn: function(/* Decimal? */n){
		return Math.pow(n, 3);
	},

	cubicOut: function(/* Decimal? */n){
		return Math.pow(n - 1, 3) + 1;
	},

	cubicInOut: function(/* Decimal? */n){
		n = n * 2;
		if(n < 1){ return Math.pow(n, 3) / 2; }
		n -= 2;
		return (Math.pow(n, 3) + 2) / 2;
	},

	quartIn: function(/* Decimal? */n){
		return Math.pow(n, 4);
	},

	quartOut: function(/* Decimal? */n){
		return -1 * (Math.pow(n - 1, 4) - 1);
	},

	quartInOut: function(/* Decimal? */n){
		n = n * 2;
		if(n < 1){ return Math.pow(n, 4) / 2; }
		n -= 2;
		return -1 / 2 * (Math.pow(n, 4) - 2);
	},

	quintIn: function(/* Decimal? */n){
		return Math.pow(n, 5);
	},

	quintOut: function(/* Decimal? */n){
		return Math.pow(n - 1, 5) + 1;
	},

	quintInOut: function(/* Decimal? */n){
		n = n * 2;
		if(n < 1){ return Math.pow(n, 5) / 2; };
		n -= 2;
		return (Math.pow(n, 5) + 2) / 2;
	},

	sineIn: function(/* Decimal? */n){
		return -1 * Math.cos(n * (Math.PI / 2)) + 1;
	},

	sineOut: function(/* Decimal? */n){
		return Math.sin(n * (Math.PI / 2));
	},

	sineInOut: function(/* Decimal? */n){
		return -1 * (Math.cos(Math.PI * n) - 1) / 2;
	},

	expoIn: function(/* Decimal? */n){
		return (n == 0) ? 0 : Math.pow(2, 10 * (n - 1));
	},

	expoOut: function(/* Decimal? */n){
		return (n == 1) ? 1 : (-1 * Math.pow(2, -10 * n) + 1);
	},

	expoInOut: function(/* Decimal? */n){
		if(n == 0){ return 0; }
		if(n == 1){ return 1; }
		n = n * 2;
		if(n < 1){ return Math.pow(2, 10 * (n - 1)) / 2; }
		--n;
		return (-1 * Math.pow(2, -10 * n) + 2) / 2;
	},

	circIn: function(/* Decimal? */n){
		return -1 * (Math.sqrt(1 - Math.pow(n, 2)) - 1);
	},

	circOut: function(/* Decimal? */n){
		n = n - 1;
		return Math.sqrt(1 - Math.pow(n, 2));
	},

	circInOut: function(/* Decimal? */n){
		n = n * 2;
		if(n < 1){ return -1 / 2 * (Math.sqrt(1 - Math.pow(n, 2)) - 1); }
		n -= 2;
		return 1 / 2 * (Math.sqrt(1 - Math.pow(n, 2)) + 1);
	},

	backIn: function(/* Decimal? */n){
		// summary:
		//		An easing function that starts away from the target,
		//		and quickly accelerates towards the end value.
		//
		//		Use caution when the easing will cause values to become
		//		negative as some properties cannot be set to negative values.
		var s = 1.70158;
		return Math.pow(n, 2) * ((s + 1) * n - s);
	},

	backOut: function(/* Decimal? */n){
		// summary:
		//		An easing function that pops past the range briefly, and slowly comes back.
		//
		// description:
		//		An easing function that pops past the range briefly, and slowly comes back.
		//
		//		Use caution when the easing will cause values to become negative as some
		//		properties cannot be set to negative values.
		
		n = n - 1;
		var s = 1.70158;
		return Math.pow(n, 2) * ((s + 1) * n + s) + 1;
	},

	backInOut: function(/* Decimal? */n){
		// summary:
		//		An easing function combining the effects of `backIn` and `backOut`
		//
		// description:
		//		An easing function combining the effects of `backIn` and `backOut`.
		//		Use caution when the easing will cause values to become negative
		//		as some properties cannot be set to negative values.
		var s = 1.70158 * 1.525;
		n = n * 2;
		if(n < 1){ return (Math.pow(n, 2) * ((s + 1) * n - s)) / 2; }
		n-=2;
		return (Math.pow(n, 2) * ((s + 1) * n + s) + 2) / 2;
	},

	elasticIn: function(/* Decimal? */n){
		// summary:
		//		An easing function the elastically snaps from the start value
		//
		// description:
		//		An easing function the elastically snaps from the start value
		//
		//		Use caution when the elasticity will cause values to become negative
		//		as some properties cannot be set to negative values.
		if(n == 0 || n == 1){ return n; }
		var p = .3;
		var s = p / 4;
		n = n - 1;
		return -1 * Math.pow(2, 10 * n) * Math.sin((n - s) * (2 * Math.PI) / p);
	},

	elasticOut: function(/* Decimal? */n){
		// summary:
		//		An easing function that elasticly snaps around the target value,
		//		near the end of the Animation
		//
		// description:
		//		An easing function that elasticly snaps around the target value,
		//		near the end of the Animation
		//
		//		Use caution when the elasticity will cause values to become
		//		negative as some properties cannot be set to negative values.
		if(n==0 || n == 1){ return n; }
		var p = .3;
		var s = p / 4;
		return Math.pow(2, -10 * n) * Math.sin((n - s) * (2 * Math.PI) / p) + 1;
	},

	elasticInOut: function(/* Decimal? */n){
		// summary:
		//		An easing function that elasticly snaps around the value, near
		//		the beginning and end of the Animation.
		//
		// description:
		//		An easing function that elasticly snaps around the value, near
		//		the beginning and end of the Animation.
		//
		//		Use caution when the elasticity will cause values to become
		//		negative as some properties cannot be set to negative values.
		if(n == 0) return 0;
		n = n * 2;
		if(n == 2) return 1;
		var p = .3 * 1.5;
		var s = p / 4;
		if(n < 1){
			n -= 1;
			return -.5 * (Math.pow(2, 10 * n) * Math.sin((n - s) * (2 * Math.PI) / p));
		}
		n -= 1;
		return .5 * (Math.pow(2, -10 * n) * Math.sin((n - s) * (2 * Math.PI) / p)) + 1;
	},

	bounceIn: function(/* Decimal? */n){
		// summary:
		//		An easing function that 'bounces' near the beginning of an Animation
		return (1 - dojo.fx.easing.bounceOut(1 - n)); // Decimal
	},

	bounceOut: function(/* Decimal? */n){
		// summary:
		//		An easing function that 'bounces' near the end of an Animation
		var s = 7.5625;
		var p = 2.75;
		var l;
		if(n < (1 / p)){
			l = s * Math.pow(n, 2);
		}else if(n < (2 / p)){
			n -= (1.5 / p);
			l = s * Math.pow(n, 2) + .75;
		}else if(n < (2.5 / p)){
			n -= (2.25 / p);
			l = s * Math.pow(n, 2) + .9375;
		}else{
			n -= (2.625 / p);
			l = s * Math.pow(n, 2) + .984375;
		}
		return l;
	},

	bounceInOut: function(/* Decimal? */n){
		// summary:
		//		An easing function that 'bounces' at the beginning and end of the Animation
		if(n < 0.5){ return dojo.fx.easing.bounceIn(n * 2) / 2; }
		return (dojo.fx.easing.bounceOut(n * 2 - 1) / 2) + 0.5; // Decimal
	}
};

}

if(!dojo._hasResource['epages.browser']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['epages.browser'] = true;
/*
	Copyright (c) 2006-2007, ePages GmbH
	All Rights Reserved.
*/
dojo.provide('epages.browser');


epages.Browser = {};

(function() {
	var browser = epages.Browser;
	browser.engine = "Unknown";
	browser.version = "Unknown";
	if(window.opera) {
		browser.engine = "Presto";
		browser.version = window.opera.version();
	}else {
		var versionString = (navigator.appVersion + navigator.userAgent).toLowerCase();
		if(versionString.match("iemobile") || versionString.match("windows ce")) {
			// ie on smartphone
			browser.engine = "MSIE";
			browser.version = 6;
		}else if(versionString.match("msie")) {
			browser.engine = "MSIE";
			browser.version = (window.XDomainRequest && dojo.isIE == 7) ? 8 : dojo.isIE;
			// ...and finally some testing for IE6 with faked IE7 appVersion & userAgent
			var testNode = document.createElement("b");
			testNode.style.cssText = "max-height:1";
			if(testNode.style.cssText.match("max-height")) {browser.version = 6}
		}else if(versionString.match("webkit")) {
			browser.engine = "WebKit";
			browser.version = parseFloat(versionString.split("/")[2]) || dojo.isSafari;
			var versionMatches = versionString.match(/applewebkit\/(\d+)\.(\d+)/);
			browser.webkitversion = {};
			browser.webkitversion.major = parseInt(versionMatches[1]);
			browser.webkitversion.minor = parseInt(versionMatches[2]);
		}else if(versionString.match("gecko")) {
			browser.engine = "Gecko";
			browser.version = dojo.isFx || dojo.isFF || parseFloat(versionString.split("/").pop()) /* handling firefox versions with alternate ua */;
		}else if(versionString.match("khtml")) {
			browser.engine = "KHTML";
		}
	}
	browser.version = browser.version ? browser.version.toString().replace(/\./g, "_") : "";
	(document.documentElement || document.body.parentNode).className += " " + browser.engine + " " + browser.engine + browser.version;
})();

}

if(!dojo._hasResource["epages.event"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["epages.event"] = true;
/*
	Copyright (c) 2006-2007, ePages GmbH
	All Rights Reserved.
*/
dojo.provide("epages.event");


epages.event.fire = function(/*Object*/DomObject,/*String*/eventType) {
	// summary: Cross browser function to throw events to dom objects.
	// description: Throws an event of the given event type to the given dom object.
	// DomObject: dom node
	// eventType: type of the event. eg. "click" or "mouseover"

		if (epages.Browser.engine == 'MSIE') {
			try {
				DomObject.fireEvent('on'+eventType);
			}catch(e) {}
		} else {
			if(eventType == 'click' ||
				eventType == 'dblclick' ||
				eventType == 'mousedown' ||
				eventType == 'mousemove' ||
				eventType == 'mouseout' ||
				eventType == 'mouseover' ||
				eventType == 'mouseup') { // create mouse events
				var newevt = document.createEvent('MouseEvents');
				newevt.initMouseEvent(eventType, true, true, window, 0, 0, 0, 0, 0, false, false, false, false, 0, null);
			} else  {										// create html events
				var newevt = document.createEvent('HTMLEvents');
				newevt.initEvent(eventType, true, true);
			}
			DomObject.dispatchEvent(newevt);
		}
};

}

if(!dojo._hasResource["epages.string"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["epages.string"] = true;
/*
	Copyright (c) 2006-2007, ePages GmbH
	All Rights Reserved.
*/
dojo.provide("epages.string");


epages.string = {

	escapeJavaScript: function (str) {
		return str.replace(/(["'\f\b\n\t\r])/gm, "\\$1");
	},

	escapeXml: function (str, noSingleQuotes) {
		str = str.replace(/&/gm, "&amp;").replace(/</gm, "&lt;").replace(/>/gm, "&gt;").replace(/"/gm, "&quot;");
		if (!noSingleQuotes) {
			str = str.replace(/'/gm, "&#39;");
		}
		return str;
	},

	toBoolean: function(/*String*/value) {
		if (value === undefined){
			return false;
		}
		if (typeof value == "boolean"){
			return value;
		}
		if (typeof value != "string"){
			throw new Error("Not a string value, got " + typeof(value) + "!");
		}
		return value.toLowerCase() === "true" || value == '1' ? true : false;
	},

	toInteger: function(/*String*/value,/*Boolean*/removeThousandDelimiter) {
	// summary: parse string to integer, optional remove thousandDelimiter
		if(removeThousandDelimiter) {
			value = value.replace(/\./g,'');
		}
		return parseInt( value.replace(/[^0-9\,\.]/g, "") );
	},

	toTime: function(/*String*/value, /*String*/epagesPattern) {
		if (value == ''){
			return null;
		}
		if (epagesPattern == null || epagesPattern.length == 0){
			epagesPattern = epages.vars.Locale.timeFormat;
		}
		var pattern = epages.localize.getDojoDateTimePattern(epagesPattern);
		var date = dojo.date.locale.parse(value, { selector: 'date', datePattern: pattern, timePattern: '' });
		if (date === null){
			 date = dojo.date.locale.parse(value, { selector: 'date', datePattern: 'HH:mm a', timePattern: '' });
		}
		if (date === null){
			date = dojo.date.locale.parse(value, { selector: 'date', datePattern: 'HH a', timePattern: '' });
		}
		if (date === null){
			date = dojo.date.locale.parse(value, { selector: 'date', datePattern: 'HH', timePattern: '' });
		}
		// allow entries like "24:00" which means "00:00"
		if (date === null) {
			date = dojo.date.locale.parse(value, { selector: 'date', datePattern: 'KK:mm', timePattern: '' });
		}
		if (date === null){
			 date = dojo.date.locale.parse(value, { selector: 'date', datePattern: 'KK', timePattern: '' });
		}
		return date;
	},

	cssSizeToNumber : function(/* string */ value) {
	// summary: convert css size to a number (float or integer)
		if(typeof value == "string") {
			if (value.match(/^(-?[0-9]+)(px|pt)$/i)) {
				var n = parseInt(RegExp.$1);
				return n;
			}

			if (value.match(/^(-?[0-9]*\.?[0-9]+)(pc|%|mm|cm|em|in|ex)$/i)) {
				var n = parseFloat(RegExp.$1);
				return n;
			}

			if (value.match(/^[0]+$/)) {
				return 0;
			}
		}
		return NaN;
	},

	// corrects a bug in IE's String.split, where IE discards all empty elements
	split: function(string, rx) {
		if (epages.Browser.engine != 'MSIE'){
			 return string.split(rx);
		}
		var unique = '';
		do {
			unique += Date.toString();
		} while(string.match(unique));
		string = string.replace(rx, '|' + unique + '|');
		var splits = string.split(unique);
		for( var i=0,iLength=splits.length ; i<iLength ; i++ ) {
			splits[i] = splits[i].replace(/^(?:\|)?(.*?)(?:\|)?$/g, function(tmp, match) { return match; } );
		}
		return splits;
	}
};

epages.string.Base64 = function () {
};

epages.string.Base64.prototype = {
	// private property
	_keyStr : "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=",

	// public method for encoding
	encode : function (input) {
		var output = "";
		var chr1, chr2, chr3, enc1, enc2, enc3, enc4;
		var i = 0;

		input = this._utf8_encode(input);

		if (typeof(window.btoa) == 'function') {
			return window.btoa(input);
		}

		while (i < input.length) {

			chr1 = input.charCodeAt(i++);
			chr2 = input.charCodeAt(i++);
			chr3 = input.charCodeAt(i++);

			enc1 = chr1 >> 2;
			enc2 = ((chr1 & 3) << 4) | (chr2 >> 4);
			enc3 = ((chr2 & 15) << 2) | (chr3 >> 6);
			enc4 = chr3 & 63;

			if (isNaN(chr2)) {
				enc3 = enc4 = 64;
			} else if (isNaN(chr3)) {
				enc4 = 64;
			}

			output = output +
			this._keyStr.charAt(enc1) + this._keyStr.charAt(enc2) +
			this._keyStr.charAt(enc3) + this._keyStr.charAt(enc4);

		}

		return output;
	},

	// public method for decoding
	decode : function (input) {
		var output = "";
		var chr1, chr2, chr3;
		var enc1, enc2, enc3, enc4;
		var i = 0;

		input = input.replace(/[^A-Za-z0-9\+\/\=]/g, "");

		if (typeof(window.atob) == 'function') {
			output = window.atob(input);
		} else {

			while (i < input.length) {

				enc1 = this._keyStr.indexOf(input.charAt(i++));
				enc2 = this._keyStr.indexOf(input.charAt(i++));
				enc3 = this._keyStr.indexOf(input.charAt(i++));
				enc4 = this._keyStr.indexOf(input.charAt(i++));

				chr1 = (enc1 << 2) | (enc2 >> 4);
				chr2 = ((enc2 & 15) << 4) | (enc3 >> 2);
				chr3 = ((enc3 & 3) << 6) | enc4;

				output = output + String.fromCharCode(chr1);

				if (enc3 != 64) {
					output = output + String.fromCharCode(chr2);
				}
				if (enc4 != 64) {
					output = output + String.fromCharCode(chr3);
				}

			}
		}

		output = this._utf8_decode(output);

		return output;

	},

	// private method for UTF-8 encoding
	_utf8_encode : function (string) {
		string = string.replace(/\r\n/g,"\n");
		var utftext = "";

		for (var n = 0,l=string.length; n < l; n++) {

			var c = string.charCodeAt(n);

			if (c < 128) {
				utftext += String.fromCharCode(c);
			}
			else if((c > 127) && (c < 2048)) {
				utftext += String.fromCharCode((c >> 6) | 192);
				utftext += String.fromCharCode((c & 63) | 128);
			}
			else {
				utftext += String.fromCharCode((c >> 12) | 224);
				utftext += String.fromCharCode(((c >> 6) & 63) | 128);
				utftext += String.fromCharCode((c & 63) | 128);
			}

		}

		return utftext;
	},

	// private method for UTF-8 decoding
	_utf8_decode : function (utftext) {
		var string = "";
		var i = 0;
		var c = c1 = c2 = 0;

		while ( i < utftext.length ) {

			c = utftext.charCodeAt(i);

			if (c < 128) {
				string += String.fromCharCode(c);
				i++;
			}
			else if((c > 191) && (c < 224)) {
				c2 = utftext.charCodeAt(i+1);
				string += String.fromCharCode(((c & 31) << 6) | (c2 & 63));
				i += 2;
			}
			else {
				c2 = utftext.charCodeAt(i+1);
				c3 = utftext.charCodeAt(i+2);
				string += String.fromCharCode(((c & 15) << 12) | ((c2 & 63) << 6) | (c3 & 63));
				i += 3;
			}
		}
		return string;
	}
};

}

if(!dojo._hasResource['epages.lang.array']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['epages.lang.array'] = true;
/*
	Copyright (c) 2006-2007, ePages GmbH
	All Rights Reserved.
*/
dojo.provide('epages.lang.array');


$A = function (/*Object?*/arrayLike, /*Boolean?*/asCopy) {
	// summary:
	//    returns a new array, same like new epages.lang.Array
	// example:
	//  | hash = $A(['value1', 'value2']);
	//  | // returns epages.lang.Array
	return new epages.lang.Array(arrayLike, asCopy);
};

epages.lang.Array = /*epages.lang.Array*/ function (/*Object?*/arrayLike, /*Boolean?*/asCopy) {
	// summary:
	//    returns a new array
	// example:
	//  | array = epages.lang.Array(['value1', 'value2']);
	//  | // returns epages.lang.Array
	if (asCopy === undefined){
		asCopy = false;
	}
	var results = [];
	if (arrayLike !== undefined) {
		if (arrayLike._data !== undefined) {
			arrayLike = arrayLike._data;
		}
		if (asCopy) {
			for (var i = 0, length = arrayLike.length; i < length; i++){
				results.push(arrayLike[i]);
			}
		} else {
			results = arrayLike;
		}
	}
	this._data  = results;
	this.length = results.length;
};

epages.lang.Array.prototype = {
	_data: undefined,
	length: 0,

	get: function (/*int*/index) {
		return this._data[index];
	},

	set: function (/*int*/index, /*mixed*/value) {
		this._data[index] = value;
	},

	insertAt: function (/*int*/index, /*mixed*/value) {
		if (!index) {
			this.unshift(value);
		} else if (index == this.length) {
			this.push(value);
		} else {
			var c = this._data;
			for(var i=this.length; i > index; i--){
				c[i] = c[i-1];
			}
			c[index] = value;
			this.length = c.length;
		}
	},

	push: function (/*arguments*/) {
		// summary:
		//    adds elements to this array
		// example:
		//  | array = new epages.lang.Array(['value1', 'value2']);
		//  | array.push('value3', 'value4');
		//  | array.push('value5');
		for (var i = 0, length = arguments.length; i < length; i++) {
			this._data.push(arguments[i]);
		}
		this.length = this._data.length;
	},

	pop: function (/*arguments*/) {
		// summary:
		//    pops the last element from this array
		// returns: 
		//    removed element
		// example:
		//  | array = new epages.lang.Array(['value1', 'value2']);
		//  | array.pop();
		//  | //returns 'value2'
		var el = this._data.pop();
		this.length = this._data.length;
		return el;
	},

	unshift: function (/*arguments*/) {
		// summary:
		//    adds elements to this array at the first position
		// example:
		//  | array = new epages.lang.Array(['value1', 'value2']);
		//  | array.unshift('value3', 'value4');
		//  | // returns ['value4', 'value3','value1', 'value2']
		for (var i = 0, length = arguments.length; i < length; i++) {
			this._data.unshift(arguments[i]);
		}
		this.length=this._data.length;
	},

	shift: function () {
		// summary:
		//    removes an elements from the beginning of this array
		// returns: 
		//    removed element
		var el = this._data.shift();
		this.length = this._data.length;
		return el;
	},

	grep: function (/*Function*/callback, /*Object?*/thisObject) {
		// summary:
		//    returns a new Array with those items from arr that match the
		//    condition implemented by callback.thisObject may be used to
		//    scope the call to callback.
		// example:
		//  | array = new epages.lang.Array([1, 2, 3, 4]);
		//  | array2 = array.grep( function(item){ return item>1; });
		//  | // returns [2, 3, 4]
		if (thisObject === undefined){
			return dojo.filter(this._data, callback); // Array
		}
		return dojo.filter(this._data, callback, thisObject); // Array
	},

	map: function (/*Function*/callback, /*Object?*/thisObject) {
		// summary:
		//    returns calls for each item of this Array the given callback function and collect the return values.
		// example:
		//  | array = new epages.lang.Array([1, 2, 3, 4]);
		//  | var newArray = array.map( function(item){ return item + 1; });
		//  | // newArray = [2,3,4,5]
		return dojo.map(this._data, callback, thisObject); //Array
	},

	each: function (/*Function*/callback, /*Object?*/thisObject) {
		// summary:
		//    returns calls for each item of this Array the given callback function.
		// example:
		//  | array = new epages.lang.Array([1, 2, 3, 4]);
		//  | var sum = 0;
		//  | array.each( function(item){ sum += item; });
		//  | // sum = 10
		dojo.map(this._data, callback, thisObject);
		return; //void
	},

	match: function (/*RegExpr*/ pattern) {
		// summary:
		//    returns a new Array with those items from arr that match the
		//    condition (matches with the given pattern).
		// example:
		//  | array = new epages.lang.Array(['Test', 'Array', 'Hash', 'epages']);
		//  | array2 = array.match(/e/);
		//  | // returns ['Test', 'epages']
		var result = [];
		for (var i = 0, length = this._data.length; i < length; i++) {
			var element = this._data[i];
			if (typeof element.match == 'function' && element.match(pattern)){
				result.push(element);
			}
		}
		return result; //Array
	},

	exists: function (/*Object*/element) {
		// summary:
		//    returns true if an item exists in the array,
		//    otherwise false.
		// example:
		//  | array = new epages.lang.Array(['Test', 'Array', 'Hash', 'epages']);
		//  | isElement = array.exists('e');
		//  | // returns false
		return (this.find(element) !== null); //boolean
	},

	find: function (/*Object*/element) {
		// summary:
		//    returns the index of the element, if it exists in the array
		// example:
		//  | array = new epages.lang.Array(['Test', 'Array', 'Hash', 'epages']);
		//  | isElement = array.find('Array');
		//  | // returns 1
		for (var i = 0, length = this._data.length; i < length; i++) {
			if (element === this._data[i]){
				return i; // integer or null
			}
		}
		return null; // integer or null
	},

	remove: function (/*integer*/index) {
		// summary:
		//    removes the element on position >index
		// example:
		//  | array = new epages.lang.Array(['Test', 'Array', 'Hash', 'epages']);
		//  | isElement = array.find('Array');
		//  | // returns 1
		this._data.splice(index, 1);
		this.length = this._data.length;
	},

	unique: function () {
		// summary:
		//    returns an (JS) array containing every element only once
		// example:
		//  | array = new epages.lang.Array(['a', 'b', 'c', 'c']);
		//  | uniqueArray = array.unique();
		//  | // returns ['a', 'b', 'c']
		var uniqueData = new epages.lang.Array();
		var i = 0;
		while (i < this._data.length) {
			if (uniqueData.find(this._data[i]) == null) {
				uniqueData.push(this._data[i]);
				i++;
			} else {
				this._data.splice(i,1);
			}
		}
		this.length = this._data.length;
	},

	equals: function(/*array|epages.lang.array*/compareArray) {
		// summary:
		//    returns true, if the given array contains the same element in the same order
		//    than this array, otherwise false.
		//    the parameter can be either a JS array or a epages.lang.Array.
		// example:
		//  | array = new epages.lang.Array(['a', 'b', 'c']);
		//  | return array.equals(['a', 'b', 'c']);
		//  | // returns true
		if (compareArray.length == this.length) {
			if (typeof compareArray._data != "undefined") {
				// compareArray seems to be a epages.lang.Array
				for( var i=0,iLength=this.length ; i<iLength ; i++ ) {
					if (this._data[i] !== compareArray._data[i]){
						return false;
					}
				}
			} else {
				// compareArray is a JS array
				for( var i=0,iLength=this.length ; i<iLength ; i++ ) {
					if (this._data[i] !== compareArray[i]){
						return false;
					}
				}
			}
			return true; //Boolean
		} else {
			// lengths differ
			return false; //Boolean
		}
	},

	merge: function(/*array|epages.lang.Array*/anotherArray) {
		var c = [];
		if (dojo.isArray(anotherArray)) {
			c = anotherArray;
		} else if(anotherArray._data !== undefined) {
			c = anotherArray._data;
		}
		for( var i=0,iLength=c.length ; i<iLength ; i++ ) {
			this._data.push(c[i]);
		}
		this.length = this._data.length;
	},

	join: function(delimiter) {
		return this._data.join(delimiter);
	},

	toString: function() {
		var a = this.map(function (v) {
			if (v === undefined){
				return 'undefined';
			}else if (v == null){
				return 'null';
			}else if (dojo.isArray(v)){
				return $A(v).toString();
			}else if (dojo.isObject(v)){
				return v.toString();
			}else{
				return '"' + epages.string.escapeJavaScript('' + v) + '"';
			}
		});
		return '[' + a.join(',') + ']';
	},

	sort: function(sortFunc) {
	  return this._data.sort(sortFunc);
	}
};

}

if(!dojo._hasResource['epages.lang.hash']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['epages.lang.hash'] = true;
/*
	Copyright (c) 2006-2007, ePages GmbH
	All Rights Reserved.
*/
dojo.provide('epages.lang.hash');



$H = function (/*Object*/object) {
	// summary:
	//		returns a new hash, same like new epages.lang.Hash
	// example:
	//	| hash = $H({ key1: 'value1', key2: 'value2'});
	//	| // returns epages.lang.Hash
	return new epages.lang.Hash(object);
};

epages.lang.Hash = /*epages.lang.Hash*/ function (/*Object?*/existingHash) {
	// summary:
	//		returns a new hash, makes a copy of the existing hash
	// example:
	//	|	hash1 = new epages.lang.Hash();
	//	|	// returns an empty epages.lang.Hash
	//	|	hash2 = new epages.lang.Hash({ key1: 'value1', key2: 'value2'});
	// 	|  hash3 = new epages.lang.Hash({
	// 	|		image       : 'test.png',
	// 	|		header      : 'Test',
	// 	|		description : 'long description, long description'
	// 	|	});
	//	|	hash4 = new epages.lang.Hash(hash2);
	//	|	// returns epages.lang.Hash
	this._data = new Object();
	var data = this._data;
	if (existingHash !== undefined) {
		if (typeof existingHash.each == 'function') {
			existingHash.each(function (pair) {
				data[pair.key] = pair.value;
			});
		} else {
			for (var key in existingHash){
				data[key] = existingHash[key];
			}
		}
	}
};

epages.lang.Hash.prototype = {
	_data: undefined,

	data : function () {
		return this._data;
	},

	each : function(iterator) {
		for (var key in this._data) {
			var value = this._data[key];
			var pair = [key, value];
			pair.key = key;
			pair.value = value;
			iterator(pair);
		}
	},

	flatten: function() {
		// summary:
		//		returns xml representation of this hash
		// example:
		// 	|  var hash = new epages.lang.Hash({
		// 	|		image       : 'test.png',
		// 	|		header      : 'Test'
		// 	|	});
		//	|	string = hash.flatten();
		//	|	// returns
		//	|	// <epages.lang.Hash>
		//	|	//  <element><key>image</key><value>test.png</value></element>
		//	|	//  <element><key>header</key><value>Test</value></element>
		//	|	// </epages.lang.Hash>
		var returnvalue = '';
		this.each( function(pair) {
			returnvalue += '<element><key>' + pair.key + '</key><value>' + pair.value + '</value></element>\n';
		});
		return '<epages.lang.Hash>\n' + returnvalue + '</epages.lang.Hash>\n'; //String
	},

	keys: function() {
		// summary:
		//		returns all keys of this hash
		// example:
		// 	|  var hash = new epages.lang.Hash({
		// 	|		image       : 'test.png',
		// 	|		header      : 'Test',
		// 	|		description : 'long description, long description'
		// 	|	});
		//	|	array = hash.keys();
		//	|	// returns ['image','header','description']
		var returnvalue = [];
		this.each( function(pair) {
			returnvalue.push(pair.key);
		});
		return returnvalue; //Array
	},

	values: function() {
		// summary:
		//		returns all values of this hash
		// example:
		// 	|  var hash = new epages.lang.Hash({
		// 	|		image       : 'test.png',
		// 	|		header      : 'Test',
		// 	|		description : 'long description, long description'
		// 	|	});
		//	|	array = hash.keys();
		//	|	// returns ['test.png','Test','long description, long description']
		var returnvalue = [];
		this.each( function(pair) {
			returnvalue.push(pair.value);
		});
		return returnvalue; //Array
	},

	set: function(/*String*/key, /*Object*/value) {
		// summary:
		//		adds a new element to the hash
		// example:
		// 	|  var hash = new epages.lang.Hash({
		// 	|		image       : 'test.png',
		// 	|		description : 'long description, long description'
		// 	|	});
		//	|	hash.set('header','Test');
		//	|	// same as
		// 	|  var hash2 = new epages.lang.Hash({
		// 	|		image       : 'test.png',
		// 	|		description : 'long description, long description',
		// 	|		header      : 'Test'
		// 	|	});
		this._data[key] = value;
	},

	get: function(/*String*/key) {
		// summary:
		//		returns the value of an specified element
		// example:
		// 	|  var hash = new epages.lang.Hash({
		// 	|		image       : 'test.png',
		// 	|		description : 'long description, long description'
		// 	|	});
		//	|	string = hash.get('image');
		//	|	// returns 'test.png'
		return this._data[key];
	},

	remove: function(/*String*/key) {
		// summary:
		//		removes the node with the given key
		// example:
		// 	|  var hash = new epages.lang.Hash({
		// 	|		image       : 'test.png',
		// 	|		description : 'long description, long description'
		// 	|	});
		//	|	bool = hash.remove('image');
		//	|	// returns true
		delete this._data[key];
	},

	existsKey: function(/*String*/key) {
		// summary:
		//		determines, if the given key exists
		// example:
		// 	|  var hash = new epages.lang.Hash({
		// 	|		image       : 'test.png',
		// 	|		description : 'long description, long description'
		// 	|	});
		//	|	bool = hash.existsKey('image');
		//	|	// returns true
		if(this._data[key] !== undefined){
			return true; //Boolean
		}
		// possible key exists but is undefined
		for (var existingKey in this._data) {
			if (existingKey == key){
				return true; //Boolean
			}
		}
		return false; //Boolean
	},

	existsValue: function(/*Object*/value) {
		// summary:
		//		returns, if the given value exists
		// example:
		// 	|  var hash = new epages.lang.Hash({
		// 	|		image       : 'test.png',
		// 	|		description : 'long description, long description'
		// 	|	});
		//	|	bool = hash.existsValue('test.png');
		//	|	// returns true
		return this.find(value) !== undefined;
	},

	find: function(/*Object*/value) {
		// summary:
		//		returns the key to the given value or undefined, when not found
		// example:
		// 	|  var hash = new epages.lang.Hash({
		// 	|		a: 'aa',
		// 	|		b: 'bb'
		// 	|	});
		//	|	bool = hash.find('aa');
		//	|	// returns 'a'
		for (var key in this._data) {
			if (this._data[key] === value){
				return key; //String
			}
		}
		return undefined; //undefined
	},

	toQueryString: function () {
		// summary:
		//		returns uri query string for a hash
		// example:
		//  | var hash = new epages.lang.Hash({ 'schluessel': 'value', 'enc&Schluessel':'enc&Value'});
		//	| var string = hash.toQueryString();
		//	| // returns 'schluessel=value&enc%26Schluessel=enc%26Value'
		var returnvalue = [];
		this.each( function(pair) {
			if (pair.value !== undefined && pair.value != null) {
				var encKey = encodeURIComponent(pair.key);
				if (dojo.isArray(pair.value)) {
					for( var i=0,iLength=pair.value.length ; i<iLength ; i++ ) {
						returnvalue.push(encKey + '=' + encodeURIComponent(pair.value[i]));
					}
				} else {
					returnvalue.push(encKey + '=' + encodeURIComponent(pair.value));
				}
			}
		});
		return returnvalue.join('&'); //String
	},

	toString: function () {
		// summary:
		//		returns uri query string for a hash
		// example:
		//  | var hash = new epages.lang.Hash({ 'schluessel': 'value', 'enc&Schluessel':'enc&Value'});
		//	| var string = hash.toQueryString();
		//	| // returns 'schluessel=value&enc%26Schluessel=enc%26Value'
		var returnvalue = [];
		this.each( function(pair) {
			var v = pair.value;
			if (v === undefined){
				v = 'undefined';
			}else if (v === null){
				v = 'null';
			}else if (dojo.isArrayLike(v)){
				v = $A(v).toString();
			}else if (dojo.isObject(v)){
				v = v.toString();
			}else{
				v = '"' + epages.string.escapeJavaScript('' + v) + '"';
			}

			returnvalue.push('"' + epages.string.escapeJavaScript(pair.key) + '":' + v);
		});
		return '{' + returnvalue.join(',') + '}'; //String
	}
};

}

if(!dojo._hasResource["epages.widget"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["epages.widget"] = true;
/*
	Copyright (c) 2006-2007, ePages GmbH
	All Rights Reserved.
*/

dojo.provide("epages.widget");



 // load all on one



//dojo.require('epages.profile'); // use this to measure time for creating widgets

epages.topDojo.publish('epages/addGlobalSymbol', ['$$', window]);

if (window.restrictedParent || $$ === undefined) {
	$$ = function(widgetId) {
		for( var i=0,iLength=epages._windows.length ; i<iLength ; i++ ) {
			if (epages._windows[i].dijit != null && typeof epages._windows[i].dijit.byId == "function") {
				var widget = epages._windows[i].dijit.byId(widgetId);
				if (widget){
					return widget;
				}
			}
		}
	};
}

epages.widget.parserClasses = $H();
/*dojo.parser.instantiateOrg = dojo.parser.instantiate;
dojo.parser.instantiate = function(nodes) {
	var w = epages.widget.parserClasses;
	if(epages.Browser.engine != "Presto") {
		var d = dojo;
		d.forEach(nodes, function(node){
			if(!node){ return; }
			var type = node.getAttribute("dojoType");
			if((!type)||(!type.length)||w.existsKey(type)){ return; }
			w.set(type,1);
			if (!dojo.isFunction(dojo.getObject(type))) {
				console.warn('add this to you script: dojo.%s("%s");', 'require', type);
				d.require(type);
			}
		});
	}
	return this.instantiateOrg(nodes);
};*/

dojo.declare("epages.widget.Popup",
	[dijit._Widget],
	{
		parent: '',					// the button etc. that is displaying this popup
		around: '',					// DOM node (typically a button); place popup relative to this node.
		hideOnBlur: false,	// flag - hide popup on blur
		o: {orient: {'BR':'TR','BL':'TL'}},	// orientation parameters
		_visible:   false,	// current visibility status of popup

		show: function() {
			// summary: show popup
			var h = {
				popup: this,
				parent: this.parent,
				around: this.around
			};

			dojo.mixin(h,this.o);
			dijit.popup.open(h);
			this._visible=true;
			dijit.focus(this.domNode);
			if(epages.zIndex) {
				this.domNode.parentNode.style.zIndex=epages.zIndex;	// set z-index of wrapper div
				epages.zIndex+=1;
			}
		},

		hide: function() {
			// summary: hide popup
			dijit.popup.close(this);
		},

		toggle: function () {
			// summary: toggle visibility of popup
			if (this._visible) {
				this.hide();
			} else {
				this.show();
			}
		},

		onClose: function () {
			// summary: event function close popup
			this._visible=false;
		},

		_onBlur: function () {
			// summary: blur event function
			if (this.hideOnBlur){
				this.hide();
			}
		}
	}
);

// register window for global widget registry
epages.onIframeStart.push(function() {
	dijit.registerWin(window);
});

//remove comment if you want to measure widget creating time
/*
dijit._Widget.prototype.create_profile = dijit._Widget.prototype.create;
dijit._Widget.prototype.create = function(){
	var me = this;
	var args = arguments;
	epages.profile.takeStatistik(me.declaredClass,function(){
		me.create_profile.apply(me,args);
	});
};

window.setTimeout(function(){
	epages.profile.showStatistik();
},25000);
*/


}

if(!dojo._hasResource["epages.widget.TypedWidget"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["epages.widget.TypedWidget"] = true;
/*
	Copyright (c) 2006-2007, ePages GmbH
	All Rights Reserved.
*/
dojo.provide("epages.widget.TypedWidget");




dojo.declare("epages.widget.TypedWidget", [dijit._Widget, dijit._Templated], {
	// summary : Base class for all widgets which can select template via templateType
	_templateDirectory: undefined,
	postMixInProperties: function() {
		this.inherited("postMixInProperties", arguments);
		if (this._templateMapping !== undefined &&  this.templateType !== undefined &&  this._templateDirectory !== undefined) {
			if(this._templateMapping[this.templateType]!==undefined) {
				// check if set to widget's default using its prototype value
				if(this.templateType != this.defaultTemplateType) {
					var tempPath = '';

					// load templates from json object..
					if(typeof this._templateMapping[this.templateType] == 'object'){
						var TempObj = this._templateMapping[this.templateType];

						// ..from shrinksafed packages
						if(TempObj.templateString){
								this.templatePath   = null;
								this.templateString = TempObj.templateString;
						}

						// ..from sources
						else if(TempObj.templatePath){
							var TempTemplateString;
							dojo.xhrGet({
								url: TempObj.templatePath,
								sync: true,
								load: function(data){
									TempTemplateString = data;
								}
							});
							this.templatePath   = null;
							this.templateString = TempTemplateString;
						} else {
								console.warn('Template type: "'+ this.templateType +'" could not be loaded from json object! Use properties templatePath or templateString to declare its template file!');
						}
					}

					// backward compatibility mode: load templates from old non object format
					else {
							tempPath = this._templateDirectory;
							tempPath.path       += '/'+this._templateMapping[this.templateType];
							tempPath.uri        += '/'+this._templateMapping[this.templateType];
							this.templatePath   = tempPath;
							this.templateString = null;
					}
				}
			} else {
				console.warn('Template type: "'+ this.templateType +'" does not exist in this widget - use default. ('+this.declaredClass+')');
			}
		}
	}

});


}

if(!dojo._hasResource['epages.io.xml']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['epages.io.xml'] = true;
/*
	Copyright (c) 2006-2007, ePages GmbH
	All Rights Reserved.
*/
dojo.provide('epages.io.xml');

epages.io.Xml = function () {}

epages.io.Xml.prototype = {

	loadSync: function (/*String*/url, /*object?*/formData) {
		// summary:
		//		returns a xml document, which was loaded via http from given url.
		// 		additional parameters will be added in the post.
		// example:
		//	|	xmlLoader = new epages.io.Xml();
		//	|	xmlDoc = xmlLoader.loadSync('http://www.mydomain.de/test.xml', { filename: "test.xml" });
		//	|	// returns XMLDocument object
		var returnData;

		// no parameters => GET
		var requestMethod = formData !== undefined ? dojo.xhrPost : dojo.xhrGet;
		requestMethod({
			url: 					url,
			handleAs:			"xml",
			sync:					true,
			preventCache: epages.constants.preventCache,
			load: 				function(data){
											returnData = data;
										},
			content:			formData
		});
		return returnData; // XMLDocument
	},

	loadAsync: function (/*String*/url, /*Function*/callback, /*object?*/formData) {
		// summary:
		//		returns a xml document, which was loaded via http from given url.
		// 		additional parameters will be added in the post.
		// example:
		//	|	xmlLoader = new epages.io.Xml();
		//	|	xmlDoc = xmlLoader.loadSync('http://www.mydomain.de/test.xml', { filename: "test.xml" });
		//	|	// returns XMLDocument object
		var requestMethod = formData !== undefined ? dojo.xhrPost : dojo.xhrGet;
		requestMethod({
			url: 					url,
			handleAs:			"xml",
			preventCache: epages.constants.preventCache,
			load: 				callback,
			content:			formData
		});
		return;
	}
}

}

if(!dojo._hasResource['epages.io.translation']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['epages.io.translation'] = true;
/*
	Copyright (c) 2006-2007, ePages GmbH
	All Rights Reserved.
*/
dojo.provide('epages.io.translation');




epages.io.Translation = function (/*String?*/ url, /*String?*/ locale) {
		this.translation = new epages.lang.Hash();
		this.loadedFiles = new epages.lang.Array();
		if( url !== undefined){
			if(locale===undefined){
				this.load(url);
			}else{
				this.loadLocale(url, (locale!="auto") ? locale : undefined);
			}
		}
};

dojo.extend(epages.io.Translation, epages.io.Xml.prototype);
dojo.extend(epages.io.Translation, {
	translation: null,
	loadedFiles: null,
	useLocale:	false,

	load: function (/*String*/ url, /*Boolean?*/ skipError) {
		if (this.loadedFiles.exists(url)){
			return true; // Boolean
		}
		if (skipError === undefined){
			skipError = false;
		}
		var urlData = this.loadSync(url);
		this.loadedFiles.push(url);
		if (urlData === undefined) {
			if (! skipError){
				throw new Error('translation undefined');
			}
			return false; //Boolean
		}
		var tags = urlData.getElementsByTagName('Translation');
		for( var i=0,iLength=tags.length ; i<iLength ; i++ ) {
			var thisElement = tags[i];
			// is translation
			if (thisElement.nodeType == 1) {
				// ignore <notrans>
				var data = '';
				for( var j=0,jLength=thisElement.childNodes.length ; j<jLength ; j++ ) {
					var child = thisElement.childNodes[j];
					data += child.nodeName.match(/notrans/i) ? child.firstChild.data : child.data;
				}
				this.translation.set(thisElement.getAttribute('Keyword'), data);
			}
		}
		return true; //Boolean
	},

	loadLocale: function (/*String*/ url, /*String?*/ locale) {
    // summary: load translation data (of language and locale), the locale will be trans
		// url: file name without locale and extension (.xml)
		// locale: locale string (RFC 3066; eg en-us)
		if (locale === undefined){
			locale = dojo.locale;
		}

		var locParts = locale.split('-');
		var loadLanguage = this.load(url + '.' + locParts[0] + '.xml', true);
		// load locale dependend translation, later
		if (this.useLocale && locParts.length > 1){
		  this.load(url + '.' + locParts[0] + '_' + locParts[1].toUpperCase() + '.xml', true);
		}
		return loadLanguage; //Boolean
	},

	exists: function (/*String*/key) {
		return this.translation.existsKey(key) || (this != epages.io.dictionary && epages.io.dictionary.translation.existsKey(key));
	},

	get: function (/*String*/ key, /*Object?*/vars) {
		// summary:
		//		returns the value of an specified language tag.
		//		returns {key} if the language tag doesnt exist.
		// example:
		//   | var translation = new epages.io.Translation();
		//   | translation.load('translation.de.xml');
		//   |   image       : 'test.png',
		//   |   description : 'long description, long description'
		//   | });
		//   | string = translation.get('Image');
		//   | // returns 'Bild'
		var locName = this.translation.get(key);
		if(locName === undefined && this != epages.io.dictionary){
			return epages.io.dictionary.get(key, vars);
		}
		return (locName === undefined) ? '{' + key + '}' : this.replaceVars(locName, vars); // String
	},

	replaceLanguageTags: function (/*String*/ txt) {
		// summary:
		//		replace all language tags in a string
		//		returns the "translated" string
		// example:
		// 	  var translation = new epages.io.Translation(translation.de.xml');
		// 		var mystring = "<div>{Image}</div>";
		// 		mystring=this.Translation.replaceLanguageTags(mystring);
		var templateRegExp = new RegExp('{(.*?)}', 'g');
		return txt.replace( templateRegExp, dojo.hitch(this, function(match, tag) {
			return this.get(tag);
		})); // String
	},

	replaceVars: function(/*String*/ txt, /*Object?*/ vars) {
	  var varsRegExp = new RegExp('#([a-zA-Z_][a-zA-Z0-9_]*)', 'g');
		return txt.replace( varsRegExp, dojo.hitch(vars, function(match, varName) {
		  return this[varName] !== undefined ? this[varName] : match;
		})); // String
	}

});

epages.io.dictionary = new epages.io.Translation(dojo.moduleUrl('epages', "translation"),"auto");

}

if(!dojo._hasResource["epages.html"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["epages.html"] = true;
/*
	Copyright (c) 2006-2007, ePages GmbH
	All Rights Reserved.
*/
dojo.provide("epages.html");


dojo.mixin(epages.html, {
	getChildNodesByTagName: function(/*XMLNode*/parentNode, /*String*/childTagName) {
		// summary:
		//  Returns an array of direct children matching the childTagName tag name.
		// description:
		//  Only direct children will be returned, so this differs from getElementsByTagName().
		// example:
		//  |/*
		//  |<ul>
		//  |  <li>node 1</li>
		//  |  <li>node 2</li>
		//  |</ul>
		//  |*/
		//  |listItems = epages.com.getChildNodesByTagName(listNode, "li");
		//  |// listItems has now two children
		var returnValue = [];
		for( var i=0,iLength=parentNode.childNodes.length ; i<iLength ; i++ ) {
			var e = parentNode.childNodes[i];
			if (e.nodeName == childTagName) {
				returnValue.push(e);
			}
		}
		return returnValue; // Array
	},

	getElementsByClassName: function (/*String*/className, /*DomNode*/domNode) {
		if(domNode == null){
			domNode = dojo.body();
		}
		return dojo.query("." + className, domNode);
	},

	replaceClass: function(/*DomNode|String*/node, /*String*/newClassStr, /*String*/ oldClassStr) {
	  var regEx = new RegExp("(^|\\s+)" + oldClassStr + "($|\\s+)", "g");
	  node.className = node.className.replace(regEx, "$1" + newClassStr + "$2");
	},

	removeChildren: function(/*DomNode|String*/node) {
		while(node.firstChild){ node.removeChild(node.firstChild); }
	},

	disableHrefsAndActions: function (/* DomNode */containerNode) {
		if(containerNode == null){
			containerNode = dojo.body();
		}

		// disable links
		var aTags = containerNode.getElementsByTagName('a');
		for( var i=0,iLength=aTags.length ; i<iLength ; i++ ) {
			var tag = aTags[i];
			if(tag.getAttribute("data-ep-preview") == "show") {break;}
			tag.style.cursor='default';
			if(typeof tag.getAttribute("onclick")=="string" && (tag.getAttribute("onclick") || "").match("openWindow")) {
				tag.onclick = function(){ return false; };
			}
			dojo.connect(tag, "onclick", function(evt) {
				dojo.stopEvent(evt);
				return;
			});
			// change href
			if(tag.getAttribute("href")){
				tag.removeAttribute("href");
			}
			// change target
			if(tag.getAttribute("target")){
				tag.removeAttribute("target");
			}
		}
		// disable all forms
		var formTags = containerNode.getElementsByTagName('form');
		for( var i=0,iLength=formTags.length ; i<iLength ; i++ ) {
			formTags[i].onsubmit = function() { return false; };
		}
		// disable all form elements
		var elementList = ['input', 'button', 'select', 'option', 'a'];
		var deleteAttributes = ['onchange','onclick','onselect','onkeyup','onkeypress','onmouseup'];
		for( var j=0,jLength=elementList.length ; j<jLength ; j++ ) {
			var inputTags = containerNode.getElementsByTagName(elementList[j]);
			for( var i=0,iLength=inputTags.length ; i<iLength ; i++ ) {
				var e = inputTags[i];
				if(e.getAttribute("data-ep-preview") == "show") {break;}
				//if (e.type == 'submit') e.type = 'button';
				for( var k=0,kLength=deleteAttributes.length ; k<kLength ; k++ ) {
					var attrib = e.getAttribute(deleteAttributes[k]);

					if(attrib) {
						if((typeof attrib == "string") || (typeof attrib == "function")) {
							dojo.connect(e, deleteAttributes[k], dojo, "stopEvent");
							e.setAttribute(deleteAttributes[k], 'return false;');
						}
					}
				}
			}
		}
	},

	getHref: function (/* DomNode<a>*/linkNode) {
		// returns: real (not transformed absolutely) href content of an A-tag-domNode
		if (linkNode.outerHTML) {
			// IE
			var html = linkNode.outerHTML;
			var href = html.match(/\s*href\s*\=\s*["'](.*?)["']/i);
			if(href && href[1]){
				return href[1];
			}
			return null;
		}
		// others
		return linkNode.getAttribute("href");
	},

	isInnerEvent: function(/* Event */evt, /* DOMNode | array.DOMNode */allowedNodes) {
		// summary: usefull for blur events, that could be triggered by clicking on another dom node
		// returns: whether the given event happened outside of the allowed nodes
		if (!dojo.isArray(allowedNodes)){
			allowedNodes = [ allowedNodes ];
		}
		var target = evt.explicitOriginalTarget;
		for( var i=0,iLength=allowedNodes.length ; i<iLength ; i++ ) {
			if (dojo.isDescendant(target, allowedNodes[i])){
				return true;
			}
		}
		return false;
	},

	encodeEntity: function(string, fixBreaks) {
		if (fixBreaks == null){
			fixBreaks = false;
		}
		var rx = new RegExp("\r?\n", "g");
		var lines = epages.string.split(string, rx);
		var textnode = document.createTextNode('');
		var node = document.createElement('span');
		node.appendChild(textnode);
		for( var i=0,iLength=lines.length ; i<iLength ; i++ ) {
			textnode.nodeValue = lines[i];
			lines[i] = node.innerHTML;
		}
		return lines.join(fixBreaks ? "<br/>\n" : "\n");
	},

	decodeEntity: function(string, fixBreaks) {
		if (fixBreaks == null){
			fixBreaks = false;
		}
		var node = document.createElement('span');
		node.innerHTML = string;
		var value = node.firstChild.nodeValue;
		if (fixBreaks){
			value = value.replace(/\<br\s*\/?\>\r?\n/gi, "\n");
		}
		return value;
	},

	insertAtCursor: function(myValue, fieldId, doc) {
		if (doc === undefined){
			doc = window.document;
		}
		var myField = $(fieldId, doc);
		//IE support
		if (doc.selection) {
			myField.focus();
			sel = doc.selection.createRange();
			sel.text = myValue;
		}
		//MOZILLA/NETSCAPE support
		else if (myField.selectionStart || myField.selectionStart == '0') {
			var startPos = myField.selectionStart;
			var endPos = myField.selectionEnd;
			myField.value = myField.value.substring(0, startPos)
				+ myValue
				+ myField.value.substring(endPos, myField.value.length);
			myField.selectionStart = startPos + myValue.length;
			myField.selectionEnd = startPos + myValue.length;
		} else {
			myField.value += myValue;
		}
		
		epages.event.fire(myField,'change');
	}
});


}

if(!dojo._hasResource['epages.html.element']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['epages.html.element'] = true;
/*
	Copyright (c) 2006-2007, ePages GmbH
	All Rights Reserved.
*/
dojo.provide('epages.html.element');



epages.html.Element = function (object) {
	//summary: epages.html.Element extends standard object with missing functions
	if (object === undefined) {
		throw new Error("epages.html.Element: initialize without object");
	} else if (typeof object == "string") {
		object = $(object);
		if (object === undefined){
			throw new Error("epages.html.Element: initialize without object, unknown element id");
		}
	}
	this.object = object;
};

dojo.extend(epages.html.Element, {
	object: undefined,

	get: function () {
		//summary: getter function for value of element with special treatment for radiobox & checkbox
		if((dojo.hasClass(this.object,"Radio")|| dojo.hasClass(this.object,"Checkbox"))  && !this.object.checked){
			return '';
		}
		return this.object.value || '';
	},

	set: function(value) {
		//summary: setter function for value of element with speacial treatment for radiobox & checkbox
		var type = this.object.type;
		switch (type) {
			case 'checkbox':
				this.object.value = value;
				if(value){
					this.object.checked = true;
				}
				else{
					this.object.checked = false;
				}
			case 'radio':
				if(this.object.form){
					var items = this.object.form.elements;
					var name = this.object.name;
					dojo.forEach(items, function(object) {
						if (object.type == type && object.name == name && object.value == value){
						object.checked = true;
						}
					});
				}
				break;
			default:
				this.object.value = value;
				break;
		}
	},

	connectChangeEvent: function (callback) {
		//summary: connect onchange event to element but in case of element is radio use click event
		dojo.connect(this.object, (this.object.type == 'radio' ? "onclick" : "onchange"), callback);
	},

	callOnChange: function () {
		//summary: fire onchange event for the element
		epages.event.fire(this.object,'change');
	},

	firstElement: function (nodeType, nodeName) {
		var element = undefined;
		var data = this.object.childNodes;
		for(var counter = 0,l=data.length; counter < l && element === undefined; counter++) {
			// find element, if nodeType == 1 then nodeName has to be equal
			if (nodeType != 1               && data[counter].nodeType == nodeType ||
					data[counter].nodeType == 1 && data[counter].nodeName == nodeName) {
				element = data[counter];
			}
		}
		return (element);
	},

	firstComment: function () {
		var commentElement = this.firstElement(8, 'COMMENT');
		return (commentElement === undefined ? undefined : commentElement.data);
	},

	getIframeDocument: function (iframe) {
		if (iframe === undefined && this.object.tagName.toLowerCase() == 'iframe'){
			 iframe = this.object;
		}
		if (iframe === undefined){
			return iframe;
		}
		return (iframe.contentDocument === undefined) ? iframe.contentWindow.document : iframe.contentDocument;
	},

	addClassName: function (/*string*/className) {
		dojo.addClass(this.object, className);
	},

	removeClassName: function (/*string*/className) {
		dojo.removeClass(this.object, className);
	},

	changeClassName: function (/*string*/fromClassName, /*string*/toClassName) {
	  epages.html.replaceClass(this.object, toClassName, fromClassName);
	},

	getAbsolutePosition: function(baseNode) {
	// summary: get absolute position to base node
		var doc = this.object.documentOwner || document,
			ret = dojo.coords(this.object);
		if( baseNode ){
			var base = dojo.coords(baseNode);
			ret.x -= base.x;
			ret.y -= base.y;
		}

		// EPG-18400
		//if(epages.Browser.engine != "WebKit"){
			ret.x += doc.body.scrollLeft;
			ret.y += doc.body.scrollTop;
		//}
		return ret;
	},

	getPositionToParent: function(parentNode) {
		// summary: calcualte position to the specified parent node
		// parentNode: should be an offsetParent (position: absolute or relative)
		var node = this.object;
		var x = 0, y = 0;
		var position = dojo.getComputedStyle(parentNode).position;
		if(position != "relative" && position != "absolute") {
			console.warn("parentNode should be an offsetParent (position: absolute or relative) in "+ this.delcaredClass);
		}
		var lastOffsetParent = undefined;
		while (node != undefined && parentNode !== node) {
			if(lastOffsetParent !== node.offsetParent) {			// skip offset values which are already added (jump from offsetparent to offsetparent)
				if (node.offsetLeft !== undefined && node.offsetTop !== undefined) {
					x += node.offsetLeft;
					y += node.offsetTop;
				}
				lastOffsetParent = node.offsetParent;
			}
			node = node.parentNode;
		}
		return {'x': x, 'y': y};
	},

	getRelativePosition: function() {
		// summary: calcualte position to the next parentnode with position relative|absolute
		var node = this.object;
		var x = 0, y = 0;
		var position = dojo.getComputedStyle(node).position;
		while (node != undefined && position != "relative" && position != "absolute") {
			if (node.offsetLeft !== undefined && node.offsetTop !== undefined) {
				x += node.offsetLeft;
				y += node.offsetTop;
			}
			node = node.offsetParent;
			position = dojo.getComputedStyle(node).position;
		}
		return {'x': x, 'y': y};
	},

	setCss: function(/*Object*/attributes) {
		for (var attributeName in attributes) {
			this.object.style[attributeName] = attributes[attributeName];
		}
	},

	makeWidgets: function() {
		dojo.parser.parse(this.object);
	},

	toQueryString: function() {
		return encodeURIComponent(this.object.name) + '=' + encodeURIComponent(this.get());
	},

	addEventListener: function(/*String*/eventName, /*function*/callBack, /*boolean?false*/bubbleUp) {
		// summary: cross-browser implementation of node.addEventListener from Mozilla
		if (bubbleUp == null){
			bubbleUp = false;
		}
		eventName = eventName.replace(/^on/, '');
		if (this.object.addEventListener) {
			return this.object.addEventListener(eventName, callBack, bubbleUp);
		} else if(this.object.attachEvent) {
			return this.object.attachEvent('on' + eventName, callBack);
		}
		throw new Error('element.addEventListener: could not add event listener for event ' + eventName);
	},

	removeEventListener: function(/*String*/eventName, /*function*/callBack, /*boolean?false*/bubbleUp) {
		//summary: cross-browser implementation of node.removeEventListener from Mozilla
		if (bubbleUp == null){
			bubbleUp = false;
		}
		eventName = eventName.replace(/^on/, '');
		if (this.object.removeEventListener) {
			return this.object.removeEventListener(eventName, callBack, bubbleUp);
		} else if(this.object.detachEvent) {
			return this.object.detachEvent('on' + eventName, callBack);
		}
		console.warn('element.removeEventListener: could not remove event listener for event ' + eventName + (typeof this.object.detachEvent));
		return false;
	},

	replaceChild: function(newNode, oldNode) {
		newNode = this._changeNodeDocument(newNode, oldNode.ownerDocument);
		this.object.replaceChild(newNode, oldNode);
	},

	appendChild: function(childNode) {
		childNode = this._changeNodeDocument(childNode, this.object.ownerDocument);
		this.object.appendChild(childNode);
	},

	_changeNodeDocument: function(node, newDoc) {
		// summary: in IE you can not insert a node in a document, that has been created in another document
		// so we simply clone the node in correct document by inserting its outer HTML
		if (node.ownerDocument != newDoc && node.outerHTML != null) {
			var nodeHtml = node.outerHTML;
			node = newDoc.createElement('span');
			node.outerHTML = nodeHtml;
		}
		return node;
	},


	getLabel: function() {
		// summary: this.object needs an id and there has to exist a label node for this id
		// returns: assigned label for this node
		var id = this.object.id;
		if (id == null){
			 return;
		}
		var labels = this.object.ownerDocument.getElementsByTagName('label');
		for( var i=0,iLength=labels.length ; i<iLength ; i++ ) {
			if (labels[i].htmlFor == id){
				return labels[i].innerHTML;
			}
		}
	},

	isEmpty: function(){
		//summary: check if object has methodes or properties assigned
		for(var el in this.object){
			return false;
		}
		return true;
	}
});

$E = function (/*Object*/object) {
	return new epages.html.Element(object); // epages.html.Element
};

}

if(!dojo._hasResource["epages.widget.LocalizedWidget"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["epages.widget.LocalizedWidget"] = true;
/*
	Copyright (c) 2006-2007, ePages GmbH
	All Rights Reserved.
*/
dojo.provide("epages.widget.LocalizedWidget");





dojo.declare("epages.widget.LocalizedWidget", [epages.widget.TypedWidget], {
	// summary:Base class for all widgets which use an additional translation file
	
	// all LocalizedWidgets share the same translation object
	translation: 			new epages.io.Translation(),
	translationName: 	null,

	constructor: function() {
		this.translation.loadLocale(this.getTranslationName());
	},

	translate: function(/*String*/key, /*Object?*/vars) {
		return this.translation.get(key, vars);
	},

	getTranslationName: function () {
		// summary: return the translation file name (without locale and .xml)
		// description: convert this.templatePath to this.translationName of translation name is null
		// example:
		//   | dojo.declare(
		//   |   "epages.cartridges.de_epages.design.Toolcollection",
		//   |   [epages.widget.LocalizedWidget], {
		//   |     widgetType  : "Toolcollection",
		//   |     templatePath: dojo.uri.dojoUri("../epages/cartridges/de_epages/design/widget/Toolcollection/Toolcollection.html"),
		//   |     translationName: dojo.uri.dojoUri("../epages/cartridges/de_epages/design/widget/templates/translation"),
		//   |     widgetsInTemplate : true
		//   | });
		// returns: translationName
		if (this.translationName == null){
			this.translationName = this.templatePath.toString().replace('.html','');
		}
		return this.translationName; //String
	},

	createNodesFromText:  function(/*String*/text){
		return dojo._toDom(this.translation.replaceLanguageTags(text));
	},


	buildRendering: function(){
		// summary:
		//    Construct the UI for this widget from a template, setting this.domNode.
    // description: 
		//    Lookup cached version of template, and download to cache if it
		//    isn't there already.  Returns either a DomNode or a string, depending on
		//    whether or not the template contains ${foo} replacement parameters.
		this._attachPoints = [];
		var cached = this.getCachedTemplate(this.templatePath, this.templateString, this._skipNodeCache, this.translation);


		var node;
		if(dojo.isString(cached)){
//##### START of epages changes #####
			var className = this.declaredClass, _this = this;
			// Cache contains a string because we need to do property replacement
			// do the property replacement
			var tstr = dojo.string.substitute(cached, this, function(value, key){
				if(key.charAt(0) == '!'){ value = _this[key.substr(1)]; }
				if(typeof value == "undefined"){ throw new Error(className+" template:"+key); } // a debugging aide
				if(!value){ return ""; }

				// Substitution keys beginning with ! will skip the transform step,
				// in case a user wishes to insert unescaped markup, e.g. ${!foo}
				return key.charAt(0) == "!" ? value :
					// Safer substitution, see heading "Attribute values" in
					// http://www.w3.org/TR/REC-html40/appendix/notes.html#h-B.3.2
					value.toString().replace(/"/g,"&quot;"); //TODO: add &amp? use encodeXML method?
			}, this);
			node = this.createNodesFromText(tstr);

//##### END of epages changes #####
		}else{
			// if it's a node, all we have to do is clone it
			node = cached.cloneNode(true);
		}

		this.domNode = node;

		// recurse through the node, looking for, and attaching to, our
		// attachment points which should be defined on the template node.
		this._attachTemplateNodes(node);

				if(this.widgetsInTemplate){
				// Make sure dojoType is used for parsing widgets in template.
				// The dojo.parser.query could be changed from multiversion support.
				var parser = dojo.parser, qry, attr;
				if(parser._query != "[dojoType]"){
					qry = parser._query;
					attr = parser._attrName;
					parser._query = "[dojoType]";
					parser._attrName = "dojoType";
				}

				// Store widgets that we need to start at a later point in time
				var cw = (this._startupWidgets = dojo.parser.parse(node, {
					noStart: !this._earlyTemplatedStartup
				}));

				// Restore the query.
				if(qry){
					parser._query = qry;
					parser._attrName = attr;
				}

				this._supportingWidgets = dijit.findWidgets(node);

				this._attachTemplateNodes(cw, function(n,p){
					return n[p];
				});
			}

		this._fillContent(this.srcNodeRef);
	},

	getCachedTemplate: function(templatePath, templateString, alwaysUseString, translation){
		// summary:
		//    static method to get a template based on the templatePath or
		//    templateString key
		// templatePath: String
		//    the URL to get the template from. dojo.uri.Uri is often passed as well.
		// templateString: String?
		//    a string to use in lieu of fetching the template from a URL
		// returns:
		//  Either string (if there are ${} variables that need to be replaced) or just
		//  a DOM tree (if the node can be cloned directly)

		// is it already cached?
		var tmplts = dijit._Templated._templateCache;
		var key = templateString || templatePath;
		var cached = tmplts[key];
		if(cached){
			return cached;
		}

		// If necessary, load template string from template path
		if(!templateString){
			templateString =dojo.cache(templatePath, {sanitize: true});
		}

		templateString = dojo.string.trim(templateString);
		if(templateString.match(/\$\{([^\}]+)\}/g) || alwaysUseString){
			// there are variables in the template so all we can do is cache the string
			return (tmplts[key] = templateString); //String
		}else{
			// there are no variables in the template so we can cache the DOM tree
			return (tmplts[key] = this.createNodesFromText(templateString)); //Node
		}
	}

});

}

if(!dojo._hasResource["epages.event.interval"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["epages.event.interval"] = true;
dojo.provide("epages.event.interval");

epages.event.interval = {
	// summary: cross browser function to set intervals
	// description: cross browser function to set intervals without using the built in function "setInterval" (which can cause serious timing problems).
	// set: set an interval (e.g.: var myinterval = epages.event.interval.set(function(){alert("test")}, 1000);
	// clear: clear the given interval (e.g.: epages.event.interval.clear(myinterval));
	
	_container: [],
	_setIntervalSecure: function(/* string */intervalId, /* function */task, /* uinteger */delay) {
		if(this._container[intervalId] != null) {
			setTimeout(dojo.hitch(this, function(){this._setIntervalSecure(intervalId, task, delay)}), delay);
			task();
		}
	},
	
	set: function(/* function */task, /* uinteger */delay) {
		var intervalId = this._container.length;
		this._container.push(intervalId);
		setTimeout(dojo.hitch(this, function(){this._setIntervalSecure(intervalId, task, delay)}), delay);
		return intervalId;
	},
	
	clear: function(/* string */intervalId) {
		this._container[intervalId] = null;
	}
	
};

}

if(!dojo._hasResource['epages.widget.Slides']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['epages.widget.Slides'] = true;
/*
	Copyright (c) 2006-2007, ePages GmbH
	All Rights Reserved.
*/
dojo.provide('epages.widget.Slides');







dojo.declare("epages.widget.Slides", [epages.widget.LocalizedWidget], {
	templateString:"<div class=\"SlideShow\">\n  <div class=\"SlideshowContainer\">\n    <div class=\"MainContainer\" dojoAttachPoint=\"mainNode\">\n      <div class=\"DescriptionContainer\" dojoAttachPoint=\"descriptionNode\"></div>\n      <div class=\"MiscContainer\" dojoAttachPoint=\"miscNode\">\n        <div class=\"MiscClipper\" dojoAttachPoint=\"miscNodeClipper\">\n          <div class=\"Counter\" dojoAttachPoint=\"countNode\"></div>\n          <div class=\"Date\" dojoAttachPoint=\"dateNode\"></div>\n        </div>\n      </div>\n      <div class=\"ImageContainer\">\n        <div class=\"ImageAligner\">\n          <table class=\"ImageTableTemplate\" dojoAttachPoint=\"imageTable\"><tr><td style=\"text-align:center;\">\n            <b></b>\n          </td></tr></table>\n        </div>\n      </div>\n      <div class=\"ControlContainer\">\n        <div class=\"ControlBar\" dojoAttachPoint=\"controlNode\"></div>\n      </div>\n    </div>\n  </div>\n</div>\n",
	imagePath:          epages.themeUrl('images'),
	translationName:    dojo.moduleUrl("epages.widget", "templates/translation"),
	loaderImagePath:    epages.themeUrl('images') + "/img_busy.gif",
	errorImagePath:     epages.themeUrl('images') + "/slideshow_img_broken_image.png",

	controlBarImages: [
		[epages.themeUrl('images') + "/slideshow_img_bg.png", epages.themeUrl('images') + "/slideshow_img_bg_small.png", epages.themeUrl('images') + "/slideshow_img_bg_smaller.png"],
		[epages.themeUrl('images') + "/slideshow_ico_l_rewind.png", epages.themeUrl('images') + "/slideshow_ico_s_rewind.png"],
		[epages.themeUrl('images') + "/slideshow_ico_l_pause.png", epages.themeUrl('images') + "/slideshow_ico_s_pause.png"],
		[epages.themeUrl('images') + "/slideshow_ico_l_play.png", epages.themeUrl('images') + "/slideshow_ico_s_play.png"],
		[epages.themeUrl('images') + "/slideshow_ico_l_overview.png", epages.themeUrl('images') + "/slideshow_ico_s_overview.png"],
		[epages.themeUrl('images') + "/slideshow_ico_l_details.png", epages.themeUrl('images') + "/slideshow_ico_s_details.png"],
		[epages.themeUrl('images') + "/slideshow_ico_l_forward.png", epages.themeUrl('images') + "/slideshow_ico_s_forward.png"]
										],

	delay:                 3500,
	protraction:           1200,
	imageWidth:            "450px",
	imageHeight:           "300px",
	tiny:                  false,
	stopped:               false,
	overview:              true,				// flag: enable / disable detailed (large) view
	imageClickOverview:    true,
	showName:              true,
	showDescription:       true,
	showControls:          true,
	showMisc:              true,
	showIndex:             0,

	_images:               undefined,
	_oldShowIndex:         0,
	_loadedIndex:          0,
	_loadedImages:         0,
	_pauseIcon:            0,
	_playIcon:             0,
	_fadeEasing:           dojo.fx.easing.quadOut,
	_overviewArea:         null,
	_overviewBackground:   null,
	_overviewImageCounter: null,
	_controlBarFadeOut:    null,
	_playerBeacon:         null,
	_hiddenDetails:        false,
	_changingImage:        true,
	_globalOverview:       true,
	_reAlignReady:         true,

	_delayExec: function(task, delay) {
		return setTimeout(dojo.hitch(this, task), delay);
	},

	_getAlphaImage: function (imageSrc, sizing) {

		var imageContainer = document.createElement("span");
		var imageHolder = document.createElement("span");

		var filterClassString = "";
		if(epages.Browser.engine == "MSIE" && imageSrc.match(".png")) {
			sizing = sizing ? "scale" : "crop";
			filterClassString = "class='AlphaImage'";
			imageHolder.style.zoom = "1";
			imageHolder.style.filter = "progid:DXImageTransform.Microsoft.AlphaImageLoader(src='" + imageSrc + "', sizingMethod='" + sizing + "')";
		}

		var image = "<img " + filterClassString + " alt='' src='" + imageSrc + "' onerror=''/>";

		imageHolder.innerHTML = image;
		imageContainer.appendChild(imageHolder);
		imageContainer.className = "Image";

		return imageContainer;
	},

	_drawControlBar: function() {

		if(this.tiny) {
			this.controlNode.className = "ControlBarMini";
		}
		if(!this.showControls) {
			this.controlNode.style.display = "none";
		}

		var backgroundImageIndex = 0;
		if(!this.overview || !this.showDescription) {
			backgroundImageIndex = 1;
		}else if(!this.overview && !this.showDescription) {
			backgroundImageIndex = 2;
		}

		var controlBarBg = this.tiny ? document.createElement("div") : this._getAlphaImage(this.controlBarImages[0][backgroundImageIndex]);
		controlBarBg.className = "ControlBarBg";
		this.controlNode.appendChild(controlBarBg);

		if(!this.overview && this.showControls) {
			controlBarBg.parentNode.style.width = (controlBarBg.offsetWidth - (this.tiny ? 16 : 32)) + "px";
		}
		if(!this.showDescription && this.showControls) {
			controlBarBg.parentNode.style.width = (controlBarBg.offsetWidth - (this.tiny ? 16 : 32)) + "px";
		}

		var rewindIcon = this._getAlphaImage(this.controlBarImages[1][this.tiny ? 1 : 0]);
		rewindIcon.className = "SlideshowButton";
		this.controlNode.appendChild(rewindIcon);
		this.connect(rewindIcon, "onclick", "_rewindClick");

		var pauseIcon = this._getAlphaImage(this.controlBarImages[2][this.tiny ? 1 : 0]);
		pauseIcon.className = "SlideshowButton";
		if(this.stopped) {pauseIcon.style.position = "absolute";pauseIcon.style.visibility = "hidden";}
		this.controlNode.appendChild(pauseIcon);
		this.connect(pauseIcon, "onclick", "_pauseClick");
		this._pauseIcon = pauseIcon;

		var playIcon = this._getAlphaImage(this.controlBarImages[3][this.tiny ? 1 : 0]);
		playIcon.className = "SlideshowButton";
		if(!this.stopped) {playIcon.style.position = "absolute";playIcon.style.visibility = "hidden";}
		this.controlNode.appendChild(playIcon);
		this.connect(playIcon, "onclick", "_playClick");
		this._playIcon = playIcon;

		if(this.overview) {
			var overviewIcon = this._getAlphaImage(this.controlBarImages[4][this.tiny ? 1 : 0]);
			overviewIcon.className = "SlideshowButton";
			this.controlNode.appendChild(overviewIcon);
			this.connect(overviewIcon, "onclick", "_overviewClick");
		}

		if(this.showDescription) {
			var detailsIcon = this._getAlphaImage(this.controlBarImages[5][this.tiny ? 1 : 0]);
			detailsIcon.className = "SlideshowButton";
			this.controlNode.appendChild(detailsIcon);
			this.connect(detailsIcon, "onclick", "_detailsClick");
		}

		var forwardIcon = this._getAlphaImage(this.controlBarImages[6][this.tiny ? 1 : 0]);
		forwardIcon.className = "SlideshowButton";
		this.controlNode.appendChild(forwardIcon);
		this.connect(forwardIcon, "onclick", "_forwardClick");

		dojo.style(this.controlNode, "opacity", 0);

		return [rewindIcon, playIcon, pauseIcon, overviewIcon ? overviewIcon : null, detailsIcon ? detailsIcon : null, forwardIcon];
	},

	_controlBarButtonOver: function(eventObject) {
		dojo.addClass(eventObject.currentTarget, "Highlighted");
		document.onselectstart = function() { return false; };
	},

	_controlBarButtonOut: function(eventObject) {
		dojo.removeClass(eventObject.currentTarget, "Highlighted");
		dojo.removeClass(eventObject.currentTarget, "Hit");
		document.onselectstart = function() { return true; };
	},

	_controlBarButtonDown: function(eventObject) {
		dojo.addClass(eventObject.currentTarget, "Hit");
	},

	_controlBarButtonUp: function(eventObject) {
		dojo.removeClass(eventObject.currentTarget, "Hit");
	},

	_showControlBar: function() {
		clearTimeout(this._controlBarFadeOut);
		dojo.fadeIn({node: this.controlNode, duration:350, easing: this._fadeEasing}).play();
	},

	_hideControlBar: function() {
		this._controlBarFadeOut = this._delayExec(function() {
			if(this.controlNode.offsetWidth == 0) {
				dojo.style(this.controlNode, "opacity", 0);
			}else{
				dojo.fadeOut({node: this.controlNode, duration:350}).play();
			}
		}, 1000);
	},

	_drawErrorImage: function(eventObject) {
		var imgNode = eventObject.currentTarget;
		imgNode.src = this.errorImagePath;
		imgNode.parentNode.style.filter = "progid:DXImageTransform.Microsoft.AlphaImageLoader(src='" + this.errorImagePath + "', sizingMethod='scale')";
	},

	_playClickStartTimer: function() {
		this._playIcon.style.position = "absolute";
		this._playIcon.style.visibility = "hidden";
		this._pauseIcon.style.position = "relative";
		this._pauseIcon.style.visibility = "visible";
		this._playerBeacon = epages.event.interval.set(dojo.hitch(this, this._setNextImage), this.delay);
	},

	_playClick: function() {
		this._setNextImage();
		this._playClickStartTimer();
	},

	_pauseClick: function() {
		this._pauseIcon.style.position = "absolute";
		this._pauseIcon.style.visibility = "hidden";
		this._playIcon.style.position = "relative";
		this._playIcon.style.visibility = "visible";
		epages.event.interval.clear(this._playerBeacon);
	},

	_overviewClick: function(image) {
		/* we don't want any mouse events in here */
		if(typeof(image) == "object") {
			image = null;
		}

		this._pauseClick();

		if(!this._overviewArea) {
			var overviewBackground = document.createElement("div");
			overviewBackground.className = "OverviewBackground";
			dojo.style(overviewBackground, "opacity", 0);

			dojo.addClass(dojo.body(), "OverviewAreaVisible");

			overviewBackground.innerHTML = "<div class='Background'></div>";

			var overviewArea = document.createElement("div");
			overviewArea.className = "OverviewArea";

			var info = this._images[this.showIndex];

			if(!info) {
				info = {};
				info.imageTable = this._createImageTable();
				info.imageTable.getElementsByTagName("b")[0].appendChild(this._getAlphaImage(epages.themeUrl('images') + "/plain.gif"));
				this._images.push(info);
			}else {
				this._oldShowIndex = this.showIndex;
			}

			var infoTable = this._images[this.showIndex].imageTable.cloneNode(true);

			var bNode = infoTable.getElementsByTagName("b")[0];
			bNode.innerHTML = "";
			bNode.appendChild(this._getAlphaImage(epages.themeUrl('images') + "/plain.gif"));

			overviewArea.appendChild(infoTable);

			var backgroundEventLayer = document.createElement("div");
			backgroundEventLayer.className = "BackgroundEventLayer";
			overviewArea.getElementsByTagName("td")[0].appendChild(backgroundEventLayer);

			var rewindIcon = this._getAlphaImage(epages.themeUrl("images") + "/slideshow_ico_l_rewind_overview.png");
			rewindIcon.className = "Image SlideshowButton";
			this.connect(rewindIcon, "onclick", "_setPrevImage");
			this.connect(rewindIcon, "onmouseover", "_controlBarButtonOver");
			this.connect(rewindIcon, "onmouseout", "_controlBarButtonOut");
			this.connect(rewindIcon, "onmousedown", "_controlBarButtonDown");
			this.connect(rewindIcon, "onmouseup", "_controlBarButtonUp");
			var rewindIconContainer = document.createElement("i");
			rewindIconContainer.appendChild(rewindIcon);
			rewindIconContainer.className = "OverviewRewindButton";

			var forwardIcon = this._getAlphaImage(epages.themeUrl("images") + "/slideshow_ico_l_forward_overview.png");
			forwardIcon.className = "Image SlideshowButton";
			this.connect(forwardIcon, "onclick", "_setNextImage");
			this.connect(forwardIcon, "onmouseover", "_controlBarButtonOver");
			this.connect(forwardIcon, "onmouseout", "_controlBarButtonOut");
			this.connect(forwardIcon, "onmousedown", "_controlBarButtonDown");
			this.connect(forwardIcon, "onmouseup", "_controlBarButtonUp");
			var forwardIconContainer = document.createElement("i");
			forwardIconContainer.appendChild(forwardIcon);
			forwardIconContainer.className = "OverviewForwardButton";

			var closeIcon = this._getAlphaImage(epages.themeUrl("images") + "/layer_img_closebutton.png");
			closeIcon.className = "Image SlideshowButton";
			closeIcon.style.clip = "rect(0, 40px, 20px, 0)";
			this.connect(closeIcon, "onclick", "_destroyOverview");
			this.connect(closeIcon, "onmouseover", "_controlBarButtonOver");
			this.connect(closeIcon, "onmouseout", "_controlBarButtonOut");
			var closeIconContainer = document.createElement("i");
			closeIconContainer.appendChild(closeIcon);
			closeIconContainer.className = "OverviewCloseButton";

			var overviewImageCounter = document.createElement("b");
			overviewImageCounter.className = "OverviewImageCounter";
			if(!this.showMisc) {
				overviewImageCounter.style.display = "none";
			}

			dojo.style(overviewArea, "background", "url(" + this.loaderImagePath + ") center no-repeat");
			dojo.style(overviewArea.getElementsByTagName("span")[0], "opacity", 0);

			var overviewImageSrc = info.imageLarge ? info.imageLarge : info.image;
			var overviewImage = overviewArea.getElementsByTagName("img")[0];
			overviewImage.src = image || overviewImageSrc;
			overviewImage.style.height = "auto";
			overviewImage.style.width = "auto";
			if(overviewImage.src.match(".ico") || overviewImage.src.match(".cur")) {
				overviewImage.parentNode.style.filter = "fade()";
			}else {
				//overviewImage.style.filter = "alpha(opacity=0)";
				//overviewImage.parentNode.style.filter = "progid:DXImageTransform.Microsoft.AlphaImageLoader(src='" + overviewImage.src + "', sizingMethod='scale')";
			};
			dojo.body().appendChild(overviewBackground);
			dojo.body().appendChild(overviewArea);

			var imageLargeDuplicated = true;
			var prevImage = this._images[0];

			for( var i=0,iLength=this._images.length ; i<iLength ; i++ ) {
				if(this._images[i].imageLarge != prevImage.imageLarge) {
					imageLargeDuplicated = false;
					break;
				}else {
					prevImage = this._images[i];
				}
			}

			if(this._images.length > 1 && !image && !imageLargeDuplicated) {
				if(this.showControls) {
					overviewArea.getElementsByTagName("span")[1].appendChild(rewindIconContainer);
					overviewArea.getElementsByTagName("span")[1].appendChild(forwardIconContainer);
				}
				overviewArea.getElementsByTagName("span")[1].appendChild(overviewImageCounter);
			}
			overviewArea.getElementsByTagName("span")[1].appendChild(closeIconContainer);

			// hack to fetch image load event. the image has to be loaded to access
			// its dimensions for calculating the resulting dimensions in relation
			// to the browser window size.
			var loadImageInstance = new Image();
			loadImageInstance.onload = dojo.hitch(this, function() {
				this._delayExec(this._drawOverviewImage, 1);
			});
			loadImageInstance.src = image || overviewImageSrc;

			this.connect(overviewImage, "onerror", "_drawErrorImage");
			this.connect(backgroundEventLayer, "onclick", "_destroyOverview");

			overviewBackground.style.zoom = 1;
			dojo.style(overviewBackground, "opacity", 1);

			this._overviewBackground = overviewBackground;
			this._overviewArea = overviewArea;
			this._overviewImageCounter = overviewImageCounter;

			this._changeOverviewDimensions();

		}
	},

	_changeOverviewDimensions: function() {
		if(this._overviewArea != null) {

			var overviewImage = this._overviewArea.getElementsByTagName("img")[0];
			var windowHeight = window.innerHeight ? window.innerHeight : document.documentElement.clientHeight;
			var windowWidth = window.innerWidth ? window.innerWidth : document.documentElement.clientWidth;

			var currentHeight = 0;

			if(windowHeight > dojo.body().offsetHeight + 10) {
				currentHeight = windowHeight;
			}else{
				currentHeight = dojo.body().offsetHeight + 10;
			}
			this._overviewBackground.style.height = currentHeight + "px";
			this._overviewArea.style.height = windowHeight + "px";

			setTimeout(dojo.hitch(this, function() {
				var absoluteWidth = this._images[this.showIndex]._overviewWidth;
				var absoluteHeight = this._images[this.showIndex]._overviewHeight;

				if(absoluteWidth && absoluteHeight && (absoluteWidth > windowWidth - 50 || absoluteHeight > windowHeight - 50)) {
					if((windowWidth / windowHeight) / (absoluteWidth / absoluteHeight) > 1) {
						overviewImage.style.height = (windowHeight - 50) + "px";
						overviewImage.style.width = "auto";
					}else {
						overviewImage.style.width = (windowWidth - 50) + "px";
						overviewImage.style.height = "auto";
					}
				}
				this._reAlignOverview();
			}), 1);

		}
	},

	_reAlignOverview: function() {
		if(this._overviewArea != null && this._reAlignReady) {
			this._reAlignReady = false;
			this._delayExec(function(){
				if(this._overviewArea != null) {
					var scrollAmount = dojo.body().pageYOffset ? dojo.body().pageYOffset : document.documentElement.scrollTop;
					dojo.fx.slideTo({
						node: this._overviewArea,
						top: scrollAmount,
						duration: 350,
						easing: this._fadeEasing
					}).play();
				}
				this._reAlignReady = true;
			}, 50);
		}
	},

	_detailsClick: function() {
		if(this._hiddenDetails) {
			this._hiddenDetails = false;
			this.descriptionNode.style.visibility = "visible";
			this.miscNode.style.visibility = "visible";
		}else{
			this._hiddenDetails = true;
			this.descriptionNode.style.visibility = "hidden";
			this.miscNode.style.visibility = "hidden";
		}
	},

	_forwardClick: function() {
		this._pauseClick();
		this._setNextImage();
	},

	_rewindClick: function() {
		this._pauseClick();
		this._setPrevImage();
	},

	_drawOverviewImage: function() {
		this._delayExec(function() {
			var overviewImage = this._overviewArea.getElementsByTagName("img")[0];

			if(!this._images[this.showIndex]._overviewWidth) {
				overviewImage.style.position = "absolute";
				this._images[this.showIndex]._overviewWidth = overviewImage.width;
				this._images[this.showIndex]._overviewHeight = overviewImage.height;
				overviewImage.style.position = "static";
			}

			if(this._images[this.showIndex]._overviewHeight < 100) {
				overviewImage.style.marginTop = (45 - this._images[this.showIndex]._overviewHeight / 2) + "px";
			}else{
				overviewImage.style.marginTop = "0";
			}

			this._changeOverviewDimensions();

			this._overviewImageCounter.innerHTML = (this.showIndex + 1) + " | " + this._images.length;

			this._overviewArea.getElementsByTagName("b")[0].style.visibility = "visible";
			dojo.style(this._overviewArea.getElementsByTagName("b")[0], "opacity", "1");
			dojo.style(this._overviewArea, "background", "");
			dojo.fadeIn({node: this._overviewArea.getElementsByTagName("span")[0], duration: this.protraction, easing: this._fadeEasing}).play();

		}, 400);
	},

	_destroyOverview: function() {
		if(this._overviewArea) {
			if(dojo.style(this._overviewArea.getElementsByTagName("span")[0], "opacity") > 0.5) {
				dojo.fadeOut({node: this._overviewArea, duration: this.protraction / 2, easing: this._fadeEasing}).play();
				this._delayExec(function() {
					if(this._overviewArea) {
						dojo.body().removeChild(this._overviewArea);
						dojo.body().removeChild(this._overviewBackground);
						delete this._overviewArea;
						delete this._overviewBackground;
						this.showIndex = this._oldShowIndex;
						delete this._oldShowIndex;

						dojo.removeClass(dojo.body(), "OverviewAreaVisible");
					}
				}, this.protraction / 2.2);
			}
		}
	},

	_checkGlobalOverview: function() {
		var isPlainImage = this._images[this.showIndex].image.match("dijit/themes/epages/images/plain.gif") || this._images[this.showIndex].image.match("transparentpixel.gif");
		if(this._globalOverview && !isPlainImage){
			this._overviewClick();
		}
	},

	_showImageDetails: function() {

		var info = this._images[this.showIndex];
		this.descriptionNode.innerHTML = info.description;
		this.dateNode.innerHTML = info.date;
		this.countNode.innerHTML = (this.showIndex + 1) + " | " + (this._images.length);

		this.miscNode.style.top = this.miscNodeClipper.offsetHeight + "px";
		this.descriptionNode.style.top = "-" + this.descriptionNode.offsetHeight + "px";

		dojo.style(this.miscNode, "opacity", 0.8);
		dojo.style(this.descriptionNode, "opacity", 0.8);

		if(this.descriptionNode.innerHTML.length > 1) {
			dojo.fx.slideTo({node: this.descriptionNode, duration: 400, easing: this._fadeEasing}).play();
		}
		dojo.fx.slideTo({node: this.miscNode, duration: 400, easing: this._fadeEasing}).play();

	},

	_hideImageDetails: function() {
		dojo.fadeOut({node: this.miscNode, duration: 200, easing: this._fadeEasing}).play();
		dojo.fadeOut({node: this.descriptionNode, duration: 200, easing: this._fadeEasing}).play();
	},

	_setNextImage: function() {
		if(this._overviewArea) {
			if(dojo.style(this._overviewArea.getElementsByTagName("span")[0], "opacity") >= 0.5) {
				this._setImage(this.showIndex > this._images.length - 2 ? 0 : this.showIndex + 1);
			}
		}else{
			this._setImage(this.showIndex > this._images.length - 2 ? 0 : this.showIndex + 1);
		}
	},

	_setPrevImage: function() {
		if(this._overviewArea) {
			if(dojo.style(this._overviewArea.getElementsByTagName("span")[0], "opacity") >= 0.5) {
				this._setImage(this.showIndex == 0 ? this._images.length - 1 : this.showIndex - 1);
			}
		}else{
			this._setImage(this.showIndex == 0 ? this._images.length - 1 : this.showIndex - 1);
		}
	},

	_setImage: function(imageIndex) {
		if(this._images[imageIndex]) {
			if(this._overviewArea) {
				if(!this._oldShowIndex) {
					this._oldShowIndex = this.showIndex;
				}

				var overviewImageContainer = this._overviewArea.getElementsByTagName("span")[0];
				var overviewImage = overviewImageContainer.getElementsByTagName("img")[0];

				dojo.style(this._overviewArea, "background", "url(" + this.loaderImagePath + ") center no-repeat");
				dojo.fadeOut({node: overviewImageContainer, duration: this.protraction, easing: this._fadeEasing}).play();

				this._delayExec(function() {
					var info = this._images[this.showIndex];
					dojo.style(overviewImageContainer, "opacity", 0);
					var overviewImageSrc = info.imageLarge || info.image;
					var overviewImage = this._overviewArea.getElementsByTagName("img")[0];
					overviewImage.src = overviewImageSrc;
					overviewImage.style.width = "auto";
					overviewImage.style.height = "auto";
					//overviewImage.style.filter = "alpha(opacity=0)";
					//overviewImage.parentNode.style.filter = "progid:DXImageTransform.Microsoft.AlphaImageLoader(src='" + overviewImageSrc + "', sizingMethod='crop')";

					// hack to fetch image load event
					var test = new Image();
					test.onload = dojo.hitch(this, function() {
						this._delayExec(this._drawOverviewImage, 1);
					});
					test.src = overviewImageSrc;
				}, this.protraction);

			}else {
				var oldContainer = this._images[this.showIndex].imageTable.getElementsByTagName("b")[0];
				var newContainer = this._images[imageIndex].imageTable.getElementsByTagName("b")[0];

				//be sure that the slideshow stops fading when it is not visible (display:none)
				if(oldContainer.offsetWidth == 0) {
					this._pauseClick();
				}else{
					var newImage = newContainer.getElementsByTagName("img")[0];
					if(typeof(newImage) == "undefined") {
						newContainer.appendChild(this._getAlphaImage(this._images[imageIndex].image, true));
						newImage = newContainer.getElementsByTagName("img")[0];
					}
					if(typeof(newImage) != "undefined") {
						if(this._images[imageIndex].fit == "true" || this._images[imageIndex].fit == true) {
							if(newImage.offsetHeight > newImage.offsetWidth) {
								newImage.style.height = (this.mainNode.offsetHeight) + "px";
								if(newImage.offsetWidth >= this.mainNode.offsetWidth) {
									newImage.style.height = "auto";
									newImage.style.width = (this.mainNode.offsetWidth) + "px";
								}
							}else{
								newImage.style.width = (this.mainNode.offsetWidth) + "px";
								if(newImage.offsetHeight >= this.mainNode.offsetHeight) {
									newImage.style.width = "auto";
									newImage.style.height = (this.mainNode.offsetHeight) + "px";
								}
							}
						}else if(this._images[imageIndex].fit == null) {
							this._delayExec(function() {
								if(newImage.offsetHeight > newImage.offsetWidth) {
									if(newImage.offsetHeight > this.mainNode.offsetHeight) {
										newImage.style.height = (this.mainNode.offsetHeight) + "px";
										newImage.style.width = "auto";
									}
								}else{
									if(newImage.offsetWidth > this.mainNode.offsetWidth) {
										newImage.style.width = (this.mainNode.offsetWidth) + "px";
										newImage.style.height = "auto";
									}
								}
							},1);
						}
					}

					if(this.showIndex != imageIndex) {
						dojo.fadeOut({node: oldContainer, duration: this.protraction, easing: this._fadeEasing}).play();
						this._delayExec(function(){
							this._fadeOutHide(oldContainer);
						}, this.protraction);
					}

					dojo.style(newContainer, "visibility", "visible");
					dojo.fadeIn({node: newContainer, duration: this.protraction, easing: this._fadeEasing}).play();

					this._hideImageDetails();
					this._delayExec(this._showImageDetails, this.protraction / 2);
					this._images[this.showIndex].imageTable.style.zIndex = 0;
					this._images[imageIndex].imageTable.style.zIndex = 1;
				}
			}
		}
		this.showIndex = imageIndex;
	},

	_fadeOutHide: function(domNode) {
		if(dojo.style(domNode, "opacity") == 0) {
			dojo.style(domNode, "visibility", "hidden");
		}
	},

	_loadImagesSoft: function() {
		dojo.disconnect(this._images[this._loadedIndex].onloadHandle);

		if(this._images[this._loadedIndex].imageToStage) {
			this._setImage(this._images[this._loadedIndex].imageToStage);
		}else if (this._loadedIndex == this.showIndex && this._overviewArea == null) {
			this._setImage(this.showIndex);
		}

		if(this._loadedImages < this._images.length) {
			this._delayExec(this._loadImages, 1000);
		}else{
			this.mainNode.style.background = "none";
		}
	},

	_loadImages: function() {
		this._loadedImages++;

		this._loadedIndex++;
		if(this._loadedIndex >= this._images.length){
			this._loadedIndex = 0;
		}

		var info = this._images[this._loadedIndex];
		var imageTable = info.imageTable;

		this.mainNode.style.background = "transparent url(" + this.loaderImagePath + ") center no-repeat";

		imageTable.style.zIndex = 0;

		var imageHolder = imageTable.getElementsByTagName("b")[0];
		imageHolder.style.zoom = "1";
		dojo.style(imageHolder, "opacity", 0);
		dojo.style(imageHolder, "visibility", "hidden");

		if(typeof(imageHolder.getElementsByTagName("img")[0]) == "undefined") {
			imageHolder.appendChild(this._getAlphaImage(info.image, true));
		}
		var image = imageHolder.getElementsByTagName("img")[0];
		this.connect(image, "onerror", "_drawErrorImage");

		// hack to fetch image load event
		var test = new Image();
		test.onload = dojo.hitch(this,'_loadImagesSoft');
		test.src = image.src;
	},

	_createImageTable: function() {
		var imageTable = this.imageTable.cloneNode(true);
		imageTable.className = "ImageTable";
		this.imageTable.parentNode.appendChild(imageTable);
		return imageTable;
	},

	_handleKeyEvent: function(evt) {
		evt = evt ? evt: window.event;
		switch (evt.which || evt.keyCode) {
			case 27:
				this._destroyOverview();
				break;
			case 37:
				this._setPrevImage();
				break;
			case 39:
				this._setNextImage();
				break;
		}
	},

	openOverview: function(image) {
		this._overviewClick(image);
	},

	setImage: function(imageIndex) {
		this._pauseClick();
		this._setImage(imageIndex);
	},

	setNextImage: function() {
		this._setNextImage();
	},

	setPrevImage: function() {
		this._setPrevImage();
	},

	addImage: function(insObject, setImage) {
		if(insObject.image) {
			var alreadyAddedIn;
			for( var i=0,iLength=this._images.length ; i<iLength ; i++ ) {
				if(this._images[i].image == insObject.image) {
					alreadyAddedIn = i;
				}
			}
			if(!alreadyAddedIn) {
				this._images.push(insObject);
				this._images[this._images.length - 1].imageTable = this._createImageTable();
				this._images[this._images.length - 1].imageToStage = this._images.length - 1;
				this._loadImages();
			}else {
				this._setImage(alreadyAddedIn);
			}
		}
	},

	postCreate: function() {
		epages.widget.Slides.superclass.postCreate.apply(this, arguments);
		this._images = [];

		this.domNode.style.width = this.imageWidth;
		this.domNode.style.height = this.imageHeight;
		this.mainNode.parentNode.style.width =  this.imageWidth;
		this.mainNode.style.height =  this.imageHeight;

		if(!this.showDescription) {
			this.descriptionNode.style.display = "none";
		}
		if(!this.showMisc) {
			this.miscNode.style.display = "none";
		}

		var controlButtons = this._drawControlBar();

		for( var i=0,iLength=controlButtons.length ; i<iLength ; i++ ) {
			if(controlButtons[i]) {
				this.connect(controlButtons[i], "onmouseover", "_controlBarButtonOver");
				this.connect(controlButtons[i], "onmouseout", "_controlBarButtonOut");
				this.connect(controlButtons[i], "onmousedown", "_controlBarButtonDown");
				this.connect(controlButtons[i], "onmouseup", "_controlBarButtonUp");
			}
		}

		if(this.srcNodeRef) {
			if(this.srcNodeRef.className) {
				this.mainNode.parentNode.parentNode.className += " " + this.srcNodeRef.className;
			}
			// build images from source node
			var sourceDom = this.srcNodeRef.getElementsByTagName("ins");
			for( var i=0,iLength=sourceDom.length ; i<iLength ; i++ ) {
				var currentImageNode = {};
				currentImageNode.image = sourceDom[i].getAttribute("image") || epages.themeUrl('images') + "/plain.gif";
				currentImageNode.imageLarge = sourceDom[i].getAttribute("imageLarge");
				currentImageNode.name = sourceDom[i].getAttribute("name");
				currentImageNode.description = sourceDom[i].innerHTML;
				currentImageNode.date = sourceDom[i].getAttribute("date");
				currentImageNode.time = sourceDom[i].getAttribute("time");
				currentImageNode.fit = sourceDom[i].getAttribute("fit");

				currentImageNode.imageTable = this._createImageTable();

				this._images.push(currentImageNode);
			}
		}

		this.showIndex = new Number(this.showIndex);
		// start one before, because loadimages increment it first
		this._loadedIndex = this.showIndex - 1;
		if (this._loadedIndex == -1){
			this._loadedIndex = this._images.length;
		}

		this.connect(this.mainNode, "onmouseover", "_showControlBar");
		this.connect(this.mainNode, "onmouseout", "_hideControlBar");

		if(this.imageClickOverview && this.overview) {
			this.connect(this.miscNode.offsetHeight ? this.miscNode : this.mainNode, "onclick", "_checkGlobalOverview");
			this.connect(this.controlNode, "onmouseover", function(){ this._globalOverview = false; } );
			this.connect(this.controlNode, "onmouseout", function(){ this._globalOverview = true; } );
		}else{
			this.mainNode.style.cursor = "default";
		}

		this.connect(window, "onscroll", "_reAlignOverview");
		this.connect(window, "onresize", "_changeOverviewDimensions");
		this.connect(document, "onkeyup", "_handleKeyEvent");

		if(this._images.length > 0) {
			this._delayExec(this._loadImages, 1);
			if(!this.stopped && this._images.length > 1){
				this._delayExec(this._playClickStartTimer, 1000);
			}
		}

	}
});

}

if(!dojo._hasResource['epages.epages_scripts']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['epages.epages_scripts'] = true;
/* ---------------------------------------------------------------------------------------------------
	 ePages 5 - epages_scripts.js
	 --------------------------------------------------------------------------------------------------- */
dojo.provide('epages.epages_scripts');





/* - General Functions ------------------------------------------------------------------------------- */

window.openWindow = function (URL, windowName, features, PathToStoreFront) {
	if (PathToStoreFront === null) {
		PathToStoreFront = undefined; // old fashion
	}
	if (features == "PopUp") {
		features = "width=1000, height=730, toolbar=no, status=no, resizable=yes, menubar=no, location=no, scrollbars=yes";
	}
	if (features == "PopUpPrint") {
		features = "width=790, height=580, toolbar=yes, status=no, resizable=yes, menubar=yes, location=no, scrollbars=yes";
	}
	if (features == "RTE" || windowName == "RTE") {
		features = "width=996, height=700, toolbar=no, status=no, resizable=yes, menubar=no, location=no, scrollbars=yes";
	}
	if (features == "HelpWindow") {
		features = "width=1000, height=730, toolbar=no, status=no, resizable=yes, menubar=no, location=no, scrollbars=yes";
		windowName = "WWHFrame";
	}
	if (features == "StylePreview") {
		features = "width=470, height=310, toolbar=no, status=no, resizable=no, menubar=no, location=no, scrollbars=no";
		windowName = "StylePreview";
	}
	if (features == "DetailedImageView") {
		features = "width=1000, height=730, toolbar=no, status=no, resizable=yes, menubar=no, location=no, scrollbars=yes";
		windowName = "StylePreview";
	}
	if (features == "CheckLink") {
		if (URL.indexOf("?") != 0) {
			if (URL.split('//').length == 1) {
				URL = "http://" + URL;
			}
		} else if (PathToStoreFront != undefined) {
			URL = PathToStoreFront + URL;
		}
		features = "toolbar=yes, status=yes, resizable=yes, menubar=yes, location=yes, scrollbars=yes";
	}

	if (URL != "http://") {
		var newwin = window.open(URL, windowName, features);
		if (newwin) {
			newwin.focus();
		}
	}

	return newwin;
};

window.linkCorrection = function (oInputField) {
	// summary: inserts http protocol if there is no protocol
	var Value = oInputField.value;
	var splittedString = Value.split('//');
	if (Value.indexOf("?") != 0) {
		if (splittedString.length == 1) {
			oInputField.value = "http://" + Value;
			epages.event.fire(oInputField, "change");
		}
	}
};

window.OnCheckAll = function (form, name, checked) {
	//summary: used to mark all checkboxes within a form as "checked"
	var originalTarget = arguments.callee.caller ? arguments.callee.caller.arguments[0].originalTarget : undefined;

	// we have to wait until the originalTarget.checked was set
	window.setTimeout(function () {
		if (form) {
			var triggerNode;

			checked = originalTarget ? originalTarget.checked : checked;
			dojo.addClass(form.getElementsByTagName("tbody")[0], "HideElement");

			for (var i = 0, l = form.length; i < l; i++) {
				var input = form[i];

				if (input.nodeName == 'INPUT') {
					if ((input.type == 'checkbox' || input.className.match("Checkbox")) && input.name == name && !input.disabled && input.checked != checked) {
						input.checked = checked;
						// we need that for the ContentList widget (row selection)
						var widget = $$(dojo.attr(input, 'widgetid') || input.id);
						if (widget) {
							widget.setChecked ? widget.setChecked(checked) : widget.attr('checked', checked);
						}
						if (checked) {
							dojo.addClass(epages.parentByTagName(input, 'tr'), "RowSelected");
						} else {
							dojo.removeClass(epages.parentByTagName(input, 'tr'), "RowSelected");
						}
						triggerNode = input;
						epages.event.fire(triggerNode, 'change');
					}
				}
			}
			dojo.removeClass(form.getElementsByTagName("tbody")[0], "HideElement");
		}
	}, 0);
};

/**
 * Mark all checkboxes within the given table as checked
 * @param {Object} table - The table where all checkboxes should be checked
 * @param {String} name - The name of all checkboxes, which should be selected
 * @param {Boolean} isChecked - The state all checkboxes should be set to
 */
window.OnCheckAllInTable = function (table, name, isChecked) {
	if (table) {
		var inputs = table.querySelectorAll('input');
		for (var i = 0; i < inputs.length; ++i) {
			var input = inputs[i];

			if (input.nodeName == 'INPUT') {
				if ((input.type == 'checkbox' || input.className.match("Checkbox")) && input.name == name && !input.disabled && input.checked != isChecked) {
					input.checked = isChecked;
				}
			}
		}
	}
};

window.DeactivateButton = function (Button) {
	// summary: function to disable an input field
	// Button must be an input/button object
	if (Button.className.match("CustomButton")) {
		fakeButton = Button.getElementsByTagName("*")[0];
		fakeButton.disabled = "true";
		dojo.addClass(Button, "Disabled");
	}
	else if (Button.getAttribute('widgetid')) {
		$$(Button.getAttribute('widgetid')).disable();
	}
	else {
		Button.disabled = "true";
		dojo.addClass(Button, "Disabled");
	}
};

window.DisableLink = function (Link) {
	// summary: function to disable a link
	if ($(Link)) {
		$(Link).className += ' Opacity40';
		$(Link).href = 'javascript:void(0)';
		$(Link).style.cursor = 'default';
	}
};

window.toggleTopic = function (/* dom node */ el, /* string? */ label1, /* string? */ label2) {
	if (el.parentNode) {
		var className = el.parentNode.className;
		if (className.match("Closed")) {
			epages.html.replaceClass(el.parentNode, "Open", "Closed");
		} else {
			epages.html.replaceClass(el.parentNode, "Closed", "Open");
		}
		if (label1 && label2) {
			if (el.innerHTML == label1) {
				el.innerHTML = label2;
			} else {
				el.innerHTML = label1;
			}
		}
	}
};

window.toggleDisplay = function (elementId) {
	// summary: switches the display properties of an element (specified by its id attribute)
	// by changing the class name (HideElement / ShowElement)
	if ($(elementId).className != 'HideElement') {
		$(elementId).className = 'HideElement';
	} else {
		$(elementId).className = 'ShowElement';
	}
};

window.hideElement = function (elementId) {
	if (!$(elementId).className.match(/HideElement/)) {
		$(elementId).className += ' HideElement';
	}
};

window.showElement = function (elementId) {
	$(elementId).className = $(elementId).className.replace(/HideElement/, "");
};

window.changeDataById = function (newdata, id) {
	myobject = $(id);
	if (myobject.firstChild) {
		myobject.firstChild.data = newdata;
	} else {
		mydata = document.createTextNode(newdata);
		myobject.appendChild(mydata);
	}
};

window.getSelectedData = function (selectElement) {
	// summary: scans all option tags in a select tag to find the data of the selected item
	if (selectElement) {
		myNode = selectElement.firstChild;
		while (myNode != null) {
			if (myNode.value == selectElement.value && myNode.value != "") {
				return myNode.firstChild.data;
			}
			myNode = myNode.nextSibling;
		}
		return "";
	}
};

window.disableFormFieldsInElement = function (ElementId) {
	// summary: disables all form fields that are sub elements of a given ElementId
	userattrinputs = $(ElementId).getElementsByTagName("input");
	for (var i = 0, iLength = userattrinputs.length; i < iLength; i++) {
		userattrinputs[i].disabled = true;
		if ($(ElementId).parentNode.className.match("Cropper")) {
			$(ElementId).parentNode.className += " Disabled";
		}
		if (userattrinputs[i].type != 'checkbox' && userattrinputs[i].type != 'radio') {
			if (userattrinputs[i].type == "button" || userattrinputs[i].type == "submit") {
				userattrinputs[i].parentNode.className = userattrinputs[i].parentNode.className + " Disabled";
			} else {
				userattrinputs[i].className = userattrinputs[i].className + " Disabled";
			}
		}
	}
	userattrtextareas = $(ElementId).getElementsByTagName("textarea");
	for (var i = 0, iLength = userattrtextareas.length; i < iLength; i++) {
		userattrtextareas[i].disabled = true;
		if ($(ElementId).parentNode.className.match("Cropper")) {
			$(ElementId).parentNode.className += " Disabled";
		}
		userattrtextareas[i].className = userattrtextareas[i].className + " Disabled";
	}
	userattrselects = $(ElementId).getElementsByTagName("select");
	for (var i = 0, iLength = userattrselects.length; i < iLength; i++) {
		userattrselects[i].disabled = true;
		if ($(ElementId).parentNode.className.match("Cropper")) {
			$(ElementId).parentNode.className += " Disabled";
		}
		userattrselects[i].className = userattrselects[i].className + " Disabled";
	}
};

window.enableFormFieldsInElement = function (ElementId) {
	// summary: enables all form fields that are sub elements of a given ElementId
	userattrinputs = $(ElementId).getElementsByTagName("input");
	for (var i = 0, iLength = userattrinputs.length; i < iLength; i++) {
		userattrinputs[i].disabled = false;
		if ($(ElementId).parentNode.className.match("Cropper")) {
			$(ElementId).parentNode.className = $(ElementId).parentNode.className.replace(/Disabled/g, "");
		}
		userattrinputs[i].parentNode.className = userattrinputs[i].parentNode.className.replace(/ Disabled/, "");
		userattrinputs[i].parentNode.className = userattrinputs[i].parentNode.className.replace(/Disabled/, "");
		userattrinputs[i].className = userattrinputs[i].className.replace(/ Disabled/, "");
		userattrinputs[i].className = userattrinputs[i].className.replace(/Disabled/, "");
	}
	userattrtextareas = $(ElementId).getElementsByTagName("textarea");
	for (var i = 0, iLength = userattrtextareas.length; i < iLength; i++) {
		userattrtextareas[i].disabled = false;
		if ($(ElementId).parentNode.className.match("Cropper")) {
			$(ElementId).parentNode.className = $(ElementId).parentNode.className.replace(/Disabled/g, "");
		}
		userattrtextareas[i].className = userattrtextareas[i].className.replace(/ Disabled/, "");
		userattrtextareas[i].className = userattrtextareas[i].className.replace(/Disabled/, "");
	}
	userattrselects = $(ElementId).getElementsByTagName("select");
	for (var i = 0, iLength = userattrselects.length; i < iLength; i++) {
		userattrselects[i].disabled = false;
		if ($(ElementId).parentNode.className.match("Cropper")) {
			$(ElementId).parentNode.className = $(ElementId).parentNode.className.replace(/Disabled/g, "");
		}
		userattrselects[i].className = userattrselects[i].className.replace(/ Disabled/, "");
		userattrselects[i].className = userattrselects[i].className.replace(/Disabled/, "");
	}
};

window.disableFormFieldById = function (ElementId) {
	// summary: function to disable an input field specified by id
	var node = $(ElementId),
		widget = $$(ElementId),
		parent = node.parentNode;

	if (dojo.hasClass(parent, 'Cropper')) {
		dojo.addClass(parent, 'Disabled');
	}
	if (widget) {
		widget.attr('disabled', true);
		dojo.addClass(node, 'Disabled');
		return;
	}

	node.disabled = false;
	if (arguments.length > 1 ? arguments[1] : true) {
		dojo.addClass(node, 'Disabled');
		dojo.addClass(parent, 'Disabled');
	}
};

window.enableFormFieldById = function (ElementId) {
	// summary: function to enable an input field specified by id
	var node = $(ElementId),
		widget = $$(ElementId),
		parent = node.parentNode;

	if (dojo.hasClass(parent, 'Cropper')) {
		dojo.removeClass(parent, 'Disabled');
	}
	if (widget) {
		widget.attr('disabled', false);
		dojo.removeClass(node, 'Disabled');
		return;
	}

	node.disabled = false;
	if (arguments.length > 1 ? arguments[1] : true) {
		dojo.removeClass(node, 'Disabled');
		dojo.removeClass(parent, 'Disabled');
	}
};

window.setValue = function (Value, TargetID) {
	$(TargetID).value = Value;
};

/* - Special Functions ------------------------------------------------------------------------------- */

window.autofillAddress = function (/*Object*/changedObject) {
	// summary: this function fills out the "DisplayName" input field (name="DisplayName") automatically
	// changedObject: object with the "changed" data --> CustomerGeneral.TabPage
	changedObject.form.DisplayName.value = changedObject.form.FirstName.value + ' ' + changedObject.form.LastName.value;
	if (changedObject.form.UserAlias) {
		changedObject.form.UserAlias.value = changedObject.form.FirstName.value.slice(0, 1) + changedObject.form.LastName.value;
	}
	if (changedObject.form.UserName) {
		changedObject.form.UserName.value = changedObject.form.FirstName.value + ' ' + changedObject.form.LastName.value;
	}
};

window.SFautofillAddress = function (FirstNameId, LastNameId, DisplayNameId) {
	// summary: fills out the "DisplayName" input field (name="DisplayName") automatically
	$(DisplayNameId).value = $(FirstNameId).value + ' ' + $(LastNameId).value;
};

window.showNotAllowedDialog = function (domNode, message) {
	dojo["require"]("epages.uimessagehandler");
	

	var position = $E(domNode).getAbsolutePosition();
	var screenW = document.getElementsByTagName("body")[0].clientWidth;
	var screenH = document.getElementsByTagName("body")[0].clientHeight;
	var orientation = 'top';
	var xoff = 0;
	var yoff = 0;

	if (position.x > (screenW / 2)) {
		orientation = 'right';
	}

	if (position.y > (screenH - 30)) {
		orientation = 'bottom';
		yoff = 0;
		xoff = 0;
	}

	if (orientation == 'top') {
		xoff = 10;
		yoff = domNode.offsetHeight;
	}

	if (orientation == 'right' || orientation == 'left') {
		yoff = domNode.offsetHeight / 2;
	}

	dojo.publish("uimessage/show", ["", message, "Bubble", {
		typeClass: "Warning Wide",
		sizeClass: "Medium",
		x: position.x + xoff,
		y: position.y + yoff,
		orientation: orientation
	}]);
	dojo.connect(dojo.body(), "mouseup", this, function () {
		dojo.publish("uimessage/hide");
	});
};

/* - Dialog Area ------------------------------------------------------------------------------------ */
window.DialogArea = function () {
	// summary: is used as constructor for an DialogArea-Object
	this.areaName = new Array();
	this.areaId = new Array();
	this.areaDisplayName = new Array();
	this.areaActivated = new Array();          // false = area disabled    ; true = area is a activated

	this.addArea = function (areaName, areaId) {
		//  this function is used to register the areas in a dialog
		this.areaName.push(areaName);
		this.areaId.push(areaId);
		if (this.addArea.arguments.length > 3) {
			this.areaDisplayName.push(this.addArea.arguments[2]);
			this.areaActivated.push(this.addArea.arguments[3]);
		} else {
			this.areaDisplayName.push("");
			this.areaActivated.push(true);
		}
	};

	this.createAreaLinks = function () {
		for (var i = 0, l = this.areaName.length; i < l; i++) {
			var oAreaLink;
			if (this.areaActivated[i]) {
				oAreaLink = document.createElement("a");
				oAreaLink.href = "javascript:epagesDialog.toggleArea('" + this.areaName[i] + "');";
				oAreaLink.name = this.areaName[i];
				oAreaLink.id = this.areaName[i];
				oAreaLink.className = "HideArea";
			} else {
				oAreaLink = document.createElement("span");
				oAreaLink.name = this.areaName[i];
				oAreaLink.id = this.areaName[i];
				oAreaLink.className = "disabled";
			}
			var oLinkText = document.createTextNode(this.areaDisplayName[i]);
			oAreaLink.appendChild(oLinkText);
			$("ToggleTabpageArea").appendChild(oAreaLink);
		}
	};

	this.showArea = function (areaName) {
		// summary: used to show an Area
		areaId = "";
		// search for the AreaName in the epagesDialog Object
		for (var i = 0, iLength = this.areaName.length; i < iLength; i++) {
			if (this.areaName[i] == areaName) {
				areaId = this.areaId[i];
			}
		}
		try {
			dojo.removeClass(document.getElementsByName(areaName)[0], "HideArea");
			dojo.addClass(document.getElementsByName(areaName)[0], "ShowArea");
		} catch (ex) {
			dojo.removeClass($(areaName), "HideArea");
			dojo.addClass($(areaName), "ShowArea");
		}
		// Replace visibility class
		$(areaId).className = $(areaId).className.replace(/HideElement/g, '');
		$(areaId).className += ' ShowElement';
		if (dojo.isIE) {
			// workaround silly MSIE display errors (elements with position:relative are still hidden after setting them visible)
			setTimeout("$('" + areaId + "').className='ShowElement';", 10);
		}
		actualDialogArea = areaName;

	};

	this.hideArea = function (areaName) {
		// summary: used to show an Area
		areaId = "";
		// search for the AreaName in the epagesDialog Object
		for (var i = 0, iLength = this.areaName.length; i < iLength; i++) {
			if (this.areaName[i] == areaName) {
				areaId = this.areaId[i];
			}
		}
		dojo.removeClass(document.getElementsByName(areaName)[0], "ShowArea");
		dojo.addClass(document.getElementsByName(areaName)[0], "HideArea");
		// Replace visibility class
		$(areaId).className = $(areaId).className.replace(/ShowElement/g, '');
		$(areaId).className += ' HideElement';
		actualDialogArea = "";
	};

	this.toggleArea = function (areaName) {
		//  summary: used to toggle the view of an area
		//  and to hide all unselected areas
		this.showArea(areaName);

		// loop through all dialog areas and hide them
		for (var i = 0, iLength = this.areaName.length; i < iLength; i++) {
			tempName = this.areaName[i]; tempId = this.areaId[i];
			if (areaName != tempName) {
				if (this.areaActivated[i]) {
					dojo.removeClass(document.getElementsByName(tempName)[0], "ShowArea");
					dojo.addClass(document.getElementsByName(tempName)[0], "HideArea");
					// Replace visibility class
					$(tempId).className = $(areaId).className.replace(/ShowElement/g, '');
					$(tempId).className += ' HideElement';
				}
			}
		}
		epages.event.fire(window, 'resize');
	};

	this.toggleAreaDisplay = function (areaName) {
		// summary: used to toggle the view of the specified area
		// display settings of other areas remain
		if (document.getElementsByName(areaName)[0].className.match('HideArea')) {
			this.showArea(areaName);
		} else {
			this.hideArea(areaName);
		}
	};

	this.selectArea = function (areaName) {
		// summary: used to show the selected area
		//  and to hide all the other areas
		this.showArea(areaName);
		// loop through all dialog areas and hide them
		for (var i = 0, iLength = this.areaName.length; i < iLength; i++) {
			tempName = this.areaName[i]; tempId = this.areaId[i];
			if (areaName != tempName) {
				dojo.removeClass(document.getElementsByName(tempName)[0], "ShowArea");
				dojo.addClass(document.getElementsByName(tempName)[0], "HideArea");
				// Replace visibility class
				$(tempId).className = $(areaId).className.replace(/ShowElement/g, '');
				$(tempId).className += ' HideElement';
			}
		}
	};
};

window.appendDialogArea = function (formElement) {
	if (actualDialogArea != "") {
		formElement.action += "&DialogArea=" + actualDialogArea;
	}
};

actualDialogArea = "";
epagesDialog = new DialogArea();

/* - Switch Wizard Pages ----------------------------------------------------------------------------- */
window.WizardPages = function () {
	// summary: constructor for wizard pages
	this.TabId = new Array();
	this.AreaId = new Array();

	this.addPage = function (TabId, AreaId) {
		this.TabId.push(TabId);
		this.AreaId.push(AreaId);
	};

	this.showStep = function (Index) {
		// summary: used to show the selected page of the wizard
		ActualWizardPage = Index;

		window.scrollTo(0, 0);

		// change tab
		$(this.TabId[Index]).className = "selected";
		$(this.TabId[Index]).getElementsByTagName("span")[0].firstChild.className = "selected";
		try { $(this.TabId[Index]).getElementsByTagName("span")[0].appendChild(oHelpTopic); } catch (ex) { }
		$(this.AreaId[Index]).className = "ShowElement";
		for (var i = 0, iLength = this.TabId.length; i < iLength; i++) {
			if (i != Index) {
				$(this.TabId[i]).className = "unselected";
				$(this.TabId[i]).getElementsByTagName("span")[0].firstChild.className = "disabled";
				$(this.AreaId[i]).className = 'HideElement';
			}
		}
	};
};

ActualWizardPage = 0;
WizardPages = new WizardPages();

/* - our "lightbox" ----------------------------------------------------------------------------- */

window.showImage = function (image) {
	
	(new epages.widget.Slides).openOverview(image);
};


/* - showMore / showLess -- */

window.showMore = function (alias, refNode) {
	// summary: wipe in hidden node / hide reference node (clicked node)if specified
	if (refNode) {
		refNode.style.display = 'none';
	}
	dojo.fx.wipeIn({ node: $(alias), duration: 200 }).play();
};
window.showLess = function (alias) {
	// summary: wipe out hidden node
	dojo.fx.wipeOut({ node: $(alias), duration: 200 }).play();
};

}

if(!dojo._hasResource['epages.lang.clipboard']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['epages.lang.clipboard'] = true;
/*
	Copyright (c) 2006-2007, ePages GmbH
	All Rights Reserved.
*/
dojo.provide('epages.lang.clipboard');


epages.lang.Clipboard = function() {
	this._hash = new epages.lang.Hash();
};

epages.lang.Clipboard.prototype = {
	_hash: undefined,
	_counter: 0,

	get: function(/*String*/key) {
		returnValue = this._hash.get("key" + key);
		return returnValue; //Object
	},

	add: function(/*Object*/value) {
		// summary:
		//		adds a value and returns its index
		// example:
		// 	| var clipboard = new epages.lang.Clipboard();
		//	| var index = clipboard.add("test");
		var addIndex = this._counter;
		this._hash.set("key" + addIndex, value);
		do {
			this._counter++;
		} while(this._hash.existsKey("key" + this._counter));
		return addIndex; //Integer
	},

	set: function(/*String*/key, /*Object*/value) {
		// summary:
		//		change the value at the specified position and returns true on success
		//		returns false if position does not exist
		// example:
		// 	| var clipboard = new epages.lang.Clipboard();
		//	| boolean = clipboard.set(5, "test");
		if (this._hash.existsKey("key" + key)) {
			this._hash.set("key" + key, value);
			return true; // Boolean
		} else {
			return false; // Boolean
		}
	}
}

}

if(!dojo._hasResource['epages.lang.hitch']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['epages.lang.hitch'] = true;
/*
	Copyright (c) 2006-2007, ePages GmbH
	All Rights Reserved.
*/
dojo.provide('epages.lang.hitch');


epages.lang.hitch = function (/*Object*/o, /*function*/func, /*Array?*/parameters) {
	if (o === undefined && func === undefined && parameters === undefined) {
		// it seems to be a constructor
		return;
	} else {
		if (parameters === undefined) {
			// no given parameters ==> use default hitch function
			return dojo.hitch(o, func);
		}
		var clipboardParams = [];
		for( var i=0,iLength=parameters.length ; i<iLength ; i++ ) {
			var clipboardIndex = epages.lang.hitch.prototype.clipBoard.add(parameters[i]);
			clipboardParams.push("epages.lang.hitch.prototype.clipBoard.get(" + clipboardIndex + ")");
		}
		// find out methods name in o
		if (typeof(func) == "string") {
			var funcName = func;
		} else {
			for (member in o) {
				if (typeof o[member] == "function" &&
						o[member] == func) {
					var funcName = member;
					break;
				}
			}
		}
		var returnValue;
		if (funcName === undefined) {
			// not a member function
			returnValue = eval("dojo.hitch(o, function() { func(" + clipboardParams.join(", ") + "); });");
		} else {
			// If possible the following line should be replaced with something
			// that does a function call and provides function parameters as an array
			// or something else non-static.
			returnValue = eval("dojo.hitch(o, function() { o." + funcName + "(" + clipboardParams.join(", ") + "); });");
		}
		return returnValue;
	}
};

epages.lang.hitch.prototype = {
	clipBoard : new epages.lang.Clipboard()
};

}

if(!dojo._hasResource["epages.lang.uri"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["epages.lang.uri"] = true;
/*
	Copyright (c) 2006-2007, ePages GmbH
	All Rights Reserved.
*/
dojo.provide("epages.lang.uri");


epages.lang.uri = {

	parts: function(uri) {
		// summary:
		//		returns an object with the parts of an uri
		// description:
		//		available parts are: protocol, host, path, query, parameters (Object), anchor
		// example:
		// 	|	var parts = epages.lang.uri.parts('http://www.host.com/path/to/somewhere?action=go#top');
		//	|	parts == {
		//	|		protocol: 'http',
		//	|		host: 'www.host.com',
		//	|		path: '/path/to/somewhere',
		//	|		query: '?action=go',
		// 	|		anchor: '#top',
		//	|		parameters: {
		//	|			action: 'go'
		//	|		}
		// 	|	}
		var parts = {};
		var regExpResult = uri.match(/^((?:(.+)\:\/\/([^\/]+))?((?:\/|[^\?#]).*?)?(\?.*?)?(#.*)?)$/);
		if (regExpResult) {
			parts = {
				'href':				regExpResult[1],
				'protocol': 	regExpResult[2],
				'host': 			regExpResult[3],
				'path': 			regExpResult[4],
				'query': 			regExpResult[5],
				'anchor':			regExpResult[6],
				'parameters': {}
			};
			parts.parameters = this.splitQueryString(parts.query);
		}
		return parts;
	},

	splitQueryString: function(uri) {
		// summary:
		//		returns an object containing the elements of a query string
		// example:
		// 	|	var parts = epages.lang.uri.parts('?action=go');
		//	|	parts == {
		//	|		action: 'go'
		// 	|	}
		var returnValue = {};
		var urlSplits = (new RegExp(/\??(.*?)(#.*)?$/g)).exec(uri);
		if (urlSplits) {
			var queryString = urlSplits[1];
			var splits = queryString.split('&');
			for( var i=0,iLength=splits.length ; i<iLength ; i++ ) {
				var querySplits = (new RegExp(/(.+)\=(.*)/)).exec(splits[i]);
				if (querySplits) {
					returnValue[decodeURIComponent(querySplits[1])] = decodeURIComponent(querySplits[2]);
				}
			}
		}
		return returnValue;
	},

	toQueryString: function(/*Object*/data) {
		// summary:
		//	Generates a query string from a data structure.
		// description:
		//	Needs a hash (hashlike) or an array[string[2]]. The first element
		//	of each sub array will be used as key, the second as the value.
		// example:
		//	| r = epages.lang.uri.toQueryString( { test: 123, test2: abc} );
		//	| r = epages.lang.uri.toQueryString( [ ['test', '123'], ['test2', 'abc'] ] );
		//	| // r (in both cases) == 'test=123&test2=abc'
		if (dojo.isArray(data)) {
			// expects strict array structure
			var returnValue = [];
			for( var i=0,iLength=data.length ; i<iLength ; i++ ) {
				if (!dojo.isArray(data[i])) {
					console.warn('data element is not an array: ' + (typeof data[i]));
					continue;
				}
				returnValue.push(encodeURIComponent(data[i][0]) + '=' + encodeURIComponent(data[i][1]));
			}
			return returnValue.join('&');
		} else if (dojo.isObject(data)) {
			// use the function provided by a hash
			return $H(data).toQueryString();
		}
	},

	fromParts: function(parts) {
		// summary:
		//		returns the uri for an object of its parts
		var result = '';
		if (parts.protocol) {
			result += parts.protocol + '://';
		}
		if (parts.host) {
			result += parts.host;
		}
		if (parts.path) {
			result += parts.path;
		}
		if (dojo.isObject(parts.parameters)) {
			var qs = this.toQueryString(parts.parameters);
			if (qs){
				result += '?' + qs;
			}
		} else if (parts.query) {
			result += parts.query;
		}
		if (parts.anchor) {
			result += parts.anchor;
		}
		return result;
	},

	isAbsolute: function(/*String*/uri) {
		// summary: uri starts with scheme or a slash
		var rx = new RegExp(/^((http|https|ftp)\:\/\/|\/)/g);
		return rx.test(uri);
	},

	getRelative: function(/*String*/baseUrl, /*String*/url) {
		var baseUrlParts = epages.lang.uri.parts(baseUrl);
		var urlParts = epages.lang.uri.parts(url);

		// protocols or hosts differ
		if ((urlParts.protocol != null && urlParts.protocol != baseUrlParts.protocol) ||
				(urlParts.host != null && urlParts.host != baseUrlParts.host)){
					return url;
		}

		baseUrlParts.path = baseUrlParts.path.replace(/\/+$/, '');
		urlParts.path = urlParts.path.replace(/\/+$/, '');
		var basePathSplitted = baseUrlParts.path.split('/');
		var urlPathSplitted = urlParts.path.split('/');

		// remove equal parts
		while (basePathSplitted.length > 0) {
			var basePathPart = basePathSplitted.shift();
			var urlPathPart = urlPathSplitted.shift();
			if (urlPathPart == null) {
				basePathSplitted.unshift(basePathPart);
				break;
			} else if (basePathPart != urlPathPart) {
				basePathSplitted.unshift(basePathPart);
				urlPathSplitted.unshift(urlPathPart);
				break;
			}
		}
		var result = '';
		for( var i=0,iLength=basePathSplitted.length ; i<iLength ; i++ ) { result += '../'; }
		return result + urlPathSplitted.join('/');
	}

};

}

if(!dojo._hasResource["dojo.date"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.date"] = true;
dojo.provide("dojo.date");

dojo.getObject("date", true, dojo);

/*=====
dojo.date = {
	// summary: Date manipulation utilities
}
=====*/

dojo.date.getDaysInMonth = function(/*Date*/dateObject){
	//	summary:
	//		Returns the number of days in the month used by dateObject
	var month = dateObject.getMonth();
	var days = [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];
	if(month == 1 && dojo.date.isLeapYear(dateObject)){ return 29; } // Number
	return days[month]; // Number
};

dojo.date.isLeapYear = function(/*Date*/dateObject){
	//	summary:
	//		Determines if the year of the dateObject is a leap year
	//	description:
	//		Leap years are years with an additional day YYYY-02-29, where the
	//		year number is a multiple of four with the following exception: If
	//		a year is a multiple of 100, then it is only a leap year if it is
	//		also a multiple of 400. For example, 1900 was not a leap year, but
	//		2000 is one.

	var year = dateObject.getFullYear();
	return !(year%400) || (!(year%4) && !!(year%100)); // Boolean
};

// FIXME: This is not localized
dojo.date.getTimezoneName = function(/*Date*/dateObject){
	//	summary:
	//		Get the user's time zone as provided by the browser
	// dateObject:
	//		Needed because the timezone may vary with time (daylight savings)
	//	description:
	//		Try to get time zone info from toString or toLocaleString method of
	//		the Date object -- UTC offset is not a time zone.  See
	//		http://www.twinsun.com/tz/tz-link.htm Note: results may be
	//		inconsistent across browsers.

	var str = dateObject.toString(); // Start looking in toString
	var tz = ''; // The result -- return empty string if nothing found
	var match;

	// First look for something in parentheses -- fast lookup, no regex
	var pos = str.indexOf('(');
	if(pos > -1){
		tz = str.substring(++pos, str.indexOf(')'));
	}else{
		// If at first you don't succeed ...
		// If IE knows about the TZ, it appears before the year
		// Capital letters or slash before a 4-digit year
		// at the end of string
		var pat = /([A-Z\/]+) \d{4}$/;
		if((match = str.match(pat))){
			tz = match[1];
		}else{
		// Some browsers (e.g. Safari) glue the TZ on the end
		// of toLocaleString instead of putting it in toString
			str = dateObject.toLocaleString();
			// Capital letters or slash -- end of string,
			// after space
			pat = / ([A-Z\/]+)$/;
			if((match = str.match(pat))){
				tz = match[1];
			}
		}
	}

	// Make sure it doesn't somehow end up return AM or PM
	return (tz == 'AM' || tz == 'PM') ? '' : tz; // String
};

// Utility methods to do arithmetic calculations with Dates

dojo.date.compare = function(/*Date*/date1, /*Date?*/date2, /*String?*/portion){
	//	summary:
	//		Compare two date objects by date, time, or both.
	//	description:
	//  	Returns 0 if equal, positive if a > b, else negative.
	//	date1:
	//		Date object
	//	date2:
	//		Date object.  If not specified, the current Date is used.
	//	portion:
	//		A string indicating the "date" or "time" portion of a Date object.
	//		Compares both "date" and "time" by default.  One of the following:
	//		"date", "time", "datetime"

	// Extra step required in copy for IE - see #3112
	date1 = new Date(+date1);
	date2 = new Date(+(date2 || new Date()));

	if(portion == "date"){
		// Ignore times and compare dates.
		date1.setHours(0, 0, 0, 0);
		date2.setHours(0, 0, 0, 0);
	}else if(portion == "time"){
		// Ignore dates and compare times.
		date1.setFullYear(0, 0, 0);
		date2.setFullYear(0, 0, 0);
	}
	
	if(date1 > date2){ return 1; } // int
	if(date1 < date2){ return -1; } // int
	return 0; // int
};

dojo.date.add = function(/*Date*/date, /*String*/interval, /*int*/amount){
	//	summary:
	//		Add to a Date in intervals of different size, from milliseconds to years
	//	date: Date
	//		Date object to start with
	//	interval:
	//		A string representing the interval.  One of the following:
	//			"year", "month", "day", "hour", "minute", "second",
	//			"millisecond", "quarter", "week", "weekday"
	//	amount:
	//		How much to add to the date.

	var sum = new Date(+date); // convert to Number before copying to accomodate IE (#3112)
	var fixOvershoot = false;
	var property = "Date";

	switch(interval){
		case "day":
			break;
		case "weekday":
			//i18n FIXME: assumes Saturday/Sunday weekend, but this is not always true.  see dojo.cldr.supplemental

			// Divide the increment time span into weekspans plus leftover days
			// e.g., 8 days is one 5-day weekspan / and two leftover days
			// Can't have zero leftover days, so numbers divisible by 5 get
			// a days value of 5, and the remaining days make up the number of weeks
			var days, weeks;
			var mod = amount % 5;
			if(!mod){
				days = (amount > 0) ? 5 : -5;
				weeks = (amount > 0) ? ((amount-5)/5) : ((amount+5)/5);
			}else{
				days = mod;
				weeks = parseInt(amount/5);
			}
			// Get weekday value for orig date param
			var strt = date.getDay();
			// Orig date is Sat / positive incrementer
			// Jump over Sun
			var adj = 0;
			if(strt == 6 && amount > 0){
				adj = 1;
			}else if(strt == 0 && amount < 0){
			// Orig date is Sun / negative incrementer
			// Jump back over Sat
				adj = -1;
			}
			// Get weekday val for the new date
			var trgt = strt + days;
			// New date is on Sat or Sun
			if(trgt == 0 || trgt == 6){
				adj = (amount > 0) ? 2 : -2;
			}
			// Increment by number of weeks plus leftover days plus
			// weekend adjustments
			amount = (7 * weeks) + days + adj;
			break;
		case "year":
			property = "FullYear";
			// Keep increment/decrement from 2/29 out of March
			fixOvershoot = true;
			break;
		case "week":
			amount *= 7;
			break;
		case "quarter":
			// Naive quarter is just three months
			amount *= 3;
			// fallthrough...
		case "month":
			// Reset to last day of month if you overshoot
			fixOvershoot = true;
			property = "Month";
			break;
//		case "hour":
//		case "minute":
//		case "second":
//		case "millisecond":
		default:
			property = "UTC"+interval.charAt(0).toUpperCase() + interval.substring(1) + "s";
	}

	if(property){
		sum["set"+property](sum["get"+property]()+amount);
	}

	if(fixOvershoot && (sum.getDate() < date.getDate())){
		sum.setDate(0);
	}

	return sum; // Date
};

dojo.date.difference = function(/*Date*/date1, /*Date?*/date2, /*String?*/interval){
	//	summary:
	//		Get the difference in a specific unit of time (e.g., number of
	//		months, weeks, days, etc.) between two dates, rounded to the
	//		nearest integer.
	//	date1:
	//		Date object
	//	date2:
	//		Date object.  If not specified, the current Date is used.
	//	interval:
	//		A string representing the interval.  One of the following:
	//			"year", "month", "day", "hour", "minute", "second",
	//			"millisecond", "quarter", "week", "weekday"
	//		Defaults to "day".

	date2 = date2 || new Date();
	interval = interval || "day";
	var yearDiff = date2.getFullYear() - date1.getFullYear();
	var delta = 1; // Integer return value

	switch(interval){
		case "quarter":
			var m1 = date1.getMonth();
			var m2 = date2.getMonth();
			// Figure out which quarter the months are in
			var q1 = Math.floor(m1/3) + 1;
			var q2 = Math.floor(m2/3) + 1;
			// Add quarters for any year difference between the dates
			q2 += (yearDiff * 4);
			delta = q2 - q1;
			break;
		case "weekday":
			var days = Math.round(dojo.date.difference(date1, date2, "day"));
			var weeks = parseInt(dojo.date.difference(date1, date2, "week"));
			var mod = days % 7;

			// Even number of weeks
			if(mod == 0){
				days = weeks*5;
			}else{
				// Weeks plus spare change (< 7 days)
				var adj = 0;
				var aDay = date1.getDay();
				var bDay = date2.getDay();

				weeks = parseInt(days/7);
				mod = days % 7;
				// Mark the date advanced by the number of
				// round weeks (may be zero)
				var dtMark = new Date(date1);
				dtMark.setDate(dtMark.getDate()+(weeks*7));
				var dayMark = dtMark.getDay();

				// Spare change days -- 6 or less
				if(days > 0){
					switch(true){
						// Range starts on Sat
						case aDay == 6:
							adj = -1;
							break;
						// Range starts on Sun
						case aDay == 0:
							adj = 0;
							break;
						// Range ends on Sat
						case bDay == 6:
							adj = -1;
							break;
						// Range ends on Sun
						case bDay == 0:
							adj = -2;
							break;
						// Range contains weekend
						case (dayMark + mod) > 5:
							adj = -2;
					}
				}else if(days < 0){
					switch(true){
						// Range starts on Sat
						case aDay == 6:
							adj = 0;
							break;
						// Range starts on Sun
						case aDay == 0:
							adj = 1;
							break;
						// Range ends on Sat
						case bDay == 6:
							adj = 2;
							break;
						// Range ends on Sun
						case bDay == 0:
							adj = 1;
							break;
						// Range contains weekend
						case (dayMark + mod) < 0:
							adj = 2;
					}
				}
				days += adj;
				days -= (weeks*2);
			}
			delta = days;
			break;
		case "year":
			delta = yearDiff;
			break;
		case "month":
			delta = (date2.getMonth() - date1.getMonth()) + (yearDiff * 12);
			break;
		case "week":
			// Truncate instead of rounding
			// Don't use Math.floor -- value may be negative
			delta = parseInt(dojo.date.difference(date1, date2, "day")/7);
			break;
		case "day":
			delta /= 24;
			// fallthrough
		case "hour":
			delta /= 60;
			// fallthrough
		case "minute":
			delta /= 60;
			// fallthrough
		case "second":
			delta /= 1000;
			// fallthrough
		case "millisecond":
			delta *= date2.getTime() - date1.getTime();
	}

	// Round for fractional values and DST leaps
	return Math.round(delta); // Number (integer)
};

}

if(!dojo._hasResource["dojo.cldr.supplemental"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.cldr.supplemental"] = true;
dojo.provide("dojo.cldr.supplemental");


dojo.getObject("cldr.supplemental", true, dojo);

dojo.cldr.supplemental.getFirstDayOfWeek = function(/*String?*/locale){
// summary: Returns a zero-based index for first day of the week
// description:
//		Returns a zero-based index for first day of the week, as used by the local (Gregorian) calendar.
//		e.g. Sunday (returns 0), or Monday (returns 1)

	// from http://www.unicode.org/cldr/data/common/supplemental/supplementalData.xml:supplementalData/weekData/firstDay
	var firstDay = {/*default is 1=Monday*/
		mv:5,
		ae:6,af:6,bh:6,dj:6,dz:6,eg:6,er:6,et:6,iq:6,ir:6,jo:6,ke:6,kw:6,
		ly:6,ma:6,om:6,qa:6,sa:6,sd:6,so:6,sy:6,tn:6,ye:6,
		ar:0,as:0,az:0,bw:0,ca:0,cn:0,fo:0,ge:0,gl:0,gu:0,hk:0,
		il:0,'in':0,jm:0,jp:0,kg:0,kr:0,la:0,mh:0,mn:0,mo:0,mp:0,
		mt:0,nz:0,ph:0,pk:0,sg:0,th:0,tt:0,tw:0,um:0,us:0,uz:0,
		vi:0,zw:0
// variant. do not use?		gb:0,
	};

	var country = dojo.cldr.supplemental._region(locale);
	var dow = firstDay[country];
	return (dow === undefined) ? 1 : dow; /*Number*/
};

dojo.cldr.supplemental._region = function(/*String?*/locale){
	locale = dojo.i18n.normalizeLocale(locale);
	var tags = locale.split('-');
	var region = tags[1];
	if(!region){
		// IE often gives language only (#2269)
		// Arbitrary mappings of language-only locales to a country:
		region = {de:"de", en:"us", es:"es", fi:"fi", fr:"fr", he:"il", hu:"hu", it:"it",
			ja:"jp", ko:"kr", nl:"nl", pt:"br", sv:"se", zh:"cn"}[tags[0]];
	}else if(region.length == 4){
		// The ISO 3166 country code is usually in the second position, unless a
		// 4-letter script is given. See http://www.ietf.org/rfc/rfc4646.txt
		region = tags[2];
	}
	return region;
};

dojo.cldr.supplemental.getWeekend = function(/*String?*/locale){
// summary: Returns a hash containing the start and end days of the weekend
// description:
//		Returns a hash containing the start and end days of the weekend according to local custom using locale,
//		or by default in the user's locale.
//		e.g. {start:6, end:0}

	// from http://www.unicode.org/cldr/data/common/supplemental/supplementalData.xml:supplementalData/weekData/weekend{Start,End}
	var weekendStart = {/*default is 6=Saturday*/
		'in':0,
		af:4,dz:4,ir:4,om:4,sa:4,ye:4,
		ae:5,bh:5,eg:5,il:5,iq:5,jo:5,kw:5,ly:5,ma:5,qa:5,sd:5,sy:5,tn:5
	};

	var weekendEnd = {/*default is 0=Sunday*/
		af:5,dz:5,ir:5,om:5,sa:5,ye:5,
		ae:6,bh:5,eg:6,il:6,iq:6,jo:6,kw:6,ly:6,ma:6,qa:6,sd:6,sy:6,tn:6
	};

	var country = dojo.cldr.supplemental._region(locale);
	var start = weekendStart[country];
	var end = weekendEnd[country];
	if(start === undefined){start=6;}
	if(end === undefined){end=0;}
	return {start:start, end:end}; /*Object {start,end}*/
};

}

if(!dojo._hasResource["dojo.regexp"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.regexp"] = true;
dojo.provide("dojo.regexp");

dojo.getObject("regexp", true, dojo);

/*=====
dojo.regexp = {
	// summary: Regular expressions and Builder resources
};
=====*/

dojo.regexp.escapeString = function(/*String*/str, /*String?*/except){
	//	summary:
	//		Adds escape sequences for special characters in regular expressions
	// except:
	//		a String with special characters to be left unescaped

	return str.replace(/([\.$?*|{}\(\)\[\]\\\/\+^])/g, function(ch){
		if(except && except.indexOf(ch) != -1){
			return ch;
		}
		return "\\" + ch;
	}); // String
};

dojo.regexp.buildGroupRE = function(/*Object|Array*/arr, /*Function*/re, /*Boolean?*/nonCapture){
	//	summary:
	//		Builds a regular expression that groups subexpressions
	//	description:
	//		A utility function used by some of the RE generators. The
	//		subexpressions are constructed by the function, re, in the second
	//		parameter.  re builds one subexpression for each elem in the array
	//		a, in the first parameter. Returns a string for a regular
	//		expression that groups all the subexpressions.
	// arr:
	//		A single value or an array of values.
	// re:
	//		A function. Takes one parameter and converts it to a regular
	//		expression.
	// nonCapture:
	//		If true, uses non-capturing match, otherwise matches are retained
	//		by regular expression. Defaults to false

	// case 1: a is a single value.
	if(!(arr instanceof Array)){
		return re(arr); // String
	}

	// case 2: a is an array
	var b = [];
	for(var i = 0; i < arr.length; i++){
		// convert each elem to a RE
		b.push(re(arr[i]));
	}

	 // join the REs as alternatives in a RE group.
	return dojo.regexp.group(b.join("|"), nonCapture); // String
};

dojo.regexp.group = function(/*String*/expression, /*Boolean?*/nonCapture){
	// summary:
	//		adds group match to expression
	// nonCapture:
	//		If true, uses non-capturing match, otherwise matches are retained
	//		by regular expression.
	return "(" + (nonCapture ? "?:":"") + expression + ")"; // String
};

}

if(!dojo._hasResource["dojo.date.locale"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.date.locale"] = true;
dojo.provide("dojo.date.locale");







dojo.getObject("date.locale", true, dojo);

// Localization methods for Date.   Honor local customs using locale-dependent dojo.cldr data.


// Load the bundles containing localization information for
// names and formats

//NOTE: Everything in this module assumes Gregorian calendars.
// Other calendars will be implemented in separate modules.

(function(){
	// Format a pattern without literals
	function formatPattern(dateObject, bundle, options, pattern){
		return pattern.replace(/([a-z])\1*/ig, function(match){
			var s, pad,
				c = match.charAt(0),
				l = match.length,
				widthList = ["abbr", "wide", "narrow"];
			switch(c){
				case 'G':
					s = bundle[(l < 4) ? "eraAbbr" : "eraNames"][dateObject.getFullYear() < 0 ? 0 : 1];
					break;
				case 'y':
					s = dateObject.getFullYear();
					switch(l){
						case 1:
							break;
						case 2:
							if(!options.fullYear){
								s = String(s); s = s.substr(s.length - 2);
								break;
							}
							// fallthrough
						default:
							pad = true;
					}
					break;
				case 'Q':
				case 'q':
					s = Math.ceil((dateObject.getMonth()+1)/3);
//					switch(l){
//						case 1: case 2:
							pad = true;
//							break;
//						case 3: case 4: // unimplemented
//					}
					break;
				case 'M':
					var m = dateObject.getMonth();
					if(l<3){
						s = m+1; pad = true;
					}else{
						var propM = ["months", "format", widthList[l-3]].join("-");
						s = bundle[propM][m];
					}
					break;
				case 'w':
					var firstDay = 0;
					s = dojo.date.locale._getWeekOfYear(dateObject, firstDay); pad = true;
					break;
				case 'd':
					s = dateObject.getDate(); pad = true;
					break;
				case 'D':
					s = dojo.date.locale._getDayOfYear(dateObject); pad = true;
					break;
				case 'E':
					var d = dateObject.getDay();
					if(l<3){
						s = d+1; pad = true;
					}else{
						var propD = ["days", "format", widthList[l-3]].join("-");
						s = bundle[propD][d];
					}
					break;
				case 'a':
					var timePeriod = (dateObject.getHours() < 12) ? 'am' : 'pm';
					s = options[timePeriod] || bundle['dayPeriods-format-wide-' + timePeriod];
					break;
				case 'h':
				case 'H':
				case 'K':
				case 'k':
					var h = dateObject.getHours();
					// strange choices in the date format make it impossible to write this succinctly
					switch (c){
						case 'h': // 1-12
							s = (h % 12) || 12;
							break;
						case 'H': // 0-23
							s = h;
							break;
						case 'K': // 0-11
							s = (h % 12);
							break;
						case 'k': // 1-24
							s = h || 24;
							break;
					}
					pad = true;
					break;
				case 'm':
					s = dateObject.getMinutes(); pad = true;
					break;
				case 's':
					s = dateObject.getSeconds(); pad = true;
					break;
				case 'S':
					s = Math.round(dateObject.getMilliseconds() * Math.pow(10, l-3)); pad = true;
					break;
				case 'v': // FIXME: don't know what this is. seems to be same as z?
				case 'z':
					// We only have one timezone to offer; the one from the browser
					s = dojo.date.locale._getZone(dateObject, true, options);
					if(s){break;}
					l=4;
					// fallthrough... use GMT if tz not available
				case 'Z':
					var offset = dojo.date.locale._getZone(dateObject, false, options);
					var tz = [
						(offset<=0 ? "+" : "-"),
						dojo.string.pad(Math.floor(Math.abs(offset)/60), 2),
						dojo.string.pad(Math.abs(offset)% 60, 2)
					];
					if(l==4){
						tz.splice(0, 0, "GMT");
						tz.splice(3, 0, ":");
					}
					s = tz.join("");
					break;
//				case 'Y': case 'u': case 'W': case 'F': case 'g': case 'A': case 'e':
//					console.log(match+" modifier unimplemented");
				default:
					throw new Error("dojo.date.locale.format: invalid pattern char: "+pattern);
			}
			if(pad){ s = dojo.string.pad(s, l); }
			return s;
		});
	}

/*=====
	dojo.date.locale.__FormatOptions = function(){
	//	selector: String
	//		choice of 'time','date' (default: date and time)
	//	formatLength: String
	//		choice of long, short, medium or full (plus any custom additions).  Defaults to 'short'
	//	datePattern:String
	//		override pattern with this string
	//	timePattern:String
	//		override pattern with this string
	//	am: String
	//		override strings for am in times
	//	pm: String
	//		override strings for pm in times
	//	locale: String
	//		override the locale used to determine formatting rules
	//	fullYear: Boolean
	//		(format only) use 4 digit years whenever 2 digit years are called for
	//	strict: Boolean
	//		(parse only) strict parsing, off by default
		this.selector = selector;
		this.formatLength = formatLength;
		this.datePattern = datePattern;
		this.timePattern = timePattern;
		this.am = am;
		this.pm = pm;
		this.locale = locale;
		this.fullYear = fullYear;
		this.strict = strict;
	}
=====*/

dojo.date.locale._getZone = function(/*Date*/dateObject, /*boolean*/getName, /*dojo.date.locale.__FormatOptions?*/options){
	// summary:
	//		Returns the zone (or offset) for the given date and options.  This
	//		is broken out into a separate function so that it can be overridden
	//		by timezone-aware code.
	//
	// dateObject:
	//		the date and/or time being formatted.
	//
	// getName:
	//		Whether to return the timezone string (if true), or the offset (if false)
	//
	// options:
	//		The options being used for formatting
	if(getName){
		return dojo.date.getTimezoneName(dateObject);
	}else{
		return dateObject.getTimezoneOffset();
	}
};


dojo.date.locale.format = function(/*Date*/dateObject, /*dojo.date.locale.__FormatOptions?*/options){
	// summary:
	//		Format a Date object as a String, using locale-specific settings.
	//
	// description:
	//		Create a string from a Date object using a known localized pattern.
	//		By default, this method formats both date and time from dateObject.
	//		Formatting patterns are chosen appropriate to the locale.  Different
	//		formatting lengths may be chosen, with "full" used by default.
	//		Custom patterns may be used or registered with translations using
	//		the dojo.date.locale.addCustomFormats method.
	//		Formatting patterns are implemented using [the syntax described at
	//		unicode.org](http://www.unicode.org/reports/tr35/tr35-4.html#Date_Format_Patterns)
	//
	// dateObject:
	//		the date and/or time to be formatted.  If a time only is formatted,
	//		the values in the year, month, and day fields are irrelevant.  The
	//		opposite is true when formatting only dates.

	options = options || {};

	var locale = dojo.i18n.normalizeLocale(options.locale),
		formatLength = options.formatLength || 'short',
		bundle = dojo.date.locale._getGregorianBundle(locale),
		str = [],
		sauce = dojo.hitch(this, formatPattern, dateObject, bundle, options);
	if(options.selector == "year"){
		return _processPattern(bundle["dateFormatItem-yyyy"] || "yyyy", sauce);
	}
	var pattern;
	if(options.selector != "date"){
		pattern = options.timePattern || bundle["timeFormat-"+formatLength];
		if(pattern){str.push(_processPattern(pattern, sauce));}
	}
	if(options.selector != "time"){
		pattern = options.datePattern || bundle["dateFormat-"+formatLength];
		if(pattern){str.push(_processPattern(pattern, sauce));}
	}

	return str.length == 1 ? str[0] : bundle["dateTimeFormat-"+formatLength].replace(/\{(\d+)\}/g,
		function(match, key){ return str[key]; }); // String
};

dojo.date.locale.regexp = function(/*dojo.date.locale.__FormatOptions?*/options){
	// summary:
	//		Builds the regular needed to parse a localized date

	return dojo.date.locale._parseInfo(options).regexp; // String
};

dojo.date.locale._parseInfo = function(/*dojo.date.locale.__FormatOptions?*/options){
	options = options || {};
	var locale = dojo.i18n.normalizeLocale(options.locale),
		bundle = dojo.date.locale._getGregorianBundle(locale),
		formatLength = options.formatLength || 'short',
		datePattern = options.datePattern || bundle["dateFormat-" + formatLength],
		timePattern = options.timePattern || bundle["timeFormat-" + formatLength],
		pattern;
	if(options.selector == 'date'){
		pattern = datePattern;
	}else if(options.selector == 'time'){
		pattern = timePattern;
	}else{
		pattern = bundle["dateTimeFormat-"+formatLength].replace(/\{(\d+)\}/g,
			function(match, key){ return [timePattern, datePattern][key]; });
	}

	var tokens = [],
		re = _processPattern(pattern, dojo.hitch(this, _buildDateTimeRE, tokens, bundle, options));
	return {regexp: re, tokens: tokens, bundle: bundle};
};

dojo.date.locale.parse = function(/*String*/value, /*dojo.date.locale.__FormatOptions?*/options){
	// summary:
	//		Convert a properly formatted string to a primitive Date object,
	//		using locale-specific settings.
	//
	// description:
	//		Create a Date object from a string using a known localized pattern.
	//		By default, this method parses looking for both date and time in the string.
	//		Formatting patterns are chosen appropriate to the locale.  Different
	//		formatting lengths may be chosen, with "full" used by default.
	//		Custom patterns may be used or registered with translations using
	//		the dojo.date.locale.addCustomFormats method.
	//
	//		Formatting patterns are implemented using [the syntax described at
	//		unicode.org](http://www.unicode.org/reports/tr35/tr35-4.html#Date_Format_Patterns)
	//		When two digit years are used, a century is chosen according to a sliding
	//		window of 80 years before and 20 years after present year, for both `yy` and `yyyy` patterns.
	//		year < 100CE requires strict mode.
	//
	// value:
	//		A string representation of a date

	// remove non-printing bidi control chars from input and pattern
	var controlChars = /[\u200E\u200F\u202A\u202E]/g,
		info = dojo.date.locale._parseInfo(options),
		tokens = info.tokens, bundle = info.bundle,
		re = new RegExp("^" + info.regexp.replace(controlChars, "") + "$",
			info.strict ? "" : "i"),
		match = re.exec(value && value.replace(controlChars, ""));

	if(!match){ return null; } // null

	var widthList = ['abbr', 'wide', 'narrow'],
		result = [1970,0,1,0,0,0,0], // will get converted to a Date at the end
		amPm = "",
		valid = dojo.every(match, function(v, i){
		if(!i){return true;}
		var token=tokens[i-1];
		var l=token.length;
		switch(token.charAt(0)){
			case 'y':
				if(l != 2 && options.strict){
					//interpret year literally, so '5' would be 5 A.D.
					result[0] = v;
				}else{
					if(v<100){
						v = Number(v);
						//choose century to apply, according to a sliding window
						//of 80 years before and 20 years after present year
						var year = '' + new Date().getFullYear(),
							century = year.substring(0, 2) * 100,
							cutoff = Math.min(Number(year.substring(2, 4)) + 20, 99),
							num = (v < cutoff) ? century + v : century - 100 + v;
						result[0] = num;
					}else{
						//we expected 2 digits and got more...
						if(options.strict){
							return false;
						}
						//interpret literally, so '150' would be 150 A.D.
						//also tolerate '1950', if 'yyyy' input passed to 'yy' format
						result[0] = v;
					}
				}
				break;
			case 'M':
				if(l>2){
					var months = bundle['months-format-' + widthList[l-3]].concat();
					if(!options.strict){
						//Tolerate abbreviating period in month part
						//Case-insensitive comparison
						v = v.replace(".","").toLowerCase();
						months = dojo.map(months, function(s){ return s.replace(".","").toLowerCase(); } );
					}
					v = dojo.indexOf(months, v);
					if(v == -1){
//						console.log("dojo.date.locale.parse: Could not parse month name: '" + v + "'.");
						return false;
					}
				}else{
					v--;
				}
				result[1] = v;
				break;
			case 'E':
			case 'e':
				var days = bundle['days-format-' + widthList[l-3]].concat();
				if(!options.strict){
					//Case-insensitive comparison
					v = v.toLowerCase();
					days = dojo.map(days, function(d){return d.toLowerCase();});
				}
				v = dojo.indexOf(days, v);
				if(v == -1){
//					console.log("dojo.date.locale.parse: Could not parse weekday name: '" + v + "'.");
					return false;
				}

				//TODO: not sure what to actually do with this input,
				//in terms of setting something on the Date obj...?
				//without more context, can't affect the actual date
				//TODO: just validate?
				break;
			case 'D':
				result[1] = 0;
				// fallthrough...
			case 'd':
				result[2] = v;
				break;
			case 'a': //am/pm
				var am = options.am || bundle['dayPeriods-format-wide-am'],
					pm = options.pm || bundle['dayPeriods-format-wide-pm'];
				if(!options.strict){
					var period = /\./g;
					v = v.replace(period,'').toLowerCase();
					am = am.replace(period,'').toLowerCase();
					pm = pm.replace(period,'').toLowerCase();
				}
				if(options.strict && v != am && v != pm){
//					console.log("dojo.date.locale.parse: Could not parse am/pm part.");
					return false;
				}

				// we might not have seen the hours field yet, so store the state and apply hour change later
				amPm = (v == pm) ? 'p' : (v == am) ? 'a' : '';
				break;
			case 'K': //hour (1-24)
				if(v == 24){ v = 0; }
				// fallthrough...
			case 'h': //hour (1-12)
			case 'H': //hour (0-23)
			case 'k': //hour (0-11)
				//TODO: strict bounds checking, padding
				if(v > 23){
//					console.log("dojo.date.locale.parse: Illegal hours value");
					return false;
				}

				//in the 12-hour case, adjusting for am/pm requires the 'a' part
				//which could come before or after the hour, so we will adjust later
				result[3] = v;
				break;
			case 'm': //minutes
				result[4] = v;
				break;
			case 's': //seconds
				result[5] = v;
				break;
			case 'S': //milliseconds
				result[6] = v;
//				break;
//			case 'w':
//TODO				var firstDay = 0;
//			default:
//TODO: throw?
//				console.log("dojo.date.locale.parse: unsupported pattern char=" + token.charAt(0));
		}
		return true;
	});

	var hours = +result[3];
	if(amPm === 'p' && hours < 12){
		result[3] = hours + 12; //e.g., 3pm -> 15
	}else if(amPm === 'a' && hours == 12){
		result[3] = 0; //12am -> 0
	}

	//TODO: implement a getWeekday() method in order to test
	//validity of input strings containing 'EEE' or 'EEEE'...

	var dateObject = new Date(result[0], result[1], result[2], result[3], result[4], result[5], result[6]); // Date
	if(options.strict){
		dateObject.setFullYear(result[0]);
	}

	// Check for overflow.  The Date() constructor normalizes things like April 32nd...
	//TODO: why isn't this done for times as well?
	var allTokens = tokens.join(""),
		dateToken = allTokens.indexOf('d') != -1,
		monthToken = allTokens.indexOf('M') != -1;

	if(!valid ||
		(monthToken && dateObject.getMonth() > result[1]) ||
		(dateToken && dateObject.getDate() > result[2])){
		return null;
	}

	// Check for underflow, due to DST shifts.  See #9366
	// This assumes a 1 hour dst shift correction at midnight
	// We could compare the timezone offset after the shift and add the difference instead.
	if((monthToken && dateObject.getMonth() < result[1]) ||
		(dateToken && dateObject.getDate() < result[2])){
		dateObject = dojo.date.add(dateObject, "hour", 1);
	}

	return dateObject; // Date
};

function _processPattern(pattern, applyPattern, applyLiteral, applyAll){
	//summary: Process a pattern with literals in it

	// Break up on single quotes, treat every other one as a literal, except '' which becomes '
	var identity = function(x){return x;};
	applyPattern = applyPattern || identity;
	applyLiteral = applyLiteral || identity;
	applyAll = applyAll || identity;

	//split on single quotes (which escape literals in date format strings)
	//but preserve escaped single quotes (e.g., o''clock)
	var chunks = pattern.match(/(''|[^'])+/g),
		literal = pattern.charAt(0) == "'";

	dojo.forEach(chunks, function(chunk, i){
		if(!chunk){
			chunks[i]='';
		}else{
			chunks[i]=(literal ? applyLiteral : applyPattern)(chunk.replace(/''/g, "'"));
			literal = !literal;
		}
	});
	return applyAll(chunks.join(''));
}

function _buildDateTimeRE(tokens, bundle, options, pattern){
	pattern = dojo.regexp.escapeString(pattern);
	if(!options.strict){ pattern = pattern.replace(" a", " ?a"); } // kludge to tolerate no space before am/pm
	return pattern.replace(/([a-z])\1*/ig, function(match){
		// Build a simple regexp.  Avoid captures, which would ruin the tokens list
		var s,
			c = match.charAt(0),
			l = match.length,
			p2 = '', p3 = '';
		if(options.strict){
			if(l > 1){ p2 = '0' + '{'+(l-1)+'}'; }
			if(l > 2){ p3 = '0' + '{'+(l-2)+'}'; }
		}else{
			p2 = '0?'; p3 = '0{0,2}';
		}
		switch(c){
			case 'y':
				s = '\\d{2,4}';
				break;
			case 'M':
				s = (l>2) ? '\\S+?' : '1[0-2]|'+p2+'[1-9]';
				break;
			case 'D':
				s = '[12][0-9][0-9]|3[0-5][0-9]|36[0-6]|'+p3+'[1-9][0-9]|'+p2+'[1-9]';
				break;
			case 'd':
				s = '3[01]|[12]\\d|'+p2+'[1-9]';
				break;
			case 'w':
				s = '[1-4][0-9]|5[0-3]|'+p2+'[1-9]';
				break;
			case 'E':
				s = '\\S+';
				break;
			case 'h': //hour (1-12)
				s = '1[0-2]|'+p2+'[1-9]';
				break;
			case 'k': //hour (0-11)
				s = '1[01]|'+p2+'\\d';
				break;
			case 'H': //hour (0-23)
				s = '1\\d|2[0-3]|'+p2+'\\d';
				break;
			case 'K': //hour (1-24)
				s = '1\\d|2[0-4]|'+p2+'[1-9]';
				break;
			case 'm':
			case 's':
				s = '[0-5]\\d';
				break;
			case 'S':
				s = '\\d{'+l+'}';
				break;
			case 'a':
				var am = options.am || bundle['dayPeriods-format-wide-am'],
					pm = options.pm || bundle['dayPeriods-format-wide-pm'];
				s = am + '|' + pm;
				if(!options.strict){
					if(am != am.toLowerCase()){ s += '|' + am.toLowerCase(); }
					if(pm != pm.toLowerCase()){ s += '|' + pm.toLowerCase(); }
					if(s.indexOf('.') != -1){ s += '|' + s.replace(/\./g, ""); }
				}
				s = s.replace(/\./g, "\\.");
				break;
			default:
			// case 'v':
			// case 'z':
			// case 'Z':
				s = ".*";
//				console.log("parse of date format, pattern=" + pattern);
		}

		if(tokens){ tokens.push(match); }

		return "(" + s + ")"; // add capture
	}).replace(/[\xa0 ]/g, "[\\s\\xa0]"); // normalize whitespace.  Need explicit handling of \xa0 for IE.
}
})();

(function(){
var _customFormats = [];
dojo.date.locale.addCustomFormats = function(/*String*/packageName, /*String*/bundleName){
	// summary:
	//		Add a reference to a bundle containing localized custom formats to be
	//		used by date/time formatting and parsing routines.
	//
	// description:
	//		The user may add custom localized formats where the bundle has properties following the
	//		same naming convention used by dojo.cldr: `dateFormat-xxxx` / `timeFormat-xxxx`
	//		The pattern string should match the format used by the CLDR.
	//		See dojo.date.locale.format() for details.
	//		The resources must be loaded by dojo.requireLocalization() prior to use

	_customFormats.push({pkg:packageName,name:bundleName});
};

dojo.date.locale._getGregorianBundle = function(/*String*/locale){
	var gregorian = {};
	dojo.forEach(_customFormats, function(desc){
		var bundle = dojo.i18n.getLocalization(desc.pkg, desc.name, locale);
		gregorian = dojo.mixin(gregorian, bundle);
	}, this);
	return gregorian; /*Object*/
};
})();

dojo.date.locale.addCustomFormats("dojo.cldr","gregorian");

dojo.date.locale.getNames = function(/*String*/item, /*String*/type, /*String?*/context, /*String?*/locale){
	// summary:
	//		Used to get localized strings from dojo.cldr for day or month names.
	//
	// item:
	//	'months' || 'days'
	// type:
	//	'wide' || 'narrow' || 'abbr' (e.g. "Monday", "Mon", or "M" respectively, in English)
	// context:
	//	'standAlone' || 'format' (default)
	// locale:
	//	override locale used to find the names

	var label,
		lookup = dojo.date.locale._getGregorianBundle(locale),
		props = [item, context, type];
	if(context == 'standAlone'){
		var key = props.join('-');
		label = lookup[key];
		// Fall back to 'format' flavor of name
		if(label[0] == 1){ label = undefined; } // kludge, in the absence of real aliasing support in dojo.cldr
	}
	props[1] = 'format';

	// return by copy so changes won't be made accidentally to the in-memory model
	return (label || lookup[props.join('-')]).concat(); /*Array*/
};

dojo.date.locale.isWeekend = function(/*Date?*/dateObject, /*String?*/locale){
	// summary:
	//	Determines if the date falls on a weekend, according to local custom.

	var weekend = dojo.cldr.supplemental.getWeekend(locale),
		day = (dateObject || new Date()).getDay();
	if(weekend.end < weekend.start){
		weekend.end += 7;
		if(day < weekend.start){ day += 7; }
	}
	return day >= weekend.start && day <= weekend.end; // Boolean
};

// These are used only by format and strftime.  Do they need to be public?  Which module should they go in?

dojo.date.locale._getDayOfYear = function(/*Date*/dateObject){
	// summary: gets the day of the year as represented by dateObject
	return dojo.date.difference(new Date(dateObject.getFullYear(), 0, 1, dateObject.getHours()), dateObject) + 1; // Number
};

dojo.date.locale._getWeekOfYear = function(/*Date*/dateObject, /*Number*/firstDayOfWeek){
	if(arguments.length == 1){ firstDayOfWeek = 0; } // Sunday

	var firstDayOfYear = new Date(dateObject.getFullYear(), 0, 1).getDay(),
		adj = (firstDayOfYear - firstDayOfWeek + 7) % 7,
		week = Math.floor((dojo.date.locale._getDayOfYear(dateObject) + adj - 1) / 7);

	// if year starts on the specified day, start counting weeks at 1
	if(firstDayOfYear == firstDayOfWeek){ week++; }

	return week; // Number
};

}

if(!dojo._hasResource["epages.localize"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["epages.localize"] = true;
/*
	Copyright (c) 2006-2007, ePages GmbH
	All Rights Reserved.
*/
dojo.provide("epages.localize");








(function(){

epages.localize = {
	// summary: formats a number by using formatting chars from the user's locale
	formatNumber: function(/*Float*/value, /*Integer?*/decimalDigits) {
		var negativeString = (value < 0) ? '-' : '';
		var valueAbs = Math.abs(value);

		if (decimalDigits == null){
			decimalDigits = 2;
		}
		var precision = Math.pow(10, decimalDigits);
		var intPart = Math.floor(Math.round(valueAbs * precision) / precision);
		var ratPart = Math.round(Math.abs(valueAbs - intPart) * precision);
		if (ratPart != 0) {
			return negativeString +intPart + epages.vars.Locale.decimalSep + ratPart;
		} else {
			return negativeString+intPart;
		}
	},

	formatBytes: function(bytes, decimalDigits) {
		if (decimalDigits == null){
			decimalDigits = 2;
		}
		var unit = "Byte";
		if (bytes >= 1073741824) {
			// GB
			bytes /= 1073741824;
			unit = "GB";
		} else if (bytes >= 1048576) {
			// MB
			bytes /= 1048576;
			unit = "MB";
		} else if (bytes >= 1024) {
			// kB
			bytes /= 1024;
			unit = "kB";
		}
		return epages.localize.formatNumber(bytes, decimalDigits) + " " + unit;
	},

  // assign epages pattern elements to dojo pattern elements
  // see http://www.unicode.org/reports/tr35/tr35-4.html#Date_Format_Patterns for more
  // information
  // IMPORTANT! Only patterns that are supported by epages (existing Locales) are defined
	_patternCache: $H(),
	_patternMap: {
	    // -- YEAR -- //
 	    // full year, 2007
	    '%{ce_year}': 'yyyy',
	    '%Y': 'yyyy',
 	    // short year, 2 digits: 07
	    '%y': 'yy',
	    // abbreviated era (BC, AD)
	    '%{era}': 'G',

	    // -- MONTH -- //
 	    // localized full month name: Oktober
	    '%B': 'MMMM',
 	    // localized month abbreviation: Okt
	    '%b': 'MMM',
 	    // month, 2 digits: 05 (may)
	    '%m': 'MM',
 	    // month, 5
	    '%{month}': 'M',

	    // -- WEEK -- //
	    // day of the week in the month
	    '%{weekday_of_month}': 'F',
	    // week of the month
	    '%{week_month}': 'W',
	    // week of the year, range 01 to 53
	    '%V': 'w',

	    // -- DAY -- //
	    // day, 2 digits, leading zero: '07'
	    '%d': 'dd',
	    // day, 2 digits, leading zero: '07'
	    '%{day}': 'd',
 	    // localized weekday name: Montag
	    '%A': 'EEEE',
	    // abbreviated weekday name
	    '%a': 'E',
	    // day of the year as decimal number
	    '%j': 'D',
	    
	    // -- HOUR -- //
 	    // hour, 24h, 2 digits, leading zero: '09'
	    '%H': 'HH',
 	    // hour, 24h, 2 digits, leading blanks
	    '%k': 'k',
 	    // hour, 24h
	    '%{hour}': 'H',
 	    // hour, 12h
	    '%{hour_12}': 'h',
	    '%l': 'h',
 	    // AM or PM, upper case: AM
	    '%p': 'a',

	    // -- MINUTE -- //
 	    // minutes, 2 digits, leading zero: '05'
	    '%M': 'mm',

	    // -- SECOND -- //
	    // seconds, 2 digits, leading zero: '05'
	    '%S': 'ss',
	    //fractional seconds digits
	    '%N': 'S',

	    // -- TIME ZONE -- //
 	    // time zone, '+0100'
    	    '%{time_zone_long_name}': 'ZZ',
    	    '%z': 'ZZ'
	},


	getDojoDateTimePattern: function(epagesPattern) {
		// summary: converts an epages date pattern to a Dojo date pattern.
	  // note: this function does not support all declared specifiers
		if (epagesPattern == null){
			epagesPattern = epages.vars.Locale.dateFormat;
		}
		if (this._patternCache.existsKey(epagesPattern)){
			return this._patternCache.get(epagesPattern); //String
		}
	  var dojoPattern = epagesPattern;
		// relace elements in string with values
		var elementPatterns = $A($A($H(epages.localize._patternMap).keys()).sort(function(a, b) { return b.length - a.length; }));

		elementPatterns.each(function(element) {
		  var rx = new RegExp(element, 'g');
		  dojoPattern = dojoPattern.replace(rx, function() {
		    return epages.localize._patternMap[element];
		  });
		});
		return dojoPattern; //String
	},

	formatDateTime: function(/*Date*/date, /*String?*/epagesPattern) {
	  // summary: replaces the pattern specifiers with real values
		if (date == null){
			return '';
		}
		if (epagesPattern == null){
			epagesPattern = epages.vars.Locale.dateFormat;
		}
		var d = dojo.date.locale.format(date, {
		  datePattern: this.getDojoDateTimePattern(epagesPattern),
		  timePattern: ' '
		});
		// trim last to chars --> spaces
		return d.substring(0, d.length - 2);
	},

	parseDateTime: function (/*String*/ dateString, /*String*/ epagesPattern) {
		if (epagesPattern == null){
			epagesPattern = epages.vars.Locale.dateFormat;
		}
		return dojo.date.locale.parse('~ ' + dateString, {
			selector: 'timeOnly',
			timePattern: this.getDojoDateTimePattern(epagesPattern),
			datePattern: '~'
		});
	},

	parseNumber: function(/*String*/ numberString){
		var thousandExp = new RegExp( dojo.regexp.escapeString(epages.vars.Locale.thousandSep), 'g' ),
			decimalExp = new RegExp( dojo.regexp.escapeString(epages.vars.Locale.decimalSep), 'g' );

		return parseFloat( numberString.replace(thousandExp,'').replace(decimalExp,'.') );
	},
	getFormat: function(type) {
		var ret = _format[type];

		if( !ret ){
			switch(type){
				case 'date':
					ret = _format[type] = epages.localize.getDojoDateTimePattern( epages.vars.Locale.dateFormat );
				break;
				case 'datetime':
					ret = _format[type] = epages.localize.getDojoDateTimePattern( epages.vars.Locale.datetimeFormat );
				break;
				case 'time':
					ret = _format[type] = epages.localize.getDojoDateTimePattern( epages.vars.Locale.timeFormat );
				break;
				case 'number':
					ret = _format[type] = dojo.regexp.escapeString('-? ?[0-9'+_separator.thousand+']+['+_separator.decimal+']?[0-9]*', '([]{}+?*)');
				break;
				case 'money':
					ret = _format[type] = dojo.regexp.escapeString('(['+_separator.money+']? ?-? ?[0-9'+_separator.thousand+']{1,}['+_separator.decimal+']?[0-9]{0,2}|-? ?[0-9'+_separator.thousand+']{1,}['+_separator.decimal+']?[0-9]{0,2} ?['+_separator.money+']?)', '([|]{}?)')
				break;
			}
		}

		return ret;
	},
	getPattern: function(type) {
		var ret = _pattern[type];

		if( !ret ){
			switch(type){

				case 'date':
				case 'datetime':
				case 'time':
				case 'month':
				case 'week':
					ret = _pattern[type] = dojo.date.locale.regexp({ selector:'date',	datePattern: this.getFormat(type),		timePattern:'' })
				break;
				case 'number':
					ret = _pattern[type] = dojo.regexp.escapeString('-? ?[0-9'+epages.vars.Locale.thousandSep+']+['+epages.vars.Locale.decimalSep+']?[0-9]*', '([]{}+?*)');
				break;
				case 'money':
					ret = _pattern[type] = dojo.regexp.escapeString('(['+epages.vars.Locale.moneySymbol||''+']? ?-? ?[0-9'+epages.vars.Locale.thousandSep+']{1,}['+epages.vars.Locale.decimalSep+']?[0-9]{0,2}|-? ?[0-9'+epages.vars.Locale.thousandSep+']{1,}['+epages.vars.Locale.decimalSep+']?[0-9]{0,2} ?['+epages.vars.Locale.moneySymbol||''+']?)', '([|]{}?)')
				break;
			}
		}

		return ret;
	},
	getSeparator: function(type) {
		return _separator[type];
	},
	date2obj: function( input , type ) {
		return dojo.date.locale.parse(
					input,
					{
						selector: 		'date',
						datePattern: 	_format[type],
						timePattern: 	''
					}
				);
	},
	obj2date: function( input , type ) {
		return dojo.date.locale.format(
					input,
					{
						selector: 		'date',
						datePattern: 	_format[type],
						timePattern: 	''
					}
				);
	},
	date2unix: function( input , type ) {
		var ret = epages.localize.date2obj( input , type );
		return ret ? ( ret.getTime() / 1000 ) : null;
	},
	unix2date: function( input , type ) {
		return epages.localize.obj2date( new Date(input*1000) , type );
	},
	str2number: function( input ) {
		var ret = parseFloat(
						(input+'')
							.replace( new RegExp( dojo.regexp.escapeString('[^0-9-' +epages.vars.Locale.decimalSep + ']', '^([]{})$'), 'g') , '' )
							.replace( new RegExp( dojo.regexp.escapeString('[' + epages.vars.Locale.decimalSep + ']', 	'^([]{})$'), 'g') , '.' )
					);

		return isNaN(ret) ? null : ret;
	},
	number2str: function( input ) {
		return (input+'').replace( /\./g , _separator.decimal );
	}
};

// private vars for use in epages.localize
var _format = {
		month:			'MM (yyyy)',
		week:			'w (yyyy)'
	},
	_pattern = {
		space: 			'\ \n\r\t\
',
		alpha: 			'A-Za-z',
		digit: 			'0-9',
		email:			dojo.regexp.escapeString('[A-Za-z0-9._-]{1,}@[A-Za-z0-9._-]{2,}[A-Za-z]{2,4}', '([]{})'),
		url: 			'([A-Z]:\\[^/:\*\?<>\|]+\.\w{2,6})|(\\{2}[^/:\*\?<>\|]+\.\w{2,6})',
		tel: 			'[0-9\/\+\-\.\ ]{0,}',
		integer:		'-?[0-9]*'
	};

})();

}

if(!dojo._hasResource['epages.io.json']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['epages.io.json'] = true;
/*
	Copyright (c) 2006-2007, ePages GmbH
	All Rights Reserved.
*/
dojo.provide('epages.io.json');

epages.io.Json = function () {};
dojo.extend(epages.io.Json, {

	defaultErrorCallback: function(error,args) {
  	var response = args.xhr.status == 404 ? {} : dojo.fromJson(args.xhr.responseText);
    console.warn('epages.io.Json: error in request');

		if (response.Errors !== undefined) {
			dojo.publish("epages/interfaceerror", [{
				code:	    response.Errors[0].Reason,
				message:  response.Errors[0].Message,
				guid:     response.Errors[0].GUID,
				vars:     {
	          				url:		  args.url,
	          				status:	  args.xhr.status,
	          				response: response
	          		  }
			}]);
		}
	},

	loadSync: function (/*String*/url, /*object?*/formData, /*Function?*/errorCallback, /*Boolean*/preventCache) {
		// summary:
		//		returns a javascript object, which was loaded via http from given url.
		// 		additional parameters will be added in the post.
		// example:
		//		JsonLoader = new epages.io.Json();
		//		Answer = JsonLoader.loadSync('http://www.mydomain.de/test.Json', { filename: "test.Json" });
		//		// returns JsonDocument object
		var returnData;
		var requestMethod = formData !== undefined ? dojo.xhrPost : dojo.xhrGet;
		var _this = this;
		var opt = {
			url: 						url,
			sync: 					true,
			preventCache:		preventCache !== undefined ? preventCache : epages.constants.preventCache,
			error:					function(error,args) {
												returnData = {
													'error': {
														error: error,
														args: args,
														data: dojo.fromJson(args.xhr.responseText)
													}
												};
												if (errorCallback != null){
													errorCallback({'error':[error,args]});
												}else{
													_this.defaultErrorCallback(error,args);
												}
											},
			load: 					function(data, evt){
												var result = dojo.fromJson(data);
												if(result.Errors){
													returnData = {
														'error': {
															data: result
														}
													};
												}
												else{
													returnData = { 'data': dojo.fromJson(data) };
												}
											}
		};
		// test cases doesnt allow posts
		if(url.match('\.(js|json)$')) {
			requestMethod = dojo.xhrGet;
		} else {
			dojo.mixin(opt, {
				headers:	{ Accept: 'text/x-json' },
				content:	formData
			});
		}
		requestMethod(opt);
		return returnData; /*Object*/
	},

	loadAsync: function (/*String*/url, /*Function*/callback, /*object?*/formData, /*Function?*/errorCallback) {
		// summary:
		//		loads a document via http from given url.
		//		the function doesnt wait for the result.
		//		if the result is available the callback function is called with the data.
		// example:
		//		JsonLoader = new epages.io.Json();
		//		JsonDoc = JsonLoader.loadAsync('http://www.mydomain.de/test.Json', function (data) { /* do what you want */ });
		//		// returns JsonDocument object
		var requestMethod = formData !== undefined ? dojo.xhrPost : dojo.xhrGet;
		var _this = this;
		var opt = {
			url: 			url,
			error:		function(error,args) {
									var c = errorCallback || callback;
									var response = dojo.fromJson(args.xhr.responseText);
									if (c) {
										c({
											'error': [ error, args ],
											'data': (typeof response == "object") ? dojo.fromJson(args.xhr.responseText) : {}
										});
									} else {
										_this.defaultErrorCallback(error,args);
									}
								},
			load: 		function(data, ioArgs) {
									if (callback != null){
										var result = dojo.fromJson(data);
										if(result.Errors){
											callback({
												'error': {
													data: result
												}
											});
										} else {
											callback({
												'data': result,
												'ioArgs': ioArgs
											});
										}
									}
								}
		};
		// test cases doesnt allow posts
		if(url.match('\.(js|json)$')) {
			requestMethod = dojo.xhrGet;
		} else {
			dojo.mixin(opt, {
				headers:	{ Accept: 'text/x-json' },
				content:	formData
			});
		}
		requestMethod(opt);
		return;
	}
});

}

if(!dojo._hasResource["epages.format"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["epages.format"] = true;
/*
	Copyright (c) 2006-2008, ePages GmbH
	All Rights Reserved.
*/
dojo.provide("epages.format");


epages.format = {

	slice: function(/*String*/value,/* integer */ length, /* boolean? */ sliceWords) {
	// summary: returns a truncated string
		if (value === undefined){
			 throw new Error("value not defined");
		}
		if (typeof value != "string"){
			throw new Error("Not a string value!");
		}
		if(!length) { 
			length = 30; 
		}
		if(!sliceWords === undefined) { 
			sliceWords= true; 
		}
		
		var valueLength = value.length;
		var absLength = length < 0 ? -length : length;
		if(valueLength < absLength){
			return value;
		}
		if (length < 0) {
				// start of new Position 70 + (- 60) + 3 = 10 + 3 = 13
				// length of new string 3 + (70 - 13) = 3 + 57 = 60
				value = value.substr((valueLength+length+3),(absLength-3));
				if(!sliceWords) {
					var firstspace = value.indexOf(" ");
					value = value.substr(firstspace,value.length);
				}
				return '...'+ value;
		}
		value = value.substr(0,length - 3);
		if(!sliceWords) {
			var lastspace = value.lastIndexOf(" ");
			value = value.substr(0,lastspace+1);
		}
		return value+'...';
	},

	scrunch: function(/*String*/ value, /* integer? */ length, /* float? */ ratio) {
		if(ratio=== undefined){
			ratio = 0.5;
		}
		if(ratio < 0 || ratio > 1){
			throw new Error("ratio value invalid only values from 0..1 allowed");
		}
		if (value === undefined){
			throw new Error("value not defined");
		}
		if (typeof value != "string"){
			throw new Error("Not a string value!");
		}
		if(!length){
			length = 30;
		}
		var absLength = length < 0 ? -length : length;
		var valueLength = value.length;
		if(valueLength <= absLength){
			return value;
		}

		var part1Length = Math.floor((length - 3) * ratio);
		var part2Length = Math.floor((length - 3) * (1 - ratio));
		var remainder = length - 3 - (part1Length + part2Length);

		var part1= value.substr(0, part1Length );
		var part2= value.substr(Math.max(0, valueLength - part2Length - remainder), valueLength);
		return part1+'...'+part2;
	}

};

}

if(!dojo._hasResource["epages.storage"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["epages.storage"] = true;
/*
	Copyright (c) 2006-2007, ePages GmbH
	All Rights Reserved.
*/

dojo.provide("epages.storage");



(function(dj){
		  
	var trim = function(string) {
			return (string+'').replace(/[\ \r\n\t]*(.*)[\ \r\n\t]*/,"$1");
		};	
	
	var storageId = function( name ) {
			return 'SiteID::'+epages.vars.SiteID+'::'+name;
		},
		domain = 			'; domain=' + document.domain,
		path = 				'; path=/',
		
		isLocalStorage = 	window.localStorage && window.localStorage.setItem ? true : false,
		isSessionStorage = 	window.sessionStorage && window.sessionStorage.setItem ? true : false,
		
		setCookie = function(name, value, expires) {
			if( typeof expires == 'number' ) {
				var date = new Date();
					date.setDate( expires );
				expires = '; expires=' + date.toUTCString();
			}
			else{
				expires = '';
			}
			document.cookie = [name, '=', dj.toJson(value), expires, path, domain].join('');
			return value;
		},
		
		getCookie = function(name) {
			var value = null;
			if (document.cookie && document.cookie != '') {
				var cookies = document.cookie.split(';');
				for (var i = 0; i < cookies.length; i++) {
					var cookie = trim(cookies[i]);
					// Does this cookie string begin with the name we want?
					if (cookie.substring(0, name.length + 1) == (name + '=')) { 
						value = dj.fromJson(cookie.substring(name.length + 1));
						break;
					}
				}
			}
			return value;
		},
		
		removeCookie = function( name ) {
			return setCookie( name, null, -365);
		},
		
		clearCookie = function( prefix ) {
			if (document.cookie && document.cookie != '') {
				var cookies = document.cookie.split(';');
				for (var i = 0; i < cookies.length; i++) {
					var cookie = trim(cookies[i]);
					if( cookie.match(new RegExp('^'+prefix,'g')) ) {
						removeCookie( cookie.substr(0,cookie.indexOf('=')) ) 
					}
				}
			}
			return null;
		};
		
	//simulate the storage fallback:
	//isLocalStorage = false;
	//isSessionStorage = false;
		
	
	dj.mixin(epages,{
		localStorage: {
			setItem: function( name, value ) {
				return isLocalStorage ? window.localStorage.setItem( storageId(name), dj.toJson(value) ) : setCookie( 'localStorage::'+storageId(name), value, 365 );
			},
			getItem: function( name ) {
				if( isSessionStorage ) {
					var ret = window.localStorage.getItem( storageId(name) );
					return ret ? dj.fromJson( ret ) : ret;
				}
				else {
					return  getCookie( 'localStorage::'+storageId(name) );
				}
			},
			removeItem: function( name ) {
				return isLocalStorage ? window.localStorage.removeItem( storageId(name) ) : removeCookie( 'localStorage::'+storageId(name) );
			},
			clear: function( name ) {
				return isLocalStorage ? window.localStorage.clear() : clearCookie('localStorage::'+siteId());
			}
		},
		sessionStorage: {
			setItem: function( name, value ) {
				return isSessionStorage ? window.sessionStorage.setItem( storageId(name), dj.toJson(value) ) : setCookie( 'sessionStorage::'+storageId(name), value );
			},
			getItem: function( name ) {
				if( isSessionStorage ) {
					var ret = window.sessionStorage.getItem( storageId(name) );
					return ret ? dj.fromJson( ret ) : ret;
				}
				else {
					return  getCookie( 'sessionStorage::'+storageId(name) );
				}
			},
			removeItem: function( name ) {
				return isSessionStorage ? window.sessionStorage.removeItem( storageId(name) ) : removeCookie( 'sessionStorage::'+storageId(name) );
			},
			clear: function( name ) {
				return isSessionStorage ? window.sessionStorage.clear() : clearCookie('sessionStorage::'+siteId());
			}
		}
	});

})(dojo);

}

if(!dojo._hasResource["epages.loaderani"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["epages.loaderani"] = true;
/*
	Copyright (c) 2006-2010, ePages GmbH
	All Rights Reserved.
*/
dojo.provide("epages.loaderani");

epages.loaderani = {

	// summary: shows a loading animation
	// example:
	// | //	to create an animation:
  // | window.aniNode = epages.loaderani.create({width:16, height:16, left:0, top:0, zIndex:1});
  // | //	or
  // | window.aniNode = epages.loaderani.create();
  // | // to remove that animation:
  // | epages.loaderani.destroy(window.aniNode);

	defaultWidth: 50,
	defaultHeight: 50,
	defaultLeft: -9999,
	defaultTop: -9999,
	defaultZIndex: 1
}


epages.loaderani.create = function(/* object */ dimensions, /* dom Node ?*/ domNode) {
// summary: moves the loader animation to the right position
// description:
// animation has to be inside the DOM with id LoaderAniNode already
// dimensions
// | width - integer
// | height - integer
// | top - integer
// | left - integer
// | zIndex - integer
	dimensions = dimensions || {};
	dimensions.width = dimensions.width || this.defaultWidth;
	dimensions.height = dimensions.height || this.defaultHeight;
	dimensions.left = dimensions.left || this.defaultLeft;
	dimensions.top = dimensions.top || this.defaultTop;
	dimensions.zIndex = dimensions.zIndex || this.defaultZIndex;
	var LoaderAniNode = $('LoaderAniNode'); //find existing animation
	if(LoaderAniNode) {
		LoaderAniNode.style.left = dimensions.left + "px";
		LoaderAniNode.style.top = dimensions.top + "px";
		LoaderAniNode.style.zIndex = dimensions.zIndex;
		if(LoaderAniNode.getElementsByTagName("object") && LoaderAniNode.getElementsByTagName("object")[0]) {
			LoaderAniNode.getElementsByTagName("object")[0].width = dimensions.width;
			LoaderAniNode.getElementsByTagName("object")[0].height = dimensions.height;
		}
	}
	return LoaderAniNode;
};

epages.loaderani.destroy = function(aniNode) {
// summary: moves the loader animation out of the visible area
		var LoaderAniNode = $('LoaderAniNode');
		if(LoaderAniNode){
			LoaderAniNode.style.left = "-9999px";
			LoaderAniNode.style.top = "-9999px";
		}
};

}

if(!dojo._hasResource["epages.event.interactionrestrictor"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["epages.event.interactionrestrictor"] = true;
/*
	Copyright (c) 2006-2007, ePages GmbH
	All Rights Reserved.
*/
dojo.provide("epages.event.interactionrestrictor");



epages.event.interactionRestrictor = function(/*Object?*/ position) {
	// summary: busylayer for a specific dom element
	// position: specifies an alternative position where the overlay is displayed 
	// example:
	//  | dojo.publish("interactionRestrictor/suspend", ["layerId", domNode, (hideLayer)]);
	//  | dojo.publish("interactionRestrictor/permit", ["layerId"]);

	dojo.subscribe("interactionRestrictor/suspend", function(layerId, domNode, hideLayer,position) {
		if(!$(layerId) && domNode && domNode.offsetWidth) {
			dojo.body().className += " SuspendUserInteraction";
			var suspendAnimation = document.createElement("div");
			if(!position){
				var suspendPosition = new epages.html.Element(domNode).getAbsolutePosition(dojo.body());
				var position = {
					Width  : domNode.offsetWidth,
					Height : domNode.offsetHeight,
					Left   : suspendPosition.x,
					Top    : suspendPosition.y
				}
			}
			suspendAnimation.id = layerId;
			suspendAnimation.className = "SuspendUserInteractionAnimation";
			suspendAnimation.style.width = position.Width + "px";
			suspendAnimation.style.height = position.Height + "px";
			suspendAnimation.style.left = position.Left + "px";
			suspendAnimation.style.top = position.Top + "px";
			suspendAnimation.style.zIndex = 11000;
			if(hideLayer) {
				dojo.addClass(suspendAnimation, "SuspendUserInteractionHidden");
			}else if(domNode.offsetHeight > 0) {
				suspendAnimation.loaderani = epages.loaderani.create({width:16,height:16,top:(position.Top + position.Height / 2 - 8),left:(position.Left + position.Width / 2 - 8),zIndex:11001});
			}
			dojo.body().appendChild(suspendAnimation);
		}
	});
	dojo.subscribe("interactionRestrictor/permit", function(layerId) {
		if($(layerId)) {
			dojo.removeClass(dojo.body(), "SuspendUserInteraction");
			epages.loaderani.destroy($(layerId).loaderani);
			dojo.body().removeChild($(layerId));
		}
	});
};

new epages.event.interactionRestrictor;

}

if(!dojo._hasResource['epages.mimetype']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['epages.mimetype'] = true;
/*
 * @charset			utf-8		(ÄÖÜ äöü ß)
 *
 * Copyright (c) 2006-2010, epages GmbH
 * All Rights Reservedojo.
 * epages.mimetype
 */

dojo.provide('epages.mimetype');

(function(dojo) {

	dojo.declare('epages.Mimetype',null,{
	// summary: 		mimetype and filetype handling
	// description: 	privat class
	// TODO:			improve this class and make it to a public class
	//					by preserving the epages.mimetype API

		/*
		 * private
		 */
		 _MIMETYPE_MAP: {
			// image
			'image/png':																	['png' ],
			'image/jpeg':																	['jpe', 'jpeg', 'jpg' ],
			'image/gif':																	['gif' ],
			'image/bmp':																	['bmp' ],
			'image/vnd.microsoft.icon':														['ico' ],
			'image/tiff':																	['tif', 'tif' ],
			'image/svg+xml':																['svg', 'svgz' ],
			'image/x-win-bitmap':															['cur' ],
			// audio/video
			'audio/basic':																	['au'],
			'audio/mpeg':																	['mp3' ],
			'audio/mp4a-latm':																['m4p', 'm4b', 'm4a' ],
			'audio/x-wav':																	['wav' ],
			'audio/ogg':																	['ogg', 'oga', 'spx' ],
			'audio/flac':																	['flac' ],
			'audio/x-ms-wma':																['wma' ],
			'audio/x-pn-realaudio':															['ra','ram' ],
			'application/vnd.rn-realmedia':													['rm' ],
			'video/ogg':																	['ogv' ],
			'video/mp4':																	['mp4' ],
			'video/mpeg':																	['mpe', 'mpg', 'mpeg', 'mpga' ],
			'video/quicktime':																['qt', 'mov' ],
			'video/x-msvideo':																['avi' ],
			'video/x-flv':																	['flv' ],
			'video/x-ms-wmv':																['wmv'],
			// web
			'text/plain':																	['txt' ],
			'text/html':																	['htm', 'html', 'shtml', 'php', 'php4' ],
			'text/css':																		['css' ],
			'text/x-actionscript':															['as'],
			'application/javascript':														['js' ],
			'application/json':																['json' ],
			'application/xml':																['xml', 'plist' ],
			'application/x-shockwave-flash':												['swf' ],
			// archives
			'application/zip':																['zip' ],
			'application/gzip':																['gz' ],
			'application/x-tar':															['tar' ],
			'application/x-rar-compressed':													['rar' ],
			'application/x-msdownload':														['exe', 'msi' ],
			'application/vnd.ms-cab-compressed':											['cab' ],
			// adobe
			'application/pdf':																['pdf' ],
			'image/vnd.adobe.photoshop':													['psd' ],
			'application/postscript':														['ai', 'eps', 'ps' ],
			// ms office
			'application/msword':															['doc' ],
			'application/x-dot':															['dot' ],
			'application/rtf':																['rtf' ],
			'application/vnd.ms-excel':														['xls','xla' ],
			'application/vnd.ms-powerpoint':												['ppt' ],
			'application/vnd.openxmlformats-officedocument.wordprocessingml.document': 		['docx'],
			'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet':			['xlsx'],
			//
			'application/x-director':														['cdr']
		},

		_MIMETYPE_GROUPS: {
			'*/*':				['*/*' ],
			'icon/*':			['image/vnd.microsoft.icon', 'image/x-win-bitmap' ],
			'image/*':			['image/png', 'image/jpeg', 'image/gif' ],
			'document/*':		['application/pdf', 'application/msword', 'application/vnd.ms-excel', 'application/rtf', 'application/vnd.openxmlformats-officedocument.wordprocessingml.document', 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet' ],
			'audio/*':			['audio/mpeg', 'audio/x-ms-wma', 'audio/ogg', 'audio/flac', 'audio/x-ms-wma' ],
			'video/*':			['video/x-msvideo', 'video/mp4', 'video/mpeg', 'video/x-ms-wmv', 'video/x-flv' ]
		},

		_MIMETYPE_PREVIEWABLE: 	['image/png', 'image/jpeg', 'image/gif', 'image/vnd.microsoft.icon' ],

		_ICON_MAP: {
			'ico_l_image':		['image/png','image/jpeg','image/gif','image/bmp','image/vnd.microsoft.icon','image/tiff','image/svg+xml','image/x-win-bitmap'],
			'ico_xl_box':		[this._UNKNOW,'text/x-actionscript','audio/basic','audio/x-wav','audio/mpeg','audio/x-ms-wma','audio/ogg','audio/flac','application/msword','application/x-dot','application/vnd.ms-excel','application/vnd.ms-powerpoint','text/plain','text/css','application/javascript','application/json','application/xml'],
			'ico_xl_file_avi':	['video/x-msvideo','video/mpeg','video/mp4'],
			'ico_xl_file_wmv':	['video/x-ms-wmv'],
			'ico_xl_file_swf':	['video/x-flv','application/x-shockwave-flash'],
			'ico_xl_file_mov':	['video/quicktime','video/ogg'],
			'ico_xl_file_rm':	['application/vnd.rn-realmedia'],
			'ico_xl_file_ram':	['audio/x-pn-realaudio'],
			'ico_xl_file_dcr':	['application/x-director'],
			'ico_xl_file_pdf':	['application/pdf'],
			'ico_xl_file_htm':	['text/html'],
			'ico_m_folder':		['application/zip','application/x-rar-compressed','application/gzip','application/x-tar']
		},

		_UNKNOW: 'application/octet-stream',

		_mimes: null,

		/*
		 * public
		 */

		/*
		 * init instance
		 */
		constructor: function( mime ) {
			this.setMime( mime || this._UNKNOW )
		},

		/*
		 * methods
		 */
		setBySuffix: function( suffix ) {
			suffix = suffix.toLowerCase();
			this._mime = this._UNKNOW;

			var map = this._MIMETYPE_MAP;

			for( var i in map ) {
				if( dojo.indexOf(map[i], suffix)>=0 ) {
					this._mime = i;
					break;
				}
			};

			return this;
		},

		setMime: function( mime ) {
			this._mime = mime;
			return this;
		},

		getMime: function() {
			return this._mime;
		},

		mimesByGroup: function( group ) {
			return this._MIMETYPE_GROUPS[group];
		},

		isGroup: function( group ) {
			var mimesGroup = this._MIMETYPE_GROUPS[group];

			if( group=='*/*' ) {
				return true;
			}
			else if(!mimesGroup) {
				return false;
			}
			else {
				if( dojo.indexOf(mimesGroup, this._mime)>=0 ) {
					return true;
				}
			}

			return false;
		},

		isPreviewable: function() {
			var prev = this._MIMETYPE_PREVIEWABLE;

			if( dojo.indexOf(prev, this._mime)>=0 ) {
				return true;
			}

			return false;
		},

		getIcon: function() {

			var map = this._ICON_MAP;
			for( var i in map ) {
				if( dojo.indexOf(map[i], this._mime)>=0 ) {
					return i;
				}
			}

			return 'ico_xl_box';
		}

	});

	var _mimetype = new epages.Mimetype();

	epages.mimetype = {
		mimeBySuffix: function( suffix ) {
		// summary: get mimetypeby suffix
			return _mimetype.setBySuffix( suffix );
		},
		isPreviewable: function( mime ) {
		// summary: to check is the givn mimetype previewable
			return _mimetype.setMime( mime ).isPreviewable();
		},
		isGroup: function( mime , group ) {
		// summary: to check is the givn mimetype in a mimetype group
			return _mimetype.setMime( mime ).isGroup( group );
		},
		getGroup: function( group ) {
		// summary: return a group of mimetypes
			return _mimetype._MIMETYPE_GROUPS[group] || ['*/*'];
		},
		isGroupBySuffix: function( suffix , group ) {
		// summary: to check is the givn fileSuffix in a mimetype group
			return _mimetype.setBySuffix( suffix ).isGroup( group );
		},
		getIcon: function( mime ) {
		// summary: returns a SpriteIconClass by the givn mimetype
			return _mimetype.setMime( mime ).getIcon();
		}
	};

})(dojo);

}

if(!dojo._hasResource["epages.widget.Form"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["epages.widget.Form"] = true;
/*
	Copyright (c) 2006-2007, ePages GmbH
	All Rights Reserved.
*/

dojo.provide("epages.widget.Form");


dojo.declare("epages.widget.Form", [dijit._Widget], {
		// summary:
		//					widget linked to the <form/> html element
		//					to enhance the functionality of that particular node

	submitButtons: 	[], 		// array of all submit buttons found inside of the form element on startup
									// you can push your dynamically built button in here hence it will be registered
	
	_formName: 		null,
	_elems:			null,
	_submits:		null,
	_invalides:		null,

	_onSubmit: function() {
		for( var i=0,iLength=this.submitButtons.length ; i<iLength ; i++ ) {
			this.submitButtons[i].disabled = true;
		}
	},

	postCreate: function() {
		this.inherited("postCreate", arguments);
		
		this._elems = {};
		this._submits = {};
		this._invalides = {};
		
		//this.submitButtons = dojo.query("[type=submit]", this.domNode);
		
		//this.connect(this.domNode, "submit", "_onSubmit");
		
		// for new formElements Widgets
		this._formName = dojo.attr( this.domNode , 'name' );
		this.connect(this.domNode, 'submit', this.validate);
	},
	
	validate: function(event) {
		var elems = this._elems;
		var error = false;
		
		for( var i in elems ) {
			error = elems[i].validate() ? error : true;
		};
		
		if( error && event ) {
			event.preventDefault();
			console.log('error')
		}
		return !error;
	},
	
	addElem: function( instance ) {
		if( instance.attr('type')=='submit' ) {
			this._submits[instance.id] = instance;
		}
		this._elems[instance.id] = instance;
	},
	
	removeElem: function( id ) {
		if( this._elems[id] ) {
			delete( this._elems[id] );
		}
		if( this._submits[id] ) {
			delete( this._submits[id] );
		}
	},
	
	change: function() {
		var elems = this._elems;
		var error = false;
		for( var i in elems ) {
			error = elems[i].attr('valide') ? error : true;
		}
		
		var elems = this._submits;
		for( var i in elems ) {
			elems[i].attr('disabled',error);
		}
	},
	
	destroy: function() {
		this.inherited('destroy', arguments);
		
		var elems = this._elems;
		
		for( var i in elems ) {
			elems[i].destroy();
		}
	}
})

}

if(!dojo._hasResource['epages.widget.FormElements']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['epages.widget.FormElements'] = true;
/*
	Copyright (c) 2010, ePages GmbH
	All Rights Reserved.
*/

dojo.provide('epages.widget.FormElements');












(function(dj){

	var windowFormWdgt = new epages.widget.Form( {}, document.createElement('form') );

	dj.declare('epages.widget.FormBase', [dijit._Widget], {

	// VARS
		wrapNode		: null,				// node of wrap which wrapped the domNode
		formNode		: null,				// node of parent form
		_type				: 'text',			// type of formElement
		_attr				: {},					// attributes of formElement
		_handles		: null,				// array of handles which will connect on this._connectHandles

		_translation	: epages.io.dictionary.translation,


	// CREATE
		postCreate: function(){
			this.inherited('postCreate', arguments);

			this._attr = {};
			this._handles = {};
			this._handleConnects = {};
			// get type
			if( this.domNode.nodeName.match(/^(TEXTAREA|SELECT)$/) ){
				this._attr.type = this.domNode.nodeName.toLowerCase();
			}
			else {
				this._attr.type = dj.attr(this.domNode,'type');
			}
			if( this._attr.type == 'datetime-local' ){
				this._attr.type = 'datetime';
			}
			if( !this._attr.type ){
				var match = (this.domNode.outerHTML).match(/[<\s]{1}type="?([A-Za-z]+)"?[>\s\/]{1}/);
				this._attr.type = match ? match[1] : 'text';
			}

			// get attributes
			var ATTR = ['autofocus','form','formaction','formmethod','formnovalidate','id','required','pattern','min','max','name','accept','checked','disabled','list','maxlength','multiple','placeholder','step','value'];

			for( var i=0,l=ATTR.length ; i<l ; i++ ){
				var name = ATTR[i];
				var value = dj.attr(this.domNode,name);
				// there is a bug in dojo.attr => use native javascript for this attribute
				if( name == 'multiple' ){
					value = this.domNode.getAttribute('multiple') === 'true' ? true : false;
				}
				// parse min, max & step to number
				if( name.match(/^(max|min|step)$/) ){
					value = epages.localize.str2number(value);
					if( value !== null ){
						this._attr[name] = value;
					}
				}
				// is defined
				else if( value !== undefined ){
					this._attr[name] = value;
				}
			}
			this._attr.id = this.id;
			this._attr.valide = dj.hasClass(this.domNode,'DialogError') ? false : true;

			// replace domNode for type compatibility
			if( this._attr.type.match(/^(color|date|datetime|email|month|number|search|tel|time|url|week)$/) ){
				var node = dj.clone(this.domNode);
				if( !(epages.Browser.engine == 'MSIE' && epages.Browser.version <= 8) ){
					dj.attr( node , 'type' , 'text' );
				}
				dj.place( node , this.domNode , 'after' );
				node.parentNode.removeChild(this.domNode);
				this.domNode = node;
			}

			// if element hidden by style copy to wrap node
			var style = '';
			var originalStyle = this.domNode.getAttribute('style') || '';
			if(originalStyle !== '') {
				// Start IE Hack
				if( typeof originalStyle === 'object' ){
					var ret = [];
					for( var i in originalStyle ){
						ret.push( i + ":" + name[i] );
					}
					originalStyle = ret.join(';');
				}
				// End IE Hack
				originalStyle.replace(/(display:\s*[a-z]+\s*;?|visibility:\s*[a-z]+\s*;?)/gi, function(str,a){
					style = style+a;
					return "";
				});
			}

			// wrap domNode
			this.wrapNode = dj.create('label',{
				'class': 'EpForm ' + this._attr.type.substr(0,1).toUpperCase() + this._attr.type.substr(1) + ' ' + dj.attr(this.domNode,'class'),
				'style': style
			},this.domNode,'after');
			this.wrapNode.appendChild(this.domNode);

				// set formWidget
				var formNode = (this._attr.form ? dj.query('form[name='+this._attr.form+']') : [this.domNode.form])[0];
				if( !formNode ){
					this.formWdgt = windowFormWdgt;
				}
				else{
					if (dj.hasClass(formNode, 'ep-js')) {
					// We do NOT want *dojo* AND *jQuery* form handling on the same *form*.
						return;
					}
					this.formWdgt = $$( dj.attr(formNode,'widgetid') );
					if( !this.formWdgt ){
						this.formWdgt = new epages.widget.Form( {}, formNode );
					}
				}

				// fix to disable validation by browser
				dj.removeAttr( this.domNode, 'formmethod' );
				dj.removeAttr( this.domNode, 'formaction' );
				dj.removeAttr( this.domNode, 'max' );
				dj.removeAttr( this.domNode, 'min' );
				dj.removeAttr( this.domNode, 'pattern' );
				dj.removeAttr( this.domNode, 'required' );
				dj.removeAttr( this.domNode, 'step' );

				// add this formElement to his form
				this.formWdgt.addElem( this );

				this.addHandle( 'domNode.focus', this.domNode, 'focus', function() {
					dj.addClass(this.domNode,'Active');
					dj.addClass(this.wrapNode,'Active');
				});
				this.addHandle( 'domNode.blur', this.domNode, 'blur', function() {
					dj.removeClass(this.domNode,'Active');
					dj.removeClass(this.wrapNode,'Active');
				});

				if( this._attr.type.match(/^(checkbox|radio)$/) ){
					this.attr( 'startupValue', this._attr.checked ? this._attr.value : '' );
				}
				else{
					this.attr( 'startupValue', this._attr.value );
				}

				// set focus
				if( this._attr.autofocus == true ){
					this._domNode.focus();
				}

				// set the id also
				this.domNode.setAttribute( 'id', this.id );

		},

		// init the readonly and disbaled status
		postAfterCreate: function(){
			this.attr( 'readonly', this._attr.readonly || false );
			this.attr( 'disabled', this._attr.disabled || false );
		},

		// METHODS
		style: function( node, name, value ){
			if( !node.nodeName && !this[node] ){
				value = name;
				name = node;
				node = this.domNode;
			}
			node = node.nodeName || this[node];

			if( typeof name == 'object' ){
				dj.style( node, name );
				return this;
			}
			else if( value == undefined ){
				return dj.style( node, name );
			}
			else{
				dj.style( node, name, value );
				return this;
			}
		},
		attr: function( name, value ){
			if( typeof name == 'object' ){
				for( var i in name ){
					this.attr( i, name[i] );
				}
				return this;
			}
			else {
				var methodName = '_attr'+name.substr(0,1).toUpperCase()+name.substr(1);
				if( value === undefined ){
					return this[methodName] ? this[methodName]() : this._attr[name];
				}
				else {
					return this._attr[name] = this[methodName] ? this[methodName](value) : value;
				}
			}
		},
		_attrMin: function( value ){
			return value ? epages.localize.str2number(value) : this._attr.min;
		},
		_attrMax: function( value ){
			return value ? epages.localize.str2number(value) : this._attr.max;
		},
		_attrStep: function( value ){
			return value ? epages.localize.str2number(value) : this._attr.step;
		},
		_attrType: function( value ){
			// value will be ignored, type can not modify
			return this._attr.type;
		},
		// set/get formElement multiple status
		_attrValue: function( value ){
			if( value ){
				this.domNode.setAttribute('value', value);
				return value;
			}
			else {
				return this._attr.value || '';
			}
		},
		// set/get formElement disabled status
		_attrForm: function( value ){
			if( value || value == '' ){
				var formNodeList;
				if( value == '' ){
					formNodeList = [this.domNodeForm];
				}
				else {
					formNodeList = dj.query('form[name='+value+']') || [];
				}
				if( formNodeList[0] ){
					this.formWdgt.removeElem( this.id );
					this.formWdgt = $$( dj.attr(formNodeList[0], 'widgetid') );
					if( !formWdgt ){
						this.formWdgt = windowFormWdgt;
						this.formWdgt.addElem( this );
					}
					this._attr.form = value;
				}
				return this._attr.form || '';
			}
			else {
				return this._attr.form;
			}
		},
		// set/get formElement disabled status
		_attrDisabled: function( bool ){
			if( bool === true ){
				dj.attr( this.domNode , 'disabled' , true );
				dj.addClass( this.wrapNode , 'Disabled' );

				this._disconnectHandles();
				return bool;
			}
			else if( bool === false ){
				dj.removeAttr( this.domNode , 'disabled' );
				dj.removeClass( this.wrapNode , 'Disabled' );

				if( !this._attr.readonly ){
					this._disconnectHandles();
					this._connectHandles();
				}
				return bool;
			}
			else {
				return this._attr.disabled || false;
			}
		},
		// set/get formElement readonly status
		_attrReadonly: function( bool ){
			if( bool === true ){
				dj.attr( this.domNode , 'readonly', true );
				dj.addClass( this.wrapNode , 'Readonly' );

				this._disconnectHandles();
				return bool;
			}
			else if( bool === false )  {
				dj.removeAttr( this.domNode , 'readonly' );
				dj.removeClass( this.wrapNode , 'Readonly' );

				if( !this.attr('disabled') ){
					this._disconnectHandles();
					this._connectHandles();
				}
				return bool;
			}
			else {
				return this._attr.readonly || false;
			}
		},
		// add handle with id
		addHandle: function( id, node, event, fn  ) {
			if( this._handleConnects[id] ) {
				this.removeHandle( id );
			}
			this._handles[id] = [ typeof node === 'string' ? this[node] : node, event, fn ];

			return this;
		},
		// remove handle by id
		removeHandle: function( id ) {
			if( this._handleConnects[id] ) {
				dj.disconnect( this._handleConnects[id] );
			}
			//delete( this._handles[id] );

			return this;
		},
		// connect handle by id
		connectHandle: function( id ) {
			if( this._handleConnects[id] ) {
				this.removeHandle( id );
			}
			if( this._handles[id] ) {
				var _handle = this._handles[id];
				this._handleConnects[id] = dj.connect( _handle[0], _handle[1], this, _handle[2] );
			}
			return this;
		},
		// disconnect handles by id
		disconnectHandle: function( id ) {
			dj.disconnect( this._handleConnects[id] );

			return this;
		},
		// connect all handles
		_connectHandles: function(){
			var _handles = this._handles;
			for( var i in _handles ){
				this.connectHandle( i );
			}
		},
		// disconnect all handles
		_disconnectHandles: function(){
			var _handles = this._handles;
			for( var i in _handles ){
				this.disconnectHandle( i );
			}
		},
		// replace pseudo pattern with regexp pattern
		_replacePattern: function(){
			if( this._attr.pattern ){
				this._attr.pattern = this._attr.pattern.replace(/(\[\:[a-z\-]+\:\])/,function( t, s ){
					return epages.localize.getPattern( s.substr(2,s.length-4) );
				} );
			}
		},
		// validate formElement
		validate: function(){
			var valide = true;

			if( !this.attr('formnovalidate') ){
				valide = this._validate();
				var maxL = Math.min( this.attr('maxlength') || epages.vars.MaxTextLength, epages.vars.MaxTextLength );
				valide =  maxL && maxL<(dj.attr(this.domNode,'value')+'').length ? false : valide;
				if( valide ){
					dj.removeClass( this.domNode , 'DialogError' );
					dj.removeClass( this.wrapNode , 'DialogError' );
				}
				else {
					dj.addClass( this.domNode , 'DialogError' );
					dj.addClass( this.wrapNode , 'DialogError' );
				}
				this.attr('valide', valide ? true : false );
				if (this.formWdgt) {
					this.formWdgt.change();
				}
			}
			else {
				this.attr('valide', valide ? true : false );
			}

			if( dj.attr( this.domNode,'value') !=  this.attr('startupValue') ){
				dj.addClass( this.domNode, 'Changed' );
				dj.addClass( this.wrapNode, 'Changed' );
			}
			else{
				dj.removeClass( this.domNode, 'Changed' );
				dj.removeClass( this.wrapNode, 'Changed' );
			}

			return valide;
		},
		// placeholder for validate function
		_validate: function(){
			return true;
		},
		// destroy a widget
		destroy: function(){
			this.inherited('destroy',arguments);

			this._disconnectHandles();
			this.formWdgt.removeElem( this.id );
		}

	});


	// WIDGET FormButton
	dj.declare('epages.widget.FormButton', [epages.widget.FormBase], {

	// CREATE
		postCreate: function(){
			this.inherited('postCreate', arguments);

			this.postAfterCreate();
		}

	});


	// WIDGET FormFile
	dj.declare('epages.widget.FormFile', [epages.widget.FormBase], {

		buttonNode: 		null,
		_domNode:				null,
		_handlesIndex: 	null,

		// CREATE
		postCreate: function(){
			this.inherited('postCreate', arguments);

			this.buttonNode = dj.create('span',{
			  'class':''
			},this.domNode,'after');

			this.buttonNode.innerHTML = this._translation.get( this._attr.multiple ? 'FileUploadMultiple' : 'FileUploadSingle');

			this.postAfterCreate();
		},

		// METHODS
		// set/get formElement multiple status
		_attrAccept: function( value ){
			if( value ){
				this.domNode.setAttribute('accept',value);
				return value;
			}
			else {
				return this._attr.accept || '';
			}
		},
		// set/get formElement multiple status
		_attrMultiple: function( bool ){
			if( bool === true ){
				this.domNode.setAttribute('multiple',true);
				return bool;
			}
			else if( bool === false ){
				this.domNode.removeAttribute('multiple');
				return bool;
			}
			else {
				return this._attr.multiple || false;
			}
		},

		_attrAccept: function( value ){
			if( value ){
				var instance = new epages.Mimetype();
				this._attr.accept = value.replace(/([A-Za-z]+\/\*)/,dj.hitch(this,function( string, found ){
					var mimes = instance.mimesByGroup( found );
					if( mimes ) {
						return mimes.join(',');
					}
					else {
						return '';
					}
				}));
				dj.attr( this.domNode, 'accept', this._attr.accept );
				return this._attr.accept;
			}
			else {
				return this._attr.accept;
			}
		},

		// set/get formElement value
		_attrValue: function( value ){
			if( value == '' ){
				this._disconnectHandles();

				var arr = [];
				for( var i=0,iLength=this._handles.length ; i<iLength ; i++ ){
					if( this._handles[i][0] == this.domNode ) {
						arr.push(i);
					}
				}

				this._domNode = this.domNode;
				if( epages.Browser.engine == "Gecko"){
					this.domNode.setAttribute('value','');
				}
				else{
					this.domNode = dj.place( this.domNode.cloneNode(), this._domNode, 'replace' );
					var node = dj.clone(this.domNode);
				}

				for( var i in this._handles ) {
					if( this._handles[i][0] == this._domNode ) {
						this._handles[i][0] = this.domNode;
					}
				}

				for( var i=0,iLength=arr.length ; i<iLength ; i++ ){
					this._handles[arr[i]][0] = this.domNode;
				}

				this._connectHandles();
			}
			else {
				return this._attr.value || '';
			}
		}

	});


// WIDGET FormFileUploder
	dj.declare('epages.widget.FormFileUploader', [epages.widget.FormFile], {

		_uploader:		null,

	// CREATE
		postCreate: function(){
			this.inherited('postCreate', arguments);

			this.addHandle( 'domNode.change.fileUploader', this.domNode, 'change', this._change  );
			this.addHandle( 'domNode.click.fileUploader', this.domNode, 'click', this._click  );

			this.attr({
				'sendaction':	dj.attr(this.domNode,'sendaction'),
				'deleteaction':	dj.attr(this.domNode,'deleteaction'),
				'finishaction':	dj.attr(this.domNode,'finishaction')
			});

			this.formWdgt.removeElem( this.id );

			this.postAfterCreate();
		},

	// METHODS
		// open the uploader if attr uploader is true
		_click:	function( event ){
			if( !this._uploader ){
				this._createUploader();
				this._uploader.hide();
			}
			else {
				event.preventDefault();
				this._uploader.show();
			}
		},
		// open the uploader if attr uploader is true
		_change: function(){
			if( !this._uploader ){
				this._createUploader();
			}
			this._uploader.show();
			this.attr('value', '');
			this._uploader.externalInput( this._domNode );
		},

		_createUploader: function(){
			dj.require('epages.widget.Uploader');

			this._uploader = new epages.widget.Uploader({
				options:{
					id: 						this.attr('id'),
					multiple : 			this.attr('multiple'),
					accept : 				this.attr('accept') || '*/*',
					sendaction : 		this.attr('sendaction'),
					deleteaction : 	this.attr('deleteaction'),
					finishaction : 	this.attr('finishaction')
				}
			}, dj.create('div'));
		}
	});


// WIDGET FormSelect
	dj.declare('epages.widget.FormSelect', [epages.widget.FormBase], {

	// CREATE
		postCreate: function(){
			this.inherited('postCreate', arguments);

			this.addHandle( 'domNode.change.select', this.domNode, 'change', this.validate  );

			this.postAfterCreate();
		},

	// METHODS
		// set/get formElement required status (required cann't be true for type select
		_attrRequired: function(){
			this._attr.required = false;
			return false;
		}
	});


// WIDGET FormText
	dj.declare('epages.widget.FormText', [epages.widget.FormBase], {

	// CREATE
		postCreate: function(){
			this.inherited('postCreate', arguments);

			this.addHandle( 'domNode.keyup.text', this.domNode 		, 'keyup' 	, this.validate );
			this.addHandle( 'domNode.blur.text',  this.domNode 		, 'blur' 		, this.validate );

			if( this.domNode.getAttribute("emptyValue") ){
				var domNode = this.domNode;
				var value = dj.attr( domNode, 'value' );
				var emptyValue = dj.attr( domNode, 'emptyValue' );

				if( !value || value == emptyValue ){
					dj.attr( domNode, 'value', emptyValue );
					dj.addClass( domNode, 'EmptyValue' );
				}

				dojo.connect( this.formWdgt.domNode, "submit", function() {
					domNode.value = domNode.value.replace(domNode.getAttribute('emptyValue'), '');
					dojo.removeClass(domNode, "EmptyValue");
				});
				this.addHandle( 'domNode.focus2.text', domNode, "focus", function() {
					domNode.value = domNode.value.replace(domNode.getAttribute('emptyValue'), '');
					dojo.removeClass(domNode, "EmptyValue");
				});
				this.addHandle( 'domNode.blur2.text', domNode, "blur", function() {
					if(domNode.value == "") {
						dojo.addClass(domNode, "EmptyValue");
						domNode.value = domNode.getAttribute("emptyValue");
					}
				});
	   }

			// replace domNode for type compatibility
			if( this._attr.type.match(/^(email|tel|url)$/) ){
				this._attr.pattern = epages.localize.getPattern( this._attr.type );
			}

			// handle mandatory
			if( dj.hasClass( this.domNode, 'Mandatory' ) && !dj.attr( this.domNode, 'value' ) ){
				dj.addClass( this.domNode, 'Unfilled' );
			}

			this.postAfterCreate();
		},

	// METHODS
		_validate: function(){
			var value = dj.attr( this.domNode , 'value' );
			var valide = false;
			if( value == '' && !this._attr.required ){
				valide = true;
			}
			if( value == '' && this._attr.required ){
				valide = false;
			}
			else if( this._attr.pattern && value.match(new RegExp('^'+this._attr.pattern+'$')) ){
				valide = true;
			}
			else if( !this._attr.pattern ){
				valide = true;
			}
			if( dj.hasClass( this.domNode, 'Mandatory' ) && !dj.attr( this.domNode, 'value' ) ){
				dj.addClass( this.domNode, 'Unfilled' );
			}
			else{
				dj.removeClass( this.domNode, 'Unfilled' );
			}
			return valide;
		}
	});


// WIDGET FormCheck
	dj.declare('epages.widget.FormCheck', [epages.widget.FormBase], {

	// CREATE
		postCreate: function(){
			this.inherited('postCreate', arguments);

			this.addHandle( 'domNode.change.check', this.domNode, 'change', this._change );
			if( epages.Browser.engine == 'MSIE' ){
				this.addHandle( 'domNode.click.check', this.domNode, 'click', this._ieChangeFix );
			}

			if( !this.domNode.value && !this.domNode.defaultValue ){
				this.domNode.defaultValue = "on";
			}

			this.attr( 'checked', this._attr.checked ? true : false );
			this.postAfterCreate();
		},

	// METHODS
		// set/get formElement checked status
		_attrChecked: function(bool){
			if(bool === true){
				if( this._attr.type == 'radio' ){
					var radios = this._getRadios();
					for( var i=0,l=radios.length ; i<l ; i++ ){
						radios[i].attr( 'checked', false );
						dojo.removeClass(radios[i].domNode,'DialogError');
						dojo.removeClass(radios[i].wrapNode,'DialogError');
					}
				}
				dj.attr( this.domNode, 'checked', true );
				dj.addClass( this.wrapNode, 'Checked' );
				return bool;
			}
			else if(bool === false){
				dj.attr( this.domNode, 'checked', false );
				dj.removeClass( this.wrapNode, 'Checked' );
				return bool;
			}
			else {
				return this._attr.checked;
			}
		},
		// a fix for the IE to fire the change action instantly
		_ieChangeFix: function(){
			this.domNode.blur();
		},
		// handle the change
		_change: function(){
			this.attr( 'checked', dj.attr( this.domNode , 'checked' ) ? true : false );
			this.validate();
		},
		// found all radio element with same name in this form and outside of this form which added to this form by the form attribute
		_getRadios: function(){
			var domNodeName = dj.attr(this.domNode,'name');
			var formNodeName = dj.attr(this.formWdgt.domNode,'name');

			var inForm;
			var exForm;
			if( this.formWdgt === windowFormWdgt ){
				inForm = dj.query('input[type=radio][name='+domNodeName+']',this.domNode.ownerDocument);
				exForm = [];
			}
			else{
				inForm = dj.query('input[type=radio][name='+domNodeName+']',this.formWdgt.domNode);
				exForm = dj.query('input[type=radio][name='+domNodeName+'][form='+formNodeName+']',document.body);
			}

			var radios = inForm.concat(exForm);
			var required = this.attr('required');

			var ret = [];
			for( var i=0,l=radios.length ; i<l ; i++ ){
				var wdgt = $$( dj.attr(radios[i],'widgetid') );
				if( wdgt ){
					ret.push( wdgt );
					required = wdgt.attr('required') || required;
					wdgt.attr('valide',true);
				}
			}
			this.attr('required',required);

			return ret;
		},

		_validate: function(){

			var value = dj.attr( this.domNode , 'value' );
			var valide = false;
			if( this._attr.type == 'checkbox' && ( (dj.attr(this.domNode,'checked') && this._attr.required) || !this._attr.required ) ){
				valide = true;
			}

			if( this._attr.type == 'radio' ){
				var radios = this._getRadios();
			}

			if( this._attr.type == 'radio' && ( (this.attr('checked') && this.attr('required')) || !this.attr('required') ) ){
				valide = true;
			}
			else if( this._attr.type == 'radio' ){
				for( var i=0,l=radios.length ; i<l ; i++ ){
					valide = radios[i].attr('checked') || valide;
				}
			}
			return valide;
		}

	});


// WIDGET FormNumber
	dj.declare('epages.widget.FormNumber', [epages.widget.FormBase], {

		stepUpNode: 			null,			// node of the step up button
		stepDownNode: 		null,			// node of the step down button

		_stepCount:				null,			// counter of intervals
		_stepSpeed: 			null,			// interval time
		_stepInterval: 		null,			// interval of step action

	// CREATE
		postCreate: function(){
			this.inherited('postCreate', arguments);

			this._replacePattern();

			this.stepUpNode = dj.create('a',{
					'href':'javascript:;',
					'class':'StepUp'
				});

			this.stepDownNode = dj.create('a',{
					'href':'javascript:;',
					'class':'StepDown'
				});

			this.wrapNode.appendChild(this.stepUpNode);
			this.wrapNode.appendChild(this.stepDownNode);

			this.addHandle( 'domNode.keyup.number',          this.domNode 			, 'keyup' 			, this.validate  );
			this.addHandle( 'domNode.blur.number',           this.domNode 			, 'blur' 				, this.validate  );

			this.addHandle( 'stepUpNode.mousedown.number',   this.stepUpNode 		, 'mousedown' 	, function(event){ event.preventDefault(); this._stepStart(+1); }  );
			this.addHandle( 'stepUpNode.mouseup.number',     this.stepUpNode 		, 'mouseup'   	, this._stepStop  );
			this.addHandle( 'stepUpNode.mouseout.number',    this.stepUpNode 		, 'mouseout'  	, this._stepStop  );

			this.addHandle( 'stepDownNode.mousedown.number', this.stepDownNode 	, 'mousedown' 	, function(event){ event.preventDefault(); this._stepStart(-1); }  );
			this.addHandle( 'stepDownNode.mouseup.number',   this.stepDownNode 	, 'mouseup' 		, this._stepStop  );
			this.addHandle( 'stepDownNode.mouseout.number',  this.stepDownNode 	, 'mouseout' 		, this._stepStop  );

			// set pattern
			this._attr.pattern = this._attr.pattern || epages.localize.getPattern( 'number' );

			this.postAfterCreate();
		},

	// METHODS
		// handle the step
		_stepAction: function( inc ){
			this._stepCount++;
			this._stepSpeedHandle( inc );

			var value = dj.attr( this.domNode , 'value' );
			var ret;
			var step = inc * (this._attr.step || 1);
			var attrMin = this._attr.min;
			var attrMax = this._attr.max;


			ret = epages.localize.str2number(value);
			ret = ret!==null ? ret : 0;
			var i = (ret+'').indexOf('.');
			var l = i>=0 ? (ret+'').substr(i+1).length : 0;
			ret = ret+step;
			ret = (attrMin && attrMin > ret.toFixed(l)) ? attrMin : (attrMax && attrMax < ret.toFixed(l)) ? attrMax : ret.toFixed(l);
			ret = epages.localize.number2str( ret );
			dj.attr( this.domNode , 'value' , ret );
		},
		// start the step action
		_stepStart: function( inc ){
			this._stepCount = 0;
			this._stepSpeed = 250;

			this._stepAction(inc);
			this._stepInterval = window.setInterval( dj.hitch(this,function(){
				this._stepAction(inc);
			}), this._stepSpeed );
		},
		// stop the step action
		_stepStop: function(){
			window.clearInterval( this._stepInterval );
			epages.event.fire(this.domNode,'blur');
		},
		// handle the speed of the step action
		_stepSpeedHandle: function( inc ){
			if( dj.indexOf([10,30,50],this._stepCount)>=0 ){
				this._stepSpeed = this._stepSpeed / 2;
				window.clearInterval( this._stepInterval );
				this._stepInterval = window.setInterval( dj.hitch(this,function(){
					this._stepAction(inc);
				}), this._stepSpeed );
			}
		},

		_validate: function(){
			var value = dj.attr( this.domNode , 'value' );
			var valide = false;

			if( value=='' && !this._attr.required ){
				valide = true;
			}
			else if( value.match(new RegExp('^'+this._attr.pattern+'$')) ){
				var val = epages.localize.str2number(value);
				var attrMin = this._attr.min;
				var attrMax = this._attr.max;
				valide = (attrMin && attrMin > val) ? false : (attrMax && attrMax < val) ? false : true;

			}
			return valide;
		}

	});


// WIDGET FormDate
	dj.declare('epages.widget.FormDate', [epages.widget.FormNumber], {

	// CREATE
		postCreate: function(){
			this.inherited('postCreate', arguments);

			// set pattern by type
			this._attr.pattern = epages.localize.getPattern( this._attr.type );
		},

	// METHODS
		_stepAction: function( inc ){
			this._stepCount++;
			this._stepSpeedHandle( inc );

			var value = dj.attr( this.domNode , 'value' );
			var ret;
			var attrStep = this._attr.step || 1;
			var attrMin = this._attr.min;
			var attrMax = this._attr.max;

			var dateObj = epages.localize.date2obj( value , this._attr.type ) || new Date();
			switch( this._attr.type ){
				case 'datetime':
					dateObj.setHours( dateObj.getHours() + (inc * attrStep) );
				break;
				case 'month':
					dateObj.setMonth( dateObj.getMonth() + (inc * attrStep) );
				break;
				case 'time':
					dateObj.setHours( dateObj.getHours() + (inc * attrStep) );
				break;
				case 'week':
					dateObj.setDate( dateObj.getDate() + (inc * attrStep * 7) );
				break;
				case 'date':
				default:
					dateObj.setDate( dateObj.getDate() + (inc * attrStep) );
				break;
			}
			var dateTime = dateObj.getTime()/1000;
				dateTime = (attrMin && attrMin > dateTime) ? attrMin : (attrMax && attrMax < dateTime) ? attrMax : dateTime;

			dateObj.setTime( dateTime*1000 );

			ret = epages.localize.obj2date( dateObj , this._attr.type );

			dj.attr( this.domNode , 'value' , ret );
		},

		_validate: function(){
			var value = dj.attr( this.domNode , 'value' );
			var valide = false;

			if( value=='' && !this._attr.required ){
				valide = true;
			}
			else if( value.match(new RegExp('^'+this._attr.pattern+'$')) ){
				var dateTime = epages.localize.date2obj( value , this._attr.type ).getTime()/1000;
				var attrMin = this._attr.min;
				var attrMax = this._attr.max;

				valide = (attrMin && attrMin > dateTime) ? false : (attrMax && attrMax < dateTime) ? false : true;

			}
			return valide;
		}

	});

})(dojo);

}

if(!dojo._hasResource["epages.widget.FormElement"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["epages.widget.FormElement"] = true;
/*
	Copyright (c) 2006-2007, ePages GmbH
	All Rights Reserved.
*/

dojo.provide("epages.widget.FormElement");


(function(dj){

	epages.changeInputType = function( node, type ){
			var _node = dj.clone(node);
				dj.attr( _node , 'type' , type );
				dj.attr( _node , 'value' , node.value );
				dj.place( _node , node , 'after' );
				_node.parentNode.removeChild(node);
			return _node;
		};

	epages.widget.FormElement = function( options, domNode ){
		this.destroy();

		var nodeName = domNode.nodeName.toLowerCase(),
			nodeType,
			widget;

		if( nodeName == 'select' ){
			widget = new epages.widget.FormSelect(options,domNode);
		}
		else if( nodeName == 'textarea' ){
			widget = new epages.widget.FormText(options,domNode);
		}
		else{
			if( dojo.hasClass( domNode, 'Checkbox' ) ){
				domNode = domNode.type != 'checkbox' ? epages.changeInputType( domNode, 'checkbox' ) : domNode;
				widget = new epages.widget.FormCheck(options,domNode);
			}
			else if( dojo.hasClass( domNode, 'Radio' ) ){
				domNode = domNode.type != 'radio' ? epages.changeInputType( domNode, 'radio' ) : domNode;
				widget = new epages.widget.FormCheck(options,domNode);
			}

			nodeType = domNode.getAttribute('type');

			if( /^(file)$/i.test(nodeType) ){
				widget = new epages.widget.FormFile(options,domNode);
			}
			else if( /^(text|password)$/i.test(nodeType) ){
				widget = new epages.widget.FormText(options,domNode);
			}
		}

		if( widget ){

			widget.elementNode = widget.domNode;
			widget.setValue = function( val ) {
				return this.attr( 'value', val );
			}

			widget.setStartUpValue = function( val ) {
				this.attr( 'startupValue', val );
				this.validate();

				return this;
			}

			widget.getValue = function() {
				return this.attr( 'value' );
			}

			widget.isChecked = function() {
				return this.attr( 'checked' );
			}

			widget.setChecked = function( val ){
				return this.attr( 'checked', val );
			}

			return widget;
		}
	};
	epages.widget.FormElement.prototype = new dijit._Widget();

})(dojo);

}


dojo.i18n._preloadLocalizations("epages.nls.ep_base", ["ROOT","ca","da","de","en","es","fi","fr","it","nl","no","pt","ro","ru","sv","xx"]);
